#!/usr/bin/python
# -*- coding: utf-8 -*-

import subprocess
from sigpdv.tools.config import config
from datetime import datetime
from sigpdv.tools.log import traced as Log
from sigpdv.models.cad_comandos_impressoras_data import CaixaCadComandosImpressoras
import os


class CadComandosImpressoras(object):
    def __init__(self, parent):
        self.parent = parent
        self.set_impressoras()

    def set_impressoras(self):
        self.impressoras = CaixaCadComandosImpressoras.query.all()

    def print_qrcode_elgin_epson(self, code):
        """
        Prints the QR code Elgin e Epson
        Parameters:
            PL and PH - defines (pL + pH * 256) for number of bytes according to
                        pH(cn, fn, and [parameters])
            cn - defines symbol type (48 - PDF417 and 49 - QR code)
            fn - defines the function
            parameters - specifies the process of each function
        """

        # QR code size - 1D 28 6B pl(3) ph(0) cn(49) fn(67) n(3)
        # 1 <= n <= 16
        qr_code = '\x1ba1'
        qr_code += '\x1d(k\x03\x00%s%s%s' % (chr(49), chr(67), chr(4))

        # Level error correction - 1D 28 6B pl(3) ph(0) cn(49) fn(67) n(48)
        # 48 <= n <=51 (levels L, M, Q, H)
        qr_code += '\x1d(k\x03\x00%s%s%s' % (chr(49), chr(69), chr(50))

        # Store data in symbols storage area - 1D 28 6B pl ph cn(49) fn(80) m(48) data
        bytes_len = 3 + len(code)
        pl = chr(bytes_len & 0xff)
        ph = chr((bytes_len >> 8 & 0xff))
        cmd = '\x1d(k%s%s%s%s%s%s' % (pl, ph, chr(49), chr(80), chr(48), code)
        qr_code += cmd

        # Print - 1D 28 6B pl(3) ph(0) cn(49) fn(81) m(48)
        qr_code += '\x1d(k\x03\x00%s%s%s' % (chr(49), chr(81), chr(48))
        return qr_code

    def print_qrcode_daruma(self, code, level='Q', wmod=4):
        #  * Print QR Code
        #  * [ESC] <129> <–Size><+Size> <Width> <Ecc> <D001> <D002> . . . <Dnnn>
        #  * Size inclui os 2 bytes de controle
        #  *    Size ≤ 402
        #  *    nnn = Size – 2
        #  *   Largura do módulo (Width): 0, 4 ≤ Width ≤ 7 ( =0 para default = 5)
        #  *   Redundância (ECC): 0, M, Q, H ( =0 para cálculo automático)
        #  * Apenas para V.02.50.00 ou superior.
        #  *
        #  * @param string $data   Dados a serem inseridos no QRCode
        #  * @param string $level  Nivel de correção L,M,Q ou H
        #  * @param int    $modelo modelo de QRCode none
        #  * @param int    $wmod   largura da barra 4 ~ 7
        #  */
        size = len(code) + 2
        if size > 402:
            return ''

        nH = int(round(size/256, 0))
        nL = size % 256
        if wmod > 7 or wmod < 4:
            wmod = 5

        # //set error correction level
        level = level.upper()
        if level in ('M', 'Q', 'H'):
            ecc = level
        else:
            ecc = 0

        return '\x1bl\x17\x1b\x81' + chr(nL) + chr(nH) + chr(wmod) + str(ecc) + code + '\x1bl\x01'

    def _comando_criar_arquivo(self, path, text):
        f = open(path, 'w')
        f.write(text)
        f.close()

    def _comando_imprimir_na_porta(self, path):
        # Manda o texto direto na porta onde a impressora esta conectada sem passar por algum driver
        os.system("echo %s | sudo -S chmod 776 %s; cat %s > %s" %
                  (config['senha_usuario'] or 'sig',
                   config['porta_impressora'],
                   path,
                   config['porta_impressora']))

    def supcci_imprimir(self, text):
        if not self.impressoras:
            self.set_impressoras()

        cmd_impressora = False
        for i in self.impressoras:
            marca , modelo = config['impressoras'].split(' | ')
            if i.supcci_marca == marca and i.supcci_modelo == modelo:
                cmd_impressora = i
                break

        path = '/tmp/documento-%s' % datetime.now().strftime('%d-%m-%Y-%H:%M:%s')
        espaco_linha = cmd_impressora.supcci_espacamento_linha or ''
        corta_papel = '\n' + cmd_impressora.supcci_corta_papel or ''

        # comandos para codigo de barras da ficha
        altura_cb = '\x1DhZ'
        margem_esquerda_cb = ''
        largura_cb = '\x1Dw\x03'
        posicao_numero_cb = '\x1DH\x02'
        # margem_esquerda_cb =  '\x1Dk\x84\x14\x00\xE0\x03'
        tipo_cb = '\x1DkI'
        if cmd_impressora.supcci_marca.upper() == 'BEMATECH':
            qr = '\x1ba\x01'          # esse código faz a centralização
            qr += '\x1dkQ'            # esse é o código do qr code
            qr += '\x02\x08\x08\x01'  # aqui é o tamanho
            qr += chr(len(self.parent.nfce_qr_code))  # tamanho de caracteres correspondente ao texto
            qr += '\x00'              # bit de partida
            qr += str(self.parent.nfce_qr_code)  # Texto
        elif cmd_impressora.supcci_marca.upper() in ('ELGIN', 'EPSON', 'BEMATECH/ELGIN'):
            qr = self.print_qrcode_elgin_epson(str(self.parent.nfce_qr_code))
        elif cmd_impressora.supcci_marca.upper() == 'DARUMA':
            text = '\x1b\x14' + text
            qr = self.print_qrcode_daruma(str(self.parent.nfce_qr_code))

        if cmd_impressora.supcci_marca.upper() == 'ELGIN':
            corta_papel = '\n\n' + corta_papel

        gaveta = cmd_impressora.supcci_gaveta or ''
        sub1 = cmd_impressora.supcci_sub_liga or ''
        sub2 = cmd_impressora.supcci_sub_desliga or ''
        f1, f2 = cmd_impressora.supcci_fonte_liga or '', cmd_impressora.supcci_fonte_desliga or ''
        cf1 = cmd_impressora.supcci_fonte_cond_liga or ''  # ligar fonte condencada
        cf2 = cmd_impressora.supcci_fonte_cond_desliga or ''  # desligar fonte condencada
        b1, b2 = cmd_impressora.supcci_negrito_liga or '', cmd_impressora.supcci_negrito_desliga or ''
        if text != '{gaveta}':
            text = str(espaco_linha + text + corta_papel)

        text = text.replace('{.', '')
        text = text.format(sub1=sub1, sub2=sub2, f1=f1, f2=f2, cf1=cf1, cf2=cf2, b1=b1, b2=b2,
                           corta=corta_papel,
                           a_cb=altura_cb, l_cb=largura_cb, pn_cb=posicao_numero_cb,
                           me_cb=margem_esquerda_cb, t_cb=tipo_cb, qrcode=qr, gaveta=gaveta)
        text = text.replace('\\\\x1b', '\x1b').replace('\\\\x1d', '\x1d').replace('\\\\x57', '\x57').\
               replace('\\\\x12', '\x12').replace('\\\\x0f', '\x0f').replace('\\\\x01', '\x01').\
               replace('\\\\x02', '\x02').replace('\\\\x03', '\x03').replace('\\\\x0c', '\x0c'). \
               replace('\\\\x08', '\x08').replace('\\\\x00', '\x00').replace('\\\\r', '\r'). \
               replace('\\\\x8c', '\x8c').replace('\\\\x20', '\x20')
        text = text.decode('string_escape')
        try:
            if cmd_impressora.supcci_marca.upper() == 'DARUMA' and len(text.split('\n')) > 24:
                novo_text = text.split('\n')
                tam = len(novo_text)
                for i in range(0, tam/24):
                    n_path = path+'_parte-%s' % (i+1)
                    self._comando_criar_arquivo(n_path, '\n'.join(novo_text[i*24: (i+1)*24]))
                    self._comando_imprimir_na_porta(n_path)
                if tam % 24:
                    n_path = path+'_parte-%s' % (i+2)
                    self._comando_criar_arquivo(n_path, '\n'.join(novo_text[(i+1)*24: (i+2)*24]))
                    self._comando_imprimir_na_porta(n_path)
            else:
                self._comando_criar_arquivo(path, text)
                if cmd_impressora.supcci_marca.upper() in ('ELGIN', 'EPSON', 'DARUMA', 'BEMATECH/ELGIN'):
                    self._comando_imprimir_na_porta(path)
                else:
                    subprocess.call(["lp", "-o raw", "-n1", '-d%s' % config['impressora'], path])

        except Exception as e:
            Log.error("Erro na impressao: %s" % e)
            return False

        return True