#!/usr/bin/python
# -*- encoding: utf-8 -*-

from sigpdv.tools.log import traced as Log
from sigpdv.models.cad_tributacao_data import CaixaCadTributacao
from sigpdv.control.utils import get_numeros


class CadTributacao():
    def __init__(self):
        pass

    @staticmethod
    def get_tributacao_by_id(tributacao_id):
        res = []
        query_tributacao = CaixaCadTributacao.query.filter(CaixaCadTributacao.id == tributacao_id)
        if query_tributacao.count():
            res = query_tributacao.one()
        return res

    @Log
    def get_tributacao(self):
        dict_trib = {}
        query_tributacao = CaixaCadTributacao.query.all()

        for trib in query_tributacao:
            cod_trib_ecf = trib.suptri_codigo_trib_ecf
            dict_trib[cod_trib_ecf] = {'Descricao': trib.suptri_name,
                                       'TribRetag': trib.suptri_codigo_trib_retag,
                                       'Aliquota': trib.suptri_aliquota,
                                       'Base': trib.suptri_base,
                                       'TipoImposto': trib.suptri_tipo_imposto,
                                       'TipoTributacao': trib.suptri_tipo_tributacao}
        return dict_trib

    @staticmethod
    def format_st(cod=False):
        """
        Retorna a Situação Tributária formatada
        Ex: T07,00%
            F1
        """
        res = {}
        query_tributacao = CaixaCadTributacao.query.all()

        for tributacao in query_tributacao:
            vr_aliquota = get_numeros(str(tributacao.suptri_aliquota)).rjust(4, '0')
            sub = ['F', 'I', 'N']
            if tributacao.suptri_tipo_imposto == 'C':
                st = '%sT%s' % (tributacao.suptri_codigo_trib_ecf, vr_aliquota)
                if tributacao.suptri_tipo_tributacao in sub:
                    st = '%s1' % tributacao.suptri_tipo_tributacao

            elif tributacao.suptri_tipo_imposto == 'S':
                st = '%sS%s' % (tributacao.suptri_codigo_trib_ecf, vr_aliquota)
                if tributacao.suptri_tipo_tributacao in sub:
                    st = '%sS1' % tributacao.suptri_tipo_tributacao

            res[tributacao.suptri_codigo_trib_ecf] = st

        if cod:
            res = res[cod]

        return res