#!/usr/bin/python
# -*- encoding: utf-8 -*-

from datetime import datetime, timedelta
from decimal import Decimal, ROUND_DOWN
from PySide import QtCore, QtGui
from stoqdrivers.exceptions import ItemAdditionError, CancelItemError, CouponNotOpenError, \
    TimeoutError, OutofPaperError, CouponOpenError
from sigpdv import root_path
from sigpdv.control.mov_cupom_servidor import MovCupomServidor
from sigpdv.control.cad_tributacao import CadTributacao
from sigpdv.control.mov_corpos import MovCorpos
from sigpdv.control.mov_fechamentos import MovFechamentos
from sigpdv.control.mov_vencimentos import MovVencimentos
from sigpdv.control.cad_kits import CadKits
from sigpdv.control.mov_kits_vendidos import MovKitsVendidos
from sigpdv.control.mov_meios_pagamentos import MovMeiosPagamentos
from sigpdv.control.cad_clientes import CadClientes
from sigpdv.control.mov_autorizacoes import MovAutorizacoes
from sigpdv.control.relatorio_gerencial import GerencialReport
from sigpdv.control.utils import ler_arq_socio_torcedor
from sigpdv.control.utils import remove_accents, checar_conexao, converte_tz
from sigpdv.control.sorteio import CadSorteio
from sigpdv.control.gerar_nfe_saida import NFCeFrente
from nfe.nfc_e import nfce as nfc_e
from nfe.nfce.consrecinfce import ProcNFCe
from sigpdv.models.cad_itens_data import CaixaCadItens
from sigpdv.models.cad_finalizadora_data import CaixaCadFinalizadora
from sigpdv.models.cad_finzalizadora_na_ecf_data import CaixaCadFinalizadoraNaEcf
from sigpdv.models.cad_tributacao_nfce_data import CaixaCadTributacaoNfce
from sigpdv.models.mov_fechamentos_data import CaixaMovFechamentos
from sigpdv.models.cad_mensagens_clientes_data import CaixaCadMensagensClientes
from sigpdv.models.cfg_mensagens_data import CaixaCfgMensagens
from sigpdv.models.cad_operada_cartao_data import CaixaCadOperadoraCartao
from sigpdv.models.supervisor import SupervisorPveFechamento
from sigpdv.models.db_utils import b_sess, exec_sql, modulo_instalado_promocoes
from sigpdv.dav_pve.dav_gestao import DavGestao
from sigpdv.tools.log import traced as Log
from sigpdv.tools.config import config
from sigpdv.tools.cardinals import NumeroPorExtenso
from sigpdv.ui.dlpPwd import password_dlg
from sigpdv.ui.messagebox import YES_ROLE, NO_ROLE, MSG_ERROR, MSG_QUESTION, MSG_INFO, \
    MSG_NONE, message_dlg
from SigUtils.cnpj import Cnpj
from SigUtils.cpf import Cpf
from SigUtils.numero import getNumeroEditado
import socket
import time
import calendar
import re
import os

TWOPLACES = Decimal(10) ** -2
TREPLACES = Decimal(10) ** -3

RESTRICOES = {1: u'Restrição de venda por item!',
              2: u'Restrição de venda por quantidade!'}

retricao_item, restricao_qtd = 1,2

# STATUS DO CUPOM
coupon_closed, coupon_opened, coupon_add_discount, coupon_add_surchage, coupon_add_payment, \
coupon_finish, mesa_closed, mesa_opened, mesa_transf, itens_mesa_transf = range(10)

# STATUS DO ITEM
item_none, item_add_quantity, item_add_discount, item_add_surchage, item_cancel, item_balanca = range(6)

# Metodos de Pagamento
pm_dinheiro, pm_cheque, pm_cartao, pm_ticket, pm_outros, pm_pagamento, pm_end = range(7)

# STATUS DA TELA
in_login, in_menu, in_menu_fiscal, in_coupon, in_ficha, in_close_turn, in_ss, in_search_client, \
in_add_payment, in_dav, in_pve, in_importacao, in_item_menu_fiscal, in_end_sale, \
in_restaurante, in_update, in_troca_ficha, in_pesar_produtos = range(18)

stk_page_menu, stk_page_login, stk_page_coupom, stk_page_ficha, stk_page_payments, stk_page_ss, \
    stk_search_client, stk_dav_pre, stk_tef, stk_fiscal, stk_importacao, stk_restaurant, \
    stk_consulta_ch, stk_update, stk_page_pesar_produtos = range(15)


class CupomFiscal(object):
    def __init__(self, parent):
        self.parent = parent
        self.rodape_cupom = self.get_rodape_cupom()

    def get_cabecalho_cupom(self):
        texto1 = self.parent.loja.supcfc_cliente.upper().center(48)
        texto2 = (self.parent.loja.supcfc_endereco or u'').upper() + u' N:' + \
                  u'%s - ' % self.parent.loja.supcfc_numero + u'B:' + \
                 (self.parent.loja.supcfc_bairro or u'').upper()
        aux = ''
        text_aux = ''
        for i in texto2.split(' '):
            if len(text_aux) + 1 + len(i) > 48:
                aux += text_aux.center(48) + '\n'
                text_aux = i
                continue

            text_aux += ' ' + i

        aux += text_aux.center(48)
        texto2 = aux
        cep = ''
        if self.parent.loja.supcfc_cep:
            cep = '%s-%s' % (self.parent.loja.supcfc_cep[:5], self.parent.loja.supcfc_cep[-3:])

        texto3 = u'CEP:%s - %s - %s' % ((cep),
                                        (self.parent.loja.supcfc_cidade or u'').upper(),
                                        (self.parent.loja.supcfc_estado))
        cnpj = ''
        if self.parent.loja.supcfc_cnpj:
            cnpj = Cnpj().format(self.parent.loja.supcfc_cnpj)

        texto4 = u'CNPJ: %s IE: %s' % (cnpj, self.parent.loja.supcfc_inscricao_estadual)
        texto5 = ''
        if self.parent.loja.supcfc_inscricao_municipal:
            texto5 = u'IM:   ' + (self.parent.loja.supcfc_inscricao_municipal or u'')

        texto6 = ''
        if self.parent.loja.supcfc_telefone:
            texto6 = 'Tel: ' + self.parent.loja.supcfc_telefone

        coo_nao_fiscal = self.get_coo_cup_gerencial()
        texto7 = u'%s                 COO:%06d ' % (datetime.now().strftime('%d/%m/%Y %H:%M:%S'),
                                                      coo_nao_fiscal)
        texto8 = u'Cupom Gerencial'
        return u'{:^48}\n{:<48}\n{:^48}\n{:<48}\n{:<48}\n{:<48}\n{:<48}\n{:^48}\n{:-<48}'.format(texto1,
                                                 texto2, texto3, texto4, texto5, texto6, texto7, texto8, '')

    def get_rodape_cupom(self):
        ini_negrito = '{b1}'
        fim_negrito = '{b2}'
        texto1 = ('CAIXA: %03d  LOJA: %04d' % (config['caixa'], config['loja'])).ljust(48)
        texto2 = 'DESENVOLVIDO POR:'.ljust(48)
        texto3 = '%sSig Solucoes em Informatica Ltda. (31)2106-1700%s' % (ini_negrito, fim_negrito)
        # todo: verificar o corta papel e o negrito
        # texto4 = '%s%s' % (chr(27), chr(109))  # corta papel
        texto4 = '{corta}'   # corta papel
        return u'{:-<48}\n{:<48}\n{:<48}\n{:<48}'.format('', texto1, texto2, texto3)

    def get_texto_cupom(self, text):
        '''
        Monta o texto completo do cupom de cordo com o texto passado como corpo
        :param text: string
        :return: string
        '''
        return '\n'.join([self.get_cabecalho_cupom(), text, self.rodape_cupom]) + '\n\n'

    @Log
    def cria_capa_cupom_gerencial(self, cod_totalizador=0, valor=Decimal(0), tipo='W',
                                  comprovante=False, estacionamento=False):
        """
        Cria a o cupom nao fiscal no banco de dados
        :param cod_totalizador: 
        :param valor: 
        :return: 
        """
        data_ini = datetime.now()
        data_cupom = data_ini.date()
        nro_cupom = self.get_coo_cup_gerencial()
        self.parent.ui.lblNumCaixa.setText(u'%03d <b>Cupom #: %06d' % (self.parent.caixa_id,
                                                                       nro_cupom))
        self.parent._coupon_id = nro_cupom
        tipo_cc = comprovante and 'CC' or 'CN'
        cupom_vinculado = comprovante and self.parent.mov_capa and \
                          self.parent.mov_capa.supcap_nro_cupom or None
        # --------------------------------- Cria a capa do cupom --------------------------------- #
        vals_capa = dict(supcap_data_cupom=data_cupom,
                         supcap_loja=self.parent.loja.supcfc_loja,
                         supcap_caixa=self.parent.caixa_id,
                         supcap_nro_cupom=nro_cupom,
                         supcap_operador_id=self.parent.user_logged.id,
                         supcap_cliente=self.parent.cliente.supcli_cnpj_cpf,
                         supcap_data_inicio=data_ini,
                         supcap_data_fim=data_ini,
                         supcap_tipo=tipo,
                         supcap_valor_desc_acres=0,
                         supcap_total_cupom=valor,
                         supcap_mf_adicional=False,
                         supcap_nro_proprietario=self.parent.loja.supcfc_loja,
                         supcap_tipo_comprovante=tipo_cc,
                         supcap_subtotal=valor,
                         supcap_indicador_cancelamento='N',
                         supcap_cancelamento_acrescimo=0,
                         supcap_cancelado_posteriormente=False,
                         supcap_nome_cliente=self.parent.cliente.name,
                         supcap_cod_vendedor=self.parent.vendedor_codigo,
                         supcap_cod_tot_nao_fiscal=cod_totalizador,
                         supcap_nro_cupom_vinculado=cupom_vinculado)
        if tipo == 'A':
            dados = exec_sql("""SELECT supcap_nro_cupom AS nro
                                FROM caixa_mov_capa 
                                WHERE supcap_state IS NOT NULL
                                ORDER BY supcap_nro_cupom DESC
                                LIMIT 1""", 'C')
            vals_capa['supcap_nro_ultima_nfce'] = dados and dados[0].get('nro') or 0

        res = self.parent.obj_mov_capa.create(vals_capa)
        self.parent._capas_gerencial.append(res)
        self.set_coo_cup_gerencial()
        if not comprovante and not estacionamento:
            self.parent._coupon = self.parent.mov_capa = res

        if not self.parent.vr_ficha: # para ficha o cupom servidor sera criado depois dw adicionar as formas de pagamento
            # Cupom Servidor do comprovante de Credito/Debito
            MovCupomServidor().create({'supcse_capa_id': res.id})

        return True

    def get_cupom_aberto(self):
        return self.parent.obj_mov_capa.search({'supcap_data_fim': None,
                                                'supcap_loja': config['loja'],
                                                'supcap_caixa': config['caixa']})
    @Log
    def get_nota_sem_enviar(self):
        """Busca a ultima nota e verifica se tem """
        res = exec_sql("""SELECT MAX(cmc.id) AS id
                          FROM caixa_mov_capa AS cmc
                          WHERE cmc.supcap_tipo = 'V'
                          """)
        if not res or not res[0].get('id'):
            return False

        capa_id = res[0].get('id')
        res = exec_sql("""SELECT id 
                    FROM caixa_mov_cupom_servidor 
                    WHERE supcse_capa_id = %s""" % capa_id)
        if res and res[0].get('id'):
            return False

        return self.parent.obj_mov_capa.search({'id': capa_id})

    @Log
    def check_coupon_is_opened(self):
        """
        Verifica se existe algum cupom fiscal aberto na ECF. Se houver, pergunta ao usuário se
        deseja tentar recuperar ou anular/cancelar o cupom fiscal.
        :return: Boolean
        """
        if self.parent._state_coupon == coupon_opened:
            return True

        cupom_aberto = self.get_cupom_aberto()
        if not cupom_aberto:
            cupom_aberto = self.get_nota_sem_enviar()

        if cupom_aberto:
            # recuperra nsu do socio torcedor se houver
            try:
                self.parent._tef_nsu_socio_torcedor = ler_arq_socio_torcedor()
                if self.parent._tef_nsu_socio_torcedor:
                    self.parent.sig_tef.usando_sitef = True
                    self.parent.sig_tef.modalidade.value = 64
            except:
                pass

            msg = 'Existe uma Nota em aberto. Deseja tentar recuperá-la?'
            if self.parent.linhas_display == 2:
                msg = 'Nota em aberto. Tentar recuperar?'

            coo = cupom_aberto.supcap_nro_cupom
            mov_capa = cupom_aberto
            self.parent.mov_capa = mov_capa
            if message_dlg(self.parent, MSG_QUESTION, msg) == NO_ROLE:
                self.parent.cancelar_nota_fiscal(automatico=True)
                message_dlg(self.parent, MSG_INFO, 'Nota %06d, cancelada.' % coo)
                self.parent._clear_globals(coupon_closed)
                self.parent.clear_widget_table(self.parent.ui.tableItens)
                return False

            self.parent._coupon = mov_capa

            acres_desc = mov_capa.supcap_alteracao
            valor_desc_acres = mov_capa.supcap_valor_desc_acres

            if acres_desc == 'D':
                self.parent._coupon_amount = self.parent._coupon_amount - valor_desc_acres
                self.parent._indicador_desc_acre = 2

            elif acres_desc == 'A':
                self.parent._coupon_amount = self.parent._coupon_amount + valor_desc_acres
                self.parent._indicador_desc_acre = 3

            self.parent.ui.lblInfoDescCapa.setText(u'%1.2f' % valor_desc_acres)
            total_cupom = mov_capa.supcap_total_cupom
            # Verifica Cliente
            if mov_capa.supcap_cliente:
                self.parent._search_cliente(mov_capa.supcap_cliente)

            mov_corpos = MovCorpos(self.parent).search({'supcor_capa_id': mov_capa.id},
                                                  order_by='supcor_sequencia')

            if not mov_corpos and total_cupom:
                coo = mov_capa.supcap_nro_cupom
                self.parent.cancelar_nota_fiscal(automatico=True)
                message_dlg(self.parent, MSG_INFO, 'Nota Cancelada!\n\n'
                                            'Não foi possível recuperar a Nota %06d.' % coo)
                self.parent._clear_globals(coupon_closed)
                self.parent._set_controls_default_values()
                return False

            if not isinstance(mov_corpos, list):
                mov_corpos = [mov_corpos]

            self.parent.mov_corpos = mov_corpos
            for corpo in mov_corpos:
                sequencia = corpo.supcor_sequencia - 1
                self.parent.last_item = int(corpo.supcor_sequencia)
                cod_ficha = False
                if config['usar_ficha']:
                    cod_ficha = corpo.supcor_codigo_ficha or False

                self.parent.add_item_in_tableitens(corpo.supcor_item, sequencia,
                                                   corpo.supcor_nome_item,
                                                   corpo.supcor_quantidade,
                                                   corpo.supcor_unidade,
                                                   corpo.supcor_preco_unitario,
                                                   corpo.supcor_preco_total,
                                                   [corpo.supcor_desconto_item,
                                                    corpo.supcor_acrescimo_item],
                                                   corpo.supcor_venda_cancelamento,
                                                   insert=False,
                                                   cod_ficha=cod_ficha)

            if config['balanca_active_auto']:
                self.parent.peso_total_venda_pesado = mov_capa.supcap_peso_pesado or 0
                self.parent.peso_total_venda_cadastro = mov_capa.supcap_peso_cadastrado or 0
                self.parent.tara = mov_capa.supcap_tara or 0

            message_dlg(self.parent, MSG_INFO, 'Nota %06d recuperada com sucesso!.' % coo)
            self.parent._coupon_id = coo
            self.parent._state_coupon = coupon_opened
            self.parent._coupon_subtotal = total_cupom
            fechamentos = CaixaMovFechamentos.query.filter_by(supfec_capa_id=self.parent.mov_capa.id).all()
            if fechamentos:
                self.parent._coupon_totalize = total_cupom
                troco = 0
                total_pago = 0
                for i in fechamentos:
                    total_pago += i.supfec_valor - i.supfec_troco
                    troco += i.supfec_troco

                dif = total_cupom - total_pago
                self.parent.ui.lblVlrTotalR.setText(u'%.2f' % total_cupom) # Valor que ja foi pago
                self.parent.ui.lblVlrPgto.setText(u'%.2f' % total_pago) # Valor que ja foi pago
                self.parent._payment_amount = total_pago
                self.parent.ui.lblVlrReceber.setText(u'%.2f' % dif) # Valor restante a pagar
                self.parent.ui.lblVlrTroco.setText(u'%1.2f' % troco)
                self.parent.ui.spinPgtoDinheiro.setValue(dif) # Valor restante a pagar
                self.parent.KEY_DINHEIRO_Click()
                if dif < 0.01:
                    self.parent.ui.stackedActions.setCurrentIndex(stk_page_payments)
                    self.parent.ui.stackedPgto.setCurrentIndex(pm_end)
                    self.parent.setControlFocus(self.parent._CONTROL_FOCUS,
                                                self.parent.ui.stackedPgto)
                    self.parent._state_coupon = coupon_finish
                    tef = self.parent.sig_tef.usando_sitef
                    self.finaliza_pagamento(tef)

                return False

            if self.parent.mov_capa.supcap_totalizado:
                self.parent.KEY_DINHEIRO_Click()
                return False

        return True

    def get_numero_nf(self):
        """
        Retorna o numero de controle da nota que esta no PdvConfig
        :return: 
        """
        return config['numero_nfce']

    def set_numero_nf(self, soma=True):
        """
        Grava no PdvConfig o numero da proxima nota. 'soma' = True faz a soma caso contrario subtrai
        :param soma: bool
        :return:
        """
        if soma:
            numero = config['numero_nfce'] + 1
        else:
            numero = config['numero_nfce'] - 1

        config.set('numero_nfce', numero)
        config.save()
        return True

    def get_numero_dav(self):
        """
        Retorna o numero de controle do DAV que esta no PdvConfig
        :return:
        """
        return config['nro_dav']

    def set_numero_dav(self, soma=True):
        """
        Grava no PdvConfig o numero do proxima DAV. 'soma' = True faz a soma caso contrario subtrai
        :param soma: bool
        :return:
        """
        if soma:
            numero = config['nro_dav'] + 1
        else:
            numero = config['nro_dav'] - 1

        config.set('nro_dav', numero)
        config.save()
        return True

    def get_numero_contingencia(self):
        """
        Retorna o numero de controle da contingencia que esta no PdvConfig
        :return:
        """
        return config['numero_nfce_contingencia']

    def set_numero_contingencia(self, soma=True):
        """
        Grava no PdvConfig o numero da proxima contingencia. 'soma' = True faz a soma caso contrario subtrai
        :param soma: bool
        :return:
        """
        if soma:
            numero = config['numero_nfce_contingencia'] + 1
        else:
            numero = config['numero_nfce_contingencia'] - 1

        config.set('numero_nfce_contingencia', numero)
        config.save()
        return True

    def get_coo_cup_gerencial(self):
        """
        Retorna o numero de controle da nota que esta no PdvConfig
        :return: 
        """
        return config['coo_cup_gerencial']

    def set_coo_cup_gerencial(self):
        """
        Grava no PdvConfig o numero do proxim cupom. 
        :param soma: bool
        :return: 
        """
        config.set('coo_cup_gerencial', config['coo_cup_gerencial'] + 1)
        config.save()
        return True
    @Log
    def open_coupon(self):
        """
        Abre o cupom fiscal na ECF e cria a capa do cupom no banco de dados.
        :return: Retorna os dados do cupom criado no banco de dados.
        """
        if not self.parent.user_logged or not self.parent.user_logged.id:
            message_dlg(self.parent, MSG_ERROR, 'Operador não está logado!')
            return False

        self.parent.limpa_variaveis_abertura_cupom()
        finalizadoras = CaixaCadFinalizadora.query.order_by('name').limit(1).all()
        if not finalizadoras:
            raise Exception('Não há finalizadoras cadastradas na base de dados.')

        if self.parent.cliente.name:
            self.identificar_cliente(endereco=config['imprimirendereco'],
                                     cidade=config['imprimircidade'],
                                     insc_estadual=config['imprimirinscricaoestadual'])
        cupom_aberto = self.get_cupom_aberto()
        if cupom_aberto:
            self.parent._coupon = self.parent.mov_capa = isinstance(cupom_aberto, list) and \
                                                         cupom_aberto[0] or cupom_aberto
            return cupom_aberto

        # NF Legal
        uf = self.parent.loja.supcfc_estado or ''
        uf = uf.upper()
        if uf == 'DF':
            self.parent.get_tot_tax()
            Log.info('Nota Legal DF')

        data_ini = datetime.now()
        data_cupom = data_ini.date()
        nro_cupom = self.get_numero_nf()
        self.parent.ui.lblNumCaixa.setText(u'%03d <b>Cupom #: %06d' % (self.parent.caixa_id,
                                                                       nro_cupom))
        self.parent._coupon_id = nro_cupom

        # --------------------------------- Cria a capa do cupom --------------------------------- #
        vals_capa = dict(supcap_data_cupom=data_cupom,
                         supcap_loja=self.parent.loja.supcfc_loja,
                         supcap_caixa=self.parent.caixa_id,
                         supcap_nro_cupom=nro_cupom,
                         supcap_operador_id=self.parent.user_logged.id,
                         supcap_cliente=self.parent.cliente.supcli_cnpj_cpf,
                         supcap_data_inicio=data_ini,
                         supcap_tipo='V',
                         supcap_valor_desc_acres=0,
                         supcap_total_cupom=0,
                         supcap_mf_adicional=False,
                         supcap_nro_proprietario=1,
                         supcap_tipo_comprovante='RV',
                         supcap_subtotal=0,
                         supcap_indicador_desconto='V',
                         supcap_indicador_acrescimo='V',
                         supcap_indicador_cancelamento='N',
                         supcap_cancelamento_acrescimo=0,
                         supcap_cancelado_posteriormente=False,
                         supcap_nome_cliente=self.parent.cliente.name,
                         supcap_cod_vendedor=self.parent.vendedor_codigo)
        if modulo_instalado_promocoes:
            vals_capa['scanntech'] = 2

        if self.parent._IdDav:
            vals_capa['supcap_numero_pedido'] = self.parent._IdDav.supdva_nro_pedido

        res = self.parent.obj_mov_capa.create(vals_capa)
        self.set_numero_nf()
        # ------------------------------------------ // ------------------------------------------ #

        self.parent._coupon = self.parent.mov_capa = res
        self.get_tara()

        if self.parent._IdPve:
            b_sess.begin()
            self.parent._IdPve.NroCupom = nro_cupom
            b_sess.commit()

        return res

    def verifica_restricao_item(self, codigo_barras, qtd_item):
        """
        Verifica se o item em questao possui algum tipo de restricao
        :param codigo_barras: Codigo de barras do item vendido
        :param qtd_item: quantidade vendida
        :return:
        """
        retricoes = self.parent.obj_restricao_item.get_by(supres_codigo_barra=codigo_barras)
        tipo_restricao = 0
        if not retricoes:
            return False

        if retricoes.supres_solicita_sup:
            tipo_restricao = retricao_item
        elif retricoes.supres_max and retricoes.supres_max < qtd_item:
            tipo_restricao = restricao_qtd

        if tipo_restricao and self.parent.user_logged.supope_operador_supervisor == 'O':
            self.parent.ativa_beep()
            msg = u"Informe ao Supervisor!\n" + RESTRICOES.get(tipo_restricao)
            dlgpwd = password_dlg(self.parent, msg_info=msg)
            if dlgpwd.retval:
                pass
            else:
                self.parent.ativa_beep()
                self.parent.ui.edtCodigo.clear()
                self.parent.ui.spinQtda.setValue(1)
                message_dlg(self.parent, MSG_INFO,
                            'Operador: %s.\n\nNão tem permissão para '
                            'executar esta operação' %
                            str(self.parent.user_logged.name))
                return True

        return False

    @Log
    def add_item(self, query_item, quantidade, vendedor_codigo=None, nome_vendedor=None,
                 discount_surcharge=[], serie='', cod_ficha=False, valor_ficha=0):
        """
        Adiciona um item ao cupom/ecf
        :param: cod_item: código do produto
        :rtype: id do item no cupom
        """
        if not quantidade:
            self.parent.ativa_beep()
            msg = '<center><h2><font color="red">A quantidade informada está incorreta.</font>' \
                  '</h2><br><center><h2><font color="white">Pressione <font color="yellow">[Sair]' \
                  '</font> para digitar a quantidade correta.</font></h2>'
            tipo_msg = MSG_NONE
            if config['autoatende']:
                tipo_msg = MSG_INFO
            message_dlg(self.parent, tipo_msg, msg)
            self.parent.ui.edtCodigo.setText(query_item.supite_codigo_item)
            self.parent._returnPressed_edtCodigo()
            QtGui.QApplication.processEvents()
            return self.parent.KEY_QTDA_Click()

        if self.verifica_restricao_item(query_item.supite_codigo_item, quantidade):
            return False

        self.parent._lock_keyboard(True)
        self.parent.ui.edtCodigo.clear()

        item_id = False
        str_serie = ', No. Serie: %s' % serie if serie else serie

        # Verifica se é kit
        itens_kit, quantidades, vr_unitario_no_kit, descontos_kit_item = CadKits(self.parent).\
                                                        get_itens_kit(query_item.supite_codigo_item)

        codigo_kit = False
        kits_vendidos = {}
        qtd_kit = quantidade
        desc_acres_rateado = []
        if itens_kit:
            codigo_kit = query_item.supite_codigo_item
            itens = itens_kit
            if discount_surcharge:
                desc_acres_rateado = CadKits(self.parent).rateia_desc_acres_kit(quantidades,
                                                 vr_unitario_no_kit, qtd_kit, discount_surcharge[1])
        else:
            itens = query_item

        if not isinstance(itens, list):
            itens = [itens]

        inc = 0
        for item in itens:
            desconto_item_kit = 0
            tributacao_id = item.supite_tributacao_id
            cad_tributacao = CadTributacao.get_tributacao_by_id(tributacao_id)
            if not cad_tributacao:
                self.parent.ativa_beep()
                msg = 'O Código: %d, não está cadastrado da tabela de Tributações' % tributacao_id
                message_dlg(self.parent, MSG_ERROR, msg)
                QtGui.QApplication.processEvents()
                return False

            cod_trib_ecf = str(cad_tributacao.suptri_codigo_trib_ecf)

            try:
                item_price = item.supite_preco if not self.parent.vlr_item_dav else self.parent.vlr_item_dav
                if itens_kit:
                    quantidade = qtd_kit * quantidades[inc] or Decimal(1)
                    item_price = vr_unitario_no_kit[inc] or Decimal(0)
                    desconto_item_kit = descontos_kit_item[inc] or Decimal(0)

                qtd_item = quantidade
                # este valor eh referente ao valor da ficha que pode ser diferente do valor do item
                # caso a ficha foi vendida a um valo X e depois o item mudou para Y
                if valor_ficha:
                    item_price = valor_ficha

                item_price = Decimal(item_price).quantize(TWOPLACES)
                desconto_acrescimo = (0, 0)

                item_amount = Decimal(item_price * qtd_item).quantize(TWOPLACES,
                                                                      rounding=ROUND_DOWN)
                if item_amount < Decimal('0.01'):
                    msg = "A Quantidade vezes valor Unitário é \n" \
                          "menor que zero considerando " \
                          "2 casas Decimais!\nQuantidade: %s\nValor Uitário: R$ %s\n" \
                          "Total: R$ %s" % (qtd_item, item_price, Decimal(item_price * qtd_item))
                    if qtd_item >= Decimal('1'):
                        msg = "Valor do item não pode ser zero!"
                    message_dlg(self.parent, MSG_ERROR, msg.replace('.', ','))
                    return False

                if discount_surcharge:
                    state_item = discount_surcharge[0]
                    value = discount_surcharge[1]
                    # desconto por valor rateado para itens do kit
                    if desc_acres_rateado:
                        value = desc_acres_rateado[inc]

                    desconto_acrescimo = self.get_desconto_acrescimo(state_item, value, item_amount,
                                                                     item_price, qtd_item)
                acrescimo_kit_item = 0
                # quando o desconto for negativo, quer dizer que o kit custa mais caro que a soma
                # dos itens do kit, portando precisa dar um acrescimo.
                if desconto_item_kit < 0:
                    acrescimo_kit_item = desconto_item_kit * -1
                    desconto_item_kit = 0
                    desconto_acrescimo = (0, desconto_acrescimo[1])
                    self.parent._state_item = item_add_surchage
                    self.parent.ui.tableItens.horizontalHeaderItem(5).setText(u'Acrésc.')
                else:
                    self.parent.ui.tableItens.horizontalHeaderItem(5).setText('Desc.')


                desconto_acrescimo = (desconto_acrescimo[0] + desconto_item_kit * qtd_kit,
                                      desconto_acrescimo[1] + acrescimo_kit_item * qtd_kit)
                und = item.supite_unidade
                if not und or len(und) < 2:
                    und = 'UN'
                else:
                    und = und[:2]

                if self.parent._state_coupon == coupon_closed:
                    # --- Cria a capa do cupom --- #
                    open_coupon = self.open_coupon()
                    if open_coupon:
                        self.parent._state_coupon = coupon_opened
                    else:
                        self.parent._set_screen_state(in_menu_fiscal)
                        QtGui.QApplication.processEvents()
                        if self.parent._coupon_id:
                            msg = 'Erro ao gravar o cupom %06d no banco de dados.' \
                                  % self.parent._coupon_id
                        else:
                            msg = 'Erro ao abrir o cupom'

                        message_dlg(self.parent, MSG_ERROR, msg)
                        return False

                if self.parent.comanda_id:
                    for x in self.parent.vendedor_comanda_ids:
                        if query_item == x[1][0] and self.parent.last_item == x[0]:
                            vendedor_codigo = x[2]
                            nome_vendedor = x[3]
                    if type(vendedor_codigo) is not int:
                        vendedor_codigo = self.parent.vendedor_codigo = 0
                        nome_vendedor = self.parent._nome_vendedor = None

                desc = desconto_acrescimo[0] * -1 if desconto_acrescimo[0] < 0 else desconto_acrescimo[0]
                acre = desconto_acrescimo[1] * -1 if desconto_acrescimo[1] < 0 else desconto_acrescimo[1]
                descricao = ''
                if item.supite_cest:
                    descricao += item.supite_cest + '#'
                if item.supite_ncm:
                    descricao += item.supite_ncm + '#'

                descricao += item.name.rstrip() + str_serie
                item_name = item.name if config['descricao_resumida'] else item.supite_descricao_completa
                total_item = item_amount
                supcor_preco_total = total_item
                vr_liquido = supcor_preco_total - desc + acre
                chave_ibpt = item.supite_ibpt_chave
                msg_trib = []
                trib_icms = CaixaCadTributacaoNfce.get_by(suptnf_codigo=item.supite_tributacao_icms_id)
                trib_pis = CaixaCadTributacaoNfce.get_by(suptnf_codigo=item.supite_tributacao_pis_id)
                trib_cofins = CaixaCadTributacaoNfce.get_by(suptnf_codigo=item.supite_tributacao_cofins_id)
                if not item.supite_tributacao_icms_id or not trib_icms:
                    msg_trib.append('ICMS')
                if not item.supite_tributacao_pis_id or not trib_pis:
                    msg_trib.append('PIS')
                if not item.supite_tributacao_cofins_id or not trib_cofins:
                    msg_trib.append('COFINS')
                if msg_trib:
                    self.parent.ativa_beep()
                    message_dlg(self.parent, MSG_ERROR, 'Item sem tributação: %s' % ', '.join(msg_trib))
                    self.parent.vlr_item_dav=False
                    self.parent.ui.spinQtda.setValue(1)
                    self.parent.tratar_except()
                    self.parent.check_button_status()
                    QtGui.QApplication.processEvents()
                    return False

                # CFOPS validos 5101; 5102; 5103; 5104; 5115; 5401; 5403; 5405; 5653; 5656; 5667 e 5933
                anp = (item.supite_codigo_anp or '').replace(' ', '') or 0
                try:
                    anp = int(anp)
                except Exception as e:
                    Log.error('Erro Codigo ANP: %s' % e)
                    message_dlg(self.parent, MSG_ERROR,
                                u"""Código ANP do item é inválido:\nCódigo: "%s" """ % anp)
                    return False

                if anp == 210203001 and ((item.supite_pglp or 0) +
                                          (item.supite_pgnn or 0) +
                                          (item.supite_pgni or 0) ) < 100:
                    message_dlg(self.parent, MSG_ERROR,
                                u'Somatório percentuais de GLP derivado do petróleo, '
                                u'GLGNn e GLGNi diferente de 100"')

                    return False

                if anp:
                    cfop = 5656  # venda de combustivel/lubrificante
                elif item.supite_producao == 'P':
                    parametros = self.parent.obj_parametros.get_by()
                    cfop = parametros.supcfp_cfop or 5101  # venda de producao propria
                else:
                    cfop = 5102  # venda de mercadoria de terceiros

                if trib_icms.suptnf_tipo_tributacao.strip() == 'U' and not anp:  # substituicao tributaria
                    cfop = 5405

                item_id = self.parent.last_item + 1
                self.parent.last_item = item_id
                # ----------------------------- Cria o corpo do cupom ---------------------------- #
                vals_item = dict(supcor_sequencia=item_id,
                                 supcor_item=item.supite_codigo_item,
                                 supcor_quantidade=qtd_item,
                                 supcor_nro_serie_item=serie,
                                 supcor_preco_total=supcor_preco_total,
                                 supcor_desconto_item=desc,
                                 supcor_desconto_item_sem_rat=desc,
                                 supcor_acrescimo_item=acre,
                                 supcor_acrescimo_item_sem_rat=acre,
                                 supcor_venda_cancelamento='V',
                                 supcor_nome_item=item_name,
                                 supcor_unidade=und,
                                 supcor_preco_unitario=item_price.quantize(TWOPLACES),
                                 supcor_tributacao_ecf=str(cad_tributacao.suptri_codigo_trib_ecf),
                                 supcor_indicador_arr_trunc='T',
                                 supcor_indicador_ppt='T',
                                 supcor_ncm=item.supite_ncm,
                                 supcor_cest=item.supite_cest,
                                 supcor_capa_id=self.parent.mov_capa.id,
                                 supcor_cod_vendedor=vendedor_codigo,
                                 supcor_nome_vendedor=nome_vendedor,
                                 supcor_ibpt_chave= chave_ibpt,
                                 supcor_nro_serie=self.parent.mov_capa.supcap_nro_serie or None,
                                 supcor_codigo_trib_icms=trib_icms.suptnf_codigo,
                                 supcor_codigo_trib_pis=trib_pis.suptnf_codigo,
                                 supcor_codigo_trib_cofins=trib_cofins.suptnf_codigo,
                                 supcor_codigo_cfop=cfop,
                                 supcor_codigo_retaguarda=item.supite_codigo_retaguarda,
                                 supcor_codigo_anp=anp,
                                 supcor_adrem=item.supite_adrem or 0,
                                 supcor_pglp=item.supite_pglp,
                                 supcor_pgnn=item.supite_pgnn,
                                 supcor_pgni=item.supite_pgni,
                                 supcor_vpart=item.supite_vpart,
                                 supcor_producao=item.supite_producao,
                                 supcor_origem_mercadoria=item.supite_origem_mercadoria or '0',
                                 supcor_aliquota_ibpt_fed=item.supite_aliquota_ibpt_fed or 0,
                                 supcor_aliquota_ibpt_est=item.supite_aliquota_ibpt_est or 0,
                                 supcor_aliquota_ibpt_mun=item.supite_aliquota_ibpt_mun or 0,
                                 supcor_getin=item.supite_getin,
                                 supcor_codigo_cst_icms=trib_icms.suptnf_codigo_cst,
                                 supcor_tipo_tributacao_icms=trib_icms.suptnf_tipo_tributacao.replace(' ', ''),
                                 supcor_codigo_csosn_icms=trib_icms.suptnf_codigo_csosn,
                                 supcor_aliquota_icms=trib_icms.suptnf_aliquota or 0,
                                 supcor_codigo_beneficio_icms=trib_icms.suptnf_codigo_beneficio,
                                 supcor_motivo_beneficio_icms=trib_icms.suptnf_motivo_beneficio.replace(' ', ''),
                                 supcor_aliquota_fcp_icms=trib_icms.suptnf_aliquota_fcp or 0,
                                 supcor_aliquota_pis=trib_pis.suptnf_aliquota or 0,
                                 supcor_codigo_cst_pis=trib_pis.suptnf_codigo_cst,
                                 supcor_aliquota_cofins=trib_cofins.suptnf_aliquota or 0,
                                 supcor_codigo_cst_cofins=trib_cofins.suptnf_codigo_cst)

                if config['usar_ficha'] and cod_ficha:
                    vals_item['supcor_codigo_ficha'] = cod_ficha

                if codigo_kit:
                    if codigo_kit not in kits_vendidos:
                        kit_vendido = MovKitsVendidos(self.parent).create(
                                                            {'supmkv_codigo_kit': codigo_kit,
                                                             'supmkv_quantidade': qtd_kit})
                        kit_vendido_id = kit_vendido.id
                        kits_vendidos.update({codigo_kit: kit_vendido_id})
                    else:
                        kit_vendido_id = kits_vendidos.get(codigo_kit)

                    vals_item['supcor_kit_vendido_id'] = kit_vendido_id

                self.parent.mov_corpos.append(MovCorpos(self.parent).create(vals_item))
                total_cupom = (self.parent.mov_capa.supcap_total_cupom or 0) + vr_liquido
                vals_capa = dict(supcap_subtotal=total_cupom,
                                 supcap_total_cupom=total_cupom)
                self.parent.obj_mov_capa.write(self.parent.mov_capa, vals_capa)
                # -------------------------------------- // -------------------------------------- #

                self.parent.vlr_item_dav = False
                self.parent.add_item_in_tableitens(item.supite_codigo_item, item_id,
                                                   item_name, qtd_item, und,
                                                   item_price.quantize(TWOPLACES), vr_liquido,
                                                   desconto_acrescimo, 'V', insert=True,
                                                   cod_ficha=cod_ficha)
                self.parent.ui.lblInfoAcresTotalItens.setText(
                    u'%1.2f' % (Decimal(self.parent.ui.lblInfoAcresTotalItens.text()) + acre))
                self.parent.ui.lblInfoDescTotalItens.setText(
                    u'%1.2f' % (Decimal(self.parent.ui.lblInfoDescTotalItens.text()) + desc))
                self.parent.desconto_total_itens += desc

                inc += 1

                self.parent.infoText('%r item(s) adicionado(s) ao cupom!' % self.parent.last_item,
                                     False)
                if self.parent.teclado:
                    self.parent.teclado.envia_linha_display(("Ad. item %s " %item.name)[:40], 2)
                    self.parent.linhas_display == 2 and self.parent.teclado.go_to(26, 1)

            except ItemAdditionError:
                self.parent.ativa_beep()
                Log.error('**************************** NAO ADICIONOU ****************************')
                self.parent.vlr_item_dav=False
                self.parent.ui.spinQtda.setValue(1)
                self.parent.tratar_except()
                self.parent.check_button_status()
                QtGui.QApplication.processEvents()
                return False
            except Exception:
                self.parent.ativa_beep()
                Log.error('**************************** NAO ADICIONOU ****************************')
                self.parent.vlr_item_dav=False
                self.parent.ui.spinQtda.setValue(1)
                self.parent.tratar_except(tipo_msg=MSG_INFO)
                self.parent.check_button_status()
                QtGui.QApplication.processEvents()
                return False

        self.parent._lock_keyboard(False)
        self.parent.check_button_status()
        # auto atendimento e nao usando comanda, calcula o peso total da compra quando a balanca estiver ativada
        if config['balanca_active_auto'] and not config['usarcomanda']:
            self.verifica_peso_produto(query_item, quantidade, item_name, item_id)

        return item_id

    def _cancelar_ultimo_item_autoatendimento(self, item_id):
        self.parent.show_info_proccess(False, '')
        msg = u'Peso do produto não está compatível com o cadastrado!<br>' \
              u'O produto será cancelado da compra!<br>' \
              u'Informe o fato ao atendente!'
        msg = u'<font color="white" size="4">%s</font>' % msg
        message_dlg(self.parent, MSG_INFO, msg)
        self.parent._tab_order = [self.parent.ui.tableItens]
        self.parent.ui.tableItens.selectRow(item_id - 1)
        self.parent.setControlFocus(self.parent._CONTROL_FOCUS, self.parent.ui.tableItens)
        self.parent._state_item = item_cancel
        self.parent._returnPressed_tableItens(item_id - 1, True)

    @Log
    def get_tara(self):
        # pega o peso da balanca antes de comecar a venda, pois pode ter sacolas na balanca
        if not config['balanca_active_auto']:
            return False

        valor = self.parent.obter_peso_balanca_auto_atendimento() or 0
        self.parent.tara = valor
        self.parent.obj_mov_capa.write(self.parent.mov_capa, dict(supcap_tara=valor))
        return valor

    @Log
    def verifica_peso_produto(self, item, quantidade, nome, item_id, msg='', cont=0):
        if not msg:
            msg = u'Coloque o item na balança ao lado!<br><br>%s-%s<br>Quantidade: %s' % \
                  (item.supite_codigo_item, nome, quantidade)
            msg = u'<font color="white" size="4">%s</font>' % msg
        self.parent.show_info_proccess(True, msg.encode('UTF-8'))
        parametros = self.parent.obj_parametros.get_by()
        peso_total = (self.parent.obter_peso_balanca_auto_atendimento() or 0) - self.parent.tara
        dif_pesagem = abs(self.parent.peso_total_venda_pesado - peso_total)
        # Aguarda colocar o produto na balanca para fazer a comparacao
        # caso o peso anterior seja o mesmo da nova pesagem, fica pesando ate colocar o peso na balanca
        while dif_pesagem <= 5:
            time.sleep(0.3)
            peso_total = (self.parent.obter_peso_balanca_auto_atendimento() or 0) - self.parent.tara
            dif_pesagem = abs(self.parent.peso_total_venda_pesado - peso_total)

        # se o novo peso for menor que o peso anterior, algum produto foi retirado da balanca
        if self.parent.peso_total_venda_pesado - peso_total > 0:
            if cont < parametros.supcfp_qtd_cancela:
                Log.info('Peso de anterior: %s' % self.parent.peso_total_venda_pesado)
                Log.info('Peso de final: %s' % peso_total)
                msg = u'Todos o itens da compra precisam estar na balaça!<br>' \
                      u'Aperte "OK" e tente novamente!'
                msg = u'<font color="white" size="4">%s</font>' % msg
                message_dlg(self.parent, MSG_INFO, msg)
                return self.verifica_peso_produto(item, quantidade, nome, item_id, cont=cont + 1)

            # cancela o ultimo item
            self._cancelar_ultimo_item_autoatendimento(item_id)
            return False

        peso_cadastro = (Decimal(item.supite_peso_da_embalagem * 1000) * quantidade) or 0
        # se o item for apenas pesavel, usa o peso do item na diferenca da pesagem total
        if item.supite_pesavel:
            peso_cadastro = dif_pesagem
            # se o item for fracionado, usa a quantidade como preco, pois nao tem preco cadastrado
            if item.supite_fracionado:
                peso_cadastro = quantidade * 1000

        peso_tot_cadastro = (self.parent.peso_total_venda_cadastro + peso_cadastro)
        dif_total = abs(peso_total - peso_tot_cadastro)
        tolerancia = parametros.supcfp_tolerancia_fixo or 0
        if parametros.supcfp_tolerancia_percentual:
            tolerancia = peso_tot_cadastro * (parametros.supcfp_tolerancia_percentual / Decimal(100))

        # a margem de erro da balanca eh de 5, por isso a tolerancia nao pode ser menor que 5
        if tolerancia < 5:
            tolerancia = 5

        # diferenca de peso do cadastrado com o pesado maior que a tolerancia
        if dif_total > tolerancia:
            if cont < parametros.supcfp_qtd_cancela:
                msg = u'Produto incorreto!<br>' \
                      u'Retire o Item da balança, aperte "OK" e tente novamente!<br><br>%s-%s<br>Quantidade: %s' % \
                      (item.supite_codigo_item, nome, quantidade)
                msg = u'<font color="white" size="4">%s</font>' % msg
                message_dlg(self.parent, MSG_INFO, msg)
                Log.info('Peso de cadastro: %s' % peso_cadastro)
                Log.info('Peso de balanca: %s' % dif_pesagem)
                return self.verifica_peso_produto(item, quantidade, nome, item_id, cont=cont+1)

            # cancela o ultimo item
            self._cancelar_ultimo_item_autoatendimento(item_id)
            return False

        # item pesado com sucesso, segue a venda
        self.parent.peso_total_venda_pesado = peso_total
        self.parent.peso_total_venda_cadastro = peso_tot_cadastro
        vals_capa = dict(supcap_peso_cadastrado=peso_tot_cadastro,
                         supcap_peso_pesado=peso_total)
        self.parent.obj_mov_capa.write(self.parent.mov_capa, vals_capa)
        self.parent.show_info_proccess(False, '')
        return True

    def set_vendedor(self):
        '''
        Grava o vendedor nos itens que nao o possuem(ex: selecionou o vendedor ao final da venda)
        '''
        mov_corpos =  MovCorpos(self.parent).search({'supcor_capa_id': self.parent.mov_capa.id,
                                          'supcor_venda_cancelamento': 'V'},
                                            order_by='supcor_sequencia') or []
        if mov_corpos and not isinstance(mov_corpos, list):
            mov_corpos = [mov_corpos]

        for corpo in mov_corpos:
            if corpo.supcor_cod_vendedor:
                continue

            corpo.supcor_cod_vendedor = self.parent.vendedor_codigo
            corpo.supcor_nome_vendedor = self.parent._nome_vendedor

        self.parent.mov_capa.supcap_cod_vendedor = self.parent.vendedor_codigo

        return True

    @Log
    def totalizar_cupom(self):
        """
        Verifica se o cupom está totalizado
        :return: 
        """
        self.parent.obj_mov_capa.write(self.parent.mov_capa, {'supcap_totalizado': True})
        return self.parent.mov_capa.supcap_total_cupom

    @Log
    def aplica_desc_socio_torcedor(self):
        '''
          Aplica o desconto do Socio Torcedor no Tef e na impressora
        '''
        if not self.parent._tef_nsu_socio_torcedor: # nao ha desconto para socio torcedor
            return False

        now = datetime.now()
        coo = str(self.parent.mov_capa.supcap_nro_cupom)
        data_fiscal = now.strftime('%Y%m%d')
        horario = now.strftime('%H%M%S')
        nome_operador = str(self.parent.user_logged.name)
        nsu_socio = self.parent._tef_nsu_socio_torcedor
        lista_produtos = ''
        sequencia = 1
        dict_desconto = {}
        desconto_total = 0
        # na impressora -----------------------
        for sequencia_item in sorted(self.parent.dict_mov_corpos_socio.keys()):
            corpo = self.parent.dict_mov_corpos_socio.get(sequencia_item).get('corpo')
            desconto = self.parent.dict_mov_corpos_socio.get(sequencia_item).get('desconto')
            codigo_produto = corpo.supcor_item[1:]
            if not desconto: # itens que nao tiveram desconto
                continue

            item = corpo.supcor_sequencia
            qtd = corpo.supcor_quantidade
            corpo.supcor_desconto_item = desconto
            corpo.supcor_desconto_item_sem_rat = desconto
            desconto_total += desconto
            if codigo_produto not in dict_desconto:
                dict_desconto[codigo_produto] = {'desconto': desconto, 'qtd': qtd}
            else:
                dict_desconto[codigo_produto]['desconto'] +=  desconto
                dict_desconto[codigo_produto]['qtd'] += qtd

        for codigo_produto in dict_desconto:
            # desconto em centavos e sem pontuacao
            desconto = str(Decimal(dict_desconto.get(codigo_produto).get('desconto')*Decimal(100))
                           .quantize(0)).replace('.','')
            qtd = dict_desconto.get(codigo_produto).get('qtd')
            lista_produtos += codigo_produto + ':' + desconto + ':' + str(int(qtd)) + ';'

        lista_produtos = str(lista_produtos[:-1])

        # no tef ------------------
        if lista_produtos:
            self.parent.sig_tef.registra_desconto_socio_torcedor(coo,
                                                                 data_fiscal,
                                                                 horario,
                                                                 nome_operador,
                                                                 nsu_socio,
                                                                 lista_produtos,
                                                                 sequencia)
        self.parent.dict_mov_corpos_socio = {}
        self.parent.controle_cancelamento_tef = True
        return "Desconto Total: %s" % desconto_total

    @Log
    def get_desconto_socio_torcedor(self):
        '''
        Esta funcao aplica no final da venda desconto para o socio torcedor caso haja algum produto na promocao.
        '''
        self.parent._tef_nsu_socio_torcedor = ler_arq_socio_torcedor()
        if not (self.parent.sig_tef.usando_sitef and self.parent._tef_nsu_socio_torcedor):
            return False

        self.parent.lista_itens_desconto_socio = '' # este campo sera atribuido no arquivo iterativo.py
        conta_produto = 1
        self.parent.dict_mov_corpos_socio = {}
        dict_itens_vendidos = {}
        desconto_promocao = map(lambda x: '%s' % x.get('sequencia_item'),
                            self.parent.list_itens_desconto_promo)
        for corpo in self.parent.mov_corpos:
            if corpo.supcor_venda_cancelamento == 'C':
                continue

            codigo_item = corpo.supcor_item[1:]
            sequencia = str(corpo.supcor_sequencia) # sequencia em que o item foi vendido na ECF
            # retira o desconto do socio torcedor nos item que estao em alguma promocao
            if sequencia in desconto_promocao:
                continue

            self.parent.dict_mov_corpos_socio[sequencia] = {'corpo':corpo}
            if not dict_itens_vendidos.get(codigo_item):
                dict_itens_vendidos[codigo_item] = {'qtd': int(corpo.supcor_quantidade),
                                                    'vr_unitario': str((Decimal(corpo.supcor_preco_unitario).
                                                                        quantize(TWOPLACES))).replace('.','')}
            else:
                dict_itens_vendidos[codigo_item]['qtd'] += int(corpo.supcor_quantidade)

        self.parent.str_lista_itens_socio = '' # lista que sera enviada para o TEF para consulta de itens
        for codigo_item in dict_itens_vendidos:
            valor_unitario = dict_itens_vendidos.get(codigo_item).get('vr_unitario')
            qtd = str(dict_itens_vendidos.get(codigo_item).get('qtd'))
            self.parent.str_lista_itens_socio += codigo_item + ':' + valor_unitario + ':' + qtd + ';'
            conta_produto +=1
            if conta_produto > 200: # a cada 200 produtos 'consulta_registra_produtos_ambev' deve ser chamado
                conta_produto = 1
                self.parent.sig_tef.consulta_registra_produtos_ambev()
                self.parent.str_lista_itens_socio = ''

        if self.parent.str_lista_itens_socio:
            self.parent.sig_tef.consulta_registra_produtos_ambev()

        total_desconto = Decimal(0).quantize(TWOPLACES)
        dados_desconto = {}
        for linha in self.parent.lista_itens_desconto_socio.split(';'):
            if not linha:
                continue
            dados_produto = linha.split(':')

            # Produto (EAN) da tabela
            codigo_produto = dados_produto[0]
            dados_desconto[codigo_produto] = {}

            # 1 se for desconto em valor inteiro, 2 se for desconto em porcentagem.
            dados_desconto[codigo_produto]['tipo_desconto'] = dados_produto[1]

            # Valor do desconto UNITÁRIO. Tanto nos casos de desconto com porcentagem e fixos, este campo virá
            # com duas casas decimais sem o ponto ou a vírgula.
            dados_desconto[codigo_produto]['desconto'] = Decimal(dados_produto[2]) / Decimal(100)

            # 0 se for um produto sem limite para a concessão de descontos, 1 se for limite por quantidade,
            # 2 se for limite por valor.
            dados_desconto[codigo_produto]['tipo_limite'] = dados_produto[3]

            # Quantidade ou valor de produtos com descontos aprovados. Caso seja um valor, este campo virá com duas
            # casas decimais sem o ponto ou a vírgula. Este campo virá nulo caso seja um produto com oferta ilimitada
            # para a concessão de descontos.
            dados_desconto[codigo_produto]['limite'] = dados_produto[4] or 0

        desconto_concedido = {}
        desconto_parcial = False
        for sequencia in sorted(self.parent.dict_mov_corpos_socio.keys()):
            corpo = self.parent.dict_mov_corpos_socio.get(sequencia).get('corpo')
            codigo_produto = corpo.supcor_item[1:]
            if not dados_desconto.get(codigo_produto): # item sem desconto do socio torcedor
                continue

            tipo_desconto = dados_desconto.get(codigo_produto).get('tipo_desconto')
            desconto = dados_desconto.get(codigo_produto).get('desconto')
            tipo_limite = dados_desconto.get(codigo_produto).get('tipo_limite')
            limite = Decimal(dados_desconto.get(codigo_produto).get('limite')).quantize(TREPLACES)
            qtd = corpo.supcor_quantidade
            qtd_vendida = desconto_concedido.get(codigo_produto).get('qtd') if desconto_concedido.get(codigo_produto) \
                                                                            else 0
            if tipo_limite == '1': # limita o desconto por quantidade
                if qtd_vendida + qtd > limite:
                    qtd = limite - qtd_vendida
                    desconto_parcial = True
                    if -0.0001 < qtd and qtd < 0.0001: # desconto maximo aplicado
                        continue

                elif qtd > limite:
                    qtd = limite
                    desconto_parcial = True

            vr_desconto = desconto * qtd
            if tipo_desconto == '2': # desconto percentual
                # o arredondamento deve ser para baixo para a porcentagem calculada nao exceda o desconto consultado
                vr_desconto = Decimal(desconto * qtd * corpo.supcor_preco_unitario / Decimal(100)).\
                    quantize(TWOPLACES, rounding=ROUND_DOWN)

            valor_vendida = desconto_concedido.get(codigo_produto).get('valor') if desconto_concedido.get(
                                                                   codigo_produto) else Decimal(0).quantize(TWOPLACES)
            if tipo_limite == '2':  # limita o desconto por valor
                limite = Decimal(limite).quantize(TWOPLACES)/100
                if valor_vendida + vr_desconto  > limite:
                    vr_desconto = limite - valor_vendida
                    desconto_parcial = True
                    if -0.001 < vr_desconto and vr_desconto < 0.001: # desconto maximo aplicado
                        continue

                elif vr_desconto > limite:
                    desconto_parcial = True
                    vr_desconto = limite

            if not desconto_concedido.get(codigo_produto):
                desconto_concedido[codigo_produto] = {'qtd': qtd, 'valor': vr_desconto}
            else:
                desconto_concedido[codigo_produto]['qtd'] += qtd
                desconto_concedido[codigo_produto]['valor'] += vr_desconto

            self.parent.dict_mov_corpos_socio[sequencia]['desconto'] = vr_desconto
            total_desconto += vr_desconto

        self.parent.desconto_socio_torcedor = total_desconto
        if desconto_parcial:
            message_dlg(self.parent, MSG_INFO, '<font size="10">Atenção!</font>' +
                        '\n\n<font size="10"> Apenas parte dos produtos adquiridos tiveram desconto!</font>')

        self.parent.desconto_socio_calculado = True
        return True

    @Log
    def aplica_desc_promocao(self):
        if not self.parent.list_itens_desconto_promo:
            return 'Sem desconto'

        itens_ja_registrados = {}
        desconto_total = 0
        for dados_desc in sorted(self.parent.list_itens_desconto_promo, key=lambda x: x.get('sequencia_item')):
            item = dados_desc.get('sequencia_item')
            desconto = dados_desc.get('desconto')
            corpo = filter(lambda x: x.supcor_sequencia == item, self.parent.mov_corpos)
            corpo[0].supcor_id_promo = int(dados_desc.get('id_promo'))
            corpo[0].supcor_quantidade_promo = dados_desc.get('quantidade')
            corpo[0].supcor_tipo_promo = dados_desc.get('tipo_promo')
            if not desconto or desconto == 0:
                # os itens que ja gravaram desconto no banco nao podem ser gravados novamente no banco
                if itens_ja_registrados.get(corpo[0].supcor_item):
                    continue

                corpo[0].supcor_desconto_item = Decimal(0)
                corpo[0].supcor_desconto_item_sem_rat = Decimal(0)
                continue

            try:
                corpo[0].supcor_desconto_item = desconto
                corpo[0].supcor_desconto_item_sem_rat = desconto
                itens_ja_registrados[corpo[0].supcor_item] = True
                desconto_total += desconto
            except Exception as err:
                msg = 'Erro ao aplicar desconto no item %s!\n' % (corpo[0].supcor_nome_item)
                msg += str(err)
                Log.error(msg)

        self.parent.list_itens_desconto_promo = []
        return "Desconto Total: %s" % desconto_total

    def aplica_desc_funcionario(self):
        self.parent.obj_cad_funcionarios.aplica_desc_funcionario()

    @Log
    def aplica_desc_itens(self):
        '''
        Aplica desconto nos item ao final da venda para as Promocoes e Para o Socio Torcedor
        '''
        if self.parent._coupon_totalize: # cupom ja totalizado, nao precisa calcular o desconto
            return 'Cupom totalizado'

        # os descontos nao sao acumulativos
        self.aplica_desc_socio_torcedor()
        self.aplica_desc_promocao()
        self.aplica_desc_funcionario()

        return True

    @Log
    def add_payment(self, value, checa_tef=True, pfinalizadora=None, parcelas=0):
        if not value and not self.parent.ficha:
            self.parent._CONTROL_FOCUS.clear()
            return message_dlg(self.parent, MSG_ERROR, 'Informe o valor para pagamento!')

        if self.parent.vr_doacao and self.parent.vr_doacao > value:
            return message_dlg(self.parent, MSG_ERROR,
                               u'O valor informado deve ser igual ao total da doação')

        elif self.parent.vr_doacao and self.parent.vr_doacao < value:
            self.parent.ui.lblVlrTroco.setText(u'%1.2f' % (value - self.parent.vr_doacao))

        if self.parent.vr_ficha and self.parent.vr_ficha < value:
            self.parent.ui.lblVlrTroco.setText((u'%1.2f' % (value - self.parent.vr_ficha)))

        nro_banco = agencia = nro_cheque = conta = troco = 0

        forma_pgto = self.parent._coupon_payment_method

        query_finalizadora = None

        if forma_pgto == pm_dinheiro:
            query_finalizadora = CaixaCadFinalizadora.query.filter("name ilike 'dinheiro'").all()
            if query_finalizadora:
                query_finalizadora = query_finalizadora[0]
            control_focus = self.parent.ui.spinPgtoDinheiro
            lbl_info = self.parent.ui.lblInfoDinheiro

        elif forma_pgto == pm_cheque:
            index = self.parent.ui.cbChTipoCheque.currentIndex()
            finalizadora_name = self.parent.ui.cbChTipoCheque.itemText(index)
            query_finalizadora = CaixaCadFinalizadora.get_by(name=finalizadora_name)
            control_focus = self.parent.ui.spinChValor
            lbl_info = self.parent.ui.lblInfoCheque
            nro_banco = self.parent.ui.spinChNroBanco.value()
            agencia = self.parent.ui.spinChNroAgencia.value()
            nro_cheque = self.parent.ui.spinChNroCheque.value()
            conta = int(self.parent.ui.spinChNroCC.text())

        elif forma_pgto == pm_cartao:
            usa_tef = False
            index = self.parent.ui.cbTipoTef.currentIndex()

            if self.parent.ui.cbTipoTef.itemText(index) == u'T.E.F - Integrado ao Sistema':
                usa_tef = True

            if not pfinalizadora:
                index = self.parent.ui.cbChTipoCartao.currentIndex()
                finalizadora_name = self.parent.ui.cbChTipoCartao.itemText(index)
                query_finalizadora = CaixaCadFinalizadora.get_by(name=finalizadora_name,
                                                                 supfin_usa_tef=usa_tef)
            elif type(pfinalizadora) is int:
                query_finalizadora = CaixaCadFinalizadora.get_by(id=pfinalizadora)

            else:
                query_finalizadora = pfinalizadora

            control_focus = self.parent.ui.spinCartaoValor
            lbl_info = self.parent.ui.lblInfoCartao

        elif forma_pgto == pm_ticket:
            usa_tef = False
            index = self.parent.ui.cbTipoTicket.currentIndex()

            if self.parent.ui.cbTipoTicket.itemText(index) == u'T.E.F':
                usa_tef = True

            if not pfinalizadora:
                index = self.parent.ui.cbFinTicket.currentIndex()
                finalizadora_name = self.parent.ui.cbFinTicket.itemText(index)
                query_finalizadora = CaixaCadFinalizadora.get_by(name=finalizadora_name,
                                                                 supfin_usa_tef=usa_tef)
            elif type(pfinalizadora) is int:
                query_finalizadora = CaixaCadFinalizadora.get_by(id=pfinalizadora,
                                                                 supfin_usa_tef=usa_tef)
            else:
                query_finalizadora = pfinalizadora

            control_focus = self.parent.ui.spinValorTicket
            lbl_info = self.parent.ui.lblInfoTicket

        elif forma_pgto == pm_outros:
            index = self.parent.ui.cbOTipoPgto.currentIndex()
            finalizadora_name = self.parent.ui.cbOTipoPgto.itemText(index)
            query_finalizadora = CaixaCadFinalizadora.get_by(name=finalizadora_name)
            control_focus = self.parent.ui.spinOValor
            lbl_info = self.parent.ui.lblInfoOutros
        elif forma_pgto == pm_pagamento:
            usa_tef = True
            if not pfinalizadora:
                index = self.parent.ui.cbChTipoCartao.currentIndex()
                finalizadora_name = self.parent.ui.cbChTipoCartao.itemText(index)
                query_finalizadora = CaixaCadFinalizadora.get_by(name=finalizadora_name,
                                                                 supfin_usa_tef=usa_tef)
            elif type(pfinalizadora) is int:
                query_finalizadora = CaixaCadFinalizadora.get_by(id=pfinalizadora)

            else:
                query_finalizadora = pfinalizadora

            control_focus = self.parent.ui.spinPgtoPagamento
            lbl_info = self.parent.ui.lblInfoCartao

        if not query_finalizadora:
            message_dlg(self.parent, MSG_ERROR, 'A Forma de pagamento selecionada '
                                         'não está cadastrada na tabela de finalizadoras.')
            return False

        total_a_pagar = Decimal(self.parent.ui.lblVlrTotalR.text())


        pgto_na_ecf = '%.2d' % query_finalizadora.supfin_codigo_na_ecf
        query_finalizadora_na_ecf = CaixaCadFinalizadoraNaEcf.get_by(id=pgto_na_ecf)

        if self.parent.vr_doacao and not self.parent._coupon_totalize:
            self.parent._coupon_totalize = self.parent.obj_doacao.efetua_doacao(
                                              config['totalizador_doacao'],
                                              query_finalizadora_na_ecf.name, self.parent.vr_doacao)
            if not self.parent._coupon_totalize:
                message_dlg(self.parent, MSG_ERROR, 'Atenção!\n'
                                                       'Erro ao tentar finalizar a Doação.\n'
                                                       'Tente novamente')
                self.parent._set_screen_state(in_add_payment)
                return False

        if self.parent.ficha and not self.parent._coupon_totalize:
            if self.parent.vr_ficha > 0:
                if self.parent._payment_amount + value > total_a_pagar:
                    permite_troco = query_finalizadora.supfin_permite_troco
                    if not permite_troco:
                        return message_dlg(self.parent, MSG_INFO, 'Finalizadora não permite troco')

                    troco = self.parent._payment_amount + value - total_a_pagar
                    troco_maximo = query_finalizadora.supfin_troco_maximo

                    if troco > troco_maximo:
                        control_focus.setValue(total_a_pagar - self.parent._payment_amount)
                        return message_dlg(self.parent, MSG_INFO, 'Troco excede valor permitido: '
                                                                  '%.2f > %.2f' % (troco, troco_maximo))
                self.parent._coupon_totalize = self.parent.obj_ficha.recebimento_ficha(
                                              config['tot_ficha'], self.parent.vr_ficha, troco)
            else: # troca de fichar
                self.parent.obj_ficha.extorno_ficha(config['tot_devolucao_ficha'],
                                                        abs(self.parent.vr_ficha),
                                                    query_finalizadora_na_ecf.supfec_codigo_na_ecf,
                                                    query_finalizadora)

                if config['usagaveta'] and not self.parent._PveCancelamento:
                    self.parent.obj_impressoras.supcci_imprimir("{gaveta}")
                    self.parent._timer_end_sale.start(500)
                else:
                    self.parent._timer_end_sale.start(1000)

                self.parent._clear_globals(0)
                self.parent._set_screen_state(in_end_sale)
                self.parent.show_info_proccess(False)
                self.parent._payment_receipt_g = []
                self.parent._texto_comprovante = None
                self.parent._payment_receipt = []
                self.parent._libera_restricao = False
                self._libera_desconto = False
                self.parent.ui.stackedActions.setCurrentIndex(stk_page_payments)
                self.parent.ui.stackedPgto.setCurrentIndex(pm_end)
                return value

        sequence = CaixaMovFechamentos.query.filter_by(supfec_capa_id=self.parent.mov_capa.id).count()
        sequence += 1
        self.set_vendedor()
        avista_aprazo = query_finalizadora.supfin_avista_aprazo
        tipo_finalizadora = query_finalizadora.supfin_tipo_finalizadora
        usa_tef = query_finalizadora.supfin_usa_tef

        emite_relatorio_g = False
        if self.parent._payment_amount + value > total_a_pagar:
            permite_troco = query_finalizadora.supfin_permite_troco
            if not permite_troco:
                return message_dlg(self.parent, MSG_INFO, 'Finalizadora não permite troco')

            troco = self.parent._payment_amount + value - total_a_pagar
            troco_maximo = query_finalizadora.supfin_troco_maximo

            if troco > troco_maximo:
                control_focus.setValue(total_a_pagar - self.parent._payment_amount)
                return message_dlg(self.parent, MSG_INFO, 'Troco excede valor permitido: '
                                                   '%.2f > %.2f' % (troco, troco_maximo))
            # Contra-Vale
            elif query_finalizadora.supfin_aceita_contra_vale and checa_tef:
                texto = 'COO............: %11d\n' % self.parent._coupon_id
                texto += 'Valor da Compra: %11.2f\n' % total_a_pagar
                valor = self.parent._payment_amount + value - total_a_pagar
                texto += 'Contra-Vale....: %11.2f\n' % valor
                self.add_payment_receipt_g(query_finalizadora, valor, sequence, texto)
                emite_relatorio_g = True

        libera_cliente = False
        limite_cliente = 0

        if self.parent.cliente.name and forma_pgto in (pm_outros, pm_cheque):
            libera_cliente = False if self.parent.cliente.supcli_status == 'B' else True


            if avista_aprazo == 'P':
                if not usa_tef:
                    limite_compras = self.parent.cliente.supcli_limite_compras or 0
                    limite = limite_compras - self.parent.cliente_total_aberto
                if limite >= value:
                    limite_cliente = 1

        dlg_info = ''
        if avista_aprazo == 'V' or tipo_finalizadora == 'C':
            limite_cliente = 1
            libera_cliente = True

        if not libera_cliente:
            if self.parent.cliente.name and avista_aprazo == 'P':
                dlg_info += '|CLI.BLOQ|'
            if tipo_finalizadora == 'H' and avista_aprazo == 'P':
                dlg_info += '|CH.PRZ|'
                if self.parent.cliente.name:
                    dlg_info += 'CLI.BLOQ|'

        if limite_cliente <= 0 and self.parent.cliente.name and avista_aprazo == 'P':
            dlg_info += '|LIM.CR|' if not dlg_info else 'LIM.CR|'
            if tipo_finalizadora == 'H':
                dlg_info += 'CH.PRZ|'

        tempo_sup_intervencao = config['temposupintervencao']
        vender_offline = False

        if self.parent._time_offline >= tempo_sup_intervencao > 0:
            vender_offline = True

        if forma_pgto in [pm_cartao, pm_ticket]:
            libera_cliente = limite_cliente = True

        if avista_aprazo == 'P' and not tipo_finalizadora == 'C'\
                and self.parent.user_logged.supope_operador_supervisor == 'O':
            if vender_offline and config['autorizacaosup']:
                dlgpwd = password_dlg(self.parent, msg_info='Supervisor OFFLINE!')
                if dlgpwd.retval:
                    self.parent._libera_restricao = dlgpwd.user.id
                else:
                    if emite_relatorio_g:
                        self.remove_payment_receipt_g()
                    return message_dlg(self.parent, MSG_INFO,
                                       'Operador: %s.\n\nNão tem permissão para '
                                       'executar esta operação' %
                                       str(self.parent.user_logged.name))
            else:
                self.parent._libera_restricao = self.parent.user_logged.id

        if avista_aprazo == 'P' and tipo_finalizadora not in ('C', 'T'):
            Log.info('Cliente: %s' % str(self.parent.cliente.supcli_codigo_interno))
            Log.info('Valor do pagamento: %s' % str(value))
            Log.info('Limite compra: %s' % str(self.parent.cliente.supcli_limite_compras or 0))
            Log.info('Total em aberto: %s' % str(self.parent.cliente_total_aberto or 0))
            Log.info('Operador: %s' % str(self.parent.user_logged.name))

        if avista_aprazo == 'P' and not tipo_finalizadora == 'C' \
                                and (not libera_cliente or limite_cliente <= 0)\
                                and not self.parent._IdDav:

                if message_dlg(self.parent, MSG_QUESTION,
                               'Deseja liberar esta venda? \n\n%s' % dlg_info) == NO_ROLE:
                    if emite_relatorio_g:
                        self.remove_payment_receipt_g()

                    return False

                if self.parent.user_logged.supope_operador_supervisor == 'O' and \
                        not self.parent.user_logged.supope_libera_restricao:
                    dlgpwd = password_dlg(self.parent, msg_info=dlg_info)
                    if dlgpwd.retval:
                        self.parent._libera_restricao = dlgpwd.user.id
                    else:
                        if emite_relatorio_g:
                            self.remove_payment_receipt_g()
                        return message_dlg(self.parent, MSG_INFO,
                                           'Operador: %s.\n\nNão tem permissão para '
                                           'executar esta operação' %
                                           str(self.parent.user_logged.name))
                else:
                    self.parent._libera_restricao = self.parent.user_logged.id

        self.aplica_desc_itens() # desconto de promocoes

        lbl_info.setText(u'%1.2f' % (Decimal(lbl_info.text()) + value))

        # Enviar o pagamento para a ECF
        if not self.parent._coupon_totalize:
            try:
                self.parent._coupon_totalize = self.totalizar_cupom()
            except Exception as err:
                Log.error(err)

        if troco > 0:
            self.parent.ui.lblVlrTroco.setText(u'%1.2f' % troco)


        if not query_finalizadora_na_ecf:
            message_dlg(self.parent, MSG_INFO,
                        'A forma de pagamento %r não está cadastrada na tabela '
                        'caixa_cad_finalizadoras_na_ecf' % query_finalizadora.name)
            if emite_relatorio_g:
                self.remove_payment_receipt_g()

            return False

        if usa_tef or avista_aprazo == 'P':
            self.add_payment_receipt_g(query_finalizadora, value, sequence,
                                       texto=self.parent._texto_comprovante,
                                       parcelas=parcelas)

        self.parent._payment_amount += value
        self.parent.ui.lblVlrPgto.setText(u'%1.2f' % self.parent._payment_amount)

        if self.parent._payment_amount >= total_a_pagar:
            troco = self.parent._payment_amount - total_a_pagar
            self.parent.ui.lblVlrTroco.setText(u'%1.2f' % troco)
            self.parent.ui.lblVlrReceber.setText(u'%1.2f' % 0.0)
            mov_corpos = MovCorpos(self.parent).search({'supcor_capa_id': self.parent.mov_capa.id,
                                             'supcor_venda_cancelamento': 'V'})

            if not isinstance(mov_corpos, list):
                mov_corpos = [mov_corpos]

            # ------------- Se tiver acréscimo/desconto na capa, efetua rateio entre os itens ------------ #
            itens_rateados = {}
            try:
                indicador_desc_acre = self.parent.mov_capa.supcap_alteracao
                valor_desc_acres = Decimal(self.parent.ui.lblInfoDescCapa.text())
                if indicador_desc_acre and valor_desc_acres:
                    itens_rateados = MovCorpos(self.parent).set_rateio_desc_acre_itens(
                                                  indicador_desc_acre, mov_corpos, valor_desc_acres)

            except Exception as e:
                Log.error(str(e))
            # ---------------------------------------- // ---------------------------------------- #


            total_imposto_fed = Decimal('0.0')
            total_imposto_est = Decimal('0.0')
            total_imposto_mun = Decimal('0.0')
            parametros = self.parent.obj_parametros.get_by()
            for corpo in mov_corpos:
                preco_total = corpo.supcor_preco_total
                desconto = corpo.supcor_desconto_item
                acrescimo = corpo.supcor_acrescimo_item
                total_item = Decimal(preco_total - desconto + acrescimo)
                item = CaixaCadItens.get_by(supite_codigo_item=corpo.supcor_item,
                                            supite_loja=self.parent.loja.supcfc_loja)
                aliq_ipbt_fed = Decimal(item.supite_aliquota_ibpt_fed or 0.0)
                aliq_ipbt_est = Decimal(item.supite_aliquota_ibpt_est or 0.0)
                aliq_ipbt_mun = Decimal(item.supite_aliquota_ibpt_mun or 0.0)
                vr_imposto_fed = Decimal(total_item * aliq_ipbt_fed / 100).quantize(TWOPLACES)
                vr_imposto_est = Decimal(total_item * aliq_ipbt_est / 100).quantize(TWOPLACES)
                vr_imposto_mun = Decimal(total_item * aliq_ipbt_mun / 100).quantize(TWOPLACES)
                total_imposto_fed += vr_imposto_fed
                total_imposto_est += vr_imposto_est
                total_imposto_mun += vr_imposto_mun
                vals_corpo = {}
                desconto_rat = acrescimo_rat = 0
                if itens_rateados and itens_rateados.get(corpo.id):
                    if indicador_desc_acre == 'D':
                        desconto_rat = itens_rateados.get(corpo.id).get('supcor_desconto_item')

                    else:
                        acrescimo_rat = itens_rateados.get(corpo.id).get('supcor_acrescimo_item')

                trib_icms = CaixaCadTributacaoNfce.get_by(suptnf_codigo=item.supite_tributacao_icms_id)
                trib_pis = CaixaCadTributacaoNfce.get_by(suptnf_codigo=item.supite_tributacao_pis_id)
                trib_cofins = CaixaCadTributacaoNfce.get_by(suptnf_codigo=item.supite_tributacao_cofins_id)
                vr_liquido = preco_total - (desconto_rat or desconto) + (acrescimo_rat or acrescimo)
                base_icms = Decimal(vr_liquido * trib_icms.suptnf_base / Decimal(100.0)).quantize(TWOPLACES)
                vr_icms = Decimal(base_icms * trib_icms.suptnf_aliquota / Decimal(100.0)).quantize(TWOPLACES)
                base_pis = Decimal(vr_liquido * trib_pis.suptnf_base / Decimal(100.0)).quantize(TWOPLACES)
                base_cofins = Decimal(vr_liquido * trib_cofins.suptnf_base / Decimal(100.0)).quantize(TWOPLACES)
                if parametros.supcfp_deduz_icms:  # retira o valor do icms do valor da base de calculo
                    base_pis -= vr_icms
                    base_cofins -= vr_icms

                vr_pis = Decimal(base_pis * trib_pis.suptnf_aliquota / Decimal(100.0)).quantize(TWOPLACES)
                vr_cofins = Decimal(base_cofins * trib_cofins.suptnf_aliquota / Decimal(100.0)).quantize(TWOPLACES)
                valores = """supcor_base_icms = {base_icms},
                             supcor_base_pis = {base_pis},
                             supcor_base_cofins = {base_cofins},
                             supcor_vr_icms = {vr_icms},
                             supcor_vr_pis = {vr_pis},
                             supcor_vr_cofins = {vr_cofins},
                             supcor_vr_imposto_fed = {vr_imposto_fed},
                             supcor_vr_imposto_est = {vr_imposto_est},
                             supcor_vr_imposto_mun = {vr_imposto_mun},
                             supcor_acrescimo_item = {acrescimo},
                             supcor_desconto_item = {desconto}""".\
                             format(base_icms=base_icms,
                                    base_pis=base_pis,
                                    base_cofins=base_cofins,
                                    vr_icms=vr_icms,
                                    vr_pis=vr_pis,
                                    vr_cofins=vr_cofins,
                                    vr_imposto_fed=vr_imposto_fed,
                                    vr_imposto_est=vr_imposto_est,
                                    vr_imposto_mun=vr_imposto_mun,
                                    acrescimo=acrescimo_rat or acrescimo,
                                    desconto=desconto_rat or desconto)
                exec_sql("""UPDATE caixa_mov_corpos 
                            SET {valores} 
                            WHERE id = {id}""".format(valores=valores,
                                                      id=corpo.id), 'C')

            descontos = self.parent.desconto_socio_torcedor + self.parent.desconto_promo + \
                        self.parent.desconto_funcionario
            total_cupom = self.parent.mov_capa.supcap_total_cupom - descontos
            self.parent.mov_capa.supcap_total_cupom = total_cupom
            # esta parte ja foi feita para doacao e ficha
            if not self.parent.vr_doacao and not self.parent.vr_ficha:
                self.parent.obj_mov_capa.write(self.parent.mov_capa,
                                {'supcap_vr_imposto_fed': total_imposto_fed,
                                 'supcap_vr_imposto_est': total_imposto_est,
                                 'supcap_vr_imposto_mun': total_imposto_mun,
                                 'supcap_cliente': self.parent.cliente.supcli_cnpj_cpf,
                                 'supcap_nome_cliente': self.parent.cliente.name,
                                 'supcap_operador_id': self.parent.user_logged.id,
                                 'supcap_cod_vendedor': self.parent.vendedor_codigo,
                                 'supcap_total_cupom': total_cupom,
                                 'supcap_subtotal': total_cupom,})

            if config['usar_ficha']:
                self.parent.obj_ficha.faturar_fichas()

            self.parent.ui.stackedActions.setCurrentIndex(stk_page_payments)
            self.parent.ui.stackedPgto.setCurrentIndex(pm_end)
            self.parent.setControlFocus(self.parent._CONTROL_FOCUS, self.parent.ui.stackedPgto)
            self.parent._state_coupon = coupon_finish

            self.parent.ui.lblNumCaixa.setText(u'%03d <b>Aberto' % self.parent.caixa_id)

            if self.parent._IdPve:
                try:
                    Log.info('Gravando PVE #: %s' % self.parent._IdPve.id)
                    b_sess.begin()
                    self.parent._IdPve.suppva_data_cupom = self.parent._date_coupon
                    self.parent._IdPve.suppva_nro_cupom = self.parent._coupon_id
                    self.parent._IdPve.suppva_loja = self.parent.loja.supcfc_loja
                    self.parent._IdPve.suppva_caixa = self.parent.caixa_id
                    self.parent._IdPve.suppva_restricao = 2
                    b_sess.commit()
                except Exception as err:
                    Log.error(str(err))

            elif self.parent.comanda_id:
                try:
                    b_sess.begin()
                    for comanda in self.parent._IdsComandas:
                        comanda.supcda_nro_cupom = self.parent.mov_capa.supcap_nro_cupom
                        comanda.supcda_loja = self.parent.loja.supcfc_loja
                        comanda.supcda_caixa = self.parent.caixa_id
                        comanda.supcda_restricao = 2
                    b_sess.commit()
                except Exception as err:
                    Log.error(str(err))

            finaliza_venda = True

        else:
            vr_receber = total_a_pagar - self.parent._payment_amount
            self.parent.ui.lblVlrReceber.setText(u'%1.2f' % vr_receber)
            self.parent._set_screen_state(in_add_payment)
            finaliza_venda = False

        # ------------------------------ Grava fechamentos do cupom ------------------------------ #
        vals_fechamento = dict(supfec_sequencia=sequence,
                               supfec_finalizadora=query_finalizadora.supfin_codigo,
                               supfec_tipo_finalizadora=query_finalizadora.supfin_tipo_finalizadora,
                               supfec_valor=value,
                               supfec_troco=troco,
                               supfec_nro_banco=nro_banco,
                               supfec_agencia=agencia,
                               supfec_nro_cheque=nro_cheque,
                               supfec_conta=conta,
                               supfec_cnpj=self.parent.cnpj_cpf_cheque,
                               supfec_nome_finalizadora=query_finalizadora.name,
                               supfec_capa_id=self.parent.mov_capa.id,
                               supfec_aut_cartao=self.parent.sig_tef.aut_cartao)
        operadora = False
        if query_finalizadora.supfin_operadora_id:
            operadora = CaixaCadOperadoraCartao.get_by(id=query_finalizadora.supfin_operadora_id)

        if self.parent.sig_tef.cnpj_operadora or (operadora and operadora.supopc_cnpj):
            vals_fechamento['supfec_cnpj_cartao'] = re.sub('[^0-9]', u'',
                                                           self.parent.sig_tef.cnpj_operadora or
                                                           operadora.supopc_cnpj or '')

        self.parent.mov_fechamentos = MovFechamentos(self.parent).create(vals_fechamento)
        self.parent.sig_tef.cnpj_operadora = ''
        self.parent.sig_tef.aut_cartao =''

        # -------------------------------- Grava meios_pagamento --------------------------------- #
        MovMeiosPagamentos().gravar_meios_pagamento(self.parent.loja.supcfc_loja,
                                                    self.parent.caixa_id,
                                                    self.parent._date_coupon,
                                                    self.parent.mov_capa.supcap_tipo,
                                                    self.parent.mov_capa.supcap_tipo_comprovante,
                                                    value, query_finalizadora.supfin_codigo,
                                                    query_finalizadora.name)
        if self.parent._IdPve:
            try:
                b_sess.begin()
                SupervisorPveFechamento(
                    suppvf_capa_id=self.parent._IdPve.id, suppvf_sequencia=sequence,
                    suppvf_finalizadora=query_finalizadora.supfin_codigo, suppvf_valor=value,
                    suppvf_cheque_banco=nro_banco,
                    suppvf_cheque_agencia=agencia, suppvf_cheque_numero=nro_cheque,
                    suppvf_cheque_conta=conta,
                    suppvf_cheque_juros=0, suppvf_cheque_cnpj=self.parent.cnpj_cpf_cheque
                )
                b_sess.commit()
                self._IdPve = False
            except Exception as err:
                Log.error(str(err))

        if self.parent.cliente.name and forma_pgto in (pm_outros, pm_cheque):
            if avista_aprazo == 'P':
                if not usa_tef:
                    # a atualizacao do total em aberto somente deve ser feita apos a venda ser finalizada
                    self.parent.cliente_total_aberto += value
                    CadClientes(self.parent).atualiza_total_em_aberto(self.parent.loja.supcfc_loja,
                                                           self.parent.cliente,
                                                           self.parent.cliente_total_aberto)

        pg_com_cartao = False
        erro = False
        if finaliza_venda:
            fin_tef = False
            finaliza_tef = False
            texto_tef = []
            texto_promissoria = []
            for p in self.parent._payment_receipt_g:
                mov_fechamentos = MovFechamentos(self.parent).search(
                    {'supfec_capa_id': self.parent.mov_capa.id,
                     'supfec_sequencia': p[3]})
                if mov_fechamentos:
                    for x in p[4]:
                        MovVencimentos().create(
                            dict(supfve_sequencia=p[3],
                                 supfve_vencimento=x[1],
                                 supfve_valor_parcela=x[2],
                                 supfve_sequencia_vencimento=x[0],
                                 supfve_fechamentos_id=mov_fechamentos.id))

                try:
                    if 'S' in [p[5], p[6]]:
                        self.parent.infoText('Imprimindo Gerencial')
                        tipo = 'P'
                        if p[6] == 'S':
                            tipo = 'O'

                        texto_promissoria.append(p[1])
                        self.cria_capa_cupom_gerencial(valor=p[2], tipo=tipo, comprovante=True)

                    if p[6] == 'T':
                        self.parent.infoText('Imprimindo Vinculado')
                        query_finalizadora_na_ecf = CaixaCadFinalizadoraNaEcf.get_by(
                            id=p[0])

                        if query_finalizadora_na_ecf:
                            fin_tef = (1, self.parent.mov_capa.supcap_nro_cupom,
                                       query_finalizadora_na_ecf.supfec_codigo_na_ecf, p[2],)

                        else:
                            message_dlg(self, MSG_INFO,
                                        'A forma de pagamento %r não está cadastrada na tabela '
                                        % str(query_finalizadora.name) +
                                        'caixa_cad_finalizadoras_na_ecf')

                        texto_tef.append((p[1], fin_tef[3]))
                        finaliza_tef = True

                except Exception as err:
                    Log.error(str(err))
                    self.parent.show_info_proccess(False)


            data_fim = datetime.now()
            self.parent.obj_mov_capa.write(self.parent.mov_capa,{'supcap_data_fim': data_fim,
                                                                 'supcap_ambiente': config['nfce_ambiente']})
            if self.parent.comanda_id:
                try:
                    data_fim = converte_tz(self, data_fim, 'America/Sao_Paulo', 'UTC')
                    b_sess.begin()
                    for comanda in self.parent._IdsComandas:
                        comanda.supcda_data_cupom = str(data_fim)[:26]
                    b_sess.commit()
                except Exception as err:
                    Log.error(str(err))

            # abrir a gaveta antes de enviar a nota para facilitar o troco
            if config['usagaveta'] and not self.parent._PveCancelamento:
                self.parent.obj_impressoras.supcci_imprimir("{gaveta}")

            self.finaliza_pagamento(finaliza_tef, pg_com_cartao, erro, dlg_info, texto_tef,
                                    texto_promissoria)

        return value

    @Log
    def gerar_dav(self):
        numero_dav = self.get_numero_dav()
        self.set_numero_dav()
        self.set_numero_nf(False)
        # altera o numero do cupom vinvulado caso haja
        for i in self.parent._capas_gerencial:
            self.parent.obj_mov_capa.write(i, {'supcap_nro_cupom_vinculado': numero_dav})

        self.parent.obj_mov_capa.write(self.parent.mov_capa, {'supcap_nro_cupom': numero_dav,
                                                              'supcap_state': '6',
                                                              'supcap_tipo': 'D'})
        self.parent.mov_capa = self.parent.obj_mov_capa.search({'id': self.parent.mov_capa.id})
        self.parent.KEY_IMPRIME_DAV()
        return True

    @Log
    def finaliza_pagamento(self,  finaliza_tef=False, pg_com_cartao=False, erro=False, dlg_info='',
                           texto_tef=[], texto_promissoria=[]):
        imprime_dav = False
        nro_antigo = self.parent.mov_capa.supcap_nro_cupom
        numero_dav = self.get_numero_dav()
        # Imprime rodapé do cupom
        if not self.parent.vr_doacao and not self.parent.vr_ficha:
            if not self.parent.btn_imprime_dav_down:
                msg = 'Enviando Nota para a SEFAZ...'
                self.parent.show_info_proccess(True, msg)
                self.parent.teclado and self.parent.teclado.envia_display(msg)
                if not self.parent.mov_capa.supcap_state and not self.envia_nota_servidor():
                    self.parent.show_info_proccess(False, '')
                    now = datetime.now()
                    data_fiscal = now.strftime('%Y%m%d')
                    horario = now.strftime('%H%M%S')
                    nro_cupom_tef = self.parent.mov_capa.supcap_nro_cupom
                    if (finaliza_tef or not pg_com_cartao) and self.parent.sig_tef.usando_sitef:
                        self.parent.show_info_proccess(True, 'Cancelando Transações SigTef')
                        if not self.parent.emitir_contingencia:
                            nro_cupom_tef -= 1

                        self.parent.sig_tef.finalizar_transacao(0,
                                                                nro_cupom_tef,
                                                                data_fiscal,
                                                                horario)

                    self.parent.show_info_proccess(False, '')
                    return False

                self.imprimir_nota(self.parent.mov_capa.supcap_nro_cupom)

            else:
                self.parent.show_info_proccess(True, 'Finalizando DAV...')
                self.gerar_dav()
                self.imprimir_dav()
                imprime_dav = True

            self.parent.show_info_proccess(False, '')
        for i in texto_promissoria:
            # troca o texto das promissorias retirando 'NFC-e' para 'DAV'
            if imprime_dav:
                texto = i.replace('Nro. NFC-e: %.06d' % nro_antigo, 'Nro. DAV: %.06d' % numero_dav)

            # caso a nota tenha sido emitida em contingencia, altera o numero vinculado da promissoria
            else:
                texto = i
                if self.parent.mov_capa.supcap_state and \
                        self.parent.mov_capa.supcap_state.strip() in ('4', '2E'):
                    if not self.parent.emitir_contingencia:
                        for j in self.parent._capas_gerencial:
                            self.parent.obj_mov_capa.write(j, {
                                'supcap_nro_cupom_vinculado': self.parent.mov_capa.supcap_nro_cupom})
                            texto = i.replace('Nro. NFC-e: %.06d' % (int(self.parent.mov_capa.supcap_nro_cupom)-1),
                                              'Nro. NFC-e: %.06d' % int(self.parent.mov_capa.supcap_nro_cupom))


            print '\n\nPromissoria\n\n', self.get_texto_cupom(texto)
            self.parent.obj_impressoras.supcci_imprimir(self.get_texto_cupom(texto))


        self.parent._capas_gerencial = [] # limpa as capas de cupons gerenciais
        if (finaliza_tef or not pg_com_cartao) and self.parent.sig_tef.usando_sitef:
            operacao = 1
            msg_tef = 'Finalizando Transações SigTef'
            if self.parent.mov_capa.supcap_state and \
                    self.parent.mov_capa.supcap_state.strip() == 'D':
                operacao = 0
                msg_tef = 'Cancelando Transações SigTef'

            self.parent.show_info_proccess(True, msg_tef)
            now = datetime.now()
            data_fiscal = now.strftime('%Y%m%d')
            horario = now.strftime('%H%M%S')
            nro_cupom_tef = self.parent.mov_capa.supcap_nro_cupom
            # def pegar o numero anterior quando for contingencia pois foi o numero que abriu a transacao TEF
            if self.parent.mov_capa.supcap_state and \
                    self.parent.mov_capa.supcap_state.strip() in ('4', '2E'):
                if not self.parent.emitir_contingencia:
                    nro_cupom_tef -= 1

            self.parent.sig_tef.finalizar_transacao(operacao,
                                                    nro_cupom_tef,
                                                    data_fiscal,
                                                    horario)
            if operacao and texto_tef:
                msg = 'Imprimindo comprovante do pagamento...'
                self.parent.teclado and self.parent.teclado.envia_display(msg)
                self.parent.show_info_proccess(True, msg)
                # Imprime recibo do pagamento
                for i in texto_tef:
                    comp_pagamento = ''
                    try:
                        comp_pagamento = self.get_texto_cupom(i[0])
                        self.cria_capa_cupom_gerencial(valor=i[1], tipo='T',
                                                       comprovante=True)
                        print '\nComprovate de Pagamento:\n\n', comp_pagamento
                        self.parent.obj_impressoras.supcci_imprimir(comp_pagamento)
                    except Exception as e:
                        self.parent.tratar_except(mostrar_tela=False)
                        Log.error('Impressao de comprovante: %s' % e)
                        Log.error('Texto impresso: \n\n%s\n\n' % (comp_pagamento or 'Sem texto'))
                        self.parent.show_info_proccess(False, '')
                        message_dlg(self.parent, MSG_ERROR, "Erro ao imprimir comprovante TEF\n"
                                                            "Use o menu do TEF para reimpressão!")

                    time.sleep(0.5)

            self.parent.show_info_proccess(False, '')

        if self.parent.codigo_estacionamento and not erro and \
                self.parent.mov_capa.supcap_state and \
                self.parent.mov_capa.supcap_state.strip() != 'D':
            while True:
                try:
                    res = self.parent.obj_nepos.envia_codigo_barras()
                    coo = self.parent.mov_capa.supcap_nro_cupom
                    self.parent.KEY_RETURN_Click()
                    msg = 'Nota vinculada: %06d Serie: %03d\n' % (coo, config['caixa']) + \
                          'Valor da compra: R$ %s' % str(self.parent.mov_capa.supcap_total_cupom).\
                                                                     replace('.',',') + '\n' + \
                          res[1].split('\x00')[0] + '\n' + \
                          'No. Ticket:' + self.parent.codigo_estacionamento
                    if res[0]:
                        print '\n\nComprovante de estacionamento\n\n', self.get_texto_cupom(msg)
                        self.parent.obj_impressoras.supcci_imprimir(self.get_texto_cupom(msg))
                        self.cria_capa_cupom_gerencial(tipo='G', estacionamento=True)
                        # Todo: Testar estacionamento
                        # GerencialReport(report_id=config['rg_relatorio_gerencial'],
                        #                 text=msg,
                        #                 loja_id=self.parent.loja.supcfc_loja,
                        #                 caixa_id=self.parent.caixa_id,
                        #                 operador_id=self.parent.user_logged.id,
                        #                 tipo='G',
                        #                 parent=self.parent)
                    msg = res[1]
                    if not res[0]:
                        msg = '%s Cartao: %s' % (msg, self.parent.codigo_estacionamento)

                    message_dlg(self.parent, MSG_INFO, msg)
                    break
                except Exception as e:
                    Log.error(str(e))
                    msg = 'Erro ao validar cartão: \n\n' + str(e) + '\n\n Deseja tentar novamente?'
                    if message_dlg(self.parent, MSG_QUESTION, msg) == NO_ROLE:
                        break

            self.parent.codigo_estacionamento = ''

        vals_aut = dict(supaut_cliente=self.parent.cliente.supcli_cnpj_cpf,
                        supaut_capa_id=self.parent.mov_capa.id,
                        supaut_liberou_limite_credito=False,
                        supaut_liberou_cheque_a_vista=False,
                        supaut_liberou_restricao_mensagem=False,
                        supaut_liberou_desconto=False)

        if self.parent._libera_desconto:
            if self.parent._libera_restricao == self.parent._libera_desconto:
                vals_aut['supaut_autorizador_id'] = self.parent._libera_desconto
                vals_aut['supaut_liberou_desconto'] = True
            else:
                vals_aut_desc = vals_aut.copy()
                vals_aut_desc['supaut_autorizador_id'] = self.parent._libera_desconto
                vals_aut_desc['supaut_liberou_desconto'] = True
                MovAutorizacoes().create(vals_aut_desc)

        libera_limite_credito = libera_cheque_avista = libera_restricao_msg = False
        if self.parent._libera_restricao:
            if not self.parent._IdDav:
                if 'LIM.CR' in dlg_info:
                    libera_limite_credito = True
                if 'CH.VIS' in dlg_info:
                    libera_cheque_avista = True
                if 'CLI.BLOQ' in dlg_info:
                    libera_restricao_msg = True
            vals_aut['supaut_autorizador_id'] = self.parent._libera_restricao
            vals_aut['supaut_liberou_limite_credito'] = libera_limite_credito
            vals_aut['supaut_liberou_cheque_a_vista'] = libera_cheque_avista
            vals_aut['supaut_liberou_restricao_mensagem'] = libera_restricao_msg

        if vals_aut.get('supaut_autorizador_id'):
            MovAutorizacoes().create(vals_aut)

        QtGui.QApplication.processEvents()

        # Grava a data final do cupom
        # ------------------- Grava cupom servidor ------------------- #

        cupserv_id = MovCupomServidor().search({'supcse_capa_id': self.parent.mov_capa.id})
        if not cupserv_id:  # caso não exista
            MovCupomServidor().create({'supcse_capa_id': self.parent.mov_capa.id})
        else:
            MovCupomServidor().write(cupserv_id, {'supcse_enviado': False})

        CadSorteio(self.parent).verifica_sorteio()
        if self.parent._IdMesa:
            self.parent._IdMesa.DataHoraFechamento = datetime.now()
            self.parent._IdMesa.Status = 'F'

        if self.parent._IdDav:
            try:
                Log.info('Salvando DAV')

                b_sess.begin()
                self.parent._IdDav.supdva_data_cupom = converte_tz(self, self.parent.mov_capa.supcap_data_fim,
                                                                'America/Sao_Paulo','UTC').strftime('%d/%m/%Y %H:%M:%S')
                self.parent._IdDav.supdva_loja = self.parent.loja.supcfc_loja
                self.parent._IdDav.supdva_caixa = self.parent.caixa_id
                self.parent._IdDav.supdva_operador = self.parent._operador_id
                self.parent._IdDav.supdva_nro_cupom = self.parent.mov_capa.supcap_nro_cupom
                self.parent._IdDav.supdva_restricao = 2
                b_sess.commit()
                if config['aplicativo_dav'] == 'SiG-Gestao':
                    o_dav = DavGestao(config.get('ip_gestao'), 5019)
                    o_dav.fatura_dav(self.parent._IdDav.id)
                    b_sess.begin()
                    self.parent._IdDav.supdva_restricao = 4
                    b_sess.commit()
                self.parent._IdDav = False
            except Exception as err:
                Log.error(str(err))

        if self.parent.vr_ficha:
            self.parent.obj_ficha.cria_cupom_fichas(self.parent.vr_ficha)

        if self.parent.vr_doacao or self.parent.vr_ficha:
            if self.parent.vr_doacao: # para ficha limpa ao voltar para tela de ficha
                self.parent._clear_globals(0)

            self.parent._set_screen_state(in_end_sale)

        if config['usagaveta'] and not self.parent._PveCancelamento:
            self.parent._timer_end_sale.start(500)
        else:
            self.parent._timer_end_sale.start(1000)

        self.parent.timer_fim_de_venda.start((config['tempo_retorno']) * 1000)
        self.parent.show_info_proccess(False)
        self.parent._payment_receipt_g = []
        self.parent._texto_comprovante = None
        self.parent._payment_receipt = []
        self.parent._libera_restricao = False
        self._libera_desconto = False

    @Log
    def remove_payment_receipt_g(self):
        if self.parent._payment_receipt_g:
            self.parent._payment_receipt_g.pop() # retira o ultimo elemento da lista

        return True

    @Log
    def add_payment_receipt_g(self, method, value, sequence, texto=None, parcelas=0):
        if parcelas:
            qtd_parcelas = parcelas
        else:
            if method.supfin_outros_qtd_parcelas > 0:
                qtd_parcelas = method.supfin_outros_qtd_parcelas
            else:
                qtd_parcelas = 1

        vlr_parcela = Decimal(value / qtd_parcelas).quantize(TWOPLACES)
        total_calculado = Decimal(vlr_parcela * qtd_parcelas).quantize(TWOPLACES)
        dif = Decimal(value - total_calculado).quantize(TWOPLACES)
        print_ = 'N'
        cc = 'N'  # contra-vale
        if method.supfin_imp_promissoria:
            print_ = 'S'
        elif method.supfin_aceita_contra_vale:
            cc = 'S'
        if method.supfin_usa_tef:
            print_ = cc = 'T'

        data = datetime.today().date()
        outros_intervalo_parcelas = method.supfin_outros_intervalo_parcelas or 0
        if method.supfin_tipo_finalizadora == 'H' and method.supfin_avista_aprazo == 'P':
            data = datetime.strptime(self.parent.ui.dateChVcto.text(), '%d/%m/%Y')
        elif method.supfin_outros_prazo_primeira:
            supfin_outros_prazo_primeira = method.supfin_outros_prazo_primeira
            data += timedelta(days=supfin_outros_prazo_primeira)

        elif method.supfin_outros_mensal_vencimento:
            # data = datetime.strptime('06-02-2021', '%d-%m-%Y')
            supfin_outros_mensal_vencimento = method.supfin_outros_mensal_vencimento
            outros_mensal_fecha_mes = method.supfin_outros_mensal_fecha_mes
            qtd_parcelas = 1
            qtd_dias_no_mes = calendar.monthrange(data.year, data.month)[1]
            if int(outros_mensal_fecha_mes or 0) > supfin_outros_mensal_vencimento:
                if data.day <= int(outros_mensal_fecha_mes or 0):
                    vencimento = (data + timedelta(days=qtd_dias_no_mes - data.day + 1))  # soma 1 mes na data
                    qtd_dias_no_mes = calendar.monthrange(data.year, data.month)[1]
                    dia = supfin_outros_mensal_vencimento
                    if dia > qtd_dias_no_mes:
                        # pega ultimo dia do mes caso o vencimento seja maior que o max de dias do mes
                        dia = qtd_dias_no_mes
                    data = datetime.strptime(str(dia) + '-' +
                                             str(vencimento.month) + '-' +
                                             str(vencimento.year), '%d-%m-%Y').date()
                else:
                    vencimento = (data + timedelta(days=qtd_dias_no_mes - data.day + 1))  # soma 1 mes na data
                    qtd_dias_no_mes = calendar.monthrange(vencimento.year, vencimento.month)[1]
                    vencimento = (vencimento + timedelta(days=qtd_dias_no_mes))  # soma 1 mes na data
                    dia = supfin_outros_mensal_vencimento
                    if dia > qtd_dias_no_mes:
                        # pega ultimo dia do mes caso o vencimento seja maior que o max de dias do mes
                        dia = qtd_dias_no_mes
                    data = datetime.strptime(str(dia) + '-' +
                                             str(vencimento.month) + '-' +
                                             str(vencimento.year), '%d-%m-%Y').date()
            else:
                if data.day <= int(outros_mensal_fecha_mes or 0):
                    dia = supfin_outros_mensal_vencimento
                    if dia > qtd_dias_no_mes:
                        # pega ultimo dia do mes caso o vencimento seja maior que o max de dias do mes
                        dia = qtd_dias_no_mes
                    data = datetime.strptime(str(dia) + '-' +
                                             str(data.month) + '-' +
                                             str(data.year), '%d-%m-%Y').date()
                else:
                    vencimento = (data + timedelta(days=qtd_dias_no_mes - data.day + 1))  # soma 1 mes na data
                    qtd_dias_no_mes = calendar.monthrange(vencimento.year, vencimento.month)[1]
                    dia = supfin_outros_mensal_vencimento
                    if dia > qtd_dias_no_mes:
                        # pega ultimo dia do mes caso o vencimento seja maior que o max de dias do mes
                        dia = qtd_dias_no_mes
                    data = datetime.strptime(str(dia) + '-' +
                                             str(vencimento.month) + '-' +
                                             str(vencimento.year), '%d-%m-%Y').date()

        data_valor_vencimentos = []
        primeira_parcela = vlr_parcela + dif # se houver diferenca, joga na primeira parcela
        if not texto:
            text = 'Reconheco a exatidao desta NOTA PROMISSORIA que pagarei a %s, CNPJ: %s. ' \
                   'Ou a sua ordem nos vencimentos indicados.\n\n' % (self.parent.loja.supcfc_cliente,
                                                                      self.parent.loja.supcfc_cnpj)
            text += 'Valor por Extenso: %s \n' % str(
                NumeroPorExtenso(value).extenso_unidade).upper()
            text += 'Nro. NFC-e: %.06d Serie: %s \n \n' % (self.parent._coupon_id,
                                                           self.parent.caixa_id)
            text += 'Parcela Vencimento Valor \n'
            inc = 0
            for x in range(qtd_parcelas):
                if not inc: # primeira parcela
                    vlr_parcela_s = primeira_parcela
                else: # demais parcelas
                    data += timedelta(days=outros_intervalo_parcelas)
                    vlr_parcela_s = vlr_parcela

                data_valor_vencimentos.append((x + 1, data, vlr_parcela_s))
                text += '%7s %s R$ %.2f\n' % ((str(x + 1) + '/' + str(qtd_parcelas)),
                                              data.strftime("%d/%m/%Y"),
                                              vlr_parcela_s,)

                inc += 1
            if self.parent.cliente.name:
                complemento = self.parent.cliente.supcli_complemento or ''
                text += ' \n \n \nAss. Emitente: --------------------------------- \n'
                text += '%s\n%s\n%s - %s%s \n%s %s\n%s\n' % (
                    self.parent.cliente.name,
                    self.parent.cliente.supcli_cnpj_cpf,
                    self.parent.cliente.supcli_endereco or 'Sem endereco',
                    self.parent.cliente.supcli_endereco_numero or 'Sem numero',
                    complemento and ('-' + complemento) or '',
                    self.parent.cliente.supcli_cep or '',
                    self.parent.cliente.supcli_bairro or 'Bairro',
                    self.parent.cliente.supcli_cidade or '')
                text += '================================================ \n'
        else:
            text = texto
            inc = 0
            for x in range(qtd_parcelas):
                if inc > 0: # demais parcelas
                    data += timedelta(days=outros_intervalo_parcelas)
                    data_valor_vencimentos.append((x + 1, data, vlr_parcela))
                else: # primeira parcela
                    data_valor_vencimentos.append((x + 1, data, primeira_parcela))

                inc += 1

        self.parent._texto_comprovante = None

        text = remove_accents(text, cod='ISO-8859-1')
        self.parent._payment_receipt_g.append(('%.2d' % method.supfin_codigo_na_ecf, str(text),
                                               value, sequence, data_valor_vencimentos, print_, cc))
        return self.parent._payment_receipt_g

    @Log
    def identificar_cliente(self, endereco=False, cidade=False, insc_estadual=False):
        try:
            str_end = []
            cnpj_cpf = self.parent.cliente.supcli_cnpj_cpf
            cliente_name = self.parent.cliente.name

            if endereco and self.parent.cliente.supcli_endereco:
                str_end.append('%s' % remove_accents(
                                                 self.parent.cliente.supcli_endereco).rstrip()[:42])

                if self.parent.cliente.supcli_endereco_numero:
                    str_end.append(', %s' % self.parent.cliente.supcli_endereco_numero)

                if self.parent.cliente.supcli_complemento:
                    str_end.append('-%s' % self.parent.cliente.supcli_complemento)

            if cidade and self.parent.cliente.supcli_cidade:
                str_end.append(', %s' % self.parent.cliente.supcli_cidade.rstrip())

                if self.parent.cliente.supcli_estado:
                    str_end.append('-%s' % self.parent.cliente.supcli_estado)

            if insc_estadual and self.parent.cliente.supcli_insc_estadual:
                str_end.append(' - IE: %s' % self.parent.cliente.supcli_insc_estadual)

            res = ''.join(str_end)
        except Exception as err:
            Log.error(str(err))
            res = None

        return res

    @Log
    def get_desconto_acrescimo(self, state_item, value, preco_total, preco_unit, quantidade):
        desconto_acrescimo = (0, 0)
        if value > 0:
            if state_item == item_add_discount:
                desconto = value
                preco_total -= desconto
                desconto_acrescimo = desconto * -1
                desconto_acrescimo = (desconto_acrescimo, 0)
                self.parent.ui.tableItens.horizontalHeaderItem(5).setText('Desc.')

                if desconto >= (preco_unit * quantidade):
                    self.parent.ui.lblItems.clear()
                    self.parent.ui.lblVlrItem.setText(u'0,00')
                    self.parent._lock_keyboard(False)
                    self.parent.check_button_status()
                    raise ValueError('Não é permitido desconto maior que valor do item.')

            elif state_item == item_add_surchage:
                acrescimo = value
                preco_total += acrescimo
                desconto_acrescimo = acrescimo
                desconto_acrescimo = (0, desconto_acrescimo)
                self.parent.ui.tableItens.horizontalHeaderItem(5).setText(u'Acrésc.')

                if acrescimo >= (preco_unit * quantidade):
                    self.parent.ui.lblItems.clear()
                    self.parent.ui.lblVlrItem.setText(u'0,00')
                    self.parent._lock_keyboard(False)
                    self.parent.check_button_status()
                    raise ValueError('Não é permitido acrescimo maior que valor do item.')

        return desconto_acrescimo

    @Log
    def cancelar_dav(self, automatico=False):
        nro_dav = self.parent.mov_capa.supcap_nro_cupom
        serie = self.parent.mov_capa.supcap_caixa
        value = {'supcap_state': '7',
                 'supcap_data_fim': datetime.now()}

        capa_id = self.parent.mov_capa.id
        self.parent.obj_mov_capa.write(self.parent.mov_capa, value)
        exec_sql("UPDATE caixa_mov_corpos SET supcor_venda_cancelamento = 'C' WHERE supcor_capa_id = %s" %
                  capa_id, 'C')
        self.parent.mov_capa = False
        exec_sql("""UPDATE caixa_mov_cupom_servidor 
                       SET supcse_enviado = False
                     WHERE supcse_capa_id = %s""" % capa_id)
        text = 'DAV CANCELADO'.center(48) + '\nNumero: %s  Caixa: %s' % (nro_dav, serie)
        comp_cancelamento = self.get_texto_cupom(text)
        self.cria_capa_cupom_gerencial(tipo='G')
        print '\nComprovate de Cancelamento:\n\n', comp_cancelamento
        self.parent.obj_impressoras.supcci_imprimir(comp_cancelamento)
        if not automatico:
            message_dlg(self.parent, MSG_INFO, "DAV %06d série %s cancelado com sucesso!" % (nro_dav, serie))

        return 'Cancelamento de DAV'

    @Log
    def cancelar_nota_fiscal_sefaz(self, capa=False, liberou_cancelamento=False):
        if not capa:
            capa = exec_sql("""SELECT id 
                               FROM caixa_mov_capa 
                               WHERE supcap_state IS NOT NULL
                               ORDER BY id DESC
                               LIMIT 1;""")

            if not capa:
                return message_dlg(self.parent, MSG_INFO, 'Não há nota para ser cacelada!')

            self.parent.mov_capa = self.parent.obj_mov_capa.search({'id': capa[0].get('id')})
        state = self.parent.mov_capa.supcap_state and \
                self.parent.mov_capa.supcap_state.strip() or ''

        if liberou_cancelamento and state and state in ('3', '6'):
            vals_aut = dict(
                supaut_cliente=self.parent.cliente.supcli_cnpj_cpf,
                supaut_capa_id=self.parent.mov_capa.id,
                supaut_liberou_limite_credito=False,
                supaut_liberou_cheque_a_vista=False,
                supaut_liberou_restricao_mensagem=False,
                supaut_liberou_cancelamento_cupom=True,
                supaut_liberou_desconto=False,
                supaut_autorizador_id=liberou_cancelamento)
            MovAutorizacoes().create(vals_aut)


        if state == '6':
            return self.cancelar_dav()

        elif state == '7':
            return message_dlg(self.parent, MSG_INFO, 'Atenção!\n\nO DAV %06d já foi cancelado!' %
                               self.parent.mov_capa.supcap_nro_cupom)

        nro_nota = self.parent.mov_capa.supcap_nro_cupom
        serie = self.parent.mov_capa.supcap_caixa
        if state != '3':
            if state in ('4', '5'):
                txt = 'foi \nemitida em CONTINGÊCIA!'
            elif state == 'C':
                txt = 'já está CANCELADA!'
            elif state == 'D':
                txt = 'está DENEGADA!'

            self.parent.mov_capa = False
            msg = "Atenção!\n\nA Nota %s série %s não pode ser cancelada " \
                  "pois %s" % (nro_nota, serie, txt)
            return message_dlg(self.parent, MSG_INFO, msg)

        chave_acesso = self.parent.mov_capa.supcap_chave_acesso
        ambiente = config['nfce_ambiente'] == 'P' and 1 or 2
        protocolo = self.parent.mov_capa.supcap_protocolo.replace(' ', '')
        if not protocolo:
            return message_dlg(self.parent, MSG_INFO, 'Atenção! \n\n'
                                                      'Nota sem protocolo, não é possível cancelá-la!')

        value = {}
        try:
            proc_nfce = nfc_e(self.parent.loja.supcfc_certificado_conteudo.decode('string_escape'),
                              self.parent.loja.supcfc_chave_conteudo.decode('string_escape'),
                              ambiente,
                              self.parent.loja.supcfc_estado)

            proc_nfce.process.tentativas = self.parent.loja.supcfc_qtd_conexoes_nfce or 20
            proc_nfce.process.timeout = self.parent.loja.supcfc_timeout_conexoes_nfce or 10
            processo = proc_nfce.process.cancelar_nfce(chave_acesso,
                                                       protocolo,
                                                       'O cliente desistiu da compra')
        except socket.gaierror:
            return message_dlg(self.parent, MSG_INFO, 'Falha no servidor da SEFAZ. \n\n'
                                                      'Verifique se existe conexao com a Internet.')

        except IOError:
            return message_dlg(self.parent, MSG_INFO, 'Falha no servidor da SEFAZ.  \n\n'
                                                      'Tente novamente mais tarde.')

        except Exception as erro:
            if str(erro).find('certificate expired') >= 1:
                erro = 'Certificado digital com validade vencida.'

            return message_dlg(self.parent, MSG_INFO, erro)

        else:
            if processo.resposta.cStat.valor == u'128':
                if processo.resposta.retEvento.infEvento.cStat.valor in (u'135-155-151'):
                    value['supcap_xml_cancelamento'] = processo.processo_cancelamento_nfe.xml
                    if processo.resposta.retEvento.infEvento.Id.valor:
                        value['supcap_protocolo_cancelamento'] = \
                            processo.resposta.retEvento.infEvento.Id.valor[2:]

                    value['supcap_status'] = '%s - %s' % (
                                                processo.resposta.retEvento.infEvento.cStat.valor,
                                                processo.resposta.retEvento.infEvento.xMotivo.valor)
                    value['supcap_state'] = 'C'
                    value['supcap_tipo'] = 'C'
                    value['supcap_data_fim'] = datetime.now()
                    capa_id = self.parent.mov_capa.id
                    self.parent.obj_mov_capa.write(self.parent.mov_capa, value)
                    exec_sql("UPDATE caixa_mov_corpos SET supcor_venda_cancelamento = 'C' WHERE supcor_capa_id = %s" %
                             capa_id, 'C')
                    self.parent.mov_capa = False
                    cont = 0
                    while self.parent.exportando_nota_nro == nro_nota:
                        time.sleep(0.5)
                        cont += 1
                        if cont > 80:
                            break

                    exec_sql("""UPDATE caixa_mov_cupom_servidor 
                                SET supcse_enviado = False
                                WHERE supcse_capa_id = %s""" % capa_id)
                    text = 'NOTA CANCELADA'.center(48) + '\nNumero: %s  Caixa: %s' % (nro_nota, serie)
                    comp_cancelamento = self.get_texto_cupom(text)
                    self.cria_capa_cupom_gerencial(tipo='G')
                    print '\nComprovate de Cancelamento:\n\n', comp_cancelamento
                    self.parent.obj_impressoras.supcci_imprimir(comp_cancelamento)
                    message_dlg(self.parent, MSG_INFO, "Nota %s série %s cancelada com sucesso!" % (nro_nota, serie))
                    return True

                else:
                    msg = '%s - %s' % (processo.resposta.retEvento.infEvento.cStat.valor,
                                       processo.resposta.retEvento.infEvento.xMotivo.valor)
                    return message_dlg(self.parent, MSG_INFO, msg)

            else:
                if not processo.resposta.cStat.valor:
                    erro = 'Erro interno na SEFAZ. Tente novamente.'
                else:
                    erro = '%s - %s.' % (processo.resposta.cStat.valor,
                                         processo.resposta.xMotivo.valor)

                return message_dlg(self.parent, MSG_INFO, erro)

    def tratar_contingencia(self, vals={}, nfce=False):
        '''A nota emitida contingencia tem a capa duplicada que ficará com a numeracao
           original, enquanto a nota antiga muda para numeracao e serie de contingencia +900'''
        serie = self.parent.caixa_id
        numero_nfce = self.parent.mov_capa.supcap_nro_cupom
        xml_antigo = nfce.xml
        # Antes de 01 de Abril de 2019 a contigencia usa a serie e numero normal, depois soma 900
        # regra_antiga = str(datetime.now().date()) < '2019-04-01'
        # if regra_antiga:
        nova_serie = serie
        nro_contingencia = numero_nfce
        if not self.parent.emitir_contingencia: # quando a sefaz estiver fora, nao aumenta o nro da nota
            nro_contingencia = numero_nfce + 1

        # else:
        #     nova_serie = serie + 900
        #     nro_contingencia = self.get_numero_contingencia()

        # elimina a possibilidade de erro ao inserir a mensagem de retorno da SEFAZ
        vals['supcap_status'] = (vals.get('supcap_status') or '')[:399].replace('"', '').replace("'", '').replace('\n', '')
        if vals['supcap_status'].split(' - ')[0] == '105':
            vals['supcap_status'] = ''

        nfce.infNFe.ide.serie.valor = nova_serie
        nfce.infNFe.ide.nNF.valor = nro_contingencia
        nfce.infNFe.ide.tpEmis.valor = 9  # contingencia
        nfce.monta_chave()
        nfce.infNFe.ide.cDV.valor = nfce.infNFe.Id.valor[-1]
        NFCeFrente(self.parent).fisnfc_set_nfe_infnfesupl(nfce)
        self.parent.nfce_qr_code = nfce.infNFeSupl.qrCode.valor
        vals.update({'supcap_caixa': nova_serie,
                     'supcap_nro_cupom': nro_contingencia,
                     'supcap_xml_gerado': nfce.xml,})
        self.parent.obj_mov_capa.write(self.parent.mov_capa, vals)
        if not self.parent.emitir_contingencia:
            dados = exec_sql("""SELECT * 
                                FROM caixa_mov_capa
                                WHERE id = %s 
                             """ % self.parent.mov_capa.id, 'C')[0]
            dados.get('id') and dados.pop('id')
            dados.get('supcap_numero_pedido') and dados.pop('supcap_numero_pedido')
            dados.update({'supcap_state': '5',
                          'supcap_caixa': serie,
                          'supcap_nro_cupom': numero_nfce,
                          'supcap_status': vals.get('supcap_status').replace('\n', ''),
                          'supcap_xml_gerado': xml_antigo})
            contingencia = self.parent.obj_mov_capa.create(dados)
            for corpo in exec_sql("""SELECT *
                                     FROM caixa_mov_corpos
                                     WHERE supcor_capa_id = %s
                                    """ % self.parent.mov_capa.id, 'C'):
                del corpo['id']
                corpo['supcor_capa_id'] = contingencia.id
                MovCorpos(self.parent).create(corpo)

            for fechamento in exec_sql("""SELECT *
                                          FROM caixa_mov_fechamentos
                                          WHERE supfec_capa_id = %s
                                       """ % self.parent.mov_capa.id, 'C'):
                fechamento_id = fechamento.pop('id')
                fechamento['supfec_capa_id'] = contingencia.id
                novo_fechamento_id = MovFechamentos(self.parent).create(fechamento).id
                for vencimento in exec_sql("""SELECT *
                                              FROM caixa_mov_vencimentos
                                              WHERE supfve_fechamentos_id = %s""" % fechamento_id, 'C'):
                    del vencimento['id']
                    vencimento['supfve_fechamentos_id'] = novo_fechamento_id
                    MovVencimentos().create(vencimento)

            MovCupomServidor().create({'supcse_capa_id': contingencia.id})

        self.parent.mov_capa.supcap_chave_acesso = nfce.chave
        self.parent.obj_mov_capa.write(self.parent.mov_capa, {'supcap_chave_acesso': nfce.chave})
        # if regra_antiga:
        if not self.parent.emitir_contingencia:
            self.set_numero_nf()
        # else:
        #     self.set_numero_contingencia()

    @Log
    def gerar_nfce(self):
        nfce = NFCeFrente(self.parent).gerar_nfce()
        return nfce

    @Log
    def envia_nota_servidor(self, loop=False):
        '''
        :param loop: booleano que controla quanto a funcao eh chamada por si mesma
        :return:
        '''
        nfce = self.gerar_nfce()
        self.parent.nfce_qr_code = nfce.infNFeSupl.qrCode.valor
        ambiente = config['nfce_ambiente'] == 'P' and 1 or 2  # 1=Producao, 2=Homologacao
        chave_acesso = nfce.chave
        lote = None
        try:
            xml = nfce.xml
            proc_nfce = nfc_e(self.parent.loja.supcfc_certificado_conteudo.decode('string_escape'),
                              self.parent.loja.supcfc_chave_conteudo.decode('string_escape'),
                              ambiente,
                              self.parent.loja.supcfc_estado)

            self.parent.emitir_contingencia = False
            if self.parent.loja.supcfc_incio_contingencia:
                data_atual = converte_tz(self, datetime.now(), 'America/Sao_Paulo',
                                         'America/Sao_Paulo')
                inicio_contingencia = converte_tz(self, self.parent.loja.supcfc_incio_contingencia,
                                                  'UTC', 'America/Sao_Paulo')
                fim_contingencia = converte_tz(self, self.parent.loja.supcfc_fim_contingencia,
                                               'UTC', 'America/Sao_Paulo')
                if inicio_contingencia <= data_atual and data_atual <= fim_contingencia:
                    self.parent.emitir_contingencia = True

            if self.parent.emitir_contingencia:
                self.tratar_contingencia({'supcap_state': '4',
                                          'supcap_chave_acesso': chave_acesso}, nfce)
                return True

            proc_nfce.process.tentativas = self.parent.loja.supcfc_qtd_conexoes_nfce or 20
            proc_nfce.process.timeout = self.parent.loja.supcfc_timeout_conexoes_nfce or 10
            proc_nfce.process.nova_versao = True
            processo = proc_nfce.process.processar_lote(nfce)
            lote = processo.resposta.infRec.nRec.valor
            if processo.resposta.cStat.valor == u'103':
                processo = proc_nfce.process.consultar_recibo(lote)


        except socket.gaierror:
            Log.error('Falha no servidor da SEFAZ. Verifique se existe conexao com a Internet.')
            self.tratar_contingencia({'supcap_state': '4',
                                      'supcap_lote': lote,
                                      'supcap_chave_acesso': chave_acesso}, nfce)

        except IOError:
            Log.error('Falha no servidor da SEFAZ. Tente novamente mais tarde.')
            self.tratar_contingencia({'supcap_state': '4',
                                      'supcap_lote': lote,
                                      'supcap_chave_acesso': chave_acesso}, nfce)

        except Exception as erro:
            if str(erro).find('certificate expired') >= 1:
                erro = ('Certificado digital com validade vencida.')
            Log.error('Erro ao faturar nota: %s' % erro)
            self.tratar_contingencia({'supcap_state': '2E',
                                      'supcap_lote': lote,
                                      'supcap_chave_acesso': chave_acesso,
                                      'supcap_status': str(erro)}, nfce)


        else:
            if processo.resposta.cStat.valor == u'105':  # Lote em processamento
                vals = {'supcap_state': '4',
                        'supcap_lote': processo.envio.nRec.valor,
                        'supcap_chave_acesso': chave_acesso,
                        'supcap_status': '105 - Lote em processamento',}
                self.tratar_contingencia(vals, nfce)

            elif processo.resposta.cStat.valor == u'104':  # Lote processado
                protocolo = processo.resposta.protNFe[0]  # Somente uma nota foi enviada
                chave_acesso = protocolo.infProt.chNFe.valor
                self.parent.mov_capa.supcap_chave_acesso = chave_acesso
                procnfce = ProcNFCe()
                procnfce.NFe.xml = nfce.xml
                procnfce.protNFe.xml = protocolo.xml
                if protocolo.infProt.cStat.valor == u'100':
                    # Nota fiscal faturada!
                    vals = {'supcap_lote': processo.envio.nRec.valor,
                            'supcap_protocolo': protocolo.infProt.nProt.valor,
                            'supcap_status': '%s - %s.' % (protocolo.infProt.cStat.valor,
                                                           protocolo.infProt.xMotivo.valor),
                            'supcap_xml_proc': procnfce.xml,
                            'supcap_state': '3',
                            'supcap_xml_gerado': xml,
                            'supcap_chave_acesso': chave_acesso}
                    self.parent.mov_capa.supcap_chave_acesso = chave_acesso
                    self.parent.obj_mov_capa.write(self.parent.mov_capa, vals)
                elif protocolo.infProt.cStat.valor == u'539': # Duplicidade com chave diferente
                    nro_nota = self.parent.mov_capa.supcap_nro_cupom
                    caixa = self.parent.mov_capa.supcap_caixa
                    self.parent.cancelar_nota_fiscal(automatico=True)
                    message_dlg(self.parent, MSG_INFO, "Atenção!\n\nNota duplicada na SEFAZ e chave diferente.\n"
                                                       "Favor entrar em contato com o SUPERVISOR!\n"
                                                       "Nota: %s \nSérie: %s" % (nro_nota,
                                                                                 caixa))

                elif protocolo.infProt.cStat.valor == u'204':  # Duplicidade de nota fiscal
                    try:
                        processo_consulta, __ = proc_nfce.consultar_nota(chave_acesso)
                    except socket.gaierror:
                        message_dlg(self.parent, MSG_ERROR, 'Falha ao comunicar com o servidor da '
                                                            'SEFAZ. Verifique se '
                                                            'existe conexao com a Internet.')
                    except Exception as erro:
                        message_dlg(self.parent, MSG_ERROR, erro.message)
                    except IOError:
                        message_dlg(self.parent, MSG_ERROR, 'Falha ao comunicar com o servidor da SEFAZ. '
                                                            'Tente novamente mais tarde.')

                    if processo_consulta.resposta.cStat.valor == u'100':  # Faturada
                        protocolo = processo_consulta.resposta.protNFe
                        vals = {'supcap_lote': processo.envio.nRec.valor,
                                'supcap_protocolo': protocolo.infProt.nProt.valor,
                                'supcap_status': '%s - %s.' % (u'100',
                                                               u'Autorizado o uso da NF-e.'),
                                'supcap_xml_proc': procnfce.xml,
                                'supcap_state': '3',
                                'supcap_xml_gerado': xml,
                                'supcap_chave_acesso': chave_acesso}
                        self.parent.mov_capa.supcap_chave_acesso = chave_acesso
                        self.parent.obj_mov_capa.write(self.parent.mov_capa, vals)

                    elif processo_consulta.resposta.cStat.valor in (u'137', u'217', u'999'):
                        nro_nota = self.parent.mov_capa.supcap_nro_cupom
                        caixa = self.parent.mov_capa.supcap_caixa
                        self.parent.show_info_proccess(False, '')
                        self.parent.cancelar_nota_fiscal(automatico=True)
                        message_dlg(self.parent, MSG_INFO, "Atenção!\n\nNota duplicada na SEFAZ.\n"
                                                           "Favor entrar em contato com o SUPERVISOR!\n"
                                                           "Nota: %s \nSérie: %s" % (nro_nota,
                                                                                     caixa))
                        return False

                elif protocolo.infProt.cStat.valor == u'213':
                    self.parent.cancelar_nota_fiscal(automatico=True)
                    msg = 'Cnpj cadastrado diferente do certificado digital'
                    message_dlg(self.parent, MSG_INFO, msg)
                    return False

                elif protocolo.infProt.cStat.valor in (u'228', u'704'):  # Data de emissao atrasada
                    if not loop:
                        self.parent.verifica_hora_servidor_sefaz(self.parent.loja, True)
                        self.parent.obj_mov_capa.write(self.parent.mov_capa,
                                                       {'supcap_ambiente': config['nfce_ambiente'],
                                                        'supcap_data_fim': datetime.now()})
                        return self.envia_nota_servidor(loop=True)
                    else:
                        self.parent.show_info_proccess(False, '')
                        nro_cupom = self.parent.mov_capa.supcap_nro_cupom
                        caixa = self.parent.mov_capa.supcap_caixa
                        self.parent.cancelar_nota_fiscal(automatico=True)
                        text = 'NOTA INUTILIZADA'.center(48) + '\nNumero: %s  Caixa: %s' % \
                               (nro_cupom, caixa)
                        texto_inutilizacao = self.get_texto_cupom(text)
                        self.cria_capa_cupom_gerencial(tipo='G')
                        self.parent.obj_impressoras.supcci_imprimir(texto_inutilizacao)
                        msg = 'Data de emissão atrasada em relação á SEFAZ!\nFavor entrar em contato com o SUPERVISOR!'
                        message_dlg(self.parent, MSG_INFO, msg)
                        return False

                elif protocolo.infProt.cStat.valor == u'703':  # Data de emissao posterior
                    if not loop:
                        self.parent.verifica_hora_servidor_sefaz(self.parent.loja, True)
                        self.parent.obj_mov_capa.write(self.parent.mov_capa,
                                                       {'supcap_ambiente': config['nfce_ambiente'],
                                                        'supcap_data_fim': datetime.now()})
                        return self.envia_nota_servidor(loop=True)

                    else:
                        self.parent.show_info_proccess(False, '')
                        # pega numero do cupom e caixa antes de cancelar pois no cancelamento self.parent.mov_capa vira false
                        nro_cupom = self.parent.mov_capa.supcap_nro_cupom
                        caixa = self.parent.mov_capa.supcap_caixa
                        self.parent.cancelar_nota_fiscal(automatico=True)
                        text = 'NOTA INUTILIZADA'.center(48) + '\nNumero: %s  Caixa: %s' % \
                               (nro_cupom, caixa)
                        texto_inutilizacao = self.get_texto_cupom(text)
                        self.cria_capa_cupom_gerencial(tipo='G')
                        self.parent.obj_impressoras.supcci_imprimir(texto_inutilizacao)
                        msg = 'Data de emissão adiantada em relação á SEFAZ!\nFavor entrar em contato com o SUPERVISOR!'
                        message_dlg(self.parent, MSG_INFO, msg)
                        return False

                elif protocolo.infProt.cStat.valor in (u'110', u'205', u'301', u'302', u'303'):
                    # Nota fiscal denegada
                    motivo = 'Rejeição: %s - %s.' % (protocolo.infProt.cStat.valor,
                                                     protocolo.infProt.xMotivo.valor)
                    vals = {'supcap_status': motivo,
                            'supcap_state': 'D',
                            'supcap_lote': lote,
                            'supcap_xml_gerado': xml,}
                    self.parent.obj_mov_capa.write(self.parent.mov_capa, vals)
                    self.parent.show_info_proccess(False, '')
                    message_dlg(self.parent, MSG_INFO,
                                '<br><br><center><h1><font color="red" size="10">*** Atenção! ***''</font></h1> '
                                '\nEsta nota foi DENEGADA!\n\n%s' % motivo)
                    return False

                else:  # Lote processado mas, houve erro
                    vals = {'supcap_status': '%s - %s.' % (protocolo.infProt.cStat.valor,
                                                           protocolo.infProt.xMotivo.valor),
                            'supcap_state': '2E',
                            'supcap_lote': lote,
                            'supcap_xml_gerado': xml,}
                    self.tratar_contingencia(vals, nfce)

            elif processo.resposta.cStat.valor == u'999':  # erro interno no sefaz ou sem internet
                motivo = u'Rejeicão: %s - %s.' % (processo.resposta.cStat.valor,
                                                 processo.resposta.xMotivo.valor)
                self.tratar_contingencia({'supcap_state': '4',
                                          'supcap_lote': lote,
                                          'supcap_chave_acesso': chave_acesso,
                                          'supcap_status': motivo}, nfce)
            else:
                vals = {'supcap_state': '2E',
                        'supcap_lote': lote,
                        'supcap_xml_gerado': xml,
                        'supcap_chave_acesso': chave_acesso}
                if not processo.resposta.cStat.valor:
                    vals['supcap_status'] =  'Erro interno na SEFAZ. Tente novamente.'
                else:
                    vals['supcap_status'] = '%s - %s.' % (processo.resposta.cStat.valor,
                                                          processo.resposta.xMotivo.valor)

                self.tratar_contingencia(vals, nfce)

        return True

    @Log
    def imprimir_dav(self):
        if self.parent.ficha or self.parent.vr_doacao:
            return True

        line_size = 48
        numero_pedido = self.parent.mov_capa.supcap_nro_cupom
        pedido_txt = []
        pedido_txt.append('{b1}' + self.parent.loja.supcfc_cliente.upper().center(line_size))
        pedido_txt.append(
            ('CNPJ: %s' % Cnpj().format(self.parent.loja.supcfc_cnpj)).center(line_size))
        pedido_txt.append(((self.parent.loja.supcfc_endereco or u'').upper() + \
                                     u', %s, ' % self.parent.loja.supcfc_numero + \
                                    (self.parent.loja.supcfc_bairro or u'').upper()).center(line_size))
        pedido_txt.append((u'%s-%s{b2}' % ((self.parent.loja.supcfc_cidade or u'').upper(),
                                       self.parent.loja.supcfc_estado)).center(line_size)),
        pedido_txt.append('')
        pedido_txt.append(''.ljust(line_size, '-'))
        linha = ('PEDIDO NRO: %s' % numero_pedido).ljust(20) + 'CAIXA: %s' % str(config['caixa']).ljust(21)
        pedido_txt.append(linha)
        pedido_txt.append(('EMISSAO: %s' % datetime.now().strftime('%H:%M:%S %d/%m/%Y')).ljust(48))
        pedido_txt.append('')
        pedido_txt.append('NAO E DOCUMENTO FISCAL. NAO E VALIDO COMO RECIBO')
        pedido_txt.append('E COMO GARANTIA DE MERCADORIA.'.center(line_size))
        pedido_txt.append('NAO COMPROVA PAGAMENTO.'.center(line_size))
        pedido_txt.append('')
        pessoa_pedido = self.parent.cliente.name or ''
        if pessoa_pedido:
            pedido_txt.append(('Cliente: %s' % pessoa_pedido[:39]).ljust(line_size))

        pedido_txt.append(''.ljust(line_size, '-'))
        pedido_txt.append('PRODUTO'.ljust(line_size))
        linha = 'CODIGO'.ljust(14) + ' ' + \
                'EMB'.ljust(5) + ' ' + \
                'QTD'.rjust(7) + ' ' + \
                'V.UNIT'.rjust(7) + ' ' + \
                'V.TOTAL'.rjust(11)
        pedido_txt.append(linha)
        pedido_txt.append(''.ljust(line_size, '-'))
        mov_corpos = MovCorpos(self.parent).search({'supcor_capa_id': self.parent.mov_capa.id,
                                                    'supcor_venda_cancelamento': 'V'},
                                                   order_by='supcor_sequencia')
        if not isinstance(mov_corpos, list):
            mov_corpos = [mov_corpos]

        total = desc_total = acres_total = 0
        for x in mov_corpos:
            pedido_txt.append(x.supcor_nome_item.ljust(line_size))
            linha = x.supcor_item.ljust(14) + ' ' + \
                    x.supcor_unidade.ljust(5) + ' ' + \
                    getNumeroEditado(x.supcor_quantidade).rjust(7) + ' ' + \
                    getNumeroEditado(x.supcor_preco_unitario or 0, 2).rjust(7) + ' ' + \
                    getNumeroEditado(x.supcor_preco_total or 0, 2).rjust(11)
            pedido_txt.append(linha)
            total += x.supcor_preco_total or 0
            desc_total += x.supcor_desconto_item or 0
            acres_total += x.supcor_acrescimo_item or 0
            if x.supcor_desconto_item_sem_rat:
                pedido_txt.append('-'.rjust(41) + getNumeroEditado(x.supcor_desconto_item_sem_rat).rjust(7))

            if x.supcor_acrescimo_item_sem_rat:
                pedido_txt.append('+'.rjust(41) + getNumeroEditado(x.supcor_acrescimo_item_sem_rat).rjust(7))

        pedido_txt.append(''.ljust(line_size, '-'))
        linha = 'Subtotal'.ljust(24)
        if not (desc_total or acres_total):
            linha += '{sub1}' + getNumeroEditado(total, 2).rjust(24) + '{sub2}'
            pedido_txt.append(linha)
        else:
            linha += getNumeroEditado(total, 2).rjust(24)
            pedido_txt.append(linha)
            if desc_total:
                linha = 'Desconto'.ljust(34)
                if not acres_total:
                    linha += '{sub1}' + ('- ' + getNumeroEditado(desc_total or 0, 2)).rjust(14) + '{sub2}'

                else:
                    linha += ('- ' + getNumeroEditado(desc_total or 0, 2)).rjust(14)
                pedido_txt.append(linha)

            if acres_total:
                linha = 'Acrescimo'.ljust(34) + '{sub1}' + \
                        ('+ ' + getNumeroEditado(acres_total or 0, 2)).rjust(14) + '{sub2}'
                pedido_txt.append(linha)

        linha = 'Total'.ljust(24) + (getNumeroEditado(self.parent.mov_capa.supcap_total_cupom, 2)).rjust(24)
        pedido_txt.append(linha)
        pedido_txt.append('')
        linha = 'Forma de pagamento'.ljust(28) + ' ' + \
                'Valor'.rjust(10) + ' '
        fechamentos = CaixaMovFechamentos.query.filter("supfec_capa_id = %s" % self.parent.mov_capa.id
                                                  ).order_by(CaixaMovFechamentos.id).all()
        troco = sum(map(lambda x: x.supfec_troco or 0, fechamentos))
        if troco:
            linha += 'Troco'.rjust(8)
        else:
            linha += ''.rjust(8)

        pedido_txt.append(linha)
        for i in fechamentos:
            linha = '%s %s ' % (i.supfec_nome_finalizadora[:28].ljust(28),
                                 getNumeroEditado(i.supfec_valor).rjust(10))
            if i.supfec_troco:
                linha += getNumeroEditado(i.supfec_troco).rjust(8)
            else:
                linha += ''.rjust(8)

            pedido_txt.append(linha)
        pedido_txt.append(''.ljust(line_size, '-'))
        pedido_txt.append('Operador: %s' % self.parent.user_logged.name.ljust(38))
        pedido_txt.extend(['', '', ''])
        print '\n\n' + '\n'.join(pedido_txt)
        self.parent.obj_impressoras.supcci_imprimir('\n'.join(pedido_txt))
        return True

    @Log
    def imprimir_nota(self, numero):
        if self.parent.ficha or self.parent.vr_doacao:
            return True

        capa = self.parent.obj_mov_capa.search({'supcap_nro_cupom': numero,
                                                'supcap_loja': config['loja'],
                                                'supcap_caixa': config['caixa']})
        if not capa:
            message_dlg(self.parent, MSG_INFO,
                        "Atenção! \n\nNúmero informado não existe no banco! \nNúmero: %s" % numero)
            return True

        cabecalho = ['{b1}%s{b2}' % self.parent.loja.supcfc_cliente.upper().center(48),
                     ((self.parent.loja.supcfc_endereco or u'').upper() + \
                                     u', %s, ' % self.parent.loja.supcfc_numero + \
                                    (self.parent.loja.supcfc_bairro or u'').upper()).center(48),
                     (u'%s-%s' % ((self.parent.loja.supcfc_cidade or u'').upper(),
                                 self.parent.loja.supcfc_estado)).center(48),
                     ('CNPJ:%s' % Cnpj().format(self.parent.loja.supcfc_cnpj)).center(48),
                     ('Tel: %s' % self.parent.loja.supcfc_telefone or '').center(48),
                     ''.ljust(48, '-'),
                     'DOCUMENTO AUXILIAR',
                     'DA NOTA FISCAL DE CONSUMIDOR ELETRONICA',
                     ''.ljust(48, '-'),
                     ' # | COD | DESC | QTD | UN | VL.UNIT | VL.TOT R$',
                     ''.ljust(48, '-') + '{cf1}']
        linha_corpos = []
        desconto_total = Decimal(0)
        acrescimo_total = Decimal(0)
        total_itens = Decimal(0)
        qtd_itens = []
        mov_corpos = MovCorpos(self.parent).search({'supcor_capa_id': self.parent.mov_capa.id},
                                                    order_by='supcor_sequencia')
        if not isinstance(mov_corpos, list):
            mov_corpos = [mov_corpos]

        for i in mov_corpos:
            if not i or i.supcor_venda_cancelamento == 'C':
                continue

            qtd = i.supcor_quantidade
            if i.supcor_item not in qtd_itens:
                qtd_itens.append(i.supcor_item)

            if qtd - int(qtd) < 0.001:
                qtd = int(i.supcor_quantidade)

            tot_item = i.supcor_preco_total
            if config['descricao_resumida'] or len(i.supcor_nome_item) <= 18:
                linha_corpos.append('%03d %s %s %7s %s X%6s %8s' %
                                    (i.supcor_sequencia, i.supcor_item[1:],
                                     i.supcor_nome_item[:18].ljust(18),
                                     qtd, i.supcor_unidade,
                                     getNumeroEditado(i.supcor_preco_unitario),
                                     getNumeroEditado(tot_item)))
            else:
                linha_corpos.append('%03d %s %s' %
                                    (i.supcor_sequencia, i.supcor_item[1:],
                                     i.supcor_nome_item[:46].ljust(46)))
                linha_corpos.append('%s %7s %s X%6s %8s' %
                                    (i.supcor_nome_item[46:82].ljust(36),
                                     qtd,
                                     i.supcor_unidade,
                                     getNumeroEditado(i.supcor_preco_unitario),
                                     getNumeroEditado(tot_item)))
            total_itens += tot_item
            desconto_item = i.supcor_desconto_item
            acrescimo_item = i.supcor_acrescimo_item
            if desconto_item:
                desconto_total += desconto_item

            if acrescimo_item:
                acrescimo_total += acrescimo_item

            if i.supcor_desconto_item_sem_rat:
                linha_corpos.append(('- ' + getNumeroEditado(i.supcor_desconto_item_sem_rat)).rjust(64))

            if i.supcor_acrescimo_item_sem_rat:
                linha_corpos.append(('+ ' + getNumeroEditado(i.supcor_acrescimo_item_sem_rat)).rjust(64))

        linha_corpos.append('{cf2}%s{cf1}' % ''.ljust(48, '-'))
        linha_corpos.append('QTD. TOTAL ITENS %47s' % len(qtd_itens))
        linha_corpos.append('Valor Total R$ %49s' % getNumeroEditado(total_itens))
        if desconto_total:
            linha_corpos.append('Desconto R$ %52s' % ('- %s' % getNumeroEditado(desconto_total)))
        if acrescimo_total:
            linha_corpos.append('Acrescimo R$ %51s' % ('+ %s' % getNumeroEditado(acrescimo_total)))

        if desconto_total or acrescimo_total:
            linha_corpos.append('{f1}{cf1}Valor a Pagar R$ %15s{f2}{cf1}' %
                                getNumeroEditado(self.parent.mov_capa.supcap_total_cupom))
        linha_corpos.append('FORMA DE PAGAMENTO %45s' % 'VALOR PAGO')
        troco = 0
        for i in CaixaMovFechamentos.query.filter("supfec_capa_id = %s" % self.parent.mov_capa.id
                                                  ).order_by(CaixaMovFechamentos.id).all():
            linha_corpos.append('%s %33s' % (i.supfec_nome_finalizadora[:30].ljust(30),
                                               getNumeroEditado(i.supfec_valor)))
            troco += i.supfec_troco or 0

        if troco:
            linha_corpos.append('%s %33s' % ('TROCO'.ljust(30), getNumeroEditado(troco)))

        chave = []
        for i in range(0, len(self.parent.mov_capa.supcap_chave_acesso)/4):
            chave.append(self.parent.mov_capa.supcap_chave_acesso[i*4:i*4+4])

        chave = ' '.join(chave)
        linha_corpos.append('{cf2}------------------------------------------------')
        if config['nfce_ambiente'] == 'H': # 'P' = Producao, 'H' = Homolocao
            linha_corpos.append('{cf1}{b1}%s{b2}{cf2}' %
                                'EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL'.center(64))
            linha_corpos.append('------------------------------------------------')

        linha_corpos.append('{b1}Consulte pela Chave de Acesso em:{b2}'.center(48))
        linha_corpos.append('{cf1}http://nfce.fazenda.mg.gov.br/portalnfce'.center(64))

        linha_corpos.append(chave.center(64) + '{cf2}')
        linha_corpos.append('------------------------------------------------{cf1}')
        cliente_incompleto = False
        cnpj_cpf = '%s' % self.parent.cliente.supcli_cnpj_cpf or ''
        if cnpj_cpf and self.parent.cliente.supcli_endereco \
                    and self.parent.cliente.supcli_endereco_numero \
                    and self.parent.cliente.supcli_cidade \
                    and self.parent.cliente.supcli_cod_municipio and int(self.parent.cliente.supcli_cod_municipio or 0)\
                    and self.parent.cliente.supcli_estado:
            if self.parent.cliente.supcli_tipo_pessoa == 'F':
                cnpj_cpf = 'CPF %s' % Cpf().format(cnpj_cpf[-11:])
            else:
                cnpj_cpf = 'CNPJ %s' % Cnpj().format(cnpj_cpf[-14:])

            texto = 'CONSUMIDOR - %s - %s' % (cnpj_cpf, self.parent.cliente.name or '')
            texto += ' - %s' % self.parent.cliente.supcli_endereco
            texto += ', %s' % self.parent.cliente.supcli_endereco_numero
            if self.parent.cliente.supcli_complemento:
                texto += '-%s' % self.parent.cliente.supcli_complemento

            texto += ', %s' % self.parent.cliente.supcli_bairro or 'Bairro'
            texto += ', %s' % self.parent.cliente.supcli_cidade
            texto += ' - %s' % self.parent.cliente.supcli_estado
            aux = ''
            for i in texto.split():
                if len(aux) + len(i) + 1 > 64:
                    linha_corpos.append(aux.center(64))
                    aux = i
                    continue

                aux += not aux and i or (' ' + i)

            linha_corpos.append(aux.center(64))
        else:
            linha_corpos.append('CONSUMIDOR NAO IDENTIFICADO'.center(64))
            cliente_incompleto = True

        data = datetime.strftime(self.parent.mov_capa.supcap_data_fim, '%d/%m/%Y %H:%M:%S')
        nro = self.parent.mov_capa.supcap_nro_cupom
        serie = int(self.parent.mov_capa.supcap_caixa)
        linha_corpos.append('NFC-e No %07d Serie %03d %s -Via Consumidor' % (nro, serie, data))
        contingencia = False
        if self.parent.mov_capa.supcap_state.strip() == '3':  # Faturada
            linha_corpos.append('Protocolo de autorizacao: %s' % self.parent.mov_capa.supcap_protocolo)
            linha_corpos.append('Data de autorizacao %s' % data)
        else:  # Contingencia
            contingencia = True
            linha_corpos.append(' {f1}EMITIDA EM CONTINGENCIA{f2} ')
            linha_corpos.append('Pendente de autorizacao'.center(48))

        linha_corpos.append('{qrcode}')

        vr_imposto_fed = getNumeroEditado(Decimal(self.parent.mov_capa.supcap_vr_imposto_fed or 0))
        vr_imposto_est = getNumeroEditado(Decimal(self.parent.mov_capa.supcap_vr_imposto_est or 0))
        vr_imposto_mun = getNumeroEditado(Decimal(self.parent.mov_capa.supcap_vr_imposto_mun or 0))
        # por questao de espaco, se nao houver imp. municipal, nao imprime o imposto
        if vr_imposto_mun:
            impostos = "Apr.Trib R$%s Fed R$%s Est R$ %s Mun Fonte:IBPT" % (vr_imposto_fed,
                                                                              vr_imposto_est,
                                                                              vr_imposto_mun)
        else:
            impostos = "Apr.Trib R$%s Fed R$%s Est Fonte:IBPT" % (vr_imposto_fed,
                                                                    vr_imposto_est)

        rodape = ['{cf2}------------------------------------------------{cf1}',
                  impostos.ljust(64),
                  ('Loja: %04d Caixa: %03d Operador: %s' % (config['loja'], config['caixa'],
                                                           self.parent.user_logged.name)).ljust(64),]
        if self.parent.vendedor_codigo:
            rodape.append(('Vendedor: %s' % (self.parent._nome_vendedor or 'Sem Nome')).ljust(64))

        if cliente_incompleto and cnpj_cpf:
              rodape.append(('%s - %s' % (cnpj_cpf, self.parent.cliente.name or '')).ljust(64))

        if self.parent._message_in_coupon:
            rodape.append(self.parent._message_in_coupon.ljust(64))

        rodape.extend([
                  '{cf2}------------------------------------------------' ,
                  'DESENVOLVIDO POR:'.ljust(48),
                  '{b1}Sig Solucoes em Informatica Ltda. (31)2106-1700{b2}',
                  '',
                  ''])

        corpo = '%s' % '\n'.join(linha_corpos)
        texto = ("%s\n%s\n%s" % ('\n'.join(cabecalho), corpo, '\n'.join(rodape)))
        print 'Nota\n\n %s \n\n' % texto
        self.parent.obj_impressoras.supcci_imprimir(texto)
        if contingencia and config['segunda_via']:
            texto = texto.replace('-Via Consumidor', '\nVia do Estabelecimento')
            print 'Nota\n\n %s \n\n' % texto
            self.parent.obj_impressoras.supcci_imprimir(texto)

        self.parent.nfce_qr_code = ''
        print 'Numero: %s' % numero
        return True
