#!/usr/bin/python
# -*- coding: utf-8 -*-

from sigpdv.tools.config import config
from sigpdv.tools.log import traced as Log
from decimal import Decimal
from sigpdv.ui.messagebox import MSG_INFO, MSG_ERROR, message_dlg
from sigpdv.models.cad_itens_data import CaixaCadItens
from sigpdv.control.mov_cupom_servidor import MovCupomServidor
from sigpdv.control.relatorio_gerencial import GerencialReport
from sigpdv.control.mov_fechamentos import MovFechamentos
from sigpdv.control.mov_meios_pagamentos import MovMeiosPagamentos
from datetime import datetime
from SigUtils.calcDigito import modulo10
from sigpdv.control.utils import exec_sql, converte_tz
from PySide import QtCore
in_coupon = 3
in_end_sale = 13
TWOPLACES = Decimal(10) ** -2


class Ficha():
    def __init__(self, parent):
        self.parent = parent

    @Log
    def recebimento_ficha(self, cod_totalizador, total_ficha, troco=0, nome_pgto=''):
        self.parent.bloqueia_botoes()
        # inicia abertura do cupom nao fiscal
        # self.parent.send_command('open_cupom_no_fiscal()')
        # self.parent.send_command('add_item_no_fiscal(%r, %s)' % (cod_totalizador, total_ficha))
        # self.parent.send_command('totalize_cupom_no_fiscal()')
        self.cria_capa_cupom_nao_fiscal(cod_totalizador, total_ficha, troco=troco, nome_pgto=nome_pgto)
        return True

    @Log
    def extorno_ficha(self, cod_totalizador, total_ficha, pgto_na_ecf, query_fin):
        # valor a devolver para o cliente, caso seja zero, apenas troca as fichas
        if total_ficha:
            self.cria_capa_cupom_nao_fiscal(cod_totalizador, total_ficha, query_fin)

        self.cria_cupom_fichas(total_ficha)
        return True

    @Log
    def pagamento_nao_fiscal(self, cod_finalizadora, value):
        # self.parent.send_command("add_payment_no_fiscal('%s', %s)" % (cod_finalizadora, value))
        return True

    @Log
    def finaliza_ficha(self):
        # self.parent.send_command('close_cupom_no_fiscal()')
        return True

    @Log
    def faturar_fichas(self):
        "Da lista de itens adidionados Fatura as fichas no banco de dados"
        tam = self.parent.ui.tableItens.rowCount()
        values_itens = []
        for i in range(0, tam):
            if self.parent.ui.tableItens.item(i, 7) and \
               self.parent.ui.tableItens.item(i, 7).checkState() == QtCore.Qt.CheckState.Checked and \
                    self.parent.ui.tableItens.item(i, 8) and \
                    self.parent.ui.tableItens.item(i, 8).text():
                codigo_barra = self.parent.ui.tableItens.item(i, 0).text()
                vr_item = self.parent.ui.tableItens.item(i, 4).text()
                cod_ficha = self.parent.ui.tableItens.item(i, 8).text()
                loja = int(cod_ficha[2:4])
                caixa = int(cod_ficha[4:6])
                cod_ficha = int(cod_ficha[6:14])
                self.atualizar_ficha('3', cod_ficha, loja, caixa)

        if values_itens:
            self.gravar_fichas(values_itens)

        return True

    def cria_fechamento_estorno(self, valor, query_finalizadora):
        self.parent.mov_fechamentos = MovFechamentos(self.parent).create(
            dict(supfec_sequencia=1,
                 supfec_finalizadora=query_finalizadora.supfin_codigo,
                 supfec_tipo_finalizadora=query_finalizadora.supfin_tipo_finalizadora,
                 supfec_valor=valor,
                 supfec_troco=0,
                 supfec_nro_banco=0,
                 supfec_agencia=0,
                 supfec_nro_cheque=0,
                 supfec_conta=0,
                 supfec_cnpj='',
                 supfec_nome_finalizadora=query_finalizadora.name,
                 supfec_capa_id=self.parent.mov_capa.id))

        # -------------------------------- Grava meios_pagamento --------------------------------- #
        MovMeiosPagamentos().gravar_meios_pagamento(self.parent.loja.supcfc_loja,
                                                    self.parent.caixa_id,
                                                    self.parent._date_coupon,
                                                    self.parent.mov_capa.supcap_tipo,
                                                    self.parent.mov_capa.supcap_tipo_comprovante,
                                                    valor, query_finalizadora.supfin_codigo,
                                                    query_finalizadora.name)

    @Log
    def cria_capa_cupom_nao_fiscal(self, totalizador, valor, query_estorno=False, troco=0, nome_pgto=''):
        # --------------------------------- Cria a capa do cupom --------------------------------- #
        texto = ''
        if query_estorno:
            texto += 'Estorno de Ficha\n'

        texto += ('Valor: %s' % valor).replace('.',',')

        if nome_pgto:
            texto += '\n%s' % nome_pgto

        if troco:
            texto += ('\n\nTroco: %s' % troco).replace('.',',')

        print 'Cupom de recebimento de ficha\n\n', self.parent.obj_cupom_fiscal.get_texto_cupom(texto)
        self.parent.obj_impressoras.supcci_imprimir(self.parent.obj_cupom_fiscal.get_texto_cupom(texto))
        self.parent.obj_cupom_fiscal.cria_capa_cupom_gerencial(cod_totalizador=totalizador, valor=valor)
        nro_cupom = self.parent._coupon_id
        self.parent.ui.lblNumCaixa.setText(u'%03d <b>Cupom #: %06d' % (self.parent.caixa_id,
                                                                       nro_cupom))
        if query_estorno:
            self.cria_fechamento_estorno(valor, query_estorno)
            #cria cupom servidor para o estorno, para o cupom normal nao eh aqui que cria
            MovCupomServidor().create({'supcse_capa_id': self.parent.mov_capa.id})

        return True

    def get_codigo_valido_ficha(self, loja, caixa):
        "Busca o primeiro codigo valido de ficha, o codigo da ultima ficha emitida mais 1"
        cod_ficha = 1
        for i in ['caixa', 'supervisor']:
            sql = """SELECT max(supfch_codigo_ficha) AS cod_ficha
                     FROM {tabela}_cad_ficha
                     WHERE supfch_loja = {loja}
                       AND supfch_caixa = {caixa}""".format(loja=loja, caixa=caixa, tabela=i)
            try:
                dados_ficha = exec_sql(sql, i)
            except:
                dados_ficha = False

            if dados_ficha and dados_ficha[0].get('cod_ficha'):
                cod_ficha = (dados_ficha[0].get('cod_ficha') or 0) + 1
                break

        if cod_ficha == 1:
            try:
                file = open('../zficha.txt', 'r')
                _loja, _caixa, _codigo = file.read().split('-')
                file.close()
                if _loja == loja and _caixa == _caixa:
                    cod_ficha = _codigo
            except:
                pass

        file = open('../zficha.txt', 'w')
        # loja - caixa - ficha
        file.write('%s-%s-%s' % (loja, caixa, cod_ficha))
        file.close()
        return cod_ficha

    def get_dados_item_ficha(self, text):
        "Recupera os dados do item da ficha no supervisor"
        if not self.parent.conectado:
            self.parent.ui.edtItemFicha.clear()
            message_dlg(self.parent, MSG_INFO, 'Atenção!\n\nSem conexão com o Supervisor!')
            return False, False, Decimal(0), Decimal(0)

        # enquanto houver fichas antigas, permitir a troca das mesmas.
        if not text.startswith('99') or len(text) < 15 or text[-1] != modulo10(text[:-1]):
            self.parent.ui.edtItemFicha.clear()
            message_dlg(self.parent, MSG_INFO, 'Atenção!\n\nCódigo de ficha inválido!')
            return False, False, Decimal(0), Decimal(0)

        loja = int(text[2:4])
        caixa = int(text[4:6])
        cod_ficha = text
        tam = self.parent.ui.tableItensFicha.rowCount()
        for i in range(0, tam):
            if self.parent.ui.tableItensFicha.item(i, 8) and \
                    self.parent.ui.tableItensFicha.item(i, 8).text() == cod_ficha:
                self.parent.ui.edtItemFicha.clear()
                message_dlg(self.parent, MSG_INFO, 'Atenção!\n\nEsta ficha já foi utilizada!')
                return False, False, Decimal(0), Decimal(0)

        tam = self.parent.ui.tableItens.rowCount()
        for i in range(0, tam):
            if self.parent.ui.tableItens.item(i, 8) and \
                    self.parent.ui.tableItens.item(i, 8).text() == cod_ficha:
                self.parent.ui.edtItemFicha.clear()
                message_dlg(self.parent, MSG_INFO, 'Atenção!\n\nEsta ficha já foi utilizada!')
                return False, False, Decimal(0), Decimal(0)


        dados_ficha = []
        descoto = Decimal(0)
        valor = Decimal(0)
        for i in ['supervisor']:
            sql = """SELECT supfch_item     AS cb,
                            supfch_valor    AS valor,
                            supfch_desconto AS desconto,
                            state 
                     FROM {tabela}_cad_ficha 
                     WHERE supfch_codigo_ficha = {cod}
                       AND supfch_loja = {loja}
                       AND supfch_caixa = {caixa}""".format(cod=int(cod_ficha[6:14]), loja=loja,
                                                            caixa=caixa, tabela=i)
            try:
                dados_ficha = exec_sql(sql, i)
            except:
                dados_ficha = []

            if dados_ficha:
                break

        if not dados_ficha:
            self.parent.ui.edtItemFicha.clear()
            message_dlg(self.parent, MSG_INFO, 'Atenção!\n\nFicha não encontrada!')
            return False, False, valor, descoto

        state = dados_ficha[0].get('state')
        if len(dados_ficha) > 1:
            dados_ficha = filter(lambda x: x.get('state') == '1', dados_ficha)
            if dados_ficha:
                state = dados_ficha[0].get('state')

        msg = ''
        if state == '2': # trocada
            msg = 'Atenção!\n\nEsta ficha já foi Trocada!'
        elif state == '3': # faturada
            msg = 'Atenção!\n\nEsta ficha já foi Faturada!'

        if msg:
            self.parent.ui.edtItemFicha.clear()
            message_dlg(self.parent, MSG_INFO, msg)
            return False, False, valor, descoto

        cod_item = dados_ficha[0].get('cb')
        valor = dados_ficha[0].get('valor')
        descoto = dados_ficha[0].get('desconto') or 0
        return cod_item, text, valor, descoto

    def gravar_fichas(self, values_itens):
        "Grava no banco as fichas emitidas"
        if values_itens:
            sql_insert = """INSERT INTO caixa_cad_ficha
                                  (supfch_codigo_ficha, state, create_date, supfch_valor , 
                                   supfch_loja, supfch_caixa, supfch_item, supfch_exportado,
                                   supfch_cz_nome_cliente, supfch_cz_data_hora_entrega,
                                   supfch_cz_observacoes, supfch_cz_state_impressao, 
                                   supfch_desconto)
                            VALUES %s """ % ',\n'.join(values_itens)
            sql_insert = sql_insert.replace("'null'", 'null')
            exec_sql(sql_insert, 'C')

    @Log
    def atualizar_ficha(self, state, cod_ficha, loja, caixa):
        "Grava a ficha como trocada/faturada, dependendo do que vier no state"
        for i in ['caixa', 'supervisor']:
            try:
                while True:
                    ficha = exec_sql("""SELECT id,
                                               state
                                        FROM %s_cad_ficha 
                                        WHERE supfch_codigo_ficha = %s 
                                          AND supfch_loja = %s 
                                          AND supfch_caixa = %s""" % (i, cod_ficha, loja, caixa), i)
                    if not ficha:
                        break

                    ficha = filter(lambda x: x.get('state') == '1', ficha)
                    ficha_id = ficha[0].get('id')
                    campos = ''
                    # Para cupons de troca marca o caixa tambem para saber onde foi a troca
                    if state == '2':
                        campos = """, supfch_caixa_faturado = %s
                        """ % self.parent.caixa_id

                    elif state == '3':
                        campos = """, supfch_caixa_faturado = %s,
                                      supfch_cupom_coo = %s,
                                      supfch_cupom_data = '%s' """ % \
                                         (self.parent.caixa_id,
                                          self.parent.mov_capa.supcap_nro_cupom,
                                          self.parent.mov_capa.supcap_data_cupom)

                    exec_sql("""UPDATE {tabela}_cad_ficha 
                                      SET state = '{state}' {campos}
                                    WHERE id = {id}""".
                             format(tabela=i,
                                    campos=campos,
                                    state=state,
                                    id=ficha_id), i)
                    break

            except Exception as e:
                Log.error('Erro ao atualizar ficha: %s' % e)
                if i == 'supervisor':
                    message_dlg(self.parent, MSG_ERROR, "Erro ao atualizar a ficha: %s\n"
                                               "Verifique se o supervisor está On-Line" % cod_ficha)
                    continue

                break

    @Log
    def cria_cupom_fichas(self, valor):
        loja = self.parent.loja.supcfc_loja
        caixa = self.parent.caixa_id
        tam = self.parent.ui.tableItensFicha.rowCount()
        cod_ficha = self.get_codigo_valido_ficha(loja, caixa)
        data_ficha = datetime.now()
        itens = []
        data = data_ficha.date().strftime('%d/%m/%Y')
        values_itens = []
        cbs = map(lambda x: self.parent.ui.tableItensFicha.item(x, 0).text(), range(0, tam))
        dict_itens = {}
        map(lambda x: dict_itens.update({x.supite_codigo_item: x.supite_producao}),
            CaixaCadItens.query.filter("supite_codigo_item in ('%s')" % "','".join(cbs)).all())
        try:
            for i in range(0, tam):
                vr_item = self.parent.ui.tableItensFicha.item(i, 3).text()
                codigo_barra = self.parent.ui.tableItensFicha.item(i, 0).text()
                # coluna 6 = False -> item cancelado, 7 = true ->item trocado
                if self.parent.ui.tableItensFicha.item(i, 6).checkState() == QtCore.Qt.CheckState.Checked and \
                   self.parent.ui.tableItensFicha.item(i, 7).checkState() != QtCore.Qt.CheckState.Checked:
                    qtd_item = int(Decimal(self.parent.ui.tableItensFicha.item(i, 2).text()))
                    # info(numero do codigo de barras, loja, caixa, sequencia)
                    for j in range(0, qtd_item):
                        if 'bematech':
                            info = '{:<16}{:>36}'.format(codigo_barra,
                             '{b1}L:%03d C:%03d S:%08d{b2}' % (loja, caixa, cod_ficha))
                        else:
                            info = '{:<16}{:>32}'.format(codigo_barra, 'L:%03d C:%03d S:%08d' %
                                                         (loja,
                                                          caixa,
                                                          cod_ficha))

                        codigo_barra_ficha = '99%02d%02d%08d' % (loja,
                                                                 caixa,
                                                                 cod_ficha)
                        codigo_barra_ficha = '%s%s' % (codigo_barra_ficha,
                                                       modulo10(codigo_barra_ficha))
                        item = []
                        item.append(self.parent.remove_accents(self.parent.loja.supcfc_cliente))
                        line = []
                        line.append('{a_cb}')  # altura do Codigo de barras
                        line.append('{l_cb}')  # largura do Codigo de barras
                        line.append('{pn_cb}')  # posicao dos numeros em relacao ao Codigo de barras
                        line.append('{me_cb}')  # margem a esquerda 1D 6B 84 0A 00 E0 03
                        line.append('{t_cb}%s%s' % (chr(len(codigo_barra_ficha)), codigo_barra_ficha))  # tipo do Codigo de barras
                        codigo_barra_ficha = ''.join(line)
                        item.append('%s' % codigo_barra_ficha)
                        item.append(info)
                        item.append(self.parent.ui.tableItensFicha.item(i, 1).text()[:48])
                        str_valor = 'Valor: R$%s' % vr_item.replace('.',',')
                        desconto = Decimal(self.parent.ui.tableItensFicha.item(i, 4).text()) / qtd_item
                        if not desconto:
                            tam = 48 - len(str_valor)
                            item.append('%s%s' % (str_valor, ('Data: %s' % data).rjust(tam)))
                        else:
                            item.append(('%s     Desconto: R$%.2f' %
                                          (str_valor, desconto)).replace('.',','))
                            item.append('Data: %s' % data)

                        itens.append('\n'.join(item))
                        cliente = self.parent.remove_accents(
                                                   self.parent.ui.tableItensFicha.item(i, 9).text())
                        data_entrega = self.parent.ui.tableItensFicha.item(i, 10).text()
                        if data_entrega:
                            data_entrega = converte_tz(self, datetime.strptime(data_entrega,
                                                                         '%H:%M %d/%m/%Y'),
                                        'America/Sao_Paulo', 'UTC').strftime('%d/%m/%Y %H:%M:%S')
                        obs = self.parent.ui.tableItensFicha.item(i, 11).text()
                        state_impressao = dict_itens.get(codigo_barra) == 'P' and '2' or '1'
                        values_itens.append("""({cod_ficha}, '1', '{data_ficha}', {vr_item},
                                                {loja}, {caixa}, '{cb}', False, '{cliente}', 
                                                '{data_entrega}', '{obs}', '{state_impressao}', 
                                                {desconto})"""
                                            .format(cod_ficha=cod_ficha,
                                                    data_ficha=data_ficha,
                                                    vr_item=Decimal(vr_item),
                                                    loja=loja,
                                                    caixa=caixa,
                                                    cb=codigo_barra,
                                                    cliente=cliente,
                                                    data_entrega=data_entrega or 'null',
                                                    obs=obs,
                                                    state_impressao=state_impressao,
                                                    desconto=desconto))

                        cod_ficha += 1
                # Troca de fichas
                elif self.parent.ui.tableItensFicha.item(i, 7).checkState() == QtCore.Qt.CheckState.Checked and \
                        self.parent.ui.tableItensFicha.item(i, 8) and \
                        self.parent.ui.tableItensFicha.item(i, 8).text():
                    cod_ficha_troca = self.parent.ui.tableItensFicha.item(i, 8).text()
                    _loja = int(cod_ficha_troca[2:4])
                    _caixa = int(cod_ficha_troca[4:6])
                    cod_ficha_troca = int(cod_ficha_troca[6:14])
                    self.atualizar_ficha('2', cod_ficha_troca, _loja, _caixa)

        except Exception as e:
            Log.error('Erro na montagem das fichas: %s' % e)
            # self.parent.send_command('cancel_last_coupon()')
            # coo = self.parent.send_command('get_coo()')
            message_dlg(self.parent, MSG_ERROR, "Atenção!\n\nErro na montagem das fihcas! "
                                                "Informe o fato ao supervisor. Cupom não fiscal "
                                                "%s cancelado." % 'Erro')
            return False

        self.gravar_fichas(values_itens)
        if not itens:
            return 'Devolucao de fichas'

        self.parent.show_info_proccess(True, 'Imprimindo Fichas...')
        self.parent.obj_impressoras.supcci_imprimir('{corta}'.join(itens))
        self.parent.show_info_proccess(False, '')
        return True