# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2017 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2017, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
from nfe.nfe_400.manual_600.nfe_400 import AutXML, Vol, DetPag
from nfe.nfce.nfce import Det
from nfe.nfe_400.webservices import UF_CODIGO
from nfe.nfce.webservices import ESTADO_WS
from nfe.pysped.xml_sped.certificado import Certificado
from nfe.nfce.webservices_flag import WS_CONSULTA_QRCODE, WS_CONSULTA_CHAVE
from nfe.nfce import qr_code
from nfe.nfce.nfce import NFCe
from datetime import datetime
from sigpdv.tools.config import config
from sigpdv.models.db_utils import exec_sql
from sigpdv.models.cad_tributacao_simples_nacional_data import CaixaCadTributacaoSimplesNacional
from decimal import Decimal
import re

HOMOLOGACAO = 2
TWOPLACES = Decimal(10) ** -2

def remove_accents(text):
    from unicodedata import normalize, category
    unicode_string = unicode(text, 'utf-8') if type(text) == str else text

    return unicode(filter(lambda c: category(c) != 'Mn', normalize('NFKD', unicode_string)))


# noinspection PyPep8Naming
class NFCeFrente(object):
    def __init__(self, parent):
        self.parent = parent
        self.total_aproveitamento_credito = 0

    def gerar_nfce(self):
        """
        Gera o XML da nota fiscal eletronica (saida)
        """
        # NF-e/NFC-e
        nfe = NFCe()
        # nfe.infNFe.Id.valor = \
        #     'NFe%s' % (dados_nf.get('nfe_chave_acesso') or self.fisnfc_monta_chave_acesso_nfe(
        #         cr, user, self.parent.mov_capa.id, context))
        # nfe.Signature.URI = nfe.infNFe.Id.valor

        self._fisnfc_set_nfe_infnfe(nfe)
        nfe.monta_chave()
        nfe.infNFe.ide.cDV.valor = nfe.infNFe.Id.valor[-1]
        self.fisnfc_set_nfe_infnfesupl(nfe)
        return nfe

    def fisnfc_set_nfe_infnfesupl(self, nfe):
        """
        Atribui os valores do QR-Code e da URL de consulta por chave da NFC-e.
        """
        # //NFe.infNFeSupl - Informacoes complementares da NFC-e
        ambiente = config['nfce_ambiente'] == 'P' and 1 or 2
        uf = self.parent.loja.supcfc_estado
        nfe.chave or nfe.gera_nova_chave()
        versao = '4.00'
        nfe.infNFeSupl.qrCode.valor = self._qrcode_v2(nfe, uf, versao, ambiente)
        nfe.infNFeSupl.urlChave.valor = ESTADO_WS[versao][uf][ambiente][WS_CONSULTA_CHAVE]['url']

    def _qrcode_v2(self, nfce, uf, versao, ambiente):
        """
        Constroi texto do QR-Code versao 2.0 da nota fiscal de consumidor.

        :param cr: cursor do banco de dados
        :param user: ID do usuario utilizado para a verificacao de perfil
        :param nf: dados da nota fiscal (browse)
        :param nfce: objeto nota fiscal eletronica - NFe()
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type user: int
        :type nf: browse
        :type nfce: NFCe
        :type context: dict
        :return: texto com o QR-Code a ser impresso no DANFE NFC-e.
        :rtype: str
        """
        nfce.chave or nfce.gera_nova_chave()
        url_qrcode = ESTADO_WS[versao][uf][ambiente][WS_CONSULTA_QRCODE]['url'] or u''
        csc = self.parent.loja.supcfc_csc or u''
        csc_id = 1 if csc else 2
        if ambiente == HOMOLOGACAO:
            csc_id = 1
        csc = [csc_id, csc]

        # DigestValue em funcao da contingencia OFFLINE
        if nfce.chave[34] in ('5', '9'):
            # Assina a NFC-e para recuperar o DigestValue
            certificado = Certificado()
            certificado.certificado = self.parent.loja.supcfc_certificado_conteudo.decode('string_escape')
            certificado.chave = self.parent.loja.supcfc_chave_conteudo.decode('string_escape')
            certificado.assina_xmlnfe(nfce)
            result = qr_code.qrcode_v2_offline(ambiente, url_qrcode, nfce.chave,
                                               nfce.Signature.DigestValue, csc,
                                               self.parent.mov_capa.supcap_data_cupom.day,
                                               self.parent.mov_capa.supcap_total_cupom)
        else:
            result = qr_code.qrcode_v2_online(ambiente, url_qrcode, nfce.chave, csc)

        return result

    def _fisnfc_set_nfe_infnfe(self, nfe):
        # Identificacao da nota
        self._fisnfc_set_nfe_infnfe_ide(nfe)

        # Dados do emitente
        self._fisnfc_set_nfe_infnfe_emit(nfe)

        # Dados do destinatario
        self._fisnfc_set_nfe_infnfe_dest(nfe)

        # Autorizados download XML
        self._fisnfc_set_nfe_infnfe_autxml(nfe)

        # Dados de pagamento
        self._fisnfc_set_nfe_infnfe_pag(nfe)

        # todo: verificar a tranportadora
        # Dados da transportadora
        self._fisnfc_set_nfe_infnfe_transp(nfe)

        # Itens da NF
        self._fisnfc_set_nfe_infnfe_prod(nfe)

        # Total produtos
        self._fisnfc_set_nfe_infnfe_total_icmstot(nfe)

    def _fisnfc_set_nfe_infnfe_ide(self, nfe):
        # //NFe.infNFe.Ide - Identificacao da NF-e
        nfe.infNFe.ide.cUF.valor = UF_CODIGO.get(self.parent.loja.supcfc_estado) or False
        nfe.infNFe.ide.cNF.valor = str(self.parent.mov_capa.id)[:8].zfill(8)
        nfe.infNFe.ide.natOp.valor = 'Venda para consumidor final'
        nfe.infNFe.ide.mod.valor = 65
        nfe.infNFe.ide.serie.valor = self.parent.mov_capa.supcap_caixa
        nfe.infNFe.ide.nNF.valor = self.parent.mov_capa.supcap_nro_cupom
        nfe.infNFe.ide.procEmi.valor = 0  # 0=Emissao de NF-e com aplicativo do contribuinte;
        nfe.infNFe.ide.dhEmi.valor = datetime.strptime(datetime.strftime(
                                        self.parent.mov_capa.supcap_data_fim or self.parent.mov_capa.supcap_data_inicio, '%d-%m-%Y %H:%M:%S'),
                                     '%d-%m-%Y %H:%M:%S')
        nfe.infNFe.ide.indPres.valor = '1' or '4'  # 4 = em operacao com entrega a domicilio
        nfe.infNFe.ide.tpNF.valor = 1 # 1 = Saida
        nfe.infNFe.ide.idDest.valor = 1 # 1 = Venda no estado
        nfe.infNFe.ide.cMunFG.valor = self.parent.loja.supcfc_cidade_ibge
        nfe.infNFe.ide.tpImp.valor = '4' # 4 = DANFE NFC-e
        nfe.infNFe.ide.verProc.valor = nfe.arquivo_esquema.split('.')[0]
        nfe.infNFe.ide.tpEmis.valor = 1 or 5 or 9 # 1 = Emissao normal, 5 e 9 = contingencias
        nfe.infNFe.ide.tpAmb.valor = config['nfce_ambiente'] == 'P' and 1 or 2
        nfe.infNFe.ide.indFinal.valor = u'1'  # 1 = Consumidor final
        nfe.infNFe.ide.finNFe.valor = 1  # 1 = NF-e normal

    def _fisnfc_set_nfe_infnfe_emit(self, nfe):
        # //NFe.infNFe.emit - Dados do emitente

        cnpj = self.parent.loja.supcfc_cnpj or u''
        nfe.infNFe.emit.CNPJ.valor = re.sub('[^0-9]', u'', cnpj[-14:])

        nfe.infNFe.emit.xNome.valor = (self.parent.loja.supcfc_cliente or u'').replace(u'\u2013', '-')
        nfe.infNFe.emit.IE.valor = re.sub('[^0-9A-Z]', u'', self.parent.loja.supcfc_inscricao_estadual or u'')
        nfe.infNFe.emit.xFant.valor = self.parent.loja.supcfc_fantasia or u''
        if self.parent.loja.supcfc_inscricao_municipal:
            nfe.infNFe.emit.IM.valor = self.parent.loja.supcfc_inscricao_municipal or u''

        nfe.infNFe.emit.enderEmit.fone.valor = \
            re.sub('[^0-9]', u'', str(self.parent.loja.supcfc_telefone) or u'')
        nfe.infNFe.emit.CRT.valor = self.parent.loja.supcfc_regime_tributario == 'S' and 1 or 3
        self._fisnfc_set_nfe_infnfe_emit_enderemit(nfe)

    def _fisnfc_set_nfe_infnfe_emit_enderemit(self, nfe):
        # //NFe.infNFe.emit.enderEmit - Endereco do emitente
        nfe.infNFe.emit.enderEmit.xLgr.valor = self.parent.loja.supcfc_endereco or ''
        nfe.infNFe.emit.enderEmit.nro.valor = u'%s' % self.parent.loja.supcfc_numero
        nfe.infNFe.emit.enderEmit.xCpl.valor = self.parent.loja.supcfc_complemento or u''
        nfe.infNFe.emit.enderEmit.xBairro.valor = self.parent.loja.supcfc_bairro or u'Bairro'
        nfe.infNFe.emit.enderEmit.xMun.valor = self.parent.loja.supcfc_cidade or u''
        nfe.infNFe.emit.enderEmit.cMun.valor = self.parent.loja.supcfc_cidade_ibge or 0
        nfe.infNFe.emit.enderEmit.UF.valor = self.parent.loja.supcfc_estado or u''
        nfe.infNFe.emit.enderEmit.cPais.valor = int('01058'[-4:]) or 0
        nfe.infNFe.emit.enderEmit.xPais.valor = u'Brasil'
        nfe.infNFe.emit.enderEmit.CEP.valor = re.sub('[^0-9]', u'', self.parent.loja.supcfc_cep or u'')

    def _fisnfc_set_nfe_infnfe_dest(self, nfe):
        # //NFe.infNFe.dest - Dados do destinatário
        if nfe.infNFe.ide.tpAmb.valor == 2:
            nfe.infNFe.dest.xNome.valor = u'NF-E EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR '\
                                            u'FISCAL'
        else:
            # Se nao houver destinatario (NFC-e) assume 'CONSUMIDOR NAO IDENTIFICADO'
            nome_consumidor = self.parent.cliente.name or 'CONSUMIDOR NAO IDENTIFICADO'
            nfe.infNFe.dest.xNome.valor = nome_consumidor

        nfe.infNFe.dest.email.valor = self.parent.cliente.supcli_email or u''
        nfe.infNFe.dest.indIEDest.valor = u'9'  # 9 = Nao contribuinte
        if not self._fisnfc_set_nfe_infnfe_dest_enderdest(nfe):
            return

        cnpj_cpf =  self.parent.cliente.supcli_cnpj_cpf
        if self.parent.cliente.supcli_tipo_pessoa == u'J':
            nfe.infNFe.dest.CNPJ.valor = re.sub('[^0-9]', u'', cnpj_cpf[-14:] or u'')

        else:
            nfe.infNFe.dest.CPF.valor = re.sub('[^0-9]', u'', cnpj_cpf[-11:] or u'') or u''

    def _fisnfc_set_nfe_infnfe_dest_enderdest(self, nfe):
        # //NFe.infNFe.dest.enderDest - Endereco do destinatario
        if not self.parent.cliente.supcli_endereco \
                or not self.parent.cliente.supcli_endereco_numero \
                or not self.parent.cliente.supcli_cidade \
                or not self.parent.cliente.supcli_cod_municipio or not int(self.parent.cliente.supcli_cod_municipio) \
                or not self.parent.cliente.supcli_estado:
            return False

        nfe.infNFe.dest.enderDest.xLgr.valor = self.parent.cliente.supcli_endereco or u''
        nfe.infNFe.dest.enderDest.nro.valor = self.parent.cliente.supcli_endereco_numero or u''
        nfe.infNFe.dest.enderDest.xBairro.valor = self.parent.cliente.supcli_bairro or u'Bairro'
        nfe.infNFe.dest.enderDest.cMun.valor = self.parent.cliente.supcli_cod_municipio or u''
        nfe.infNFe.dest.enderDest.xMun.valor = self.parent.cliente.supcli_cidade or u''
        nfe.infNFe.dest.enderDest.UF.valor = self.parent.cliente.supcli_estado or u''
        nfe.infNFe.dest.enderDest.xCpl.valor = u''
        nfe.infNFe.dest.enderDest.CEP.valor = re.sub('[^0-9]', u'', self.parent.cliente.supcli_cep
                                                     or u'')
        telefone = self.parent.cliente.supcli_telefone or u''
        nfe.infNFe.dest.enderDest.fone.valor = re.sub('[^0-9]', u'', telefone)
        nfe.infNFe.dest.enderDest.cPais.valor = int('01058'[-4:]) or 0
        nfe.infNFe.dest.enderDest.xPais.valor = u'Brasil'
        return True

    def _fisnfc_set_nfe_infnfe_autxml(self, nfe):
        # //NFe.infNFe.autXML - Pessoas autorizadas para o download do XML da NF-e
        if nfe.infNFe.emit.enderEmit.UF.valor == 'BA':  # Contemplado apenas pela Bahia
            if (nfe.infNFe.dest.CPF.valor or nfe.infNFe.dest.CNPJ.valor) != u'13937073000156':
                autxml = AutXML()
                autxml.CNPJ.valor = '13937073000156'  # CNPJ da SEFAZ Bahia
                nfe.infNFe.autXML.append(autxml)

        # CNPJ da Sig para consultar notas dos clientes.
        if nfe.infNFe.ide.tpAmb.valor == 1: # apenas em producao
            autxml = AutXML()
            autxml.CNPJ.valor = '01526558000178'
            nfe.infNFe.autXML.append(autxml)

        cnpj = self.parent.loja.supcfc_contador_cnpj or ''
        cpf = self.parent.loja.supcfc_contador_cpf or ''
        if cnpj and cnpj != nfe.infNFe.dest.CNPJ.valor:
            autxml = AutXML()
            autxml.CNPJ.valor = cnpj
            nfe.infNFe.autXML.append(autxml)

        elif cpf and cpf != nfe.infNFe.dest.CPF.valor:
            autxml = AutXML()
            autxml.CPF.valor = cpf
            nfe.infNFe.autXML.append(autxml)

    def _fisnfc_set_nfe_infnfe_pag(self, nfe):
        # //NFe.infNFe.pag - Dados de pagamento
        # {'01': 'Dinheiro',
        #  '02': 'Cheque',
        #  '03': 'Cartão de Crédito',
        #  '04': 'Cartão de Débito',
        #  '05': 'Crédito Loja',
        #  '10': 'Vale Alimentação',
        #  '11': 'Vale Refeição',
        #  '12': 'Vale Presente',
        #  '13': 'Vale Combustível',
        #  '17': 'PIX'
        #  '18': 'Carteiras Digitais'
        #  '99': 'Outros'}
        dados_pag = exec_sql("""
                    SELECT ccf.supfin_avista_aprazo     AS avista,
                           ccf.supfin_cartao_tipo       AS tipo_cartao,
                           ccf.supfin_tipo_finalizadora AS tipo_fin,
                           ccf.supfin_usa_tef           AS usa_tef,
                           scb.supban_tipo              AS tipo_ban,
                           scb.name                     AS nome_ban,
                           cmf.supfec_valor             AS valor,
                           cmf.supfec_troco             AS troco,
                           cmf.supfec_cnpj_cartao       AS cnpj_fec,
                           cmf.supfec_aut_cartao        AS aut_cartao,
                           cmf.supfec_nome_finalizadora AS nome_fin,
                           coc.supopc_cnpj              AS cnpj_op
                    FROM caixa_mov_fechamentos            AS cmf
                    INNER JOIN caixa_cad_finalizadora     AS ccf ON ccf.supfin_codigo = cmf.supfec_finalizadora
                    LEFT  JOIN caixa_cad_bandeiras        AS scb ON scb.id = ccf.supfin_bandeira_cre
                    LEFT  JOIN caixa_cad_operadora_cartao AS coc ON coc.id = ccf.supfin_operadora_id
                    WHERE cmf.supfec_capa_id = %s """ % self.parent.mov_capa.id)
        troco = 0
        for condicao in dados_pag:
            troco += condicao.get('troco') or 0
            detpag = DetPag()

            avista = 0
            if condicao.get('avista') == 'P':  # a prazo
                avista = 1

            hoje = str(datetime.now())
            detpag.indPag.valor = str(avista)  # 0=A vista ,1=A Prazo ,2=Outros
            tipo_pagamento = u'01'  # Dinheiro
            if condicao.get('tipo_fin') == 'H':
                tipo_pagamento = u'02'  # cheque
            elif condicao.get('tipo_fin') in ('C', 'T'):  # cartao e ticket
                if condicao.get('tipo_cartao') == 'C':
                    tipo_pagamento = u'03'  # credito
                    if condicao.get('tipo_ban') == 'O':  # Carteira digital
                        detpag.indPag.valor = str(0)
                        if hoje >= '2021-09-01' or config['nfce_ambiente'] == 'H':
                            tipo_pagamento = u'18'
                        if (condicao.get('nome_ban') or '').upper() == 'PIX' and \
                                (hoje >= '2021-09-01' or config['nfce_ambiente'] == 'H'):  # PIX
                            tipo_pagamento = u'17'
                else:
                    tipo_pagamento = u'04'  # debto

            elif condicao.get('tipo_fin') == 'O':
                tipo_pagamento = u'99'  # outros
                detpag.xPag.valor = (condicao.get('nome_fin') or '')[:60]

            detpag.tPag.valor = tipo_pagamento
            detpag.vPag.valor = '%.2f' % condicao.get('valor')
            if detpag.tPag.valor in (u'03', u'04'):  # Cartao credito/debito
                cnpj = condicao.get('cnpj_fec') or condicao.get('cnpj_op') or u''
                if cnpj:
                    cnpj = cnpj.rjust(14, '0')

                detpag.card.tpIntegra.valor = condicao.get('usa_tef') and 1 or 2  # 1=TEF, 2=POS
                detpag.card.CNPJ.valor = cnpj
                detpag.card.cAut.valor = condicao.get('aut_cartao') or u'123456'

            nfe.infNFe.pag.detPag.append(detpag)

        if troco:
            nfe.infNFe.pag.vTroco.valor = troco

    def _fisnfc_set_nfe_infnfe_transp(self, nfe):
        # //NFe.infNfE.transp - Dados da transportadora
        load = '_classic_write'

        # O DANFE da versao 4.0 ainda nao contempla todas as opcoes de frete
        # frete = {'0': u'0', '1': u'1', '2': u'2', '3': u'0', '4': u'1'}.get(dados_nf.get('frete'))
        # todo: quando tiver frete, tratar o frete
        nfe.infNFe.transp.modFrete.valor = u'9'
        return True

        if dados_nf.get('transportador_id') and dados_nf.get('frete') != u'9':
            campos = ['name', 'pessoa_id', 'admtra_annt']
            dados_transp = self.pool.get('adm.transportador').read(
                cr, user, dados_nf.get('transportador_id'), campos, context, load)
            campos = ['endereco_nf_id', 'pessoa_tipo', 'cpf_cnpj', 'admpes_jur_ie', 'endereco_ids']
            pessoa_transp = self.pool.get('adm.pessoa').read(
                cr, user, dados_transp.get('pessoa_id'), campos, context, load)
            nfe.infNFe.transp.transporta.xNome.valor = (dados_transp.get('name') or u'')[:45]
            if pessoa_transp.get('pessoa_tipo') == 'J':
                nfe.infNFe.transp.transporta.CNPJ.valor = re.sub(
                    '[^0-9]', u'', pessoa_transp.get('cpf_cnpj') or u'')
                nfe.infNFe.transp.transporta.IE.valor = re.sub(
                    '[^0-9A-Z]', u'', pessoa_transp.get('admpes_jur_ie') or '').upper()
            else:
                nfe.infNFe.transp.transporta.CPF.valor = re.sub(
                    '[^0-9A-z]', u'', pessoa_transp.get('cpf_cnpj') or u'')
            if pessoa_transp:
                obj_endereco = self.pool.get('adm.pessoa.endereco')
                campos = ['logradouro_id', 'cidade_id']
                ender_transp = pessoa_transp.get('endereco_nf_id') and obj_endereco.read(
                    cr, user, pessoa_transp.get('endereco_nf_id'), campos, context, load) or {}
                ender_transp = ender_transp or obj_endereco.read(
                    cr, user, pessoa_transp.get('endereco_ids')[0], campos, context, load)
                campos = ['titulo', 'descricao']
                logradouro = ender_transp.get('logradouro_id') and self.pool.get('adm.logradouro').read(
                    cr, user, ender_transp.get('logradouro_id'), campos, context) or {}
                campos = ['uf_id', 'name']
                cidade = ender_transp.get('cidade_id') and self.pool.get('adm.cidade').read(
                    cr, user, ender_transp.get('cidade_id'), campos, context, load) or {}
                uf = cidade.get('uf_id') and self.pool.get('adm.uf').read(
                    cr, user, cidade.get('uf_id'), ['sigla'], context) or {}
                nfe.infNFe.transp.transporta.xEnder.valor = \
                    logradouro and ((logradouro.get('titulo') or u'') + u' ' +
                                    (logradouro.get('descricao') or u'')).strip() or u''
                nfe.infNFe.transp.transporta.xMun.valor = cidade.get('name') or u''
                nfe.infNFe.transp.transporta.UF.valor = uf.get('sigla') or u''

        self._fisnfc_set_nfe_infnfe_transp_vol(cr, user, dados_nf, nfe, context)

    def _fisnfc_set_nfe_infnfe_transp_vol(self, cr, user, dados_nf, nfe, context=None):
        # //NFe.infNFe.transp.vol - Dados dos volumes
        context = context and context.copy() or {}
        vol = Vol()
        vol.qVol.valor = dados_nf.get('quantidade_volume') or 0
        vol.esp.valor = dados_nf.get('especie') or u''
        vol.marca.valor = dados_nf.get('marca') or u''
        vol.nVol.valor = dados_nf.get('numero_volume') or u''
        vol.pesoB.valor = '%.3f' % dados_nf.get('peso_bruto') or 0
        vol.pesoL.valor = '%.3f' % dados_nf.get('peso_liquido') or 0
        nfe.infNFe.transp.vol.append(vol)

    def _fisnfc_set_nfe_infnfe_prod(self, nfe):
        # //NFe.infNFe.prod - Itens da nota fiscal
        tot_ncm_vr_tributo_fed = tot_ncm_vr_tributo_est = tot_ncm_vr_tributo_mun = 0
        ncm_chave = u''
        self.total_aproveitamento_credito = 0
        dados_itens = exec_sql("""
                        SELECT cmc.*,
                               supcor_codigo_retaguarda     AS codigo_interno,
                               supcor_codigo_anp            AS codigo_anp,
                               supcor_adrem                 AS adrem,
                               supcor_pglp                  AS pglp,
                               supcor_pgnn                  AS pgnn,
                               supcor_pgni                  AS pgni,
                               supcor_vpart                 AS vpart,
                               supcor_producao              AS producao,
                               supcor_unidade               AS unidade,
                               supcor_origem_mercadoria     AS origem_mercadoria,
                               supcor_aliquota_ibpt_fed     AS supcor_aliquota_ibpt_fed,
                               supcor_aliquota_ibpt_est     AS supcor_aliquota_ibpt_est,
                               supcor_aliquota_ibpt_mun     AS supcor_aliquota_ibpt_mun,
                               supcor_cest                  AS supcor_cest,
                               supcor_ncm                   AS supcor_ncm,
                               supcor_getin                 AS supcor_getin,
                               supcor_codigo_trib_icms      AS codigo_icms,
                               supcor_codigo_cst_icms       AS codigo_cst,
                               supcor_tipo_tributacao_icms  AS tipo_trib,
                               supcor_codigo_csosn_icms     AS csosn,
                               supcor_base_icms             AS base_icms,
                               suptnf_base                  AS percent_base_icms,
                               supcor_aliquota_icms         AS aliquota_icms,
                               supcor_codigo_beneficio_icms AS cod_beneficio,
                               supcor_motivo_beneficio_icms AS mot_beneficio,
                               supcor_aliquota_fcp_icms     AS aliquota_fcp,
                               supcor_codigo_trib_pis       AS codigo_pis,
                               supcor_aliquota_pis          AS aliquota_pis,
                               supcor_codigo_cst_pis        AS codigo_cst_pis,
                               supcor_base_pis              AS base_pis,
                               supcor_codigo_trib_cofins    AS codigo_cofins,
                               supcor_aliquota_cofins       AS aliquota_cofins,
                               supcor_codigo_cst_cofins     AS codigo_cst_cofins,
                               supcor_base_cofins           AS base_cofins,
                               supite_med_codigo_anvisa     AS med_codigo_anvisa,
                               supite_med_motivo_isencao    AS med_motivo_isencao,
                               supite_med_preco_maximo      AS med_preco_maximo,
                               supite_fator_multiplicador   AS fator_multiplicador
                               
                        FROM caixa_mov_corpos               AS cmc
                        LEFT JOIN caixa_cad_tributacao_nfce AS cct ON cct.suptnf_codigo = cmc.supcor_codigo_trib_icms
                        LEFT JOIN caixa_cad_itens           AS sci ON sci.supite_codigo_item = cmc.supcor_item
                        WHERE cmc.supcor_capa_id = {capa_id}
                          AND cmc.supcor_venda_cancelamento = 'V'
                          AND sci.supite_loja = {loja}
                        ORDER BY cmc.supcor_sequencia""".format(capa_id=self.parent.mov_capa.id,
                                                                loja=self.parent.loja.supcfc_loja))
        if not dados_itens:
            return False

        numero_item = 1
        for item in dados_itens:
            if item.get('supcor_item') and item.get('supcor_getin'):
                codigo_barra = item.get('supcor_item')
                # Na nota 4.0 o primeiro digito do codigo de barras nao pode ser 0 quando tiver
                # 14 digitos, o primeiro digito deve ser de 1 a 9 que indicara a quantidade na
                # embalagem
                if len(codigo_barra) == 14:
                    if codigo_barra[0] == '0':
                        codigo_barra = codigo_barra[1:]

                p_cean = codigo_barra
                if int(p_cean) < 10000000:
                    p_cean = u'SEM GTIN'

            else:
                p_cean = u'SEM GTIN'

            ncm_vr_tributo_fed = 0
            if item.get('supcor_aliquota_ibpt_fed') > 0:
                ncm_vr_tributo_fed = item.get('supcor_quantidade') * item.get('supcor_preco_unitario')
                ncm_vr_tributo_fed *= item.get('supcor_aliquota_ibpt_fed') or 0
                ncm_vr_tributo_fed /= Decimal(100)

            ncm_vr_tributo_est = item.get('supcor_quantidade') * item.get('supcor_preco_unitario')
            ncm_vr_tributo_est *= item.get('supcor_aliquota_ibpt_est') or 0
            ncm_vr_tributo_est /= Decimal(100)
            ncm_vr_tributo_mun = item.get('supcor_quantidade') * item.get('supcor_preco_unitario')
            ncm_vr_tributo_mun *= item.get('supcor_aliquota_ibpt_mun') or 0
            ncm_vr_tributo_mun /= Decimal(100)
            if not ncm_chave:
                ncm_chave = item.get('fisnc_ibpt_chave') or u''

            tot_ncm_vr_tributo_fed += ncm_vr_tributo_fed
            tot_ncm_vr_tributo_est += ncm_vr_tributo_est
            tot_ncm_vr_tributo_mun += ncm_vr_tributo_mun
            p_ucom = item.get('unidade')
            p_qcom = '%.3f' % item.get('supcor_quantidade')
            p_vuncom = '%f' % item.get('supcor_preco_unitario')
            p_vuncom = p_vuncom.rstrip('0')  # Desconsidera zeros aa direita
            _valor = item.get('supcor_preco_total')
            p_vprod = '%.2f' % _valor
            p_ceantrib = p_cean
            det = Det()
            det.nItem.valor = numero_item
            numero_item += 1
            det.prod.cProd.valor = item.get('codigo_interno') or u''
            det.prod.cEAN.valor = p_cean or u''

            # Se NFC-e em ambiente de homologacao, a descricao do primeiro item deve ser
            # obrigatoriamente NOTA FISCAL EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL
            nome_item = u'NOTA FISCAL EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL'
            if item == dados_itens[0] and nfe.infNFe.ide.tpAmb.valor == 2:  # Homologacao
                det.prod.xProd.valor = u'NOTA FISCAL EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM ' \
                                       u'VALOR FISCAL'
            else:
                nome_item = (item.get('supcor_nome_item') or u'').replace('&', '&amp;')
                det.prod.xProd.valor = nome_item

            # Tratamento para combustiveis. Verifica se o produto tem CFOP para combustiveis e
            # faz a insercao das tags
            det.prod.NCM.valor = re.sub('[^0-9]', u'', item.get('supcor_ncm') or '') or u''
            if det.prod.NCM.valor[:4] in ('3001', '3002', '3003', '3004', '3005', '3006'):
                det.prod.med.cProdANVISA.valor = item.get('med_codigo_anvisa') or ''
                det.prod.med.xMotivoIsencao.valor = item.get('med_motivo_isencao') or ''
                det.prod.med.vPMC.valor = Decimal(item.get('med_preco_maximo') or 0).quantize(TWOPLACES) or 0

            #todo: verificar nFCI
            # det.prod.nFCI.valor = produto.get('estpro_fci') or u''

            det.prod.CFOP.valor = item.get('supcor_codigo_cfop')
            det.prod.uCom.valor = p_ucom or u''
            det.prod.qCom.valor = p_qcom or 0
            det.prod.vUnCom.valor = p_vuncom or 0
            if item.get('supcor_codigo_cfop') == '5656':
                det.prod.comb.cProdANP.valor = item.get('codigo_anp') or 0
                det.prod.comb.descANP.valor = (item.get('supcor_nome_item') or u'').replace('&', '&amp;')
                det.prod.comb.UFCons.valor = self.parent.loja.supcfc_estado
                if str(item.get('codigo_anp')) == '210203001':
                    # apenas para codigo anp 210203001, a unidade tributavel deve ser 'kg',
                    # se nao ocorre a rejeicao '854'
                    p_ucom = 'kg'
                    det.prod.comb.pGLP.valor = item.get('pglp') or 0
                    det.prod.comb.pGNn.valor = item.get('pgnn') or 0
                    det.prod.comb.pGNi.valor = item.get('pgni') or 0
                    det.prod.comb.vPart.valor = item.get('vpart') or 0

            det.prod.vDesc.valor = '%.2f' % (item.get('supcor_desconto_item') or 0)
            # det.prod.vFrete.valor = '%.2f' % item.get('vr_frete_rateado') or 0
            # det.prod.vSeg.valor = '%.2f' % item.get('vr_seguro_rateado') or 0

            det.prod.vOutro.valor = '%.2f' % item.get('supcor_acrescimo_item') or 0
            det.prod.vProd.valor = p_vprod or 0
            det.prod.cEANTrib.valor = p_ceantrib or u''
            det.prod.uTrib.valor = p_ucom or u''
            det.prod.qTrib.valor = '%.3f' % (item.get('supcor_quantidade') *
                                            (Decimal(item.get('fator_multiplicador')) or 1))
            det.prod.vUnTrib.valor = det.prod.vUnTrib.valor = '%.4f' % (det.prod.vProd.valor / det.prod.qTrib.valor)
            det.prod.indTot.valor = 1  # Valor do item entra no valor total da nota, 1-sim, 0-nao

            # //NFe.infNFe.imposto - Encargos e tributacoes
            det.imposto.vTotTrib.valor = '%.2f' % ((item.get('supcor_vr_imposto_fed') or 0) +
                                                   (item.get('supcor_vr_imposto_est') or 0) +
                                                   (item.get('supcor_vr_imposto_mun') or 0))

            if nfe.infNFe.emit.CRT.valor == 1:
                self.set_regime_tributario_simples_nacional(det, item)
            else:
                self.set_regime_tributario_outros(det, item)

            nfe.infNFe.det.append(det)

        # Aproveitamento de ICMS para Simples Nacional
        if self.parent.loja.supcfc_valor_faturamento > 0:
            if nfe.infNFe.emit.CRT.valor == 1:
                nfe.infNFe.infAdic.infCpl.valor += \
                    (u'\n Documento emitido por ME ou EPP optante pelo Simples Nacional. - NAO '
                      u'GERA DIREITO DE CREDITO FISCAL DE ISS E IPI. - Permite aproveitamento '
                      u'de credito ICMS no valor de R$ %.2f correspondente a aliq. %.2f%% nos '
                      u'termos ART.23 LC123/06') % (
                        self.total_aproveitamento_credito or 0, self._fisnfc_get_aliquota_sn())

        total_nf = self.parent.mov_capa.supcap_total_cupom
        valor_percent_aprox_fed = tot_ncm_vr_tributo_fed / total_nf * 100
        valor_percent_aprox_est = tot_ncm_vr_tributo_est / total_nf * 100
        valor_percent_aprox_mun = tot_ncm_vr_tributo_mun / total_nf * 100
        nfe.infNFe.infAdic.infCpl.valor += \
            (u' \n Trib.Aprox.: R$ %.2f (%.2f%%) federal R$ %.2f (%.2f%%) estadual R$ %.2f '
              u'(%.2f%%) municipal. Fonte IBPT %s') % (
                tot_ncm_vr_tributo_fed, valor_percent_aprox_fed, tot_ncm_vr_tributo_est,
                valor_percent_aprox_est, tot_ncm_vr_tributo_mun, valor_percent_aprox_mun,
                ncm_chave)
        # valor_cfop_por_aliquota = \
        #     self.pool.get('fiscal.nf.item.tributacao').fisnit_get_cfop_por_aliquota(
        #         dados_itens[0].get('tributacoes_ids'))

        # todo: verificar como fazer esta parte
        str_resumo = u''
        # for vcpa in valor_cfop_por_aliquota:
        #     str_resumo += '[%s - CFOP: %s ' % (vcpa.get('encargo'), vcpa.get('cfop'))
        #     if vcpa.get('vr_icms_substituicao'):
        #         str_resumo += ('Substituicao') + ': %s]' % getNumeroEditado(
        #             vcpa.get('vr_icms_substituicao'))
        #         continue
        #     if vcpa.get('vr_isento'):
        #         str_resumo += 'Isentos: %s ' % getNumeroEditado(vcpa.get('vr_isento'))
        #     if vcpa.get('aliquota'):
        #         str_resumo += 'Trib.a: %s%% ' % getNumeroEditado(vcpa.get('aliquota'), 3)
        #     if vcpa.get('vr_base_calculo'):
        #         str_resumo += 'Base: %s ' % getNumeroEditado(vcpa.get('vr_base_calculo'))
        #     if vcpa.get('vr_encargo'):
        #         str_resumo += 'Imp.: %s' % getNumeroEditado(vcpa.get('vr_encargo'))
        #     str_resumo += ']'
        if str_resumo:
            nfe.infNFe.infAdic.infCpl.valor += u'\n Resumo: ' + str_resumo
            
        total_icms_uf_dest = sum(map(lambda m: float(m.imposto.ICMSUFDest.vICMSUFDest.valor),
                                     nfe.infNFe.det))
        total_fcp = sum(map(lambda m: float(m.imposto.ICMS.vFCP.valor), nfe.infNFe.det))
        if total_icms_uf_dest:
            nfe.infNFe.infAdic.infCpl.valor += (u' \n Parcela do diferencial de aliquota devido '
                                                 u'ao estado destino: R$%.2f\n Valor do fundo de '
                                                 u'combate a pobreza: R$%.2f') % (
                                                   total_icms_uf_dest, total_fcp)
        # if self.pool.get('adm.utils').admutl_check_modulo_instalado(cr, user, 'ecf', context):
        #     for cond in dict_condicoes.values():
        #         cond_forma = dict_cond_forma.get(cond.get('condicao_forma_id'))
        #         if cond_forma.get('tipo_finalizadora') == u'C':
        #             nfe.infNFe.infAdic.infCpl.valor += \
        #                 (u'\n Autorizacao do cartao: %s') % cond.get('fiscpg_nro_autorizacao_cartao')

    def _fisnfc_set_nfe_infnfe_total_icmstot(self, nfe, context=None):
        # NFe.infNFe.total.ICMSTot - Total do grupo ICMS - Produtos
        context = context and context.copy() or {}
        nfe.infNFe.total.ICMSTot.vBC.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vBC.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vICMS.valor = '%.2f' % (sum(map(
            lambda m: float(m.imposto.ICMS.vICMS.valor), nfe.infNFe.det)) or 0)
        nfe.infNFe.total.ICMSTot.vICMSDeson.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vICMSDeson.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.qBCMonoRet.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.qBCMonoRet.valor), nfe.infNFe.det) or 0)
        nfe.infNFe.total.ICMSTot.vICMSMonoRet.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vICMSMonoRet.valor), nfe.infNFe.det) or 0)
        nfe.infNFe.total.ICMSTot.vFCP.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vFCP.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vBCST.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vBCST.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vST.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vICMSST.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vFCPST.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMS.vFCPST.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vProd.valor = '%.2f' % sum(map(
            lambda m: float(m.prod.vProd.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vFrete.valor = '%.2f' %  0
        nfe.infNFe.total.ICMSTot.vSeg.valor = '%.2f' % 0
        nfe.infNFe.total.ICMSTot.vDesc.valor = '%.2f' % sum(map(
            lambda m: float(m.prod.vDesc.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vII.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.II.vII.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vIPI.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.IPI.vIPI.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vPIS.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.PIS.vPIS.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vCOFINS.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.COFINS.vCOFINS.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vOutro.valor = '%.2f' % sum(map(
            lambda m: float(m.prod.vOutro.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vNF.valor = '%.2f' % self.parent.mov_capa.supcap_total_cupom
        nfe.infNFe.total.ICMSTot.vTotTrib.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.vTotTrib.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vICMSUFDest.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMSUFDest.vICMSUFDest.valor), nfe.infNFe.det))
        nfe.infNFe.total.ICMSTot.vICMSUFRemet.valor = '%.2f' % sum(map(
            lambda m: float(m.imposto.ICMSUFDest.vICMSUFRemet.valor), nfe.infNFe.det))

    def set_beneficio_fiscal(self, det, item, valor_desonerado):
        # CSTs: 20, 30, 40, 41, 50, 51, 70, 90
        pass
        # vr_liquido = item.get('supcor_preco_total') - item.get('supcor_desconto_item') + item.get('supcor_acrescimo_item')
        # base_icms = Decimal(vr_liquido * item.get('base_icms') / Decimal(100.0)).quantize(TWOPLACES)
        # vr_icms = Decimal(base_icms * Decimal(18) / Decimal(100.0)).quantize(
        #     TWOPLACES)
        # valor_desonerado = vr_icms
        # det.imposto.ICMS.vICMSDeson.obrigatorio = True
        # det.imposto.ICMS.motDesICMS.obrigatorio = True
        # det.imposto.ICMS.vICMSDeson.valor = '%.2f' % valor_desonerado
        # det.imposto.ICMS.motDesICMS.valor = int(item.get('mot_beneficio') or 0)

    def set_regime_tributario_outros_icms(self, det, item):
        codigo_cst = item.get('codigo_cst')
        det.imposto.ICMS.CST.valor = codigo_cst # deve ser diferente de 50, 51
        det.imposto.ICMS.orig.valor = int(item.get('origem_mercadoria') or '0')
        vr_encargo = Decimal(item.get('supcor_vr_icms') or 0)
        vr_base_icms =  Decimal(item.get('supcor_base_icms') or 0)
        det.imposto.ICMS.pICMS.valor = '%.2f' % item.get('aliquota_icms') or 0
        det.imposto.ICMS.vICMS.valor = '%.2f' % vr_encargo

        # Em operacoes interestaduais (idDest=2) para consumidor final (indFinal=1) nao
        # contribuinte (indIEDest=9), o valor do FCP deve ser informado no campo vFCPUFDest

        det.imposto.ICMS.pFCP.valor = '%.4f' % 0
        det.imposto.ICMS.vFCP.valor = '%.2f' % 0
        if codigo_cst == '00':  # Tributado integralmente
            det.imposto.ICMS.modBC.valor = u'3'
            det.imposto.ICMS.vBC.valor = '%.2f' % vr_base_icms

        elif codigo_cst == '20':  # Com reducao de base de calculo
            det.imposto.ICMS.modBC.valor = u'3'
            det.imposto.ICMS.pRedBC.valor = '%.4f' % (Decimal(100) - (Decimal(item.get('percent_base_icms') or 0)))
            det.imposto.ICMS.vBC.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.vBCFCP.valor = '%.2f' % vr_base_icms
            self.set_beneficio_fiscal(det, item, vr_encargo)

        elif codigo_cst == '30':
            aliq_fcp = Decimal(item.get('aliquota_fcp') or 0)
            vr_fcp = 0
            if self.parent.cliente and self.parent.cliente.supcli_tipo_pessoa == u'F':
                vr_fcp = vr_base_icms * aliq_fcp / Decimal(100)

            det.imposto.ICMS.pICMS.valor = '%.2f' % 0
            det.imposto.ICMS.vICMS.valor = '%.2f' % 0
            det.imposto.ICMSUFDest.pFCPUFDest.valor = '%.4f' % 0
            det.imposto.ICMSUFDest.vFCPUFDest.valor = '%.2f' % 0
            det.imposto.ICMS.modBCST.valor = '4'
            det.imposto.ICMS.vBCST.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.pICMSST.valor = '%.2f' % item.get('aliquota_icms') or 0
            det.imposto.ICMS.vICMSST.valor = '%.2f' % vr_encargo
            det.imposto.ICMS.vBCFCPST.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.pFCPST.valor = '%.4f' % aliq_fcp
            det.imposto.ICMS.vFCPST.valor = '%.2f' % vr_fcp
            det.prod.CEST.valor = item.get('supcor_cest') or ''
            self.set_beneficio_fiscal(det, item, vr_encargo)

        elif codigo_cst in ('40','41','50'):  # 40=Isento, 41=Nao tributado, 50=Com suspensao
            det.imposto.ICMS.pFCP.valor = '%.4f' % 0
            det.imposto.ICMS.vFCP.valor = '%.2f' % 0
            self.set_beneficio_fiscal(det, item, vr_encargo)

        elif codigo_cst == '51': # Com difereimento
            det.imposto.ICMS.vBC.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.vBCFCP.valor = '%.2f' % vr_base_icms
            self.set_beneficio_fiscal(det, item, vr_encargo)

        elif codigo_cst == '60':  # Cobrado anteriormente com substituicao tributaria
            det.prod.CEST.valor = item.get('supcor_cest') or ''
            det.imposto.ICMS.vBCSTRet.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.pFCP.valor = '%.4f' % 0
            det.imposto.ICMS.vFCP.valor = '%.2f' % 0
            det.imposto.ICMS.vICMSSTRet.valor = '%.2f' % vr_encargo

        elif codigo_cst == '61':  # Monofásico combustiveis
            det.prod.CEST.valor = item.get('supcor_cest') or ''
            det.imposto.ICMS.vBCSTRet.valor = '%.2f' % vr_base_icms
            if det.prod.comb.cProdANP.valor == 210203001:
                det.imposto.ICMS.qBCMonoRet.valor = '%.4f' % float(det.prod.qTrib.valor)
                det.imposto.ICMS.adRemICMSRet.valor = '%.4f' % item.get('adrem') or 0
                vICMSMonoRet = item.get('adrem') * Decimal(det.prod.qTrib.valor)
                det.imposto.ICMS.vICMSMonoRet.valor = '%.2f' % vICMSMonoRet

        elif codigo_cst == '70': # Com redução da BC e cobrança do ICMS por substituição tributária
            aliq_fcp = Decimal(item.get('aliquota_fcp') or 0)
            vr_fcp = 0
            if self.parent.cliente and self.parent.cliente.supcli_tipo_pessoa == u'F':
                vr_fcp = vr_base_icms * aliq_fcp / Decimal(100)

            det.imposto.ICMS.modBC.valor = '3'
            det.imposto.ICMS.pRedBC.valor = '%.4f' % (Decimal(100) - (Decimal(item.get('percent_base_icms') or 0)))
            det.imposto.ICMS.vBC.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.vBCFCPST.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.pFCPST.valor = '%.4f' % aliq_fcp
            det.imposto.ICMS.vFCPST.valor = '%.2f' % vr_fcp
            det.imposto.ICMS.modBCST.valor = '4'
            det.imposto.ICMS.vBCST.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.vICMSST.valor = '%.2f' % vr_encargo
            det.imposto.ICMS.vBCFCP.valor = '%.2f' % vr_base_icms
            det.prod.CEST.valor = item.get('supcor_cest') or ''
            self.set_beneficio_fiscal(det, item, vr_encargo)

        elif codigo_cst == '90':  # Outras tributacoes)
            det.imposto.ICMS.modBC.valor = '3'
            det.imposto.ICMS.vBC.valor = '%.2f' % vr_base_icms
            det.imposto.ICMS.vBCFCP.valor = '%.2f' % vr_base_icms
            det.prod.CEST.valor = item.get('supcor_cest') or ''

        # Isentos
        if item.get('tipo_trib').strip() == u'I':
            det.imposto.ICMS.vBC.valor = '%.2f' % 0

    def set_regime_tributario_outros_pis(self, det, item):
        codigo_cst = item.get('codigo_cst_pis')
        codigo_cst = '99' if codigo_cst == '05' else codigo_cst
        det.imposto.PIS.CST.valor = codigo_cst
        vr_base_pis = item.get('supcor_base_pis') or 0
        aliq_pis = item.get('aliquota_pis') or 0
        vr_pis = item.get('supcor_vr_pis') or 0
        if codigo_cst in ('01', '02'):
            det.imposto.PIS.vBC.valor = '%.2f' % vr_base_pis
            det.imposto.PIS.pPIS.valor = '%.4f' % item.get('aliquota_pis')
            det.imposto.PIS.vPIS.valor = '%.2f' % vr_pis
        elif codigo_cst == '03':
            det.imposto.PIS.qBCProd.valor = '%.4f' % item.get('supcor_quantidade')
            det.imposto.PIS.vAliqProd.valor = '%.4f' % \
                                              (vr_base_pis * (aliq_pis / Decimal(100)))
            det.imposto.PIS.vPIS.valor = '%.2f' % vr_pis

        elif codigo_cst in ('49', '50', '51', '52', '53', '54', '55', '56', '60', '61', '62', '63',
                            '64', '65', '66', '67', '70', '71', '72', '73', '74', '75', '98', '99'):
            det.imposto.PIS.vBC.valor = '%.2f' % vr_base_pis
            det.imposto.PIS.pPIS.valor = '%.4f' % aliq_pis
            det.imposto.PIS.vPIS.valor = '%.2f' % vr_pis
            det.imposto.PIS.qBCProd.valor = '%.4f' % item.get('supcor_quantidade')
            det.imposto.PIS.vAliqProd.valor = '%.4f' % (vr_base_pis * (aliq_pis / Decimal(100)))
            det.imposto.PIS.vPIS.valor = '%.2f' % vr_pis

    def set_regime_tributario_outros_cofins(self, det, item):
        codigo_cst = item.get('codigo_cst_cofins')
        codigo_cst = '99' if codigo_cst == '05' else codigo_cst
        det.imposto.COFINS.CST.valor = codigo_cst
        vr_base_cofins = item.get('supcor_base_cofins') or 0
        aliq_cofins = item.get('aliquota_cofins') or 0
        vr_cofins = item.get('supcor_vr_cofins') or 0
        if codigo_cst in ('01', '02'):
            det.imposto.COFINS.vBC.valor = '%.2f' % vr_base_cofins
            det.imposto.COFINS.pCOFINS.valor = '%.4f' % aliq_cofins
            det.imposto.COFINS.vCOFINS.valor = '%.2f' % vr_cofins
        elif codigo_cst == '03':
            det.imposto.COFINS.qBCProd.valor = '%.4f' % item.get('supcor_quantidade')
            det.imposto.COFINS.vAliqProd.valor = '%.4f' % \
                                                 (vr_base_cofins * (aliq_cofins / Decimal(100)))
            det.imposto.COFINS.vCOFINS.valor = '%.2f' % vr_cofins
        elif codigo_cst in ('04', '06', '07', '08', '09'):
            pass
        elif codigo_cst in ('49', '50', '51', '52', '53', '54', '55', '56', '60', '61', '62', '63',
                            '64', '65', '66', '67', '70', '71', '72', '73', '74', '75', '98', '99'):
            det.imposto.COFINS.vBC.valor = '%.2f' % vr_base_cofins
            det.imposto.COFINS.pCOFINS.valor = '%.4f' % aliq_cofins
            det.imposto.COFINS.qBCProd.valor = '%.2f' % item.get('supcor_quantidade')
            det.imposto.COFINS.vAliqProd.valor = '%.4f' % \
                                                 (vr_base_cofins * (aliq_cofins / Decimal(100)))
            det.imposto.COFINS.vCOFINS.valor = '%.2f' % vr_cofins

    def set_regime_tributario_outros(self, det, item):
        self.set_regime_tributario_outros_icms(det, item)
        self.set_regime_tributario_outros_pis(det, item)
        self.set_regime_tributario_outros_cofins(det, item)

    def set_regime_tributario_simples_nacional_icms(self, det, item):
        """
        Atribui os valores de ICMS para regime tributario Simples Nacional.
        :param det: item de movimentacao
        :param item: dados do item
        :type det: Det (objeto ´nfe.infNFe.det´)
        :type item: dict
        :return: n/a
        :rtype: n/a
        """
        csosn = item.get('csosn')
        # todo: ver como fica essa parte de teste de csosn
        # if not csosn:  # Como o CSOSN nao eh obrigatorio evita-se erro ao validar o arquivo .xsd
        #     produto_nome = self.pool.get('estoque.produto').read(
        #         cr, user, produto['id'], ['name'], context).get('name')
        #     sequencia = self.pool.get('estoque.movimento.itens').read(
        #         cr, user, item['id'], ['sequencia'], context).get('sequencia')
        #     raise osv.except_osv(('Atencao!'),
        #                          ('Nenhum CSOSN cadastrado para ICMS/%s no item %s-%s.') % (
        #                              cst.get('codigo'), sequencia, produto_nome))

        det.imposto.ICMS.orig.valor = item.get('origem_mercadoria')
        det.imposto.ICMS.CSOSN.valor = csosn
        aliquota = self._fisnfc_get_aliquota_sn()
        vr_cred_icms_sn = item.get('supcor_base_icms') * aliquota / Decimal(100)
        if csosn == '101':
            det.imposto.ICMS.pCredSN.valor = '%.2f' % aliquota
            det.imposto.ICMS.vCredICMSSN.valor = '%.2f' % vr_cred_icms_sn
        elif csosn == '500':
            det.imposto.ICMS.vBCSTRet.valor = '%.2f' % item.get('supcor_base_icms')
        # elif csosn == '900':
        #     det.imposto.ICMS.vCredICMSSN.valor = 0

        codigo_cst = item.get('codigo_cst')
        if codigo_cst == '60':
            det.prod.CEST.valor = item.get('supcor_cest') or ''

        # Para tipo tributado soma o aproveitamento de credito
        if item.get('supcor_tipo_tributacao') == u'T':
            # Aproveitamento de credito
            self.total_aproveitamento_credito += \
                (item.get('supcor_base_icms') * aliquota / Decimal(100))

    def set_regime_tributario_simples_nacional(self, det, item):
        """
        Atribui valores para o regime Simples Nacional
        :param det: item de movimentacao
        :param item: dados do item
        :type det: Det (objeto ´nfe.infNFe.det´)
        :type item: dict
        :return: n/a
        :rtype: n/a
        """
        det.imposto.ICMS.regime_tributario = 1  # Simples Nacional
        self.set_regime_tributario_simples_nacional_icms(det, item)
        det.imposto.PIS.CST.valor = u'99'  # Outras operacoes
        det.imposto.COFINS.CST.valor = u'99'  # Outras operacoes


    def _fisnfc_get_aliquota_sn(self):
        """
        Recupera o valor da aliquota do Simples Nacional.
        """
        aliquota_sn = 0
        receita_anual = self.parent.loja.supcfc_valor_faturamento
        for trib in CaixaCadTributacaoSimplesNacional.query.order_by(
                                  CaixaCadTributacaoSimplesNacional.suptsn_valor_faturamento).all():
            if receita_anual <= trib.suptsn_valor_faturamento:
                aliquota_sn = trib.suptsn_aliq_icms or 0
                break

        return aliquota_sn
