#!/usr/bin/python
# -*- encoding: utf-8 -*-

__autor__ = 'Dener Rodrigues'

from datetime import datetime, date
from PySide import QtGui
import psycopg2
import psycopg2.extras
from sigpdv.tools.log import traced as Log
from sigpdv.tools.config import config
from sigpdv.models import db_utils

# Tabelas que sempre serão truncadas na importação
TABELAS_TRUNCATE = ['cad_kits', 'cad_finalizadora_na_ecf', 'cad_bandeiras', 'cad_finalizadora',
                    'cad_operadora_cartao',
                    'cad_vendedor', 'cfg_parametros', 'cad_comandos_impressoras',
                    'cfg_configuracao', 'cfg_mensagens', 'mov_capa', 'mov_meios_pagamentos',
                    'cad_restricao_item', 'cad_tributacao_nfce',
                    'cad_promo_capa', 'cad_promo1', 'cad_promo2', 'cad_promo3f',
                    'cad_promo3p', 'cad_promo4', 'cad_promo5f', 'cad_promo5p',
                    'cad_promo6', 'cad_promo7', 'cad_lista_clientes', 'cad_mensagens_clientes',
                    'cad_sorteio', 'tributacao_simples_nacional', 'cad_clientes']

COLUNAS_A_IMPORTAR = [] # eh carregada em execucao

# Nome de todas as tabelas a serem importadas
NOMES_TABELAS = {'cad_itens': 'Itens',
                 'cad_restricao_item': 'Restricao',
                 'cad_promo_capa': 'Promocoes',
                 'cad_promo1': 'Promocoes',
                 'cad_promo2': 'Promocoes',
                 'cad_promo3f': 'Promocoes',
                 'cad_promo3p': 'Promocoes',
                 'cad_promo4': 'Promocoes',
                 'cad_promo5f': 'Promocoes',
                 'cad_promo5p': 'Promocoes',
                 'cad_promo6': 'Promocoes',
                 'cad_promo7': 'Promocoes',
                 'cad_lista_clientes': 'Promocoes',
                 'cad_tributacao': 'Tributacao',
                 'cad_tributacao_nfce': 'Tributacao NFC-e',
                 'tributacao_simples_nacional': 'Tributacao Simples Nacional',
                 'cad_departamento': 'Departamentos',
                 'cad_mensagens_clientes': 'Mensagem',
                 'cad_operadores': 'Operadores',
                 'cad_finalizadora_na_ecf': 'FinalizadorasNaEcf',
                 'cad_bandeiras': 'Bandeiras',
                 'cad_finalizadora': 'Finalizadoras',
                 'cad_operadora_cartao': 'Operadora de cartao',
                 'tipo_recebimento': 'TipoRecebimento',
                 'cad_clientes': 'Clientes',
                 'cfg_parametros': 'Parametros',
                 'cfg_configuracao': 'Parametros',
                 'cad_comandos_impressoras': 'Parametros',
                 'cad_kits': 'Kits',
                 'cad_vendedor': 'Vendedor',
                 'cfg_mensagens': 'Mensagem',
                 'reducao_z': 'Reducao_Z',
                 'layouts_de_cheques': 'Layouts de Cheques',
                 'cad_sorteio': 'Sorteios',}


# dicionario para guaradar os ids das capas importadas na importacao full, a chave eh o id que vem
# do supoervisor e o valor eh o id do caixa
dict_capa_ids = {}
# idem de dict_capa_ids
dict_fechamentos_ids = {}
# idem de dict_capa_ids
dict_bandeira_ids = {}


class ImportacaoAutomatica(object):
    def __init__(self, parent, full=False):
        self.parent = parent
        self.total_geral = 0
        self.contador = 0
        self.full = full
        self.erro_importacao = False
        self.total_registros = {}
        self.contador_registros = 0
        self.connection_caixa = db_utils.get_connection('C')
        self.connection_supervisor = db_utils.get_connection('S')
        self.cursor_caixa = self.connection_caixa.cursor(cursor_factory=psycopg2.extras.DictCursor)
        self.cursor_supervisor = self.connection_supervisor.cursor(cursor_factory=psycopg2.extras.DictCursor)

    def get_sup_cad_pdvs(self, loja_id, caixa_id):
        sql = """SELECT *
                 FROM   supervisor_cad_pdvs
                 WHERE  suppdv_loja=%d
                 AND    suppdv_caixa=%d LIMIT 1 ;""" % (loja_id, caixa_id)
        try:
            res = db_utils.exec_sql(sql, 'S', ret=True, connection=self.connection_supervisor,
                                    close_connection=False, auto_commit=False)
        except Exception as err:
            raise err
        if res:
            res = res[0]
        return res

    def update_cad_pdvs(self, coluna_pdv, pdv_id):
        sql = """UPDATE   supervisor_cad_pdvs
                 SET %s = (NOW() at time zone 'UTC')
                 WHERE    id=%d ; """ % (coluna_pdv, pdv_id)
        try:
            res = db_utils.exec_sql(sql, 'S', ret=False, connection=self.connection_supervisor,
                                    close_connection=False)
        except Exception as err:
            raise err
        return res

    def get_acao_pendente(self, loja_id, caixa_id):
        write_date = str(datetime.utcnow())
        res = {}
        sql = """SELECT DISTINCT supape_tabela,
                        id
                 FROM  supervisor_mov_acao_pendente
                 WHERE supeape_loja=%s
                   AND supeape_pdv=%s
                   AND supape_situacao != 'B';""" % (loja_id, caixa_id)
        try:
            acao_pendentes = db_utils.exec_sql(sql, 'S', ret=True,
                                               connection=self.connection_supervisor,
                                               close_connection=False, auto_commit=False)
        except Exception as err:
            raise err
        acao_pendentes_ids = []
        for acao in acao_pendentes:
            if not res.get(acao.get('supape_tabela')):
                res[acao.get('supape_tabela')] = [acao.get('id')]
            else:
                res[acao.get('supape_tabela')].append(acao.get('id'))
            acao_pendentes_ids.append(acao.get('id'))
        if acao_pendentes_ids:
            # Marca como bloqueada a acao que encontrou para que o supervisor saiba quam esta sendo usada
            sql = """
                UPDATE supervisor_mov_acao_pendente
                SET write_date = '%s',
                    supape_situacao = 'B'
                WHERE id IN (%s)
                  """ % (write_date, str(acao_pendentes_ids).replace('[','').replace(']',''))
            db_utils.exec_sql(sql, 'S')
        return res

    def del_acao_pendente(self, table, ids=[], loja_id=None, caixa_id=None):
        '''
        Deleta as acoes pendentes que ja foram executadas
        :param table: nome da tabela que ha uma acao pendente
        :param ids: ids a serem deletados da tabela supervisor_mov_acao_pendente
        :param loja_id: id da loja
        :param caixa_id: id do caixa
        :return:
        '''
        sql = """DELETE FROM supervisor_mov_acao_pendente
                 WHERE       supape_tabela = '%s' """ % NOMES_TABELAS[table]

        if ids:
            sql +='''AND id IN (%s) ''' % str(ids).replace('[','').replace(']','')
        if loja_id:
            sql += " AND supeape_loja=%s " % loja_id
        if caixa_id:
            sql += " AND supeape_pdv=%s " % caixa_id

        try:
            res = db_utils.exec_sql(sql.replace('Kits', 'Itens'), 'S', ret=False,
                                    connection=self.connection_supervisor, close_connection=False)
        except Exception as err:
            raise err

        return res

    @Log
    def importar(self, loja_id, caixa_id):
        """
        Método responsável por realizar a importação automática.
        """

        # Colocar campos Defaults na lista de importação
        list_importacao = ('Itens', u'Restrições', u'Promoções', u'Tributação', 'Departamentos',
                           'Finalizadoras',
                           'Finalizadoras na ECF', 'Tipo Recebimento', 'Operadores', 'Clientes',
                           'Mensagem Clientes', 'Parametros', 'Kits', 'Vendedores',
                          u'Importação Full')
        self.promo_capa_ids = []
        for lst in range(len(list_importacao)):
            self.parent.ui.listImportacao.item(lst).setIcon(self.parent.icon_red)
            self.parent.ui.listImportacao.item(lst).setText(str(lst) + ' - ' + list_importacao[lst])

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("gui/ui/images/16x16/check2.png"),
                       QtGui.QIcon.Disabled,
                       QtGui.QIcon.Off)

        self.parent.icon_check = icon

        if not self.parent.conectado:
            raise Exception('Falha ao obter conexão com o Servidor!')

        sup_cad_pdvs = self.get_sup_cad_pdvs(loja_id, caixa_id)

        if not sup_cad_pdvs:
            err = str('ECF %s nao cadastrada no banco de dados do Supervisor!' % config['caixa'])
            raise Exception(err)

        suppdv_caixa = sup_cad_pdvs['suppdv_caixa']
        self.del_acao_pendente('layouts_de_cheques')

        if config['truncar_tabelas']:
            TABELAS_TRUNCATE.append('cad_itens')
            TABELAS_TRUNCATE.append('cad_mensagens_clientes')
        acao_pendente = {}
        if self.full:
            self.parent.infoText('Executando Importação Full', False)
            self.parent.ui.listImportacao.item(14).setIcon(self.parent.icon_arrow)
            QtGui.QApplication.processEvents()
            importacao = self.get_vals_importacao_full(loja_id, caixa_id)
        else:
            importacao, acao_pendente = self.get_vals_importacao(loja_id, caixa_id, sup_cad_pdvs)


        total_geral = 0
        data_inicio = datetime.now()

        cont = 0
        nome_tabela_old = ''
        self.erro_cliente = False
        importei_clientes = False
        itens = []
        clientes = []
        mensagens = []
        atualizar_cfg = False
        for i in importacao:
            self.erro_importacao = False

            if self.full:
                idx = 14
                join, where, nome_tabela, desc, condicao_del, dados = i
                registros = self.get_dados_tabela('supervisor', nome_tabela, dados, join, where)
            else:
                condicao_del = None
                idx, total, registros, nome_tabela, coluna_pdv, trunc, where = i

            if nome_tabela != nome_tabela_old:
                self.parent.ui.listImportacao.item(idx).setIcon(self.parent.icon_blue)
                self.atualiza_contador(idx, nome_tabela)
                if nome_tabela in TABELAS_TRUNCATE:
                    self.truncate(nome_tabela, cascade=True)

                cont = self.total_registros.get(nome_tabela)
                nome_tabela_old = nome_tabela

                info_text = 'Importando Tabela %s' % nome_tabela

                if self.full:
                    info_text = 'Executando Importação Full: %s' % info_text
                    self.parent.ui.listImportacao.item(14).setText(
                        self.parent.trUtf8('Importação Full: %s (%d)' % (desc, cont)))

                self.parent.infoText(info_text, False)
                QtGui.QApplication.processEvents()
                self.parent._close_window = False

                acao_pendente_ids = acao_pendente.get(nome_tabela)
                if acao_pendente_ids:
                    if not isinstance(acao_pendente_ids, list):
                        acao_pendente_ids = [acao_pendente_ids]
                    self.del_acao_pendente(nome_tabela, acao_pendente_ids,  loja_id=loja_id, caixa_id=caixa_id)
                    acao_pendente.pop(nome_tabela)

            total_geral += cont

            if nome_tabela == 'cad_itens':
                itens.extend(registros)
            elif nome_tabela == 'cad_clientes':
                importei_clientes = True
                clientes.extend(registros)
            elif nome_tabela == 'cad_mensagens_clientes':
                mensagens.extend(registros)
            else:
                self.importar_tabelas(registros, nome_tabela, where, condicao_del)

            if nome_tabela == 'cfg_configuracao':
                atualizar_cfg = True

            self.cursor_caixa.connection.commit()

            if not self.erro_importacao:
                try:
                    prox_tabela = importacao[importacao.index(i) + 1][3]
                except IndexError:
                    prox_tabela = None

                if prox_tabela != nome_tabela:
                    if self.full:
                        self.parent.ui.listImportacao.item(14).setText(
                            self.parent.trUtf8('Importação Full: %s (%d)' % (desc, 0)))
                    else:
                        if coluna_pdv:
                            self.update_cad_pdvs(coluna_pdv, sup_cad_pdvs['id'])
                        self.parent.ui.listImportacao.item(idx).setText(
                            u'%d - %s (%d)' % (idx, NOMES_TABELAS[nome_tabela], 0))

        self.importar_cad_itens(itens)
        self.cursor_caixa.connection.commit()
        self.importar_cad_clientes(clientes)
        self.cursor_caixa.connection.commit()
        self.importar_cad_mensagens_clientes(mensagens)
        self.cursor_caixa.connection.commit()
        # deleta os clientes que estao no caixa e nao estao mais no supervisor
        if not config['truncar_tabelas'] and acao_pendente.get('cad_clientes'):
            clientes_ativos = self.get_dados_tabela('supervisor', 'cad_clientes', 'supcli_cnpj_cpf')
            clientes_ativos = tuple(map(lambda l: l.get('supcli_cnpj_cpf'), clientes_ativos))
            if clientes_ativos:
                sql = """
                    DELETE FROM caixa_cad_clientes
                    WHERE       supcli_cnpj_cpf NOT IN %s
                """ % str(clientes_ativos).replace(',)', ')')
                self.cursor_caixa.execute(sql)
                self.cursor_caixa.connection.commit()

        if self.full:
            self.parent.ui.listImportacao.item(14).setIcon(self.parent.icon_green)
            self.parent.ui.listImportacao.item(14).setText(
                self.parent.trUtf8('Importação Full: Concluída!'))
            config.set('importacao_full', False)
            config.save()

        for acao in acao_pendente:
            acao_pendente_ids = acao_pendente.get(acao)
            if acao_pendente_ids:
                if not isinstance(acao_pendente_ids, list):
                    acao_pendente_ids = [acao_pendente_ids]
                self.del_acao_pendente(acao, acao_pendente_ids,  loja_id=loja_id, caixa_id=caixa_id)

        QtGui.QApplication.processEvents()

        self.connection_caixa.close()

        tempo_gasto = datetime.now() - data_inicio
        Log.info('Importado %d registros em %ss.' % (total_geral, tempo_gasto))
        self.parent.infoText('Importado %d registros em %ss.' % (total_geral, tempo_gasto), False)
        QtGui.QApplication.processEvents()
        if atualizar_cfg:
            self.parent.loja = self.parent.get_loja()

        return True

    def get_vals_importacao_full(self, loja_id, caixa_id):
        """
        Retorna uma lista de tuplas com os dados das tabelas à serem importadas na importação full.
        """
        # join, where, nome_tabela, descricao, condicao_delete, dados, vals
        importacao_full = [
            ('', '', 'cad_cfop', 'Cadastro de CFOP', '', '*'),

            ('', '', 'cad_clientes', 'Cadastro de Clientes', 'supcli_cnpj_cpf', '*'),

            ('', '', 'cad_vendedor', 'Cadastro de Vendedores', '', '*'),

            ('', '', 'cad_bandeiras', 'Cadastro de Bandeiras', '', '*'),

            ('', '', 'cad_finalizadora_na_ecf', 'Cadastro de Finalizadoras Na Ecf', '', '*'),

            ('', '', 'cad_finalizadora', 'Cadastro de Finalizadoras', '', '*'),

            ('', '', 'cad_operadora_cartao', 'Cadastro de Operadoras de Cartao', '', '*'),

            ('', '', 'cad_mensagens_clientes', 'Cadastro de Mensagens de Clientes',
             'supclm_cnpj_cpf', '*'),

            ('', '', 'cad_modelos_nfe', 'Cadastro de Modelos de NF', '', '*'),

            ('', '', 'cad_municipios', 'Cadastro de Municipios', '', '*'),

            ('', '', 'cad_operadores', 'Cadastro de Operadores', 'supope_codigo', '*'),

            ('', '', 'cad_tributacao', 'Cadastro de Tributação', 'suptri_codigo_trib_ecf', '*'),

            ('', '', 'cad_tributacao_nfce', 'Cadastro de Tributação', '', '*'),

            ('', '', 'tributacao_simples_nacional', 'Cadastro de Tributação Simples Nacional', '', '*'),

            ('', '', 'cfg_parametros', 'Parametros', '', "*"),

            ('', '', 'cad_comandos_impressoras', 'Comandos_impressora', '', "*"),

            ('', '', 'cfg_configuracao', 'Configuracao', '', "*"),

            ('', '', 'cfg_mensagens', 'Mensagens ECF', '', "*"),

            ('', "supkit_loja = '%d'" % loja_id, 'cad_kits', 'Cadastro de Kits', '', '*'),

            ('', "supite_loja = '%d'" % loja_id, 'cad_itens', 'Cadastro de Itens',
             '[supite_loja, supite_codigo_item]', '*'),

            ('','', 'cad_restricao_item', 'Restrições', '', '*'),

            ('','', 'cad_promo_capa', 'Promoções', '', '*'),

            ('','', 'cad_promo1', 'Promoção Tipo 1', '', '*'),

            ('','', 'cad_promo2', 'Promoção Tipo 2', '', '*'),

            ('','', 'cad_promo3f', 'Promoção Tipo 3', '', '*'),

            ('','', 'cad_promo3p', 'Promoção Tipo 3', '', '*'),

            ('','', 'cad_promo4', 'Promoção Tipo 4', '', '*'),

            ('','', 'cad_promo5f', 'Promoção Tipo 5', '', '*'),

            ('','', 'cad_promo5p', 'Promoção Tipo 5', '', '*'),

            ('','', 'cad_promo6', 'Promoção Tipo 6', '', '*'),

            ('','', 'cad_promo7', 'Promoção Tipo 7', '', '*'),

            ('','', 'cad_lista_clientes', 'Lista de clientes', '', '*'),

            ('', "supcax_loja = '%d' AND supcax_caixa = '%d'" % (loja_id, caixa_id),
             'mov_caixa', 'Caixa', '', '*'),

            ('', "supcap_loja = '%d' AND supcap_caixa = '%d'" % (loja_id, caixa_id),
             'mov_capa', 'Movimento Capa', '', '*'),

            ("cor inner join supervisor_mov_capa movCap on cor.supcor_capa_id = movCap.id ",
             "movCap.supcap_loja = '%d' AND movCap.supcap_caixa = '%d'" % (loja_id, caixa_id),
             'mov_corpos', 'Movimento Corpos', '', 'cor.*'),

            ("fec inner join supervisor_mov_capa movCap on fec.supfec_capa_id = movCap.id ",
             "movCap.supcap_loja = '%d' AND movCap.supcap_caixa = '%d'" % (loja_id, caixa_id),
             'mov_fechamentos', 'Movimento Fechamentos', '', 'fec.*'),

            ("fve inner join supervisor_mov_fechamentos movFec on fve.supfve_fechamentos_id = "
             "movFec.id inner join supervisor_mov_capa movCap on movFec.supfec_capa_id = movCap.id ",
             "movCap.supcap_loja = '%d' AND movCap.supcap_caixa = '%d'" % (loja_id, caixa_id),
             'mov_vencimentos', 'Movimento Vencimentos', '', 'fve.*'),

            ("cxz inner join supervisor_mov_capa movCap on cxz.supcxz_capa_id = movCap.id ",
             "movCap.supcap_loja = '%d' AND movCap.supcap_caixa = '%d'" % (loja_id, caixa_id),
             'mov_corpo_xz', 'Movimento Corpos XZ', 'supcxz_capa_id', 'cxz.*'),

            ("txz inner join supervisor_mov_capa movCap on txz.suptxz_capa_id = movCap.id ",
             "movCap.supcap_loja = '%d' AND movCap.supcap_caixa = '%d'" % (loja_id, caixa_id),
             'mov_totalizadores_xz', 'Movimento Totalizadores XZ', 'suptxz_capa_id', 'txz.*'),

            ("cse inner join supervisor_mov_capa movCap on cse.supcse_capa_id = movCap.id ",
             "movCap.supcap_loja = '%d' AND movCap.supcap_caixa = '%d'" % (loja_id, caixa_id),
             'mov_cupom_servidor', 'Movimento Cupom Servidor', 'supcse_capa_id',
             'cse.id, True as supcse_enviado, cse.supcse_capa_id '),

            ('', " supmpa_loja = '%d' AND supmpa_caixa = '%d'" % (loja_id, caixa_id),
             'mov_meios_pagamentos', 'Movimento Meios de Pagamento',
             '[supmpa_loja, supmpa_caixa, supmpa_finalizadora]', '*'),
        ]
        if db_utils.modulo_instalado_sorteio:
            importacao_full.append(
            ('','', 'cad_sorteio', 'Sorteios', '', '*'), )

        self.parent.ui.progressBarImport.setValue(1)
        for i in importacao_full:
            join, where, nome_tabela, desc, condicao_del, dados = i

            self.parent.ui.listImportacao.item(14).setText(
                self.parent.trUtf8('Importação Full: Calculando: %s...' % nome_tabela))
            QtGui.QApplication.processEvents()
            vals = self.get_dados_tabela('supervisor', nome_tabela, 'COUNT(id)', join, where)
            if vals and isinstance(vals, list):
                total = int(vals[0].get('count'))
                self.total_geral += total
                self.total_registros[nome_tabela] = total
        self.parent.ui.listImportacao.item(14).setText(
            self.parent.trUtf8('Importação Full: Cálculo Concluído!'))
        return importacao_full

    # @Log
    def get_vals_importacao(self, loja_id, caixa_id, sup_cad_pdvs):

        itens = (0, "Itens", 'cad_itens', 'suppdv_lastup_itens', 'supite_loja , supite_codigo_item')

        restricao = (1, "Restricao", 'cad_restricao_item', '', '')

        promo_capa = (2, "Promocoes", 'cad_promo_capa', '', '')

        promo1 = (2, "Promocoes", 'cad_promo1', '', '')

        promo2 = (2, "Promocoes", 'cad_promo2', '', '')

        promo3f = (2, "Promocoes", 'cad_promo3f', '', '')

        promo3p = (2, "Promocoes", 'cad_promo3p', '', '')

        promo4 = (2, "Promocoes", 'cad_promo4', '', '')

        promo5f = (2, "Promocoes", 'cad_promo5f', '', '')

        promo5p = (2, "Promocoes", 'cad_promo5p', '', '')

        promo6 = (2, "Promocoes", 'cad_promo6', '', '')

        promo7 = (2, "Promocoes", 'cad_promo7', '', '')

        sorteio = (2, "Sorteios", 'cad_sorteio', '', '')

        lista_clientes = (2, "Promocoes", 'cad_lista_clientes', '', '')


        tributacao = (3, "Tributacao", 'cad_tributacao', 'suppdv_lastup_tributacao',
                      'suptri_codigo_trib_ecf')

        tributacao_nfce = (3, "Tributacao NFC-e", 'cad_tributacao_nfce', '', '')

        departamento = (4, "Departamentos", 'cad_departamento', 'write_date', '')

        finalizadora_ecf = (5, "Finalizadoras", 'cad_finalizadora_na_ecf',
                            '', '')

        bandeiras = (6, "Bandeiras", 'cad_bandeiras', 'write_date', '')

        finalizadora = (6, "Finalizadoras", 'cad_finalizadora', '', '')

        operadora_cartao = (6, "Finalizadoras", 'cad_operadora_cartao', '', '')

        tipo_recebimento = (7, "TipoRecebimento", 'cad_tipo_recebimento',
                            'suppdv_lastup_tipo_recebimento', '')

        operadores = (8, "Operadores", 'cad_operadores', 'suppdv_lastup_operadores',
                      'supope_codigo')

        clientes = (9, "Clientes", 'cad_clientes', 'suppdv_lastup_clientes', 'supcli_cnpj_cpf')

        mensagens_clientes = (10, "Mensagem", 'cad_mensagens_clientes',
                              'suppdv_lastup_mensagens_clientes',
                              '[supclm_cnpj_cpf, supclm_sequencia]')

        mensagens_cfg = (10, 'Mensagem', 'cfg_mensagens', 'write_date', '')

        parametros = (11, "Parametros", 'cfg_parametros', 'write_date', '')

        config = (11, "Parametros", 'cfg_configuracao', '', '')

        comandos = (11, "Parametros", 'cad_comandos_impressoras', '', '')

        trib_simples_nacional = (11, "Parametros", 'tributacao_simples_nacional', '', '')

        kits = (12, 'Itens', 'cad_kits', 'suppdv_lastup_kits', 'supite_loja , supite_codigo_item')

        vendedor = (13, 'Vendedor', 'cad_vendedor', 'suppdv_lastup_vendedor', 'supven_codigo')

        acoes = (tributacao, tributacao_nfce, itens, restricao,
                 promo_capa ,promo1, promo2, promo3f, promo3p, promo4, promo5f, promo5p, promo6,
                 promo7, lista_clientes,
                 departamento,
                 bandeiras, finalizadora_ecf, finalizadora, operadora_cartao, tipo_recebimento,
                 operadores, clientes,
                 mensagens_clientes, mensagens_cfg, config, parametros, comandos, trib_simples_nacional,
                 kits, vendedor)

        if not db_utils.modulo_instalado_sorteio:
            try:
                res_sql = db_utils.exec_sql("""SELECT name 
                                               FROM ir_module_module 
                                               WHERE name = 'supervisor_sorteio'
                                                 AND state = 'installed';
                                             """, 'S', True)
                if res_sql:
                    db_utils.modulo_instalado_sorteio = True
                    acoes += sorteio,
            except:
                pass
        else:
            acoes += sorteio,

        importacao = []

        acao_pendente = self.get_acao_pendente(loja_id, caixa_id)

        if 'Reducao_Z' in acao_pendente:
            self.parent.acao_pendente_z = True
            self.parent.ui.listImportacao.item(14).setIcon(self.parent.icon_wait)
            QtGui.QApplication.processEvents()

        total_itens = 0
        total_geral = 0

        self.parent.ui.progressBarImport.setValue(1)

        time_ini = datetime.now()
        Log.info("Inicio do calculo: %s" % time_ini)
        aux = {}
        for acao in acoes:
            if not self.parent.status_conexao_supervisor:
                raise Exception('Falha ao obter conexão com o Servidor!')
            try:
                self.parent._close_window = False
                idx, tabela, nome_tabela, coluna_pdv, where = acao

                aux[nome_tabela] = acao_pendente.get(tabela)
                self.parent.ui.listImportacao.item(idx).setIcon(self.parent.icon_wait)
                QtGui.QApplication.processEvents()

                self.parent.infoText('Calculando: %s...' % nome_tabela, False)
                QtGui.QApplication.processEvents()

                if tabela == 'Bandeiras' and 'Finalizadoras' in acao_pendente and tabela not in acao_pendente:
                    acao_pendente['Bandeiras'] = acao_pendente.get('Finalizadoras')
                    aux[nome_tabela] = acao_pendente.get(tabela)
                if tabela == 'Finalizadoras' and 'Bandeiras' in acao_pendente and tabela not in acao_pendente:
                    acao_pendente['Finalizadoras'] = acao_pendente.get('Finalizadoras')
                    aux[nome_tabela] = acao_pendente.get(tabela)

                if tabela not in acao_pendente:
                    self.parent.ui.listImportacao.item(idx).setIcon(self.parent.icon_red)
                elif tabela == 'Reducao_Z':
                    self.parent.acao_pendente_z = True
                if tabela in acao_pendente:
                    dados_tabela = self.get_dados_tabela('caixa', nome_tabela, fields='id', limit=1)

                    last_up = sup_cad_pdvs.get(coluna_pdv)

                    if not last_up or not dados_tabela or nome_tabela in TABELAS_TRUNCATE:
                        trunc = True
                        vals = self.get_dados_tabela('supervisor', nome_tabela, fields='*')
                    else:
                        trunc = False
                        vals = self.get_dados_tabela('supervisor', nome_tabela, fields='*',
                                                     where="write_date >= '%s'" % last_up)

                    total = len(vals)
                    QtGui.QApplication.processEvents()
                    self.parent.ui.listImportacao.item(idx).setText(
                        u'%d - %s (%d)' % (idx, NOMES_TABELAS[nome_tabela], total))

                    QtGui.QApplication.processEvents()

                    if total > 0:
                        if tabela == u'Itens' and not nome_tabela == u'cad_kits':
                            total_itens += total
                            total = total_itens
                        elif tabela == u'Tributacao':
                            self.truncate(nome_tabela, True)
                            if u'Itens' not in acao_pendente:
                                acao_pendente[u'Itens'] = []
                            trunc = True
                            vals = self.get_dados_tabela('supervisor', nome_tabela, fields='*')
                            total = len(vals)

                        total_geral += total

                        lista = []

                        bloco = 50

                        self.total_registros[nome_tabela] = total

                        if total > bloco:
                            for x in vals:
                                tamanho_lista = len(lista)
                                if tamanho_lista <= bloco:
                                    lista.append(x)
                                if tamanho_lista >= bloco or x == vals[-1]:
                                    importacao.append((idx, total, lista, nome_tabela, coluna_pdv, trunc, where),)
                                    lista = []
                        else:
                            importacao.append((idx, total, vals, nome_tabela, coluna_pdv, trunc, where),)

                        self.parent.ui.listImportacao.item(idx).setIcon(self.parent.icon_blue)

                    elif tabela == u'Itens' and nome_tabela != u'cad_itens':
                        continue
                    else:
                        self.parent.ui.listImportacao.item(idx).setIcon(self.parent.icon_red)

                    QtGui.QApplication.processEvents()

            except Exception as err:
                Log.info(str(err))
                raise err

        Log.info('Cálculo Concluído em %s' % (datetime.now() - time_ini))
        self.parent.infoText('Cálculo Concluído em %s' % (datetime.now() - time_ini))
        self.total_geral = total_geral

        acao_pendente = aux
        return importacao, acao_pendente

    def atualiza_contador(self, idx, nome_tabela):
        self.total_registros[nome_tabela] -= 1

        if self.full:
            self.parent.ui.listImportacao.item(idx).setText(
                self.parent.trUtf8(
                    'Importação Full: %s (%d)' % (nome_tabela,
                                                  self.total_registros.get(nome_tabela))))

        else:
            self.parent.ui.listImportacao.item(idx).setText(
                u'%d - %s (%d)' % (idx, NOMES_TABELAS[nome_tabela],
                                   self.total_registros.get(nome_tabela)))

        QtGui.QApplication.processEvents()
        QtGui.QApplication.processEvents()

        return True

    def atualiza_progress_bar(self):
        """
        Atualiza a barra de progressão.
        """
        progress_value = self.parent.ui.progressBarImport.value()

        QtGui.QApplication.processEvents()

        add_value = 100 / float(self.total_geral)

        if progress_value >= 99:
            self.parent.ui.progressBarImport.setValue(progress_value)

        elif self.contador >= 1 / add_value:
            self.contador = 0
            self.parent.ui.progressBarImport.setValue(progress_value + 1)
            QtGui.QApplication.processEvents()

        self.contador += 1

        return self.parent.ui.progressBarImport.value()

    def condicao_promo(self, caixa_supervisor, nome_tabela):
        if (caixa_supervisor != 'supervisor' or nome_tabela not in ['cad_promo1', 'cad_promo2', 'cad_promo3p',
                                                                   'cad_promo3f', 'cad_promo4', 'cad_promo5p',
                                                                   'cad_promo5f', 'cad_promo6', 'cad_promo7']) \
            or not self.promo_capa_ids:
            return ''
        condicao = ''
        if nome_tabela == 'cad_promo1':
            condicao += '''WHERE suppro1_capa_id IN (%s)''' % str(self.promo_capa_ids).replace('[','').replace(']','')

        elif nome_tabela == 'cad_promo2':
            condicao += '''WHERE suppro2_capa_id IN (%s)''' % str(self.promo_capa_ids).replace('[','').replace(']','')

        elif nome_tabela == 'cad_promo3p':
            condicao += '''WHERE suppro3p_capa_id IN (%s)''' % str(self.promo_capa_ids).replace('[','').replace(']','')

        elif nome_tabela == 'cad_promo3f':
            condicao += '''WHERE suppro3f_capa_id IN (%s)''' % str(self.promo_capa_ids).replace('[','').replace(']','')

        elif nome_tabela == 'cad_promo4':
            condicao += '''WHERE suppro4_capa_id IN (%s)''' % str(self.promo_capa_ids).replace('[','').replace(']','')

        elif nome_tabela == 'cad_promo5p':
            condicao += '''WHERE suppro5p_capa_id IN (%s)''' % str(self.promo_capa_ids).replace('[','').replace(']','')

        elif nome_tabela == 'cad_promo5f':
            condicao += '''WHERE suppro5f_capa_id IN (%s)''' % str(self.promo_capa_ids).replace('[','').replace(']','')

        elif nome_tabela == 'cad_promo6':
            condicao += '''WHERE suppro6_capa_id IN (%s)''' % str(self.promo_capa_ids).replace('[','').replace(']','')

        elif nome_tabela == 'cad_promo7':
            condicao += '''WHERE suppro7_capa_id IN (%s)''' % str(self.promo_capa_ids).replace('[','').replace(']','')

        return condicao

    def get_colunas(self, nome_tabela):
        # sera feita uma interseccao entre os campos do supervisor e do caixa
        global COLUNAS_A_IMPORTAR
        colunas_caixa = set(map(lambda x: x.get('column_name'),
                                db_utils.exec_sql("""SELECT column_name 
                                             FROM information_schema.columns 
                                             WHERE table_name ='caixa_%s';""" % nome_tabela, 'C')))
        colunas_super = set(map(lambda x: x.get('column_name'),
                                db_utils.exec_sql("""SELECT column_name 
                                             FROM information_schema.columns 
                                             WHERE table_name ='supervisor_%s';""" % nome_tabela, 'S')))
        COLUNAS_A_IMPORTAR = list(colunas_caixa.intersection(colunas_super))
        return True

    def get_dados_tabela(self, caixa_supervisor, nome_tabela, fields='*', join=None, where=None,
                         limit=None):

        res = []

        sql = "SELECT %s FROM %s_%s " % (fields, caixa_supervisor, nome_tabela)

        if join:
            sql += ' %s ' % join
            if fields == 'COUNT(id)':
                sql = sql.replace('COUNT(id)', 'COUNT(%s.id)' % join[:3])

        if where:
            sql += ' WHERE %s ' % where

        if caixa_supervisor == 'supervisor' and nome_tabela=='cad_promo_capa':
            sql += " WHERE state = '2'"

        if caixa_supervisor == 'supervisor' and nome_tabela=='cad_sorteio':
            sql += " WHERE state = '2'"

        sql += self.condicao_promo(caixa_supervisor, nome_tabela)
        if limit:
            sql += 'LIMIT %s ' % limit

        try:
            registros = db_utils.exec_sql(sql, caixa_supervisor, ret=True, auto_commit=False)
            # evita pegar promocoes inativas quando nao tem promocoes ativas
            if nome_tabela[:10] in ['cad_promo1','cad_promo2','cad_promo3','cad_promo4',
                               'cad_promo5','cad_promo6', 'cad_promo7'] and not self.promo_capa_ids:
                registros = []
                if 'COUNT' in fields:
                    registros = [{'count':0}]

            self.get_colunas(nome_tabela)
            for registro in registros:
                delete_id = False
                if nome_tabela == 'cad_promo_capa' and registro.get('id'):
                    self.promo_capa_ids.append(registro.get('id'))

                if not(nome_tabela.startswith('cad') or self.full):
                    index_id = COLUNAS_A_IMPORTAR.index('id')
                    del COLUNAS_A_IMPORTAR[index_id]
                    delete_id = True

                res.append(dict(filter(lambda l: l[0] in COLUNAS_A_IMPORTAR, registro.items())))

                if delete_id:
                    COLUNAS_A_IMPORTAR.append('id')

        except Exception as err:
            raise err

        return res

    @Log
    def truncate(self, table, cascade=False):
        self.parent.infoText('Truncando a Tabela: %s' % table, False)
        sql_truncate = 'TRUNCATE caixa_%s ' % table
        if cascade:
            sql_truncate += ' CASCADE '
        try:
            res = db_utils.exec_sql(sql_truncate, 'C', connection=self.connection_caixa,
                                    close_connection=False)
        except Exception as err:
            raise err
        return res

    def importar_cad_itens(self, registros):
        if not registros:
            return True

        sql_insert = """
                    INSERT INTO caixa_cad_itens
                                (supite_loja,
                                supite_codigo_item,
                                supite_getin,
                                supite_med_codigo_anvisa,
                                supite_med_motivo_isencao,
                                supite_med_preco_maximo,
                                active,
                                name,
                                supite_tributacao_id,
                                supite_tributacao_pis_id,
                                supite_tributacao_cofins_id,
                                supite_tributacao_icms_id,
                                supite_aliquota,
                                supite_pesavel,
                                supite_fracionado,
                                supite_data_atualizacao,
                                supite_preco,
                                supite_unidade,
                                supite_peso_da_embalagem,
                                supite_codigo_retaguarda,
                                supite_fator_multiplicador,
                                supite_producao,
                                supite_iat,
                                supite_ncm,
                                supite_cest,
                                supite_aliquota_ibpt_fed,
                                supite_aliquota_ibpt_est,
                                supite_aliquota_ibpt_mun,
                                supite_ibpt_chave,
                                supite_descricao_completa,
                                supite_peso_tara,
                                supite_codigo_anp,
                                supite_adrem,
                                supite_pglp,
                                supite_pgnn,
                                supite_pgni,
                                supite_vpart,
                                supite_origem_mercadoria,
                                id)
                    VALUES
                  """
        loja = list(set(map(lambda i: i.get('supite_loja'), registros)))
        itens = list(set(map(lambda i: i.get('supite_codigo_item'), registros)))
        sql = """
        DELETE FROM caixa_cad_itens WHERE supite_loja IN {loja} AND supite_codigo_item IN {itens}
        """.format(loja=str(loja).replace('[', '(').replace(']', ')'),
                   itens=str(itens).replace('[', '(').replace(']', ')'))
        self.cursor_caixa.execute(sql)
        inserts = []
        parcela_registros = []
        qtd_reg = 0
        for registro in registros:
            self.atualiza_progress_bar()
            QtGui.QApplication.processEvents()
            # self.atualiza_contador(idx, nome_tabela)
            QtGui.QApplication.processEvents()
            supite_loja = registro.get('supite_loja') or 'null'
            supite_codigo_item = registro.get('supite_codigo_item') or 'null'
            supite_getin = registro.get('supite_getin') or False
            supite_med_codigo_anvisa = registro.get('supite_med_codigo_anvisa') or False
            supite_med_motivo_isencao = registro.get('supite_med_motivo_isencao') or False
            supite_med_preco_maximo = registro.get('supite_med_preco_maximo') or 0
            active = registro.get('active') or False
            name = registro.get('name') or 'null'
            supite_tributacao_id = registro.get('supite_tributacao_id') or 'null'
            supite_tributacao_pis_id = registro.get('supite_tributacao_pis_id') or 'null'
            supite_tributacao_cofins_id = registro.get('supite_tributacao_cofins_id') or 'null'
            supite_tributacao_icms_id = registro.get('supite_tributacao_icms_id') or 'null'
            supite_aliquota = registro.get('supite_aliquota') or 0.0
            supite_pesavel = registro.get('supite_pesavel') or 'null'
            supite_fracionado = registro.get('supite_fracionado') or 'null'
            supite_data_atualizacao = registro.get('supite_data_atualizacao') or 'null'
            supite_preco = registro.get('supite_preco') or 0.0
            supite_unidade = registro.get('supite_unidade') or 'null'
            supite_peso_da_embalagem = registro.get('supite_peso_da_embalagem') or 0.0
            supite_codigo_retaguarda = registro.get('supite_codigo_retaguarda') or 'null'
            supite_fator_multiplicador = registro.get('supite_fator_multiplicador') or 0.0
            supite_producao = registro.get('supite_producao') or 'null'
            supite_iat = registro.get('supite_iat') or 'null'
            supite_ncm = registro.get('supite_ncm') or 'null'
            supite_cest = registro.get('supite_cest') or 'null'
            supite_aliquota_ibpt_fed = registro.get('supite_aliquota_ibpt_fed') or 0.0
            supite_aliquota_ibpt_est = registro.get('supite_aliquota_ibpt_est') or 0.0
            supite_aliquota_ibpt_mun = registro.get('supite_aliquota_ibpt_mun') or 0.0
            supite_ibpt_chave = registro.get('supite_ibpt_chave') or 0.0
            supite_descricao_completa = registro.get('supite_descricao_completa') or 'null'
            supite_peso_tara = registro.get('supite_peso_tara') or 0.0
            supite_codigo_anp = registro.get('supite_codigo_anp') or 0
            supite_adrem = registro.get('supite_adrem') or 0
            supite_pglp = registro.get('supite_pglp') or 0
            supite_pgnn = registro.get('supite_pgnn') or 0
            supite_pgni = registro.get('supite_pgni') or 0
            supite_vpart = registro.get('supite_vpart') or 0
            supite_origem_mercadoria = registro.get('supite_origem_mercadoria') or ''
            valores = [supite_loja,
                       supite_codigo_item,
                       supite_getin,
                       supite_med_codigo_anvisa,
                       supite_med_motivo_isencao,
                       supite_med_preco_maximo,
                       active,
                       name,
                       supite_tributacao_id,
                       supite_tributacao_pis_id,
                       supite_tributacao_cofins_id,
                       supite_tributacao_icms_id,
                       supite_aliquota,
                       supite_pesavel,
                       supite_fracionado,
                       supite_data_atualizacao,
                       supite_preco,
                       supite_unidade,
                       supite_peso_da_embalagem,
                       supite_codigo_retaguarda,
                       supite_fator_multiplicador,
                       supite_producao,
                       supite_iat,
                       supite_ncm,
                       supite_cest,
                       supite_aliquota_ibpt_fed,
                       supite_aliquota_ibpt_est,
                       supite_aliquota_ibpt_mun,
                       supite_ibpt_chave,
                       supite_descricao_completa,
                       supite_peso_tara,
                       supite_codigo_anp,
                       supite_adrem,
                       supite_pglp,
                       supite_pgnn,
                       supite_pgni,
                       supite_vpart,
                       supite_origem_mercadoria]
            item_id = registro.get('id')
            valores.append(item_id)
            valores = "%s, '%s', %s, '%s', '%s', %s, %s, '%s', %s, %s, %s, %s, %s, %s, %s, '%s', %s, '%s', %s, '%s', %s, " \
                      "'%s', '%s', '%s', '%s', %s,  %s,  %s,  '%s', '%s',%s, %s, %s, %s, %s, %s, %s, '%s', %s" % tuple(valores)
            sql = """\n(%s)
            """ % valores
            sql = sql.replace("'null'", "null")
            if qtd_reg >= 10000: # evita que o postgres fique sobrecarregado.
                qtd_reg = 0
                inserts.append(parcela_registros)
                parcela_registros = []

            parcela_registros.append(sql)
            qtd_reg += 1

        inserts.append(parcela_registros) # insere a ultima parcela
        for parcela in inserts:
            try:
                sql = sql_insert + ','.join(parcela)
                self.cursor_caixa.execute(sql)
            except Exception as e:
                Log.error(str(sql))
                Log.error('Erro ao inserir itens: ' + str(e))
                self.erro_importacao = True
                self.cursor_caixa.connection.commit()

        return True

    def importar_cad_clientes(self, registros):
        if not registros:
            return True

        clientes = str(map(lambda i: i.get('supcli_cnpj_cpf'), registros)).\
            replace('[', '(').replace(']', ')')
        sql = """
                DELETE FROM caixa_cad_clientes WHERE supcli_cnpj_cpf IN {clientes}
              """.format(clientes=clientes)
        self.cursor_caixa.execute(sql)
        sql_insert = """
                INSERT INTO caixa_cad_clientes
                            (supcli_cnpj_cpf,
                            supcli_tipo_pessoa,
                            name,
                            supcli_endereco,
                            supcli_status,
                            supcli_limite_compras,
                            supcli_total_em_aberto,
                            supcli_codigo_interno,
                            supcli_cod_municipio,
                            supcli_insc_estadual,
                            supcli_bairro,
                            supcli_cidade,
                            supcli_estado,
                            supcli_endereco_numero,
                            supcli_complemento,
                            supcli_cep,
                            supcli_telefone,
                            supcli_rota,
                            supcli_email,
                            id)
                VALUES
        """
        inserts = []
        parcela_registros = []
        qtd_reg = 0
        for registro in registros:
            self.atualiza_progress_bar()
            QtGui.QApplication.processEvents()
            # self.atualiza_contador(idx, nome_tabela)
            QtGui.QApplication.processEvents()
            supcli_cnpj_cpf = registro.get('supcli_cnpj_cpf') or 'null'
            supcli_tipo_pessoa = registro.get('supcli_tipo_pessoa') or 'null'
            name = registro.get('name') or 'null'
            supcli_endereco = registro.get('supcli_endereco') or 'null'
            supcli_status = registro.get('supcli_status') or 'null'
            supcli_limite_compras = registro.get('supcli_limite_compras') or 0
            supcli_total_em_aberto = registro.get('supcli_total_em_aberto') or 0
            supcli_codigo_interno = registro.get('supcli_codigo_interno') or 'null'
            supcli_cod_municipio = registro.get('supcli_cod_municipio') or 'null'
            supcli_insc_estadual = registro.get('supcli_insc_estadual') or 'null'
            supcli_bairro = registro.get('supcli_bairro') or 'null'
            supcli_cidade = registro.get('supcli_cidade') or 'null'
            supcli_estado = registro.get('supcli_estado') or 'null'
            supcli_endereco_numero = registro.get('supcli_endereco_numero') or 'null'
            supcli_complemento = registro.get('supcli_complemento') or 'null'
            supcli_cep = registro.get('supcli_cep') or 'null'
            supcli_telefone = registro.get('supcli_telefone') or 'null'
            supcli_rota = registro.get('supcli_rota') or 'null'
            supcli_email = registro.get('supcli_email') or 'null'
            valores = [supcli_cnpj_cpf,
                       supcli_tipo_pessoa,
                       name,
                       supcli_endereco,
                       supcli_status,
                       supcli_limite_compras,
                       supcli_total_em_aberto,
                       supcli_codigo_interno,
                       supcli_cod_municipio,
                       supcli_insc_estadual,
                       supcli_bairro,
                       supcli_cidade,
                       supcli_estado,
                       supcli_endereco_numero,
                       supcli_complemento,
                       supcli_cep,
                       supcli_telefone,
                       supcli_rota,
                       supcli_email]
            cliente_id = registro.get('id')
            valores.append(cliente_id)
            valores = "'%s', '%s', '%s', '%s', '%s', %s, %s, %s, %s, '%s', '%s', '%s', '%s', " \
                      "'%s', '%s', '%s', '%s', %s, '%s', %s" % tuple(valores)
            sql = '\n(%s)' % valores
            sql = sql.replace("'null'", "null")
            if qtd_reg >= 10000: # evita que o postgres fique sobrecarregado.
                qtd_reg = 0
                inserts.append(parcela_registros)
                parcela_registros = []

            parcela_registros.append(sql)
            qtd_reg += 1

        inserts.append(parcela_registros) # insere a ultima parcela
        for parcela in inserts:
            try:
                sql = sql_insert + ','.join(parcela)
                self.cursor_caixa.execute(sql)
            except Exception as e:
                self.erro_cliente = True
                Log.error(str(sql))
                Log.error('Erro ao inserir clientes: ' + str(e))
                self.cursor_caixa.connection.commit()

        return True

    def importar_cad_mensagens_clientes(self, registros):
        if not registros:
            return True

        sql_insert = """
                    INSERT INTO caixa_cad_mensagens_clientes
                                (supclm_cnpj_cpf,
                                supclm_sequencia,
                                supclm_mensagem,
                                supclm_tipo,
                                id)
                    VALUES
        """
        clientes = str(map(lambda i: i.get('supclm_cnpj_cpf'), registros)).\
            replace('[', '(').replace(']', ')')
        sql = """
                        SELECT  id
                        FROM    caixa_cad_mensagens_clientes
                        WHERE   supclm_cnpj_cpf IN {clientes}
                    """.format(clientes=clientes)
        self.cursor_caixa.execute(sql)
        inserts = []
        parcela_registros = []
        qtd_reg = 0
        for registro in registros:
            self.atualiza_progress_bar()
            QtGui.QApplication.processEvents()

            # self.atualiza_contador(idx, nome_tabela)
            QtGui.QApplication.processEvents()

            supclm_cnpj_cpf = registro.get('supclm_cnpj_cpf') or 'null'
            supclm_sequencia = registro.get('supclm_sequencia') or 0
            supclm_mensagem = registro.get('supclm_mensagem') or 'null'
            supclm_tipo = registro.get('supclm_tipo') or 'null'

            valores = [supclm_cnpj_cpf,
                       supclm_sequencia,
                       supclm_mensagem,
                       supclm_tipo]
            msg_cliente_id = registro.get('id')
            valores.append(msg_cliente_id)
            valores = "'%s', %s, '%s', '%s', %s" % tuple(valores)
            sql = '\n(%s)' % valores
            sql = sql.replace("'null'", "null")
            if qtd_reg >= 10000: # evita que o postgres fique sobrecarregado.
                qtd_reg = 0
                inserts.append(parcela_registros)
                parcela_registros = []

            parcela_registros.append(sql)
            qtd_reg += 1

        inserts.append(parcela_registros) # insere a ultima parcela
        for parcela in inserts:
            try:
                sql = sql_insert + ','.join(parcela)
                self.cursor_caixa.execute(sql)
            except Exception as e:
                Log.error(str(sql))
                Log.error('Erro ao inserir mensagem de clientes: ' + str(e))
                self.cursor_caixa.connection.commit()

        return True

    def altera_capa_id_registro(self, nome_tabela, registro):
        '''
        Para importacao full altera no registro a capa_id nas tabelas 'mov_corpos', 'mov_fechamentos', 'mov_corpo_xz'
        'mov_totalizadores_xz'e 'mov_cupom_servidor'
        :param nome_tabela: nome da tabela
        :param registro: dicionario com os campos da tabela a ser inserida na importacao
        :return: retorna os registro
        '''
        if nome_tabela == 'mov_corpos':
            registro['supcor_capa_id'] = dict_capa_ids.get(registro.get('supcor_capa_id'))
        elif nome_tabela == 'mov_fechamentos':
            registro['supfec_capa_id'] = dict_capa_ids.get(registro.get('supfec_capa_id'))
        elif nome_tabela == 'mov_corpo_xz':
            registro['supcxz_capa_id'] = dict_capa_ids.get(registro.get('supcxz_capa_id'))
        elif nome_tabela == 'mov_totalizadores_xz':
            registro['suptxz_capa_id'] = dict_capa_ids.get(registro.get('suptxz_capa_id'))
        elif nome_tabela == 'mov_cupom_servidor':
            registro['supcse_capa_id'] =  dict_capa_ids.get(registro.get('supcse_capa_id'))
        elif nome_tabela == 'mov_vencimentos':
            registro['supfve_fechamentos_id'] =  dict_fechamentos_ids.get(registro.get('supfve_fechamentos_id'))
        elif nome_tabela == 'cad_finalizadora':
            if registro.get('supfin_bandeira_cre'):
                registro['supfin_bandeira_cre'] = dict_bandeira_ids.get(registro.get(
                                                                        'supfin_bandeira_cre'))
            elif registro.get('supfin_bandeira_deb'):
                registro['supfin_bandeira_deb'] = dict_bandeira_ids.get(registro.get(
                                                                        'supfin_bandeira_deb'))

        return registro

    def importar_tabelas(self, registros, nome_tabela, where=None, condicao_del=None):
        global dict_capa_ids
        global dict_fechamentos_ids
        global dict_bandeira_ids
        for registro in registros:
            try:
                self.atualiza_progress_bar()
                QtGui.QApplication.processEvents()

                # self.atualiza_contador(idx, nome_tabela)
                QtGui.QApplication.processEvents()

                insert_update = 'I'

                if self.full:
                    condicao_del = filter(lambda element: element[0] in condicao_del, registro.items())
                    condicao_del = dict(condicao_del)
                else:
                    condicao_del = filter(lambda element: element[0] in where, registro.items())
                    condicao_del = dict(condicao_del)

                if nome_tabela not in TABELAS_TRUNCATE:
                    condicao_where = ''

                    for condicao in condicao_del:
                        data_value = condicao_del[condicao]

                        if type(data_value) in [unicode, str, date, datetime]:
                            str_value = "AND %s = '%s' "
                        else:
                            str_value = "AND %s = %s "

                        condicao_where += str_value % (condicao, data_value)

                    if condicao_where:
                        condicao_where = condicao_where[3:]
                        sql = 'SELECT 1 FROM caixa_%s WHERE %s' % (nome_tabela, condicao_where)

                        self.cursor_caixa.execute(sql)
                        if db_utils.fetchall_2_dictfetchall(self.cursor_caixa):
                            insert_update = 'U'
                if self.full:
                    if nome_tabela == 'mov_capa':
                        capa_id = registro.pop('id')


                    elif nome_tabela in ['mov_caixa', 'mov_corpos', 'mov_corpos', 'mov_fechamentos','mov_vencimentos',
                                         'mov_corpo_xz', 'mov_totalizadores_xz', 'mov_cupom_servidor',
                                         'mov_meios_pagamentos']:
                        if nome_tabela == 'mov_fechamentos':
                            fechamento_id = registro.get('id')

                        registro.pop('id')
                        registro = self.altera_capa_id_registro(nome_tabela, registro)

                if nome_tabela == 'cad_bandeiras':
                    bandeira_id = registro.pop('id')

                if nome_tabela == 'cad_finalizadora':
                    fechamento_id = registro.pop('id')
                    registro = self.altera_capa_id_registro(nome_tabela, registro)

                if nome_tabela == 'cfg_configuracao':
                    chave = registro.pop('supcfc_chave_conteudo') or ''
                    cert = registro.pop('supcfc_certificado_conteudo') or ''

                sql = db_utils.dict_2_sql('caixa_' + nome_tabela, registro, _type=insert_update,
                                          where=condicao_del)

                self.cursor_caixa.execute(sql)
                if nome_tabela == 'cfg_configuracao':
                    self.cursor_caixa.execute("""UPDATE caixa_cfg_configuracao 
                                                 SET supcfc_chave_conteudo = '{chave}',
                                                     supcfc_certificado_conteudo = '{cert}'""".
                                              format(chave=chave, cert=cert))

                if self.full and nome_tabela in ['mov_capa','mov_fechamentos']:
                    id = db_utils.fetchall_2_dictfetchall(self.cursor_caixa)
                    if isinstance(id, list):
                        id = id[0]
                    if isinstance(id, dict):
                        id = id.get('id')
                    if nome_tabela == 'mov_capa':
                        dict_capa_ids[capa_id] = id
                    elif nome_tabela == 'mov_fechamentos':
                        dict_fechamentos_ids[fechamento_id] = id

                if nome_tabela == 'cad_bandeiras':
                    id = db_utils.fetchall_2_dictfetchall(self.cursor_caixa)
                    if isinstance(id, list):
                        id = id[0]
                    if isinstance(id, dict):
                        id = id.get('id')
                    dict_bandeira_ids[bandeira_id] = id

            except Exception as err:
                Log.info(str(sql))
                Log.info(str(err))
                self.erro_importacao = True
                self.cursor_caixa.connection.commit()

        return True

