#!/usr/bin/python
# -*- encoding: utf-8 -*-

from sigpdv.tools.log import traced as Log
from sigpdv.tools.config import config
from sigpdv.models.mov_corpos_data import CaixaMovCorpos
from sigpdv.control.cad_tributacao import CadTributacao
from sigpdv.models.cad_tributacao_data import CaixaCadTributacao
from sigpdv.models.cad_itens_data import CaixaCadItens
from sigpdv.models.db_utils import sess, exec_sql
from sigpdv.control.fiscal_printer import FiscalPrinterHelper
from datetime import datetime, date
from decimal import Decimal, ROUND_DOWN, ROUND_UP
from sigpdv.ui.messagebox import YES_ROLE, MSG_QUESTION, message_dlg

TWOPLACES = Decimal(10) ** -2
TREPLACES = Decimal(10) ** -3


class MovCorpos():
    def __init__(self, parent=None):
        self.parent = parent

    @Log
    def set_rateio_desc_acre_itens(self, indicador_desc_acre, mov_corpos, valor):
        '''
        Calcula o rateio do desconto ou acrescimo, dado na capa, nos itens do cupom
        :param totalizadores: diferenca dos tots. de depois e antes da venda do cupom. Eh uma lisca contendo tuplas,
         na tupla temos nome do totalizador e valor nesta ordem.
        :param indicador_desc_acre: indica se eh desconto('D') ou acrescimo('A')
        :param mov_corpos: itens do cupom
        :return:
        '''
        dados_itens = {}
        maior_item = False
        total_cupom = sum(map(lambda x: x.supcor_preco_total - x.supcor_desconto_item + \
                                        x.supcor_acrescimo_item, mov_corpos)) # total liquido
        percent = valor / total_cupom
        desc_acres_total = 0
        for item in mov_corpos:
            tot_liquido = item.supcor_preco_total - item.supcor_desconto_item + \
                          item.supcor_acrescimo_item
            des_acres = Decimal(tot_liquido * percent).quantize(TWOPLACES)
            dif = valor - desc_acres_total
            if not (dif <= Decimal('-0.01') or Decimal('0.01') <= dif): # diferenca igual a zero
                break

            dif = valor - (desc_acres_total + des_acres)
            if dif <= Decimal('-0.01'):
                des_acres = valor - desc_acres_total

            if indicador_desc_acre == 'D':
                desconto = item.supcor_desconto_item + des_acres
                if item.supcor_desconto_item and desconto >= item.supcor_preco_total:
                    continue

                dados_itens[item.id] = {'supcor_desconto_item': item.supcor_desconto_item +
                                                                des_acres}
            else:
                dados_itens[item.id] = {'supcor_acrescimo_item': item.supcor_acrescimo_item +
                                                                 des_acres}

            if not maior_item or item.supcor_preco_total > maior_item.supcor_preco_total:
                maior_item = item

            desc_acres_total += des_acres

        dif = valor - desc_acres_total
        if dif <= Decimal('-0.01') or Decimal('0.01') <= dif:
            if indicador_desc_acre == 'D':
                dados_itens[maior_item.id]['supcor_desconto_item'] += dif
            else:
                dados_itens[maior_item.id]['supcor_acrescimo_item'] += dif

        return dados_itens

    @staticmethod
    def search(vals, order_by=None, limit=0):
        res = []
        try:
            filters = ''
            for key in vals:
                value = vals[key]
                if type(value) in [unicode, str, date, datetime]:
                    filters += ".filter(CaixaMovCorpos.%s == '%s')" % (key, value)
                else:
                    filters += ".filter(CaixaMovCorpos.%s == %s)" % (key, value)

            query = 'CaixaMovCorpos.query%s' % filters

            if order_by:
                query += ".order_by('%s')" % order_by

            if limit:
                query += '.limit(%s)' % limit

            res_query = eval(query)
            if res_query.count():
                if res_query.count() == 1:
                    res = res_query.one()
                else:
                    res = res_query.all()
        except Exception as err:
            res = False
            Log.error(str(err))

        return res

    @Log
    def create(self, vals):
        timeout = 0
        try:
            dict_trib = CadTributacao().get_tributacao()
            cod_trib_ecf = vals['supcor_tributacao_ecf']

            vals['supcor_totalizador_parcial'] = CadTributacao().format_st(cod_trib_ecf)
            vals['supcor_tipo_tributacao'] = dict_trib[cod_trib_ecf].get('TipoTributacao')
            vals['supcor_tributacao_retag'] = dict_trib[cod_trib_ecf].get('TribRetag')
            vals['supcor_aliquota'] = dict_trib[cod_trib_ecf].get('Aliquota')
            vals['supcor_base'] = dict_trib[cod_trib_ecf].get('Base')
            values = []
            for key in vals:
                value = vals[key]
                if type(value) in [unicode, str, date, datetime]:
                    values.append("%s='%s'" % (key, self.parent.remove_accents(value)))
                else:
                    values.append("%s=%s" % (key, self.parent.remove_accents(value)))

            sess.begin()
            res = eval("CaixaMovCorpos(%s)" % ', '.join(values))
            sess.commit()
        except Exception as err:
            res = False
            sess.rollback()
            Log.error('Erro ao gravar Corpo do Cupom! \n%s' % str(err))

        return res

    @Log
    def write(self, query_corpo, vals):
        res = False
        try:
            sess.begin()
            query_corpo.from_dict(vals)
            sess.commit()
            res = True
        except Exception as err:
            sess.rollback()
            Log.error(str(err))

        return res
