#!/usr/bin/python
# -*- encoding: utf-8 -*-

import os
from datetime import datetime
from PySide import QtGui
from sigpdv import root_path as ROOT_PATH
from sigpdv.models.db_utils import exec_sql
from sigpdv.models.cfg_configuracao_data import CaixaCfgConfiguracao
from sigpdv.models.mov_capa_data import CaixaMovCapa
from sigpdv.models.mov_corpos_data import CaixaMovCorpos
from sigpdv.models.mov_fechamentos_data import CaixaMovFechamentos
from sigpdv.control.cad_tributacao import CadTributacao
from sigpdv.control.fiscal_printer import FiscalPrinterHelper
from sigpdv.control.utils import get_info_pafecf, get_numeros, md5_file, remove_accents
from sigpdv.tools.config import config
from sigpdv.tools.log import traced as Log
from sigpdv.tools.crypto import assinatura_ead
from sigpdv.ui.messagebox import MSG_ERROR, MSG_INFO, message_dlg

cr_lf = chr(13) + chr(10)


class MovimentoEcf(object):
    def __init__(self, parent):
        self.parent = parent

    @Log
    def gera_movimento_ecf(self, start, end):

        caixa = config['caixa']
        loja = config['loja']
        self.parent.show_info_proccess(True, 'Gerando Movimento por ECF')
        QtGui.QApplication.processEvents()

        try:
            r1 = self.registro_tipo_r01(loja, caixa, start, end)
            if isinstance(r1, tuple):
                self.parent.show_info_proccess(False, '')
                message_dlg(self.parent, MSG_ERROR, r1[1])
                return False

            r2 = self.registro_tipo_r02(loja, caixa, start, end)
            r3 = self.registro_tipo_r03(loja, caixa, start, end)
            r4 = self.registro_tipo_r04(loja, caixa, start, end)
            r5 = self.registro_tipo_r05(loja, caixa, start, end)
            r6 = self.registro_tipo_r06(loja, caixa, start, end)
            r7 = self.registro_tipo_r07(loja, caixa, start, end)
            res = r1 + r2 + r3 + r4 + r5 + r6 + r7
        except Exception as err:
            res = str(err)
            Log.error(res)

        ecf = FiscalPrinterHelper().get_ecf_autorizada(config['loja'],
                                                       config['caixa'],
                                                       os.path.join(ROOT_PATH, 'pdvsig.sig'))
        filename = '%s%s%s.txt' % (ecf.CNacional,
                                   ecf.CNSerie[20 - 14:],
                                   datetime.now().strftime('%d%m%Y'))

        filename = os.path.join(ROOT_PATH, filename)

        f = open(filename, 'w')
        f.write(remove_accents(res, 'utf-8'))
        f.close()

        assinatura_ead(filename)

        self.parent.show_info_proccess(False, '')
        message_dlg(self.parent, MSG_INFO, 'Arquivo gerado:', file=str(filename))

        return True

    @Log
    def registro_tipo_r01(self, loja, caixa, data_ini, data_fim):
        """
        R01. Identificação do ECF, do Usuário, do PAF-ECF 1o registro (único)
                e da Empresa Desenvolvedora e Dados do Arquivo
        """
        self.parent.show_info_proccess(True, 'Gerando Movimento por ECF\n\nRegistro: (1-7)')
        QtGui.QApplication.processEvents()

        paf_ecf = get_info_pafecf('sw.ini')
        ecf_autorizada = FiscalPrinterHelper().get_ecf_autorizada(loja, caixa, 'pdvsig.sig')

        loja = CaixaCfgConfiguracao.get_by(supcfc_loja=loja)
        if not loja:
            return ''

        nserie = ecf_autorizada.CNSerie.ljust(20, ' ')

        r1 = 'R01'  # Tipo
        r1 += nserie[:20].ljust(20, ' ')  # Numero de Fabricação (20)
        r1 += ' '  # Letra indicativa de MF adicional (1)
        r1 += 'ECF-IF'.ljust(7, ' ')  # Tipo de ECF (7)
        r1 += ecf_autorizada.CFabricante[:20].ljust(20, ' ')  # Marca do ecf (20)
        r1 += ecf_autorizada.CModelo[:20].ljust(20, ' ')
        r1 += ecf_autorizada.CVersao[:10].ljust(10,
                                                ' ')  # Versão do Softeare básico do ECF gravado da
        # MFD (10)

        CDataSB = ecf_autorizada.CDataSB.replace('-', '').replace('False', '').ljust(8, ' ')
        r1 += CDataSB

        CHoraSB = ecf_autorizada.CHoraSB.replace(':', '').replace('False', '').ljust(6, ' ')
        r1 += CHoraSB

        r1 += '%.3d' % caixa  # Numero sequencial do ecf (3)
        r1 += get_numeros(loja.supcfc_cnpj[:14]).rjust(14, '0')  # CNPJ do usuario (14)
        if not loja.supcfc_inscricao_estadual:
            Log.error('Inscrição estadual não configurada na tabela de configuraçẽs.')
            return ''

        r1 += get_numeros(loja.supcfc_inscricao_estadual[:14]).rjust(14, '0')  # IE do usuario (14)
        r1 += get_numeros(paf_ecf.cnpj[:14]).rjust(14, '0')  # CNPJ do desenvolvedor (14)
        r1 += get_numeros(paf_ecf.ie[:14]).rjust(14, '0')  # IE do usuario (14)
        r1 += get_numeros(paf_ecf.im[:14]).rjust(14, '0')  # IM do usuario (14)
        r1 += paf_ecf.razao_social[:40].ljust(40, ' ')  # Denominação da empresa desenvolvedora (40)
        r1 += paf_ecf.nome[:40].ljust(40, ' ')  # Nome do PAF-ECF (40)
        r1 += paf_ecf.versao[:10].ljust(10, ' ')  # Versão do PAF ECF (10)
        r1 += md5_file('lista_de_aplicativos.txt')  # Código MD-5 do PAF ECF (32)
        r1 += data_ini.strftime('%Y%m%d')  # Data inicial (8)
        r1 += data_fim.strftime('%Y%m%d')  # Data Final (8)
        r1 += get_numeros(paf_ecf.er[:4]).rjust(4, '0')  # Versão da ER-PAF-ECF
        r1 += cr_lf  # Quebra de linha
        return r1

    @Log
    def registro_tipo_r02(self, loja, caixa, data_ini, data_fim):
        """
        R02. Relação de Reduções Z
        """
        self.parent.show_info_proccess(True, 'Gerando Movimento por ECF\n\nRegistro: (2-7)')
        QtGui.QApplication.processEvents()

        sql = """SELECT a.supcap_operador_id AS operador_id, 
                        a.supcap_crz         AS crz, 
                        a.supcap_nro_cupom   AS coo, 
                        z.supcxz_cro         AS cro, 
                        a.supcap_data_cupom  AS data_cupom,
                        a.supcap_data_inicio AS data_inicio, 
                        (z.supcxz_gt_atual - z.supcxz_gt_anterior) AS total,
                        a.supcap_nro_serie   AS serie, 
                        a.supcap_modelo_ecf  AS modelo 
                FROM caixa_mov_capa          AS a
                LEFT JOIN caixa_mov_corpo_xz AS z on z.supcxz_capa_id=a.id
                WHERE a.supcap_loja=%d 
                  AND a.supcap_caixa=%d
                  AND z.supcxz_data_movimento >= '%s' 
                  AND z.supcxz_data_movimento <= '%s' 
                  AND a.supcap_tipo='Z'
                ORDER BY a.supcap_crz, z.supcxz_cro;""" % (loja, caixa, data_ini, data_fim)
        ret_sql = exec_sql(sql, 'C', ret=True)
        if not ret_sql:
            return ''

        tipo = 'R02'
        lines = ''
        CaixaMovCapa.get_by().refresh()
        for z in ret_sql:
            lines += tipo  # Tipo
            lines += '%s' % z.get('serie')[:20].ljust(20, ' ')  # Numero de fabricacao
            lines += ' '  # MF Adicional
            lines += '%s' % z.get('modelo').ljust(20, ' ')  # Modelo do ECF
            lines += '%.2d' % z.get('operador_id') or '0'.rjust(2, '0')
            lines += '%.6d' % z.get('crz') or '0'.rjust(6, '0')
            lines += '%.6d' % z.get('coo') or '0'.rjust(6, '0')
            lines += '%.6d' % z.get('cro') or '0'.rjust(6, '0')
            lines += z.get('data_cupom').strftime('%Y%m%d')
            lines += z.get('data_inicio').strftime('%Y%m%d')
            lines += z.get('data_inicio').strftime('%H%M%S')
            lines += get_numeros('%.2f' % z.get('total')).rjust(14, '0') or '0'.rjust(14, '0')
            lines += 'N'
            lines += cr_lf

        return lines

    @Log
    def registro_tipo_r03(self, loja, caixa, data_ini, data_fim):
        """
        R03. Detalhe da Redução Z
        """
        self.parent.show_info_proccess(True, 'Gerando Movimento por ECF\n\nRegistro: (3-7)')
        QtGui.QApplication.processEvents()

        sql = """SELECT a.supcap_data_cupom  AS data_cupom,
                        a.supcap_nro_cupom   AS coo,
                        a.supcap_crz         AS crz,
                        a.supcap_operador_id AS operador_id,
                        a.supcap_nro_serie   AS serie,
                        a.supcap_modelo_ecf  AS modelo,
                        a.supcap_loja        AS loja,
                        a.supcap_caixa       AS caixa
                FROM caixa_mov_corpo_xz  AS c
                LEFT JOIN caixa_mov_capa AS a ON a.id = c.supcxz_capa_id
                WHERE a.supcap_loja =%d
                  AND a.supcap_caixa =%d
                  AND a.supcap_data_cupom BETWEEN '%s' AND '%s'
                  AND a.supcap_tipo = 'Z'
                ORDER  BY a.supcap_crz; """ % (loja, caixa, data_ini, data_fim)

        sql_t = """SELECT xz.suptxz_codigo_trib_ecf AS cod_trib,
                          xz.suptxz_valor           AS valor,
                          xz.suptxz_nro_serie       AS serie,
                          xz.suptxz_crz             AS crz
                FROM   caixa_mov_totalizadores_xz AS xz
                INNER JOIN caixa_mov_capa         AS ca ON xz.suptxz_capa_id = ca.id
                WHERE ca.supcap_data_cupom = '%s'
                  AND ca.supcap_loja =%d
                  AND ca.supcap_caixa =%d
                  AND ca.supcap_nro_cupom =%d; """

        ret_sql = exec_sql(sql, 'C', ret=True)
        if not ret_sql:
            return ''

        tipo = 'R03'
        lines = ''
        lines_i = ''
        CaixaMovCapa.get_by().refresh()
        for z in ret_sql:
            lines_i += tipo
            for t in exec_sql(sql_t % (z.get('data_cupom'), loja, caixa, z.get('coo')), 'C', ret=True):
                lines += lines_i
                lines += t.get('serie').ljust(20, ' ')
                lines += ' '
                crz = z.get('crz')
                ope = z.get('operador_id')
                ser = z.get('serie')
                mod = z.get('modelo')

                if not mod:
                    mod = FiscalPrinterHelper().get_ecf_autorizada(loja, caixa,
                                                                   'pdvsig.sig').CModelo
                lines += '%s' % mod.ljust(20, ' ')
                lines += '%.2d' % ope
                lines += '%.6d' % crz
                try:
                    lines += CadTributacao().format_st()[t.get('cod_trib')]
                except:
                    lines += ' '
                lines += get_numeros('%.2f' % t.get('valor')).rjust(13, '0')
                lines += cr_lf

            lines_i = ''

        return lines

    @Log
    def registro_tipo_r04(self, loja, caixa, data_ini, data_fim):
        """
        R04 Cupom Fiscal, Nota Fiscal de Venda a Consumidor
        ou Bilhete de Passagem
        """
        self.parent.show_info_proccess(True, 'Gerando Movimento por ECF\n\nRegistro: (4-7)')
        QtGui.QApplication.processEvents()

        sql = """SELECT supcap_tipo                     AS tipo,          -- 0
                        supcap_operador_id              AS operador_id,   -- 1
                        supcap_ccf                      AS ccf,           -- 2
                        supcap_nro_cupom                AS coo,           -- 3
                        supcap_data_inicio              AS data_inicio,   -- 4
                        supcap_total_cupom              AS total,         -- 5
                        CASE supcap_alteracao 
                             WHEN 'D' 
                             THEN (supcap_total_cupom + supcap_valor_desc_acres)
                             WHEN 'A' 
                             THEN (supcap_total_cupom - supcap_valor_desc_acres)
                             ELSE supcap_total_cupom 
                        END                             AS subtotal,      -- 6
                        supcap_alteracao                AS alteracao,     -- 7
                        supcap_valor_desc_acres         AS desc_acres,    -- 8
                        supcap_cancelado_posteriormente AS cancelado,     -- 9
                        supcap_cliente                  AS cliente,       --10
                        supcap_nome_cliente             AS nome_cli,      --11
                        supcap_data_cupom               AS data_cupom,    --12
                        supcap_nro_serie                AS serie,         --13
                        supcap_modelo_ecf               AS modelo         --14
                 FROM caixa_mov_capa
                 WHERE supcap_loja=%d
                   AND supcap_caixa=%d
                   AND supcap_data_cupom BETWEEN '%s' AND '%s'
                   AND supcap_tipo IN ('V', 'N')
                 ORDER BY supcap_data_inicio;""" % (loja, caixa, data_ini, data_fim)
        ret_sql = exec_sql(sql, 'C', ret=True)
        if not ret_sql:
            return ''

        tipo = 'R04'
        lines = ''
        CaixaMovCapa.get_by().refresh()
        for x in ret_sql:
            total = get_numeros('%.2f' % x.get('total')).rjust(14, '0')
            subtotal = get_numeros('%.2f' % x.get('subtotal')).rjust(14, '0')
            # Indicador de Cancelamento
            if (x.get('tipo') == 'V') and (x.get('cancelado') == 1):
                cancelado = 'S'
            else:
                cancelado = 'N'

            if x.get('tipo') == 'N':
                subtotal = get_numeros('%.2f' % 0).rjust(14, '0')
                total = subtotal
                cancelado = 'S'
            # Indicador de Desconto/Acrescimo
            tipo_acres = ' '
            tipo_desc = ' '
            desconto = '0' * 13
            acrescimo = '0' * 13
            if (x.get('alteracao') == 'D'):
                desconto = get_numeros('%.2f' % x.get('desc_acres')).rjust(13, '0')
                tipo_desc = 'V'
            elif (x.get('alteracao') == 'A'):
                acrescimo = get_numeros('%.2f' % x.get('desc_acres')).rjust(13, '0')
                tipo_acres = 'V'

            lines += tipo
            lines += '%s' % x.get('serie').ljust(20, ' ')
            lines += ' '
            lines += '%s' % x.get('modelo').ljust(20, ' ')
            lines += '%.2d' % x.get('operador_id')
            lines += '%.6d' % x.get('ccf')
            lines += '%.6d' % x.get('coo')
            lines += x.get('data_inicio').strftime('%Y%m%d')
            lines += subtotal
            lines += desconto
            lines += tipo_desc
            lines += acrescimo
            lines += tipo_acres
            lines += total
            lines += cancelado
            lines += '0' * 13
            lines += x.get('alteracao') or ' '
            lines += x.get('nome_cli').rjust(40, '0') or ''.rjust(40, ' ')
            lines += get_numeros(x.get('cliente')[1:]).rjust(14, '0') or ''.rjust(14, '0')
            lines += cr_lf
        return lines

    @Log
    def registro_tipo_r05(self, loja, caixa, data_ini, data_fim):
        """
        R05. Detalhe do Cupom Fiscal, Nota Fiscal de Venda a Consumidor ou Bilhete de Passagem
        """
        self.parent.show_info_proccess(True, 'Gerando Movimento por ECF\n\nRegistro: (5-7)')
        QtGui.QApplication.processEvents()

        sql = """SELECT ca.supcap_operador_id         AS _0,   -- 0
                        ca.supcap_nro_cupom           AS _1,   -- 1
                        co.supcor_ccf                 AS _2,   -- 3
                        co.supcor_sequencia           AS _3,   -- 4
                        co.supcor_item                AS _4,   -- 5
                        co.supcor_nome_item           AS _5,   -- 6
                        co.supcor_quantidade          AS _6,   -- 7
                        co.supcor_unidade             AS _7,   -- 8
                        co.supcor_preco_unitario      AS _8,   -- 9
                        co.supcor_desconto_item       AS _9,   --10
                        co.supcor_acrescimo_item      AS _10,   --11
                        (co.supcor_preco_total - co.supcor_desconto_item + co.supcor_acrescimo_item)
                                                      AS _11,   --12
                        co.supcor_tributacao_ecf      AS _12,   --13
                        co.supcor_tipo_tributacao     AS _13,   --14
                        co.supcor_aliquota            AS _14,   --15
                        co.supcor_venda_cancelamento  AS _15,   --16
                        co.supcor_indicador_arr_trunc AS _16,   --17
                        co.supcor_indicador_ppt       AS _17,   --18
                        ca.supcap_data_cupom          AS _18,   --19
                        co.supcor_nro_serie           AS _19,   --20
                        ca.supcap_modelo_ecf          AS _20,   --21
                        co.supcor_totalizador_parcial AS _21,   --22
                        ca.supcap_tipo                AS _22    --23
                    FROM caixa_mov_corpos    AS co 
                    LEFT JOIN caixa_mov_capa AS ca ON ca.id=co.supcor_capa_id
                    WHERE ca.supcap_loja=%d
                      AND ca.supcap_caixa=%d
                      AND ca.supcap_data_cupom between '%s'
                      AND '%s'
                      AND ca.supcap_tipo IN ('V', 'N', 'C') OR ca.supcap_tipo is null;""" % (
            loja, caixa, data_ini, data_fim)
        ret_sql = exec_sql(sql, 'C', ret=True)
        if not ret_sql:
            return ''

        tipo = 'R05'
        lines = ''
        CaixaMovCorpos.get_by().refresh()
        modelo_ecf = ret_sql[0]['_20']
        for x in ret_sql:
            lines += tipo
            lines += '%s' % x.get('_19').ljust(20, ' ')
            lines += ' '
            if not x.get('_20'):
                modelo_ecf = modelo_ecf
            else:
                modelo_ecf = x.get('_20')

            lines += '%s' % modelo_ecf.ljust(20, ' ')
            lines += '%.2d' % x.get('_0') or '%.2d' % 1
            lines += '%.6d' % x.get('_1')  # COO
            lines += '%.6d' % x.get('_2')  # CCF
            lines += '%.3d' % x.get('_3')  # Sequencia Item
            lines += x.get('_4')  # Codigo Item
            lines += x.get('_5').ljust(100, ' ') or ''.rjust(100, ' ')  # Descrição Item
            lines += get_numeros('%.3f' % x.get('_6')).rjust(7, '0') or ''.rjust(7, '0') # Quantidade
            lines += x.get('_7').ljust(3, ' ') or ''.rjust(3, ' ') # supite_unidade
            lines += get_numeros('%.2f' % x.get('_8')).rjust(8, '0') or ''.rjust(8, '0')  # Valor Item
            lines += get_numeros('%.2f' % x.get('_9')).rjust(8, '0') or ''.rjust(8, '0')  # Desconto
            lines += get_numeros('%.2f' % x.get('_10')).rjust(8, '0') or ''.rjust(8, '0')  # Acrescimo
            lines += get_numeros('%.2f' % x.get('_11')).rjust(14, '0') or ''.rjust(14, '0')  # Total Liquido
            lines += '%s' % x.get('_21').rjust(7, ' ')  # Tributação
            inf_cancel = 'S'

            if x.get('_22') in ('C', 'N'):
                inf_cancel = 'N'
            else:
                if x.get('_15') == 'V':
                    inf_cancel = 'N'
            # print '>>', inf_cancel, x.get('_22')
            lines += inf_cancel
            lines += ''.rjust(7, '0')
            lines += ''.rjust(13, '0')
            lines += ''.rjust(13, '0')
            lines += x.get('_16')
            lines += x.get('_17')
            lines += '3'  # Casas decimais quantidade
            lines += '2'  # casas decimais valor comercializado
            lines += cr_lf
            # print lines
        return lines

    @Log
    def registro_tipo_r06(self, loja, caixa, data_ini, data_fim):
        """
        R06. Demais documentos emitidos pelo ECF
        """
        self.parent.show_info_proccess(True, 'Gerando Movimento por ECF\n\nRegistro: (6-7)')
        QtGui.QApplication.processEvents()

        sql = """SELECT supcap_operador_id      AS _0,
                        supcap_nro_cupom        AS _1,
                        supcap_gnf              AS _2,
                        supcap_grg              AS _3,
                        supcap_cdc              AS _4,
                        supcap_tipo_comprovante AS _5,
                        supcap_data_fim         AS _6,
                        supcap_data_cupom       AS _7,
                        supcap_nro_serie        AS _8,
                        supcap_modelo_ecf       AS _9,
                        supcap_tipo             AS _10,
                        supcap_tipo_comprovante AS _11 
                 FROM caixa_mov_capa
                 WHERE supcap_loja=%d
                   AND supcap_caixa=%d
                   AND supcap_data_cupom between '%s'
                   AND '%s'
                   AND supcap_tipo NOT IN ('C', 'V', 'N', 'Z');""" \
              % (loja, caixa, data_ini, data_fim)
        ret_sql = exec_sql(sql, 'C', ret=True)
        if not ret_sql:
            return ''

        tipo = 'R06'
        lines = ''
        CaixaMovCapa.get_by().refresh()
        for x in ret_sql:
            lines += tipo
            lines += x.get('_8').ljust(20, ' ')
            lines += ' '
            lines += x.get('_9').ljust(20, ' ').ljust(20, ' ')
            lines += '%.2d' % x.get('_0')  # Operador
            lines += '%.6d' % x.get('_1')  # COO
            lines += '%.6d' % x.get('_2')  # GNF
            if x.get('_10') == 'G':
                lines += '%.6d' % (x.get('_3') or 0)  # GRG
            else:
                lines += ''.rjust(6, '0')  # GRG

            if x.get('_11') == 'CC':
                lines += '%.4d' % x.get('_4')  # CDC
            else:
                lines += ''.rjust(4, '0')  # GRG

            lines += x.get('_5') or ''  # Tipo Comprovante
            lines += x.get('_6').strftime('%Y%m%d') or ''.rjust(8, ' ')  # Data Fim
            lines += x.get('_6').strftime('%H%M%S') or ''.rjust(8, ' ')  # Hora Fim
            lines += cr_lf

        return lines

    @Log
    def registro_tipo_r07(self, loja, caixa, data_ini, data_fim):
        """
        R07. Detalhe do Cupom Fiscal e do Documento Não Fiscal
        """
        self.parent.show_info_proccess(True, 'Gerando Movimento por ECF\n\nRegistro: (7-7)')
        QtGui.QApplication.processEvents()

        sql = """SELECT c.supcap_operador_id              AS _0,
                        c.supcap_nro_cupom                AS _1,
                        f.supfec_ccf                      AS _2,
                        f.supfec_gnf                      AS _3,
                        f.supfec_nome_finalizadora        AS _4,
                        f.supfec_valor                    AS _5,
                        c.supcap_data_cupom               AS _6,
                        f.supfec_sequencia                AS _7,
                        f.supfec_nro_serie                AS _8,
                        c.supcap_modelo_ecf               AS _9,
                        c.supcap_cancelado_posteriormente AS _10,
                        c.supcap_total_cupom              AS _11,
                        f.supfec_valor                    AS _12
                 FROM caixa_mov_fechamentos AS f 
                 LEFT JOIN caixa_mov_capa   AS c ON c.id=f.supfec_capa_id
                 WHERE c.supcap_loja=%d
                   AND c.supcap_caixa=%d
                   AND c.supcap_data_cupom BETWEEN '%s'
                   AND '%s'
                   AND c.supcap_tipo <> 'A' AND c.supcap_tipo <> 'C' or
                   c.supcap_data_cupom IS NULL;""" % (loja, caixa, data_ini, data_fim)
        ret_sql = exec_sql(sql, 'C', ret=True)
        if not ret_sql:
            return ''

        tipo = 'R07'
        lines = ''
        CaixaMovFechamentos.get_by().refresh()
        modelo_ecf = ret_sql[0]['_9']

        for x in ret_sql:
            if not x.get('_9'):
                modelo_ecf = modelo_ecf
            else:
                modelo_ecf = x.get('_9')

            lines += tipo
            lines += '%s' % x.get('_8').ljust(20, ' ')  # NRO SERIE
            lines += ' '
            lines += '%s' % modelo_ecf.ljust(20, ' ')
            lines += '%.2d' % x.get('_0') if x.get('_0') else '%.2d' % 1  # Operador
            lines += '%.6d' % x.get('_1')  # COO
            lines += '%.6d' % x.get('_2')  # CCF
            lines += '%.6d' % x.get('_3')  # GNF
            lines += x.get('_4')[:15].ljust(15, ' ') if x.get('_4') else ' ' * 15  # Meio de Pagamento
            lines += get_numeros('%.2f' % x.get('_5')).rjust(13, '0')  # Valor
            ind_estorno = 'N'
            if x.get('_10') == 1:
                ind_estorno = 'S'
                vr = x.get('_12')
            else:
                vr = 0
            lines += ind_estorno  # Indicador de estorno
            # vr = x.get('_11') if x.get('_11') else x.get('_12')
            lines += get_numeros('%.2f' % vr).rjust(13, '0')  # Valor estornado
            lines += cr_lf

        return lines