#!/usr/bin/python
# -*- encoding: utf-8 -*-

import os
from datetime import datetime
from decimal import Decimal
from PySide import QtGui
from sigpdv.control.mov_cupom_servidor import MovCupomServidor
from sigpdv.models.mov_totalizadores_xz_data import CaixaMovTotalizadoresXZ
from sigpdv.models.mov_corpo_xz_data import CaixaMovCorpoXZ
from sigpdv.models.cad_operadores_data import CaixaCadOperadores
from sigpdv.models.cad_tributacao_data import CaixaCadTributacao
from sigpdv.models.cad_finzalizadora_na_ecf_data import CaixaCadFinalizadoraNaEcf
from sigpdv.models.db_utils import exec_sql, sess
from sigpdv.tools.config import config
from sigpdv.tools.log import traced as Log
from sigpdv.ui.messagebox import MSG_ERROR, message_dlg
from SigUtils.numero import getNumeroEditado
TWOPLACES = Decimal(10) ** -2



class ReducaoXZ(object):
    def __init__(self, parent):
        self.parent = parent

    @Log
    def gerar_reducao_xz(self, loja, caixa, tipo, codigo_operador):
        """
        Emiti redução x/z gerando os dados na base de dados.
        ------------------------------------------------------------------------------------------
        Venda Bruta Diaria = Total itens + Acrescimo Itens sem Rateio + Acrescimo capa
                            (Cupons de venda, venda(cancelados) e nulos)
        Cancelamento ICMS = Total itens + Acrescimo Itens sem Rateio
                            (Para itens com venda_cancelamento = 'C' independente do cupom)
                            + Acrescimo da capa (Cupons Nulos e Cancelados)
        Desconto ICMS = Soma do (Desconto dos itens com rateio (Cupons de venda)
                                - Desconto dos itens com rateio (Cupons de venda cancelados))
        Venda Liquida = Total liquido dos cupons de venda
                       - Total liquido dos cupons de venda cancelados
        Acrescimo ICMS = Acrescimo das Capas (Cupons de venda)
                       + Acrescimo dos itens (Cupons de venda, itens com venda_cancelamento = 'V')
                       - Acrescimo das Capas (Cupons de venda cancelados)
                       - Acrescimo dos itens (Cupons de venda cancelados)
        """
        msg = []
        if not config['sangria_dinheiro']:
            msg.append('Sangria')
        if not config['suprimento']:
            msg.append('Suprimento')

        if msg:
            return message_dlg(self.parent, MSG_ERROR,
                               'Atenção!\n\nSem totalizador cadastrado no PDVConfig para:\n%s' % '\n'.join(msg))

        cad_operador = False
        if codigo_operador:
            cad_operador = CaixaCadOperadores.get_by(supope_codigo=codigo_operador)

        try:
            texto = ''
            if tipo == 'X':  # Leitura X
                texto = 'Redução X'
            elif tipo == 'Z':
                texto = 'Redução Z'

            if self.parent.teclado:
                self.parent.teclado.envia_display('Imprimindo %s...' % texto)

            sql = """SELECT cmc.id                                           AS capa_id, 
                            cmc.supcap_nro_cupom                             AS coo,
                            cmc.supcap_tipo                                  AS tipo_cupom,
                            COALESCE(cmc.supcap_total_cupom, 0)              AS total_liquido_cupom,
                            CASE WHEN cmc.supcap_alteracao = 'A' 
                                 THEN COALESCE(cmc.supcap_valor_desc_acres, 0)  
                                 ELSE 0
                            END                                              AS acres_capa,
                            COALESCE(cmcor.supcor_venda_cancelamento)        AS venda_cancelamento,
                            COALESCE(cmcor.supcor_preco_total, 0)            AS total_bruto_item,
                            COALESCE(cmcor.supcor_vr_icms, 0)                AS total_icms,
                            COALESCE(cmcor.supcor_desconto_item, 0)          AS desconto_item,
                            COALESCE(cmcor.supcor_acrescimo_item_sem_rat, 0) AS acrescimo_item,
                            COALESCE(cmcor.supcor_acrescimo_item, 0)         AS acrescimo_item_rat,
                            cmcor.supcor_tributacao_ecf                      AS tributacao_ecf,
                            COALESCE(cmcor.supcor_aliquota, 0)               AS aliquota,
                            COALESCE(cmcor.supcor_base_icms, 0)              AS base,
                            cmc.supcap_state                                 AS supcap_state,
                            cmc.supcap_cod_tot_nao_fiscal                    AS tot_nao_fiscal 
                     FROM caixa_mov_capa             AS cmc
                     LEFT JOIN caixa_mov_corpos      AS cmcor ON supcor_capa_id = cmc.id
                     WHERE cmc.supcap_data_cupom = '{data_cupom}'
                       AND cmc.supcap_tipo IN ('A', 'W', 'U', 'V', 'D', 'C', 'N', 'R')
                       AND (cmc.supcap_state IN ('3', '4', '6', '7', 'C', '2E') OR cmc.supcap_state IS NULL)
                    """.format(data_cupom=datetime.now().date())  #'2018-10-23' # datetime.now().date()
            dados = exec_sql(sql, 'C')
            total_bruto_venda = total_liquido = desconto_icms = acrescimo_icms = \
                                total_cancelamento = 0

            tot_n_fiscais = {int(config['sangria_dinheiro'][:2]): 0,
                             int(config['suprimento'][:2]): 0,
                             config['totalizador_recarga_cel']: 0,
                             config['tot_ficha']: 0,
                             config['tot_devolucao_ficha']: 0,
                             config['totalizador_doacao']: 0}
            capa_ids = []
            totais_tributacao = {}
            coos = []
            acrescimos_capa = []
            cancel_acrescimos_capa = []
            for i in dados:
                i['supcap_state'] = (i.get('supcap_state') or '').strip()
                capa_ids.append("'%s'" % i.get('capa_id'))
                coo = i.get('coo')
                if coo not in coos and i.get('tipo_cupom') in ('V', 'C', 'D'):
                    coos.append(coo)
                    if i.get('tipo_cupom') == 'V' or i.get('supcap_state') == '6':
                        total_liquido += i.get('total_liquido_cupom')
                    elif not i.get('supcap_state'): # para cupons cancelados na ECF
                        total_liquido -= i.get('total_liquido_cupom')

                if i.get('tipo_cupom') == 'V' or i.get('supcap_state') == '6':
                    total_bruto_venda += i.get('total_bruto_item') + i.get('acrescimo_item')
                    if i.get('venda_cancelamento') == 'V':
                        desconto_icms += i.get('desconto_item')
                        acrescimo_icms += i.get('acrescimo_item')
                    else:
                        total_cancelamento += i.get('total_bruto_item') + i.get('acrescimo_item')
                        continue

                    if coo not in acrescimos_capa:
                        acrescimos_capa.append(coo)
                        total_bruto_venda += i.get('acres_capa')
                        acrescimo_icms += i.get('acres_capa')

                    base = Decimal(i.get('base')).quantize(TWOPLACES)
                    vr_imposto = Decimal(i.get('total_icms')).quantize(TWOPLACES)
                    if totais_tributacao.get(i.get('tributacao_ecf')):
                        totais_tributacao[i.get('tributacao_ecf')][0] +=  vr_imposto
                        totais_tributacao[i.get('tributacao_ecf')][1] +=  Decimal(i.get('base')).quantize(TWOPLACES)
                    else:
                        totais_tributacao[i.get('tributacao_ecf')] = [vr_imposto, base]


                elif i.get('tipo_cupom') == 'N':
                    total_bruto_venda += i.get('total_bruto_item') + i.get('acrescimo_item')
                    total_cancelamento += i.get('total_bruto_item') + i.get('acrescimo_item')
                    if coo not in acrescimos_capa:
                        acrescimos_capa.append(coo)
                        total_bruto_venda += i.get('acres_capa')
                        total_cancelamento += i.get('acres_capa')

                # notas canceladas ou davs cancelados
                elif i.get('tipo_cupom') == 'C' or i.get('supcap_state') == '7':
                    if i.get('supcap_state'):
                        total_bruto_venda += i.get('total_bruto_item') + i.get('acrescimo_item')
                        total_cancelamento += i.get('total_bruto_item') + i.get('acrescimo_item')
                        if coo not in cancel_acrescimos_capa:
                            cancel_acrescimos_capa.append(coo)
                            total_bruto_venda += i.get('acres_capa')
                            total_cancelamento += i.get('acres_capa')
                    else: # retira o valor de cupons cancelados na ECF
                        desconto_icms -= i.get('desconto_item')
                        acrescimo_icms -= i.get('acrescimo_item')
                        total_cancelamento += i.get('total_bruto_item') + i.get('acrescimo_item')
                        if coo not in cancel_acrescimos_capa:
                            cancel_acrescimos_capa.append(coo)
                            acrescimo_icms -= i.get('acres_capa')
                            total_cancelamento += i.get('acres_capa')

                        liquido = (i.get('total_bruto_item') or 0) + \
                                  (i.get('acrescimo_item_rat') or 0) - \
                                  (i.get('desconto_item') or 0)

                        vr_imposto = Decimal(i.get('total_icms')).quantize(TWOPLACES)
                        if totais_tributacao.get(i.get('tributacao_ecf')):
                            totais_tributacao[i.get('tributacao_ecf')][0] -= vr_imposto
                            totais_tributacao[i.get('tributacao_ecf')][1] -= liquido
                        else:
                            totais_tributacao[i.get('tributacao_ecf')] = [-vr_imposto, -liquido]

                elif i.get('tipo_cupom') == 'A':
                    tot_n_fiscais[int(config['sangria_dinheiro'][:2])] += i.get('total_liquido_cupom') or 0

                elif i.get('tipo_cupom') == 'U':
                    tot_n_fiscais[int(config['suprimento'][:2])] += i.get('total_liquido_cupom') or 0

                elif i.get('tipo_cupom') == 'W':
                    if i.get('tot_nao_fiscal') == config['totalizador_recarga_cel']:
                        tot_n_fiscais[config['totalizador_recarga_cel']] += \
                                                                   i.get('total_liquido_cupom') or 0
                    elif i.get('tot_nao_fiscal') == config['totalizador_doacao']:
                        tot_n_fiscais[config['totalizador_doacao']] += \
                                                                   i.get('total_liquido_cupom') or 0
                    elif i.get('tot_nao_fiscal') == config['tot_ficha']:
                        tot_n_fiscais[config['tot_ficha']] += i.get('total_liquido_cupom') or 0

                    elif i.get('tot_nao_fiscal') == config['tot_devolucao_ficha']:
                        tot_n_fiscais[config['tot_devolucao_ficha']] += \
                                                                   i.get('total_liquido_cupom') or 0

                elif i.get('tipo_cupom') == 'R':
                    tot_n_fiscais[config['totalizador_recarga_cel']] += \
                                                                   i.get('total_liquido_cupom') or 0

            self.tributacao_ecf = [['CANC', total_cancelamento, 'ICMS'],
                                   ['DESC', desconto_icms, 'ICMS'],
                                   ['ACRESC', acrescimo_icms, 'ICMS']]
            self.parent.show_info_proccess(True, texto)
            if tipo == 'X':  # Leitura X
                tribtacoes_icms = []
                tribtacoes_iss = []
                subst = isento = outros = 0
                cod_subst = cod_isento = cod_outros = False
                for i in CaixaCadTributacao.query.all():
                    if i.suptri_tipo_imposto == 'C':  # ICMS
                        cod_trib = i.suptri_codigo_trib_ecf
                        if i.suptri_tipo_tributacao == 'T':  # Tributado
                            tribtacoes_icms.append(i)
                        elif i.suptri_tipo_tributacao == 'F':  # Substituicao
                            subst += totais_tributacao.get(cod_trib) and \
                                     totais_tributacao.get(cod_trib)[1] or 0
                            cod_subst = (not cod_subst and not cod_trib.isdigit() and cod_trib.zfill(2)) or cod_subst
                        elif i.suptri_tipo_tributacao == 'I':  # Isento
                            isento += totais_tributacao.get(cod_trib) and \
                                     totais_tributacao.get(cod_trib)[1] or 0
                            cod_isento = (not cod_isento and not cod_trib.isdigit() and cod_trib.zfill(2)) or cod_isento
                        elif i.suptri_tipo_tributacao == 'N':  # Outros
                            outros += totais_tributacao.get(cod_trib) and \
                                     totais_tributacao.get(cod_trib)[1] or 0
                            cod_outros = (not cod_outros and not cod_trib.isdigit() and cod_trib.zfill(2)) or cod_outros
                            
                    elif i.suptri_tipo_imposto == 'S':  # ISS
                        tribtacoes_iss.append(i)

                self.tributacao_ecf.extend([[cod_subst or 'F1', subst, 'ICMS'],
                                            [cod_isento or 'I1', isento, 'ICMS'],
                                            [cod_outros or 'N1', outros, 'ICMS']])

                texto_icms = []
                dados = sorted(tribtacoes_icms, key=lambda x: x.suptri_codigo_trib_ecf)
                tot_base_icms = tot_imposto_icms = 0
                for i in dados:
                    base = totais_tributacao.get(i.suptri_codigo_trib_ecf) and \
                           totais_tributacao.get(i.suptri_codigo_trib_ecf)[1] or 0
                    imposto = totais_tributacao.get(i.suptri_codigo_trib_ecf) and \
                              totais_tributacao.get(i.suptri_codigo_trib_ecf)[0] or 0
                    aliquota = ('%s' % i.suptri_aliquota).rjust(5, '0').replace('.', ',')
                    tot_base_icms += base
                    tot_imposto_icms += imposto
                    tipo_trib = ({'C': 'ICMS', 'S': 'ISS'}.get(i.suptri_tipo_imposto)) or \
                                'Tributacao Sem Tipo'
                    self.tributacao_ecf.append([str(i.suptri_codigo_trib_ecf).zfill(2), imposto, tipo_trib])
                    if i == dados[-1]:
                        t = '%s                 %8s  {sub1}        %8s{sub2}' % (aliquota,
                                                                                 getNumeroEditado(base),
                                                                                 getNumeroEditado(imposto))
                    else:
                        t = '%s                 %8s          %8s' % (aliquota,
                                                                     getNumeroEditado(base),
                                                                     getNumeroEditado(imposto))

                    texto_icms.append(t)
                texto_iss = []
                dados = sorted(tribtacoes_iss, key=lambda x: x.suptri_codigo_trib_ecf)
                tot_base_iss = tot_imposto_iss = 0
                for i in dados:
                    base = totais_tributacao.get(i.suptri_codigo_trib_ecf) and \
                           totais_tributacao.get(i.suptri_codigo_trib_ecf)[1] or 0
                    imposto = totais_tributacao.get(i.suptri_codigo_trib_ecf) and \
                              totais_tributacao.get(i.suptri_codigo_trib_ecf)[0] or 0
                    aliquota = ('%s' % i.suptri_aliquota).rjust(5, '0').replace('.', ',')
                    tot_base_iss += base
                    tot_imposto_iss += imposto
                    if i == dados[-1]:
                        t = '%s                 %8s  {sub1}        %8s{sub2}' % (aliquota,
                                                                                 '%.2f' % base,
                                                                                 '%.2f' % imposto)
                    else:
                        t = '%s                 %8s          %8s' % (aliquota,
                                                                     '%.2f' % base,
                                                                     '%.2f' % imposto)

                    texto_iss.append(t)

                dict_tot_n_fiscais = {int(config['sangria_dinheiro'][:2]): ('sangria_dinheiro', 'Sangria'),
                                      int(config['suprimento'][:2]): ('suprimento', 'Suprimento')}
                if config['totalizador_recarga_cel']:
                    dict_tot_n_fiscais[config['totalizador_recarga_cel']] = ('totalizador_recarga_cel',
                                                                             'Recarga Celular')
                if config['tot_ficha']:
                    dict_tot_n_fiscais[config['tot_ficha']] = ('tot_ficha', 'Fichas')

                if config['tot_devolucao_ficha']:
                    dict_tot_n_fiscais[config['tot_devolucao_ficha']] = ('tot_devolucao_ficha',
                                                                         'Devolucao de Fichas')
                if config['totalizador_doacao']:
                    dict_tot_n_fiscais[config['totalizador_doacao']] = ('totalizador_doacao',
                                                                        'Doacao')
                texto_n_fisc = []
                total_n_fiscal = 0
                dados = sorted(dict_tot_n_fiscais.items(), key=lambda x: x[0])
                coo_x = self.parent.obj_cupom_fiscal.get_coo_cup_gerencial()
                for i in dados:
                    k, v = i
                    descricao = (v[1] or 'Tot. Sem Nome').ljust(26)
                    if i == dados[-1]:
                        t = '%02d %s   {sub1}        %8s{sub2}' % \
                            (not isinstance(config[v[0]], int) and ',' in config[v[0]] and
                             int(config[v[0]].split(',')[0]) or int(config[v[0]]),
                             descricao,
                             getNumeroEditado(tot_n_fiscais.get(k)))
                    else:
                        t = '%02d %s           %8s' % \
                                 (not isinstance(config[v[0]], int) and ',' in config[v[0]]
                                  and int(config[v[0]].split(',')[0]) or int(config[v[0]]),
                                  descricao,
                                  getNumeroEditado(tot_n_fiscais.get(k)))
                        
                    total_n_fiscal += tot_n_fiscais.get(k) or 0
                    texto_n_fisc.append(t)

                total_n_fiscal = getNumeroEditado(total_n_fiscal)
                texto_pag = []
                sql = """SELECT supfec_valor                AS valor,
                                COALESCE(  supfec_troco, 0) AS troco,
                                supfin_codigo_na_ecf        AS finalizadora,
                                supcap_tipo                 AS tipo,
                                supcap_state                AS state
                         FROM caixa_mov_fechamentos        AS cmf
                         INNER JOIN caixa_cad_finalizadora AS ccf ON ccf.supfin_codigo = cmf.supfec_finalizadora
                         INNER JOIN caixa_mov_capa         AS cmc ON cmc.id = cmf.supfec_capa_id
                         WHERE cmc.id IN (%s)
                           AND cmc.supcap_tipo IN ('V', 'b', 'R', 'W', 'C', 'D')
                           AND (cmc.supcap_state IN ('3', '4', '6', '2E') OR cmc.supcap_state IS NULL)
                      """ % ','.join(capa_ids)
                dict_pgto = {}
                total_troco = 0

                if capa_ids:
                    for i in exec_sql(sql, 'C'):
                        troco = i.get('troco')
                        valor = i.get('valor')
                        total_troco += troco
                        if i.get('tipo') in ('V', 'D', 'W', 'R') or (i.get('state') or '').strip() == '6':
                            if not dict_pgto.get(i.get('finalizadora')):
                                dict_pgto[i.get('finalizadora')] = valor
                            else:
                                dict_pgto[i.get('finalizadora')] += valor

                        elif not i.get('state'):  # para Cupons cancelados na ECF
                            if not dict_pgto.get(i.get('finalizadora')):
                                dict_pgto[i.get('finalizadora')] = -valor
                            else:
                                dict_pgto[i.get('finalizadora')] -= valor


                total_troco = getNumeroEditado(total_troco)
                for i in CaixaCadFinalizadoraNaEcf.query.order_by('supfec_codigo_na_ecf').all():
                    valor = getNumeroEditado((dict_pgto.get(i.id) or 0))
                    texto_pag.append('%02d   %s               %8s' %
                                     (i.supfec_codigo_na_ecf, (i.name or 'Sem Nome').ljust(20), valor))
                data_cupom = datetime.now()
                texto = ['{f1}%s{f2}' % 'REDUCAO X'.center(24, ' '),
                         'LOJA: %04d CAIXA: %03d' % (config['loja'], config['caixa']) + ('COO: %s ' % coo_x).rjust(27),
                         'DATA: %s' % data_cupom.strftime('%d/%m/%Y %H:%M:%S'),
                         '{b1}TOTALIZADORES{b2}'.center(56, '-'),
                         'VENDA BRUTA DIARIA:               %14s' % getNumeroEditado(total_bruto_venda),
                         'CANCELAMENTO ICMS:                %14s' % getNumeroEditado(total_cancelamento),
                         'DESCONTO ICMS:                    %14s' % getNumeroEditado(desconto_icms),
                         'Total de ISSQN:                   %14s' % getNumeroEditado(0),
                         'CANCELAMENTO ISSQN:               %14s' % getNumeroEditado(0),
                         'DESCONTO ISSQN:                 {sub1}  %14s{sub2}' % getNumeroEditado(0),
                         'VENDA LIQUIDA:                    %14s' % getNumeroEditado(total_liquido),
                         'ACRESCIMO ICMS:                   %14s' % getNumeroEditado(acrescimo_icms),
                         'ACRESCIMO ISSQN:                  %14s' % '0,00',
                         '{b1}ICMS{b2}'.center(56, '-'),
                         'Totalizador   Base Calculo( R$)     Imposto( R$)',
                         '\n'.join(texto_icms),
                         'Total ICMS:     %14s    %14s' % (getNumeroEditado(tot_base_icms),
                                                           getNumeroEditado(tot_imposto_icms)),
                         'Nao Tributados              Valor Acumulado( R$)',
                         'F1                                %14s' % getNumeroEditado(subst),
                         'I1                                %14s' % getNumeroEditado(isento),
                         'N1                                %14s' % getNumeroEditado(outros),
                         '{b1}ISSQN{b2}'.center(56, '-'),
                         'Totalizador   Base Calculo( R$)     Imposto( R$)',
                         '\n'.join(texto_iss),
                         'Total ISSQN:    %14s    %14s' % (getNumeroEditado(tot_base_iss),
                                                           getNumeroEditado(tot_imposto_iss)),
                         'Nao Tributados              Valor Acumulado( R$)',
                         # todo: pegar valores para nota de servico
                         'FS1                                     %8s' % '0,00',
                         'IS1                                     %8s' % '0,00',
                         'NS1                                     %8s' % '0,00',
                         '{b1}TOTALIZADORES NAO FISCAIS{b2}'.center(56, '-'),
                         'No.Operacao             CON Valor Acumulado( R$)',
                         '\n'.join(texto_n_fisc),
                         'Total Oper Nao Fiscal             %14s' % total_n_fiscal,
                         '{b1}MEIOS DE PAGAMENTO{b2}'.center(56, '-'),
                         'No.   Meio Pagamento        Valor Acumulado( R$)',
                         '\n'.join(texto_pag),
                         'TROCO:                            %14s' % total_troco,
                         ''.ljust(48, '-'),
                         ('LOJA: %04d CAIXA: %03d' % (config['loja'], config['caixa'])).ljust(48),
                         'DESENVOLVIDO POR:'.ljust(48),
                         '{b1}Sig Solucoes em Informatica Ltda. (31)2106-1700{b2}',
                         '',
                         '']

            self.parent.obj_impressoras.supcci_imprimir('\n'.join(texto))
            # Grava totalizadores e corpo xz
            # Grava cupom servidor
            self.parent.obj_cupom_fiscal.cria_capa_cupom_gerencial(tipo='X')
            self.totalizar_xz(data_cupom.date())

            MovCupomServidor().create({'supcse_capa_id': self.parent.mov_capa.id})
            QtGui.QApplication.processEvents()

        except Exception as e:
            Log.error(str(e))
            self.parent.show_info_proccess(False)
            return False

        self.parent.show_info_proccess(False)
        return True

    @Log
    def totalizar_xz(self, data_movimento):
        try:
            dados = exec_sql("SELECT MAX(supcap_nro_cupom) AS nro FROM caixa_mov_capa WHERE supcap_tipo IN ('V', 'C','D')")
            dados_x = exec_sql("SELECT MAX(supcxz_cupom_final) AS nro FROM caixa_mov_corpo_xz")
            coo = 0
            if dados:
                coo = dados[0].get('nro')

            ccf = self.parent.mov_capa.supcap_ccf
            gnf = self.parent.mov_capa.supcap_gnf
            crz = self.parent.mov_capa.supcap_crz
            nro_serie = self.parent.mov_capa.supcap_nro_serie
            cro = None
            cupom_ini = None
            if dados_x:
                cupom_ini = dados_x[0].get('nro')

            cupom_fin = coo
            gt_atual = None
            gt_anterior = None
            tributacao = {}
            acrescimos_iss = 0
            acrescimos = 0
            cancelamentos_iss = 0
            cancelamentos = 0
            descontos_iss = 0
            descontos = 0
            for trib in self.tributacao_ecf:
                if (trib[0] == 'ACRESC') and (trib[2] == 'ISS'):
                    acrescimos_iss = trib[1]
                elif (trib[0] == 'ACRESC') and (trib[2] == 'ICMS'):
                    acrescimos = trib[1]
                elif (trib[0] == 'CANC') and (trib[2] == 'ISS'):
                    cancelamentos_iss = trib[1]
                elif (trib[0] == 'CANC') and (trib[2] == 'ICMS'):
                    cancelamentos = trib[1]
                elif (trib[0] == 'DESC') and (trib[2] == 'ISS'):
                    descontos_iss = trib[1]
                elif (trib[0] == 'DESC') and (trib[2] == 'ICMS'):
                    descontos = trib[1]
                else:
                    key = str(trib[0].zfill(2)) + '-' + trib[2]
                    tributacao[key] = trib[1]

            for cad_tributacao in CaixaCadTributacao.query.all():
                tipo_trib = ({'C': 'ICMS', 'S':'ISS'}.get(cad_tributacao.suptri_tipo_imposto)) or \
                            'Tributacao Sem Tipo'
                key = str(cad_tributacao.suptri_codigo_trib_ecf).zfill(2) + '-' + tipo_trib
                try:
                    valor = tributacao[key]
                except:
                    Log.error("Erro ao pegar valor da tributacao(cod. na ecf): %s" % key)
                    valor = Decimal('0')

                sess.begin()
                CaixaMovTotalizadoresXZ(
                    suptxz_codigo_trib_ecf=cad_tributacao.suptri_codigo_trib_ecf,
                    suptxz_descricao_totalizador=cad_tributacao.suptri_name,
                    suptxz_valor=valor,
                    suptxz_aliquota=cad_tributacao.suptri_aliquota,
                    suptxz_base=cad_tributacao.suptri_base,
                    suptxz_tipo_imposto=cad_tributacao.suptri_tipo_imposto,
                    suptxz_tipo_tributacao=cad_tributacao.suptri_tipo_tributacao,
                    suptxz_crz=crz,
                    suptxz_nro_serie=nro_serie,
                    suptxz_capa_id=self.parent.mov_capa.id
                )
                sess.commit()

            sess.begin()
            CaixaMovCorpoXZ(
                supcxz_data_movimento=data_movimento,
                supcxz_ccf=ccf, supcxz_gnf=gnf, supcxz_cro=cro,
                supcxz_cupom_inicial=cupom_ini, supcxz_cupom_final=cupom_fin,
                supcxz_gt_atual=gt_atual, supcxz_gt_anterior=gt_anterior,
                supcxz_cancelamentos=cancelamentos, supcxz_descontos=descontos,
                supcxz_acrescimos=acrescimos, supcxz_descontos_iss=descontos_iss,
                supcxz_acrescimos_iss=acrescimos_iss, supcxz_cancelamentos_iss=cancelamentos_iss,
                supcxz_capa_id=self.parent.mov_capa.id)
            sess.commit()
        except Exception as err:
            Log.error(str(err))
            sess.rollback()
            return False

        return coo