#!/usr/bin/python
# -*- encoding: utf-8 -*-

from sigpdv.tools.log import traced as Log
from sigpdv.models.cad_sorteio_data import CaixaCadSorteio  # deve ficar para executar 'eval()
from sigpdv.models.db_utils import exec_sql
from sigpdv.control.relatorio_gerencial import GerencialReport
from sigpdv.tools.config import config
from sigpdv.ui.messagebox import NO_ROLE, MSG_QUESTION, message_dlg
from datetime import datetime
from decimal import Decimal
from SigUtils.cep import Cep
from SigUtils.cpf import Cpf
from SigUtils.cnpj import Cnpj

TWOPLACES = Decimal(10) ** -2


class CadSorteio(object):
    def __init__(self, parent):
        self.parent = parent

    @staticmethod
    def search(vals, order_by=None, limit=0):
        res = []
        try:
            filters = ''
            for key in vals:
                value = vals[key]
                if type(value) in [unicode, str]:
                    filters += ".filter(CaixaCadSorteio.%s == '%s')" % (key, value)
                else:
                    filters += ".filter(CaixaCadSorteio.%s == %s)" % (key, value)

            query = 'CaixaCadSorteio.query%s' % filters
            if order_by:
                query += '.order_by(%s)' % order_by

            if limit:
                query += '.limit(%s)' % limit

            res_query = eval(query)
            if res_query.count():
                if res_query.count() == 1:
                    res = res_query.one()
                else:
                    res = res_query.all()

        except Exception as err:
            res = False
            Log.error(str(err))

        return res

    @Log
    def get_sorteios(self, valor_compra):
        DataPesq = str(datetime.now().date())
        sql = '''SELECT ccs.supsor_valor     AS valor,
                        ccs.supsor_descricao AS descricao                         
                 FROM caixa_cad_sorteio AS ccs
                 WHERE ccs.supsor_data_inicio <= '{data}' 
                   AND ccs.supsor_data_fim >= '{data}'
                   AND ccs.supsor_valor <= {valor}
                 '''.format(data=DataPesq, valor=valor_compra)
        dados_sorteio = exec_sql(sql, 'C')
        return dados_sorteio

    @Log
    def verifica_sorteio(self):
        if self.parent.ficha or self.parent.vr_doacao:
            return 'Sem soteio para Docao/Ficha'

        valor_compra = self.parent.mov_capa.supcap_total_cupom
        sorteios = self.get_sorteios(valor_compra)
        if not sorteios:
            return 'Nenhum sorteio encontrado!'

        codigo_cliente = self.parent.mov_capa.supcap_cliente  # cpf/cnpj
        if not codigo_cliente:
            print 'Sem cliente'
            return self.imprimir_cupom_sorteio(sorteios, valor_compra, False)

        dados_funcionario = self.parent.obj_cad_funcionarios.get_dados_funcionario(codigo_cliente)
        if dados_funcionario:
            print 'Cliente eh funcionario'
            return self.imprimir_cupom_sorteio(sorteios, valor_compra, False)

        cpf_cnpj = ''
        if self.parent.cliente.supcli_cnpj_cpf:
            cpf = Cpf()
            cnpj = Cnpj()
            if cpf.validar(self.parent.cliente.supcli_cnpj_cpf[-11:]):
                cpf_cnpj = cpf.format(self.parent.cliente.supcli_cnpj_cpf[-11:])

            elif cnpj.validar(self.parent.cliente.supcli_cnpj_cpf):
                cpf_cnpj = cnpj.format(self.parent.cliente.supcli_cnpj_cpf)

        cep = ''
        if self.parent.cliente.supcli_cep:
            cep = Cep(self.parent.cliente.supcli_cep)
            cep = cep.getCepFormatado()

        complemento = self.parent.cliente.supcli_complemento and ('-' + self.parent.cliente.supcli_complemento) or ''
        msg = [u'Deseja imprimir os dados do cliente no cupom da Promoção?',
               u'',
               u'NOME: %s' % (self.parent.cliente.name or ''),
               u'CFP/CNPJ: %s' % cpf_cnpj,
               u'TELEFONE: %s' % (self.parent.cliente.supcli_telefone or ''),
               u'E-MAIL: %s' % (self.parent.cliente.supcli_email or ''),
               u'ENDEREÇO: %s, %s%s, %s - %s, %s - %s' %
                                           ((self.parent.cliente.supcli_endereco or ''),
                                            (self.parent.cliente.supcli_endereco_numero or ''),
                                            complemento,
                                            (self.parent.cliente.supcli_bairro or ''),
                                            (self.parent.cliente.supcli_cidade or ''),
                                            (self.parent.cliente.supcli_estado or ''),
                                            (cep)),
               ]
        if message_dlg(self.parent, MSG_QUESTION, '\n'.join(msg)) == NO_ROLE:
            return self.imprimir_cupom_sorteio(sorteios, valor_compra, False)
            
        return self.imprimir_cupom_sorteio(sorteios, valor_compra, True)

    def get_endereco(self, rua, numero, complemento, bairro, cidade, estado, cep):
        endereco_1 = []
        endereco_2 = []
        endereco_3 = []
        for i in ('%s%s%s%s%s%s%s' % (rua, numero, complemento, bairro, cidade, estado, cep)).split(' '):
            if not endereco_2 and len(i) + len(''.join(endereco_1)) < 39:
                endereco_1.append(i)
                if len(endereco_1) < 39:
                    endereco_1.append(' ')

            elif not endereco_3 and len(i) + len(''.join(endereco_2)) < 48:
                endereco_2.append(i)
                if len(endereco_2) < 48:
                    endereco_2.append(' ')

            elif len(i) + len(''.join(endereco_3)) <  48:
                endereco_3.append(i)
                if len(endereco_3) < 48:
                    endereco_3.append(' ')

        return ''.join(endereco_1), ''.join(endereco_2), ''.join(endereco_3)

    @Log
    def imprimir_cupom_sorteio(self, sorteios, valor_compra, cliente):
        self.parent.show_info_proccess(True, 'Aguarde!\n\nImprimindo cupons de promoção......')
        coo_vinculado = self.parent.mov_capa and self.parent.mov_capa.supcap_nro_cupom or None
        loja = self.parent.loja.supcfc_loja
        caixa = self.parent.caixa_id
        cpf_cnpj = ''

        nome = ''
        rua = ''
        bairro = ''
        numero = ''
        cidade = ''
        estado = ''
        cep = ''
        telefone = '(    )'
        email = ''
        complemento = ''
        if cliente:
            if self.parent.cliente.supcli_cnpj_cpf:
                cpf = Cpf()
                cnpj = Cnpj()
                if cpf.validar(self.parent.cliente.supcli_cnpj_cpf[-11:]):
                    cpf_cnpj = cpf.format(self.parent.cliente.supcli_cnpj_cpf[-11:])

                elif cnpj.validar(self.parent.cliente.supcli_cnpj_cpf):
                    cpf_cnpj = cnpj.format(self.parent.cliente.supcli_cnpj_cpf)

            nome = self.parent.remove_accents(self.parent.cliente.name or '')[0:43]
            rua = self.parent.remove_accents(self.parent.cliente.supcli_endereco or '')
            numero = self.parent.cliente.supcli_endereco_numero and \
                     (', %s' % self.parent.cliente.supcli_endereco_numero) or ''
            complemento = self.parent.cliente.supcli_complemento and \
                     ('-%s' % self.parent.cliente.supcli_complemento) or ''
            bairro = self.parent.cliente.supcli_bairro and ', %s' % \
                               self.parent.remove_accents(self.parent.cliente.supcli_bairro) or ''
            cidade = self.parent.cliente.supcli_cidade and ' - %s' % \
                               self.parent.remove_accents(self.parent.cliente.supcli_cidade) or ''
            estado = self.parent.cliente.supcli_estado and ', %s' % \
                                                           self.parent.cliente.supcli_estado or ''

            if self.parent.cliente.supcli_cep:
                cep = Cep(self.parent.cliente.supcli_cep)
                cep = cep.getCepFormatado()
                cep = ' - %s' % cep

            telefone = self.parent.cliente.supcli_telefone or ''
            email = self.parent.cliente.supcli_email or ''

        endereco_1, endereco_2, endereco_3  = self.get_endereco(rua, numero, complemento, bairro, cidade, estado, cep)

        for i in sorteios:
            texto = ['%s' % i.get('descricao').replace('\\n', '\n'),
                     '------------------------------------------------',
                     'Nome:{sub1}{b1}%s{sub2}{b2}' % nome.ljust(43),
                     '',
                     'Endereco:{sub1}{b1}%s{sub2}{b2}' % endereco_1.ljust(39),
                     '',
                     '{sub1}{b1}%s{sub2}{b2}' % endereco_2.ljust(48),
                     '',
                     '{sub1}{b1}%s{sub2}{b2}' % (endereco_3 or '').ljust(48),
                     '',]

            texto.extend(
                    ['Tel:{sub1}{b1} %s{sub2}{b2}' % telefone.ljust(43),
                     '',
                     'CPF/CNPJ:{sub1}{b1} %s{sub2}{b2}' % cpf_cnpj.ljust(38),
                     '',
                     'RG:{sub1}{b1} %s{sub2}{b2}' % ''.ljust(44),
                     '',
                     'E-mail:{sub1}{b1} %s{sub2}{b2}' % email.ljust(40)])
            valor = i.get('valor')
            qtd_sorteio = int(valor_compra / valor)
            for j in range(0, qtd_sorteio):
                print 'Cupom de sorteio'
                txt = str(self.parent.remove_accents('\n'.join(texto)))
                txt = self.parent.obj_cupom_fiscal.get_texto_cupom(txt)
                self.parent.obj_impressoras.supcci_imprimir(txt)

        self.parent.show_info_proccess(False)
        return True