#!/usr/bin/python
# -*- encoding: utf-8 -*-

from sigpdv.models.cad_finalizadora_data import CaixaCadFinalizadora
from sigpdv.control.mov_capa import MovCapa
from sigpdv.control.mov_fechamentos import MovFechamentos
from sigpdv.control.mov_cupom_servidor import MovCupomServidor
from sigpdv.control.fiscal_printer import FiscalPrinterHelper
from sigpdv.tools.config import config
from sigpdv.tools.log import traced as Log
from sigpdv.control.mov_meios_pagamentos import MovMeiosPagamentos
from sigpdv.ui.messagebox import MSG_INFO, MSG_ERROR, message_dlg


class Suprimento():
    def __init__(self, parent):
        self.parent = parent
        self.printer = FiscalPrinterHelper(self.parent)

    @Log
    def add_suprimento(self, loja_id, caixa_id, value):
        self.parent.infoText('Suprimento')

        if value <= 0:
            message_dlg(self, MSG_INFO, 'Valor não pode ser menor ou igual a zero')
            return False
        try:
            try:
                totalizador = int(config['suprimento'][:2])
            except:
                totalizador = False

            if not totalizador or not isinstance(totalizador, int):
                message_dlg(self.parent, MSG_ERROR, 'Não foi possível efetuar o Suprimento',
                                   'Totalizador de suprimento não configurado no '
                                   'PDV-Config')
                return False

            texto = self.parent.obj_cupom_fiscal.get_texto_cupom('Suprimento valor:%s' % value)
            self.parent.obj_impressoras.supcci_imprimir(texto)

        except Exception as e:
            Log.error(str(e))
            return False

        self.gravar_suprimento(loja_id, caixa_id, value)
        if config['usagaveta']:
            self.parent.obj_impressoras.supcci_imprimir("{gaveta}")
            self.parent._timer_end_sale.start(500)

        return True

    @Log
    def gravar_suprimento(self, loja_id, caixa_id, value):
        try:

            self.parent.obj_cupom_fiscal.cria_capa_cupom_gerencial(int(config['suprimento'][:2]),
                                                                   value, 'U')
            query_cad_finalizadora = CaixaCadFinalizadora.get_by(supfin_tipo_finalizadora='D')
            MovFechamentos().create(dict(
                supfec_capa_id=self.parent.mov_capa.id, supfec_sequencia=1, supfec_valor=value,
                supfec_finalizadora=query_cad_finalizadora.supfin_codigo,
                supfec_tipo_finalizadora=query_cad_finalizadora.supfin_tipo_finalizadora,
                supfec_nome_finalizadora=query_cad_finalizadora.name))

            MovMeiosPagamentos().gravar_meios_pagamento(loja_id, caixa_id,
                                                        self.parent._date_coupon,
                                                        self.parent.mov_capa.supcap_tipo,
                                                        self.parent.mov_capa.supcap_tipo_comprovante,
                                                        value,
                                                        query_cad_finalizadora.supfin_codigo,
                                                        query_cad_finalizadora.name)

            MovCapa().write(self.parent.mov_capa, {'supcap_data_fim': self.printer.get_data_ecf()})

            MovCupomServidor().create({'supcse_capa_id': self.parent.mov_capa.id})
        except Exception as e:
            Log.error(str(e))
            return False

        return True