#!/usr/bin/env python
# -*- Mode: Python; coding: utf-8 -*-

import sys
from PySide import QtCore, QtGui
from sigpdv.ui.busca_produto import BuscaProduto
from sigpdv.ui.busca_cliente import BuscaCliente
from sigpdv.ui.login import LoginForm
from ui_dav import Ui_MainWindow
from sigpdv.models.tbl_pve_dav import itens, dav_capa, dav_corpo, finalizadora, dav_fechamento, b_session as sess
from decimal import Decimal
import os
from report import gerar_rel_dav
from sigpdv import root_path

form_dav, form_pve = range (2)
class DavForm(QtGui.QMainWindow):
    def __init__(self, parent=None, form_type=form_dav):
        super(DavForm, self).__init__()
        
        self.ui = Ui_MainWindow()

        self.ui.setupUi(self)
        
        self.ui.tableItens.resizeRowsToContents()

        self._form_type = form_type
         
        # SINAIS
        self.ui.buttonAddItem.pressed.connect(self._button_add_item_click)
        self.ui.buttonBuscarProduto.pressed.connect(self._button_busca_item_click)
        self.ui.spinAcrescItem.valueChanged.connect(self._calcula_total)
        self.ui.spinDescItem.valueChanged.connect(self._calcula_total)
        self.ui.spinQtda.valueChanged.connect(self._calcula_total)
        self.ui.buttonCancelaItem.pressed.connect(self._cancela_item)
        self.ui.buttonNovoDav.pressed.connect(self._button_novo_dav_click)
        self.ui.buttonAddPgto.pressed.connect(self._button_add_pgto_click)
        self.ui.buttonEditarDav.pressed.connect(self._editar_dav)
        self.ui.ButtonImprimirDav.pressed.connect(self._imprimir_dav)
        self.ui.buttonBuscaDav.pressed.connect(self._buscar_dav)
        self.ui.buttonAlternar.pressed.connect(self._alternar)
        self.ui.buttonUndoPgtos.pressed.connect(self._undo_pgtos)
        # OUTROS
        self._dav = None
        self.ui.stackedWidget.setCurrentIndex(1)
        self._buscar_dav()
        # FINALIZADORAS
        f = finalizadora.query.all()
        self._finalizadoras_inc = {}
        self._finalizadoras_cod = {}
        inc = 0
        self.ui.label_7.setVisible(False)
        self.ui.spinAcrescItem.setVisible(False)
        for x in f:
            self._finalizadoras_inc[inc] = (x.supfin_codigo, x.name)
            self._finalizadoras_cod[x.supfin_codigo] = (inc, x.name)
            
            self.ui.comboFinalizadora.addItem(x.name)
            inc += 1

    def _undo_pgtos(self):
        total = self.ui.spinTotalDav.value()
        pgtos = dav_fechamento.query.filter(dav_fechamento.supdvf_capa_id == self._dav.IdDav)
        sess.begin()
        # Apagar os pagamentos
        for p in pgtos:
            p.delete()
        sess.commit()
        # Limpar a o widget table
        self.clear_widget_table(self.ui.tablePgtos)
        self.ui.spinTotalReceber.setValue(total)
        self.ui.spinTotalPago.setValue(0)

    def _alternar(self):
        self.ui.stackedWidget.setCurrentIndex(1)
        self.ui.editBuscaDav.selectAll()
        self.ui.editBuscaDav.setFocus()
        self._buscar_dav()
        
    def _clear_widgets(self):
        self.clear_widget_table(self.ui.tableItens)
        self.clear_widget_table(self.ui.tablePgtos)
        self.ui.editNumDav.clear()
        self.ui.dateEmissaoDav.clear() 
        self.ui.comboStatusDav.setCurrentIndex(0)
        self.ui.editDocCliente.clear()
        self.ui.labelNomeCliente.clear()
        self.ui.spinTotalDav.setValue(0)
        self.ui.editCodItem.clear()
        self.ui.spinAcrescItem.setValue(0)
        self.ui.spinDescItem.setValue(0)
        self.ui.spinQtda.setValue(1)
        self.ui.spinTotalDav.setValue(0)
        self.ui.spinTotalPago.setValue(0)
        self.ui.spinTotalReceber.setValue(0)
        self.ui.spinVlItem.setValue(0)
        self.ui.spinVlPgto.setValue(0)
        self.ui.spinVlTotal.setValue(0)
    
    
    def _buscar_dav(self):
        table_dav = self.ui.tableBuscaDav
        self.clear_widget_table(table_dav)
        index = self.ui.comboFiltroDav.currentIndex()
        text = self.ui.editBuscaDav.text()
        if text:
            if index == 0:
                dav = dav_capa.get_by(id=text)
                if dav:
                    dav = [dav]
            elif index == 1:
                s = dav_capa.NomeCliente.like('%s%s%s' % ('%', text, '%'))
                dav = dav_capa.query.filter(s)
            elif index == 2:
                dav = dav_capa.get_by(supdva_cliente=text.rjust(15, 0))
                if dav:
                    dav = [dav]
        else:
            dav = dav_capa.query.filter(dav_capa.supdva_restricao==0).all()
        row = 0
        if not dav and self.isActiveWindow():
            QtGui.QMessageBox.information(self, 'DAV', u'A busca não retornou resultados.')
            self.ui.editBuscaDav.selectAll()
            return
        for d in dav:
            print d.TotalDav
            table_dav.insertRow(row)
            # Status
            icon3 = QtGui.QIcon()
            if d.supdva_restricao == 1:
                rest = 'Fechado'
                icon3.addPixmap(QtGui.QPixmap("%s/gui/ui/images/art/16x16/actions/fileprint.png" % root_path), QtGui.QIcon.Normal, QtGui.QIcon.Off)
            else:        
                icon3.addPixmap(QtGui.QPixmap("%s/gui/ui/images/art/16x16/actions/gtk-open.png" % root_path), QtGui.QIcon.Normal, QtGui.QIcon.Off)
                rest = 'Aberto'
            
            item_check = QtGui.QTableWidgetItem(rest)
            item_check.setIcon(icon3)
            # Codigo
            item_cod = QtGui.QTableWidgetItem('%010d' % d.id)
            # data
            item_data = QtGui.QTableWidgetItem('%s' % d.supdva_data_hora)
            # CPF
            item_cpf = QtGui.QTableWidgetItem(d.supdva_cliente)
            # Nome Cliente
            item_nome = QtGui.QTableWidgetItem(d.supdva_nome_cliente)
            # Valor
            item_vr = QtGui.QTableWidgetItem('%1.2f' % d.supdva_total_dav)
            item_vr.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
    
            # WIDGET OS NEW ITENS NA TABLE ITENS
            table_dav.setItem(row, 0, item_check)
            table_dav.setItem(row, 1, item_cod)
            table_dav.setItem(row, 2, item_data)
            table_dav.setItem(row, 3, item_cpf)
            table_dav.setItem(row, 4, item_nome)
            table_dav.setItem(row, 5, item_vr)
            row += 1
        table_dav.resizeColumnsToContents()

    def _imprimir_dav(self):
            idx = self.ui.stackedWidget.currentIndex()
            if idx == 1:
                if self.ui.tableBuscaDav.selectedItems():
                    id_dav = self.ui.tableBuscaDav.item(self.ui.tableBuscaDav.currentRow(), 1).text()
                    status = self.ui.tableBuscaDav.item(self.ui.tableBuscaDav.currentRow(), 0).text()
                    id_dav = int(id_dav)
                    dav = dav_capa.get_by(id=id_dav)
                else:
                    QtGui.QMessageBox.information(self, 'DAV', u'Selecione um DAV.')
                    return
            else:
                dav = self._dav
                id_dav = self._dav.IdDav
                status = self.ui.comboStatusDav.currentText()

            total_dav = dav_capa.get_by(id=id_dav).supdva_total_dav
            pgto = dav_fechamento.query.filter(dav_fechamento.supdvf_capa_id==id_dav)
            total_pgto = Decimal(0)
            for p in pgto:
                total_pgto += p.Valor

            if status == 'Aberto':
                msg = False
                if total_dav == 0:
                    msg = u'Não posso imprimir!\n\nO Total do DAV é igual a zero.'
                elif total_dav != total_pgto:
                    msg = u'Não posso imprimir!\n\nO Total do DAV diferente do Total do Pagamento.'
                if msg:
                    QtGui.QMessageBox.information(self, 'DAV', msg)
                    return
                if QtGui.QMessageBox.question(self, 'DAV', u'Deseja imprimir o DAV: %010d\n\nObs. Despois de impresso, o DAV não pode mais ser alterado.' % id_dav, QtGui.QMessageBox.Yes|QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes:
#                    try:
                        sess.begin()
                        dav.supdva_restricao = 1
                        gerar_rel_dav(id_dav, 1)
                        sess.commit()
#                        if idx == 0:
#                            self.ui.comboStatusDav.setCurrentIndex(1)
#                        else:
#                            item = self.ui.tableBuscaDav.item(self.ui.tableBuscaDav.currentRow(), 0)
#                            item.setText('Fechado')
#                            icon3 = QtGui.QIcon()
#                            icon3.addPixmap(QtGui.QPixmap("../art/16x16/actions/fileprint.png"), QtGui.QIcon.Normal,
#                                            QtGui.QIcon.Off)
#                            item.setIcon(icon3)
                        self.ui.stackedWidget.setCurrentIndex(1)
                        self._buscar_dav()
#                    except Exception as e:
#                        sess.rollback()
#                        QtGui.QMessageBox.information(self, 'DAV', str(e))
            else:
                QtGui.QMessageBox.information(self, 'DAV', u'Dav: %010d, já foi Fechado.' % id_dav)
    
    def _editar_dav(self):   
        if self.ui.tableBuscaDav.selectedItems():
            id_dav = self.ui.tableBuscaDav.item(self.ui.tableBuscaDav.currentRow(), 1).text()
            status = self.ui.tableBuscaDav.item(self.ui.tableBuscaDav.currentRow(), 0).text()
            id_dav = int(id_dav)
            self.ui.editCodItem.setFocus()
            if status == 'Aberto':
                self.editar_dav(id_dav)
            else:
                self.editar_dav(id_dav, ro=False)
        else:
            QtGui.QMessageBox.information(self, 'DAV', u'Selecione um DAV.')
    
    def editar_dav(self, id_dav, ro=True):
        self._clear_widgets()
        self._dav = dav = dav_capa.get_by(id=id_dav)
        self.ui.stackedWidget.setCurrentIndex(0)
        self.ui.tabWidget.setCurrentIndex(0)
        self.ui.editNumDav.setText('%010d' % dav.IdDav)
        d = dav.DataHora.timetuple()
        d = QtCore.QDateTime(d[0], d[1], d[2], d[3], d[4], d[5])
        self.ui.dateEmissaoDav.setDateTime(d) 
        self.ui.comboStatusDav.setCurrentIndex(0)
        self.ui.editDocCliente.setText(dav.supdva_cliente)
        self.ui.labelNomeCliente.setText('<p style="font:bold; color:#aa0000;">%s</p>' % dav.NomeCliente)
        self.ui.spinTotalDav.setValue(dav.supdva_total_dav)
        self.ui.comboStatusDav.setCurrentIndex(dav.supdva_restricao)
        corpos = dav_corpo.query.filter(dav_corpo.supdvo_capa_id == id_dav).order_by('supdvo_sequencia')
        for c in corpos:
            item = itens.get_by(supite_codigo_item=c.supdvo_item)
            icon3 = QtGui.QIcon()
            if c.VendaCancelamento == 'V':
                icon3.addPixmap(QtGui.QPixmap("%s/gui/ui/images/16x16/check2.png" % root_path), QtGui.QIcon.Normal, QtGui.QIcon.Off)
            else:        
                icon3.addPixmap(QtGui.QPixmap("%s/gui/ui/images/16x16/delete.png" % root_path), QtGui.QIcon.Normal, QtGui.QIcon.Off)
            table_item = self.ui.tableItens
            row = table_item.rowCount()
            table_item.insertRow(row)
            # Codigo Item
            item_check = QtGui.QTableWidgetItem('')
            item_check.setIcon(icon3)                        
            # Codigo Item
            item_cod = QtGui.QTableWidgetItem(c.supdvo_item)
            # Descrição
            item_descr = QtGui.QTableWidgetItem(item.name)
            # Quantidade
            item_qtda = QtGui.QTableWidgetItem('%1.2f' % c.supdvo_quantidade)
            # Valor Unitário
            item_vuni = QtGui.QTableWidgetItem('R$ %1.2f' % item.supite_preco)
            item_vuni.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            # Acrescimo
            item_acresc = QtGui.QTableWidgetItem('%1.2f' % c.supdvo_acrescimo_item)
            # Desconto
            item_desc = QtGui.QTableWidgetItem('%1.2f' % c.supdvo_desconto_item)
            # Total
            item_total = QtGui.QTableWidgetItem('R$ %1.2f' % c.supdvo_vr_total)
    
            # WIDGET OS NEW ITENS NA TABLE ITENS
            table_item.setItem(row, 0, item_check)
            table_item.setItem(row, 1, item_cod)
            table_item.setItem(row, 2, item_descr)
            table_item.setItem(row, 3, item_qtda)
            table_item.setItem(row, 4, item_vuni)
            table_item.setItem(row, 5, item_acresc)
            table_item.setItem(row, 6, item_desc)
            table_item.setItem(row, 7, item_total)
            table_item.resizeColumnsToContents()
            table_item.scrollToBottom()
        pgtos = dav_fechamento.query.filter(dav_fechamento.supdvf_capa_id == id_dav).order_by('supdvf_sequencia')
        for p in pgtos:
            self.add_pgto_in_table(p.supdvf_finalizadora, float(str(p.Valor)))

        self.ui.tab.setEnabled(ro)
        self.ui.tab_2.setEnabled(ro)
        self.ui.buttonCancelaItem.setEnabled(ro)

    def _button_add_pgto_click(self):
        if self.ui.spinVlPgto.value():
            fin = self._finalizadoras_inc[self.ui.comboFinalizadora.currentIndex()]
            count = dav_fechamento.query.filter(dav_fechamento.IdDav == self._dav.IdDav).count()
            try:
                if not self.ui.spinTotalDav:
                    raise Exception('Adicione itens ao DAV p/ poder efetuar o Pagamento')
                elif self.ui.spinVlPgto.value() > self.ui.spinTotalReceber.value():
                    raise Exception('Valor do Pagamento Ultrapassa o Total à Receber')
                                    
                sess.begin()
                dav_fechamento(supdvf_capa_id=self._dav.IdDav,
                            supdvf_sequencia=count + 1,
                            supdvf_finalizadora=fin[0],
                            supdvf_valor=self.ui.spinVlPgto.value())
                sess.commit()
                self.add_pgto_in_table(fin[0], self.ui.spinVlPgto.value())
            except Exception as e:
                QtGui.QMessageBox.critical(self, 'DAV', str(e))
                self.ui.spinVlPgto.setValue(0)
                sess.rollback()
                return
        else:
            QtGui.QMessageBox.information(self, 'DAV', u'Informe o Valor do Pagamento')
            self.ui.spinVlPgto.setFocus()
            self.ui.spinVlPgto.setValue(self.ui.spinTotalDav.value() - self.ui.spinTotalPago.value())
                        
    def add_pgto_in_table(self, cod_fin, valor):
        table = self.ui.tablePgtos
        row = table.rowCount()
        table.insertRow(row)
        # Descrição
        item_fin = QtGui.QTableWidgetItem(self._finalizadoras_cod[cod_fin][1])
        # Valor
        item_vl = QtGui.QTableWidgetItem('R$ %1.2f' % valor)
        item_vl.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        # WIDGET OS NEW ITENS NA TABLE ITENS
        table.setItem(row, 0, item_fin)
        table.setItem(row, 1, item_vl)
        table.resizeColumnsToContents()    
        table.scrollToBottom()
        
        total_pago = self.ui.spinTotalPago.value() + valor
        total_receber = self.ui.spinTotalDav.value() - total_pago
        self.ui.spinTotalPago.setValue(total_pago)
        self.ui.spinTotalReceber.setValue(total_receber)
    
    def _cancela_item(self):
        if not self.ui.tableItens.selectedItems():
            QtGui.QMessageBox.information(self, 'DAV', u'Selecione um item.')
            QtGui.QMessageBox.information(self, 'DAV', u'Informe o Valor do Pagamento')
            return
        row = self.ui.tableItens.currentRow()
        tb_item = self.ui.tableItens.item(row, 0)
        item_des = self.ui.tableItens.item(row, 2).text()
        item_vr = self.ui.tableItens.item(row, 7).text()
        if QtGui.QMessageBox.question(self, 'DAV', u'Deseja cancelar o item:\n%d-%s' % (row + 1, item_des),  QtGui.QMessageBox.Yes| QtGui.QMessageBox.No) ==  QtGui.QMessageBox.No:
            return
         
        item = dav_corpo.get_by(supdvo_capa_id=self._dav.IdDav,
                                supdvo_sequencia=row + 1)
        if item.VendaCancelamento == 'C':
            QtGui.QMessageBox.critical(self, 'DAV', u'O Item selecionado já foi cancelado!')
            return
        else:
            try:
                total = self.ui.spinTotalDav.value() - float(item_vr.split(' ')[-1])
                pgtos = dav_fechamento.query.filter(dav_fechamento.supdvf_capa_id == self._dav.IdDav)
                sess.begin()
                self._dav.supdva_total_dav = total
                item.supdvo_venda_cancelamento = 'C'
                # Apagar os pagamentos
                for p in pgtos:
                    p.delete()
                sess.commit()
                # Limpar a o widget table
                self.clear_widget_table(self.ui.tablePgtos)
                self.ui.spinTotalDav.setValue(total)
                self.ui.spinTotalReceber.setValue(self.ui.spinTotalDav.value())
                self.ui.spinTotalPago.setValue(0)                
            except Exception as e:
                sess.rollback()
                QtGui.QMessageBox.critical(self, 'DAV', str(e))
                return
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap("%s/gui/ui/images/16x16/delete.png" % root_path), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        tb_item.setIcon(icon3)
        
    def _calcula_total(self):
        total = self.ui.spinVlItem.value() * self.ui.spinQtda.value() + \
                self.ui.spinAcrescItem.value() - self.ui.spinDescItem.value()
        self.ui.spinVlTotal.setValue(total)
        return total
    
    def _button_novo_dav_click(self):
        busca_cli_form = BuscaCliente(self, 'S')
        if busca_cli_form.exec_():
            cli = busca_cli_form.retorno
            doc_cli = cli.CNPJCPF
            nome_cli = cli.Nome
            self.ui.editCodItem.setFocus()
            try:
                sess.begin()
                self._dav = dav = dav_capa(supdva_cliente=doc_cli, supdva_nome_cliente=nome_cli)
                sess.commit()
            except Exception as e:
                QtGui.QMessageBox.critical(self, 'DAV', str(e))
                sess.rollback()
                return
            self._clear_widgets()
            self.ui.stackedWidget.setCurrentIndex(0)
            self.ui.tabWidget.setCurrentIndex(0)
            self.ui.editNumDav.setText('%010d' % dav.id)
            self.ui.dateEmissaoDav.setDateTime(QtCore.QDateTime.currentDateTime())
            self.ui.comboStatusDav.setCurrentIndex(dav.supdva_restricao)
            self.ui.editDocCliente.setText(doc_cli)
            self.ui.labelNomeCliente.setText('<p style="font:bold; color:#aa0000;">%s</p>' % nome_cli)
            
            self.ui.editCodItem.setFocus()
        
    def _button_busca_item_click(self):
        cod_item = self.ui.editCodItem.text()
        item = None
        if cod_item:
            item = itens.get_by(supite_codigo_item=cod_item.rjust(14, '0'))
            if item:
                self.ui.editCodItem.setText(item.supite_codigo_item)
                self.ui.spinVlItem.setValue(item.supite_preco)
                self._calcula_total()
                self.ui.spinQtda.setFocus()
                self.ui.labelItem.setText('<b>%s' % item.name)
            else:
                busca_produto_form = BuscaProduto(self)
                if busca_produto_form.exec_():
                    res = busca_produto_form.retorno
                    if res:
                        self.ui.editCodItem.setText(res.supite_codigo_item)
                        self.ui.spinVlItem.setValue(res.supite_preco)
                        self._calcula_total()
                        self.ui.spinQtda.setFocus()
                        self.ui.labelItem.setText('<b>%s' % res.supite_codigo_item)
                else:
                    self.ui.editCodItem.selectAll()
        return item           


    def _button_add_item_click(self):
        item = self._button_busca_item_click()
        if item:
            self._adicionar_item(item)
    
    def _adicionar_item(self, item):
        # GET
        cod = item.supite_codigo_item
        descr = item.name
        qtda = self.ui.spinQtda.value()
        vuni = item.supite_preco
        acresc = self.ui.spinAcrescItem.value()
        desc = self.ui.spinDescItem.value()
        total = Decimal(str(self.ui.spinVlTotal.value()))
        descricao = descr 
        unidade = item.supite_unidade
        preco = item.supite_preco

        # ADD ON TABLE
        if self.ui.spinDescItem.value() >= self.ui.spinVlItem.value():
            QtGui.QMessageBox.information(self, 'DAV', u'Desconto maior ou igual que o valor do item.')
            self.ui.spinDescItem.setValue(0)
            self.ui.spinDescItem.setFocus()
            return
        try:
            total_dav = self._dav.supdva_total_dav
            sess.begin()
            self._dav.supdva_total_dav = total_dav + total
            sess.commit()
            self.ui.spinVlTotal.setValue(total_dav + total)
            sess.begin()
            dav_corpo(supdvo_capa_id=self._dav.IdDav,
                supdvo_numero_do_dav=self._dav.IdDav,
                supdvo_sequencia=self.ui.tableItens.rowCount() + 1 ,
                supdvo_item=cod,
                supdvo_quantidade=qtda,
                supdvo_vr_total=total,
                supdvo_desconto_item=desc,
                supdvo_acrescimo_item=acresc,
                supdvo_descricao = descricao,
                supdvo_unidade = unidade,
                supdvo_vr_unit = preco,
                supdvo_totalizador_parcial = ''
                )
            sess.commit()
        except Exception as e:
            sess.rollback()
            QtGui.QMessageBox.critical(self, 'SigNfe', str(e))
            return
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap("%s/gui/ui/images/16x16/check2.png" % root_path), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        table_item = self.ui.tableItens
        row = table_item.rowCount()
        table_item.insertRow(row)
        # Codigo Item
        item_check = QtGui.QTableWidgetItem('')
        item_check.setIcon(icon3)                        
        # Codigo Item
        item_cod = QtGui.QTableWidgetItem(cod)
        # Descrição
        item_descr = QtGui.QTableWidgetItem(descr)
        # Quantidade
        item_qtda = QtGui.QTableWidgetItem('%1.2f' % qtda)
        # Valor Unitário
        item_vuni = QtGui.QTableWidgetItem('R$ %1.2f' % vuni)
        item_vuni.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        # Acrescimo
        item_acresc = QtGui.QTableWidgetItem('%1.2f' % acresc)
        # Desconto
        item_desc = QtGui.QTableWidgetItem('%1.2f' % desc)
        # Total
        item_total = QtGui.QTableWidgetItem('R$ %1.2f' % total)

        # WIDGET OS NEW ITENS NA TABLE ITENS
        table_item.setItem(row, 0, item_check)
        table_item.setItem(row, 1, item_cod)
        table_item.setItem(row, 2, item_descr)
        table_item.setItem(row, 3, item_qtda)
        table_item.setItem(row, 4, item_vuni)
        table_item.setItem(row, 5, item_acresc)
        table_item.setItem(row, 6, item_desc)
        table_item.setItem(row, 7, item_total)
        table_item.resizeColumnsToContents()
        table_item.scrollToBottom()
        self.ui.editCodItem.clear()
        self.ui.editCodItem.setFocus()
        self.ui.spinVlItem.setValue(0)
        self.ui.spinQtda.setValue(1)
        self.ui.spinAcrescItem.setValue(0)
        self.ui.spinDescItem.setValue(0)
        self.ui.spinVlTotal.setValue(0)
        self.ui.spinTotalDav.setValue(self.ui.spinTotalDav.value() + float(total))
        self.ui.spinTotalReceber.setValue(self.ui.spinTotalDav.value() - self.ui.spinTotalPago.value())
        self.ui.labelItem.setText(u'<b>Digite o código do item e pressiona o botão [+] para adicionar o item')
    def clear_widget_table(self, table):
        # Apaga lista de itens
        rows_count = table.rowCount()
        # Verifica se hà dados
        if rows_count:
            rows = range(0, rows_count + 1)
            rows.reverse()
            for r in rows:
                table.removeRow(r)
        return rows_count

def run():
    app = QtGui.QApplication(sys.argv)
    login_form = LoginForm(db='S')
    # Tradução
    qtTranslator = QtCore.QTranslator()
    if qtTranslator.load(os.path.join(root_path, "qt_pt.qm")):
        app.installTranslator(qtTranslator)
        print('installTranslator pt_BR')
    else:
        print('Arquivo de tradução não encontrado')
    if login_form.exec_():
        form = DavForm()
        form.setWindowTitle('SigPDV - Documento Auxiliar de Venda (DAV)')
        form.showMaximized()
        sys.exit(app.exec_())
    else:
        sys.exit(-1)

if __name__ == "__main__":
    print '%s/gui/ui/images/16x16/check2.png' % root_path
    run()