# -*- coding: utf-8 -*-

__author__ = 'Christian Douglas'
__version__ = (1, 0, 0)

from PySide import QtCore, QtGui
from ui.ui_sigpdv import Ui_Dialog
import sys

class Animation(QtCore.QPropertyAnimation):
    LinearPath, CirclePath = range(2)

    def __init__(self, target, prop):
        super(Animation, self).__init__(target, prop)
        self.setPathType(Animation.LinearPath)

    def setPathType(self, pathType):
        self.m_pathType = pathType
        self.m_path = QtGui.QPainterPath()

    def updateCurrentTime(self, currentTime):
        if self.m_pathType == Animation.CirclePath:
            if self.m_path.isEmpty():
                end = self.endValue()
                start = self.startValue()
                self.m_path.moveTo(start)
                self.m_path.addEllipse(QtCore.QRectF(start, end))

            dura = self.duration()
            if dura == 0:
                progress = 1.0
            else:
                progress = (((currentTime - 1) % dura) + 1) / float(dura)

            easedProgress = self.easingCurve().valueForProgress(progress)
            if easedProgress > 1.0:
                easedProgress -= 1.0
            elif easedProgress < 0:
                easedProgress += 1.0

            pt = self.m_path.pointAtPercent(easedProgress)
            self.updateCurrentValue(pt)
            self.valueChanged.emit(pt)
        else:
            super(Animation, self).updateCurrentTime(currentTime)

# PyQt doesn't support deriving from more than one wrapped class so we use
# composition and delegate the property.
class Pixmap(QtCore.QObject):
    def __init__(self, pix):
        super(Pixmap, self).__init__()

        self.pixmap_item = QtGui.QGraphicsPixmapItem(pix)
        self.pixmap_item.setCacheMode(QtGui.QGraphicsItem.DeviceCoordinateCache)

    def set_pos(self, pos):
        self.pixmap_item.setPos(pos)

    def get_pos(self):
        return self.pixmap_item.pos()

    pos = QtCore.Property(QtCore.QPointF, get_pos, set_pos)


class DlgSigPdv(QtGui.QDialog):
    def __init__(self):
        super(DlgSigPdv, self).__init__()

        self.m_scene = QtGui.QGraphicsScene()

        self.ui = Ui_Dialog()
        self.ui.setupUi(self)
        self.ui.btnKEY_ACRESCIMO.clicked.connect(self._hide)
        self.ui.btnKEY_BALANCA.clicked.connect(self.run_s)


        self.info_anim = Animation(self.ui.lblInfo, 'pos')     # Animação do Texto de Informação
        self.font = QtGui.QFont("Ubuntu", 18, QtGui.QFont.Bold) # Fonte usada em alguns widgets da Aplicação
        self.ui.lblInfo.setFont(self.font) # Defini a fonte padrão p/ o Label de Informação

    def iniciar_animacao_info(self, tipo_curva=QtCore.QEasingCurve.InOutElastic):
        """
        Executa a animação do texto de informação
        """
        w = (self.ui.frmInfo.width() - self.ui.lblInfo.width()) / 2

        self.info_anim.setEasingCurve(tipo_curva)
        h = (self.ui.frmInfo.height() - self.ui.lblInfo.height()) / 2
        self.info_anim.setStartValue(QtCore.QPointF(w, self.ui.lblInfo.height() * -1))

        self.info_anim.setEndValue(QtCore.QPointF(w, h))
        self.info_anim.setDuration(500)
        self.info_anim.setLoopCount(1)
        self.info_anim.start()

    def _hide(self):
        self.ui.frame_keyboard.setVisible(False)
    def run_s(self):

        self.infoText('Sig Soluções em Informática')

    def infoText(self, text):
        """
        Ajusta o texto a ser exibido no widget 'lblInfo' e executa a animação
        """
        text = self.trUtf8(text)
        metrics = QtGui.QFontMetrics(self.font)
        w, h = metrics.width(text), metrics.height() # Pega a largura/altura do texto em pixels
        self.ui.lblInfo.setGeometry(0, 0, w, h)
        self.ui.lblInfo.setText(text)
        self.iniciar_animacao_info() # Executa a animação

if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    mainWin = DlgSigPdv()
    mainWin.show()
    sys.exit(app.exec_())
