#!/usr/bin/python
# -*- coding: utf-8 -*-


from sigpdv.models.db_utils import a_metadata, a_session
from elixir import Entity, Field, using_table_options
from elixir.options import using_options
from sqlalchemy import CHAR, SmallInteger, DateTime, Enum, Integer, String, \
    Date, DECIMAL, BOOLEAN, ForeignKey, UniqueConstraint, Text


class CaixaCadFicha(Entity):
    """
    Tabela de Fichas
    Prefixo: supfch
    """
    using_options(metadata=a_metadata, session=a_session, tablename='caixa_cad_ficha')

    id = Field(Integer(11), primary_key=True, autoincrement=True)
    supfch_codigo_ficha = Field(Integer(11), index=True)
    state = Field(Enum('1', '2', '3', name='state')) # 1=Emitida, 2=Trocada, 3=Faturada
    create_date = Field(DateTime(), index=True)
    supfch_valor = Field(DECIMAL(10, 2))
    supfch_desconto = Field(DECIMAL(10, 2))
    supfch_loja = Field(SmallInteger(6), index=True)
    supfch_caixa = Field(SmallInteger(6), index=True)
    supfch_item = Field(String(14), index=True)
    supfch_exportado = Field(BOOLEAN)
    supfch_caixa_faturado = Field(SmallInteger(6), index=True)
    supfch_cupom_coo = Field(Integer(11), index=True)
    supfch_cupom_data = Field(Date(), index=True)
    supfch_cz_nome_cliente = Field(String(40))
    supfch_cz_data_hora_entrega = Field(DateTime())
    supfch_cz_observacoes = Field(Text())
    supfch_cz_state_impressao = Field(Enum('1', '2', '3', name='state_impressao')) # 1=Nao imprimir,
                                                                         # 2=A imprimir, 3=Impresso

    # _sql_constraints
    using_table_options(UniqueConstraint('supfch_codigo_ficha', 'supfch_loja', 'supfch_caixa'))

    def __repr__(self):
        _repr = "<caixa_cad_ficha: 'Loja=%d', 'Data Emissao=%s', 'Numero=%d'>"
        vals = self.supfch_loja, self.supfch_caixa, self.create_date, self.supfch_codigo_ficha
        return _repr % vals