#!/usr/bin/python
# -*- encoding: utf-8 -*-

from datetime import datetime
from sigpdv.models.db_utils import a_metadata, a_session
from elixir import Entity, Field
from elixir.options import using_options, using_table_options
from sqlalchemy import SmallInteger, String, DateTime, UniqueConstraint


class CaixaCfgEcfAutorizada(Entity):
    """
    Tabela de ecfs autorizadas
    Prefixo: supecf
    """
    using_options(metadata=a_metadata, session=a_session, tablename='caixa_cfg_ecf_autorizada')
    using_table_options(useexisting=True)

    supecf_loja = Field(SmallInteger(6))
    supecf_caixa = Field(SmallInteger(6))
    supecf_create_date = Field(DateTime(), default=datetime.today())
    supecf_data_sb = Field(String(10))
    supecf_fabricante = Field(String(100))
    supecf_hora_sb = Field(String(10))
    supecf_modelo = Field(String(30))
    supecf_nserie = Field(String(20))
    supecf_nacional = Field(String(10))
    supecf_versao = Field(String(10))
    supecf_empresa = Field(String(100))
    supecf_sign = Field(String(256))
    supecf_md5 = Field(String(32))

    # _sql_constraints
    using_table_options(UniqueConstraint('supecf_loja', 'supecf_caixa'))

    def __repr__(self):
        _repr = "<caixa_mov_ecf_autorizada: 'Loja=%d', 'Caixa=%d', 'Serie=%s'>"
        vals = self.supecf_loja, self.supecf_caixa, self.supecf_nserie
        return _repr % vals