#!/usr/bin/python
# -*- encoding: utf-8 -*-

from datetime import datetime
from sigpdv import get_version
from sigpdv.models.db_utils import a_metadata, a_session
from elixir import Entity, Field
from elixir.options import using_options, using_table_options
from elixir.ext.perform_ddl import preload_data
from sqlalchemy import DateTime, String


class CaixaCfgVersao(Entity):
    """
    Tabela de versão
    Prefixo: supver

    Significado da coluna Versao:
    3.1.01.01
     |   |  |_ Versão do Sql que foi rodado na base.
     |   |____ Versão das Alterações Internas do Frente.
     |________ Versão do SigFrente para a Secretaria da Fazenda.
    """
    using_options(metadata=a_metadata, session=a_session, tablename='caixa_cfg_versao')
    using_table_options(useexisting=True)

    last_update = Field(DateTime, default=datetime.now())
    supver_versao = Field(String(40))

    # --------------- defaults --------------- #
    fields = ('last_update', 'supver_versao')
    vals = [(datetime.now(), get_version())]
    preload_data(fields, vals)
    # ---------------------------------------- #

    def __repr__(self):
        _repr = "<caixa_cfg_versao: %s, Versao=%s>"
        vals = self.last_update, self.supver_versao
        return _repr % vals