#!/usr/bin/python
# -*- coding: utf-8 -*-

from sigpdv.tools.log import traced as Log
from sigpdv.models.db_utils import sess, exec_sql
from datetime import datetime
from sigpdv.tools.config import config
import socket
import struct

STATUS = {0: 'Cartao validado ate DD/MM/AAAA - HH:MM.',
          1: 'Cartao nao validado.',
          2: 'Cartão ja validado.',
          3: 'Valor da compra insuficiente para validacao.',
          4: 'Cartao invalido.',
          5: 'Comando invalido.',
          6: 'Operacao invalida.',
          7: 'Terminal nao cadastrado.',
          8: 'Tempo de desconto excedido.',
          9: 'Tipo de cartão inválido.',
          99: 'Sem resposta'}

class Nepos(object):
    def __init__(self, parent):
        self.parent = parent
        self.cmdConsult = 15
        self.cmdValidation = 16
        self.sequencia = 0 # numero de comandos enviados
        self.ENTRADA = 100
        self.PASSAGEM = 110
        self.SAIDA = 120
        self.TCP_IP = config['nepos_ip']
        self.TCP_PORT = config['nepos_port_tcp']
        self.BUFFER_SIZE = 1024
        self.str_cmd = ''

    class respcmdData(object):
        def __init__(self, parent, resp):
            if resp == '99':
                self.rspStatus = 99 # sem resposta
                return

            self.parent = parent
            # rspLength
            self.rspBlockSize = struct.unpack('<h', resp[0:2])[0]
            # rspHeader
            self.rspFiller = resp[2:4] # uso futuro
            self.rspType = struct.unpack('<l', resp[4:8])[0]
            self.rspSignature = resp[8:23]
            self.rspCompanySign = resp[23:39]
            self.rspTmt = struct.unpack('<l', resp[39:43])[0]
            self.rspSeqNo = struct.unpack('<l', resp[43:47])[0]
            #rspData
            self.rspTermId = struct.unpack('<l', resp[47:51])[0]
            self.rspCardId = resp[51:115]
            self.rspStatus = struct.unpack('<l', resp[115:119])[0]
            self.rspOpDisplayTxt = resp[119:247].decode('iso-8859-1').encode('utf8').replace('\x00',' ')
            self.rspCustDisplayTxt = resp[247:375].decode('iso-8859-1').encode('utf8').replace('\x00',' ')
            self.rspPrinterLineTxt = resp[375:503].decode('iso-8859-1').encode('utf8').replace('\x00',' ')
            self.rspEntryTimeStamp = struct.unpack('<l', resp[503:507])[0]
            self.rspVehicleType = struct.unpack('<h', resp[507:509])[0]
            self.rspRUF_1 = resp[509:511] # uso futuro
            self.rspRUF_2 = resp[511:515] # uso futuro
    @Log
    def length(self, cmd):
        '''
        Retorna o tamanho do comando
        :param cmd: eh o 'hedear' + 'cmdData'
        :return:
        '''
        return struct.pack('<h',len(cmd)) # 2 bytes

    @Log
    def hedear(self, tipo_comando):
        cmdFiller = '\x00\x00' # 2 bytes # reservados para uso futuro. Complemento do campo msgBlockSize
        cmdType = struct.pack('<l', tipo_comando) # 4 bytes  tipo de comando:
        cmdSignature = str(self.parent.cnpj_empresa).ljust(15,'\x00') # 15 bytes (ASCII)
        cmdCompanySign = 'NEPOS          \x00' # 16 bytes (ASCII) 
        dif = datetime.now() - datetime(1970,1,1)
        segundos = struct.pack('<l', dif.days * 24*60*60 + dif.seconds)
        cmdTmt = segundos # 4 bytes 
        cmdSeqNo = struct.pack('<l', self.sequencia) # 4 bytes  Sequencia de comandos
        self.sequencia += 1

        return cmdFiller + cmdType + cmdSignature + cmdCompanySign + cmdTmt + cmdSeqNo

    @Log
    def cmdData(self):
        codigo_barras = self.parent.codigo_estacionamento
        vr_compra = self.parent.mov_capa.supcap_total_cupom
        coo = self.parent.mov_capa.supcap_nro_cupom
        caixa = self.parent.caixa_id
        cmdTermId = struct.pack('<l', caixa) # 4 bytes  PDV
        cmdCardId = str(codigo_barras).ljust(64, '\x00')
        cmdOpValue = struct.pack('<l', vr_compra * 100) # valor da compra em centavos
        cmdOpSeqNo = struct.pack('<l', coo)
        cmdRUF_0 = '\xff\xff\xff\xff' # uso futuro
        cmdRUF_1 = '\xff\xff\xff\xff' # uso futuro
        cmdSaleType = '\xff\xff\xff\xff'
        cmdOptDisplayLen = struct.pack('<l', 40)
        cmdCustDisplayLen = struct.pack('<l', 40)
        cmdPrinterLineLen = struct.pack('<l', 40)
        Log.info(
            'Dados enviados ao Nepos: Caixa=%s; CodigoBarras=%s; COO=%s; TotalCupom=%s;'
            % (caixa, codigo_barras, coo, vr_compra))
        return cmdTermId + cmdCardId + cmdOpValue + cmdOpSeqNo + cmdRUF_0 + cmdRUF_1 + \
               cmdSaleType + cmdOptDisplayLen + cmdCustDisplayLen + cmdPrinterLineLen

    @Log
    def envia_comando(self, comando):
        self.s.send(comando)
        data = self.s.recv(self.BUFFER_SIZE)
        res = self.respcmdData(self.parent, data)
        return res

    @Log
    def inicia_conexao(self):
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.settimeout(10)
        self.s.connect((self.TCP_IP, self.TCP_PORT))
        self.sequencia = 0
        return True

    @Log
    def fecha_conexao(self):
        self.s.close()
        return True

    @Log
    def envia_codigo_barras(self):
        self.inicia_conexao()
        return self.libera_cartao_estacionamento()

    @Log
    def libera_cartao_estacionamento(self):
        header = self.hedear(self.cmdValidation)
        cmdData = self.cmdData()
        cmd = self.length(header+cmdData) + header + cmdData
        self.inicia_conexao()
        res = self.envia_comando(cmd)
        self.fecha_conexao()
        if res.rspStatus == 0:
            return (1,res.rspOpDisplayTxt) # 1 = sem erro
        else:
            return (0,res.rspOpDisplayTxt) # 0 = com erro


