#!/usr/bin/env python
# -*- coding: utf-8 -*-

__autor__ = 'Dener Rodrigues'

try:
    import subprocess
    import os
    from sigpdv.tools.log_restaurante import traced as log
    sub_process = subprocess.Popen(['ps', '-ef'], stdout=subprocess.PIPE)
    out, err = sub_process.communicate()
    # caso a comanda levante duas vezes, a segunda istancia fecha automaticamente.
    for line in out.splitlines():
        dados_linha = line.split()
        if 'consulta_comanda' == dados_linha[-1] and not str(os.getpid()) in line:
                log.info('Dois Comandas abertos!')
                cmd = 'kill -9 %s' % os.getpid()
                log.info(cmd)
                os.system(cmd)

    from PyQt4 import QtCore, Qt
    import sys
    from sigpdv.ui.messagebox_timeout import YES_ROLE, MSG_ERROR, MSG_QUESTION, MSG_INFO, \
        MSG_NONE, message_dlg
    from PySide import QtCore, QtGui
    from sigpdv.gui.ui.ui_ConsultaRest import Ui_MainWindow
    from sigpdv.models.supervisor import SupervisorCadVendedor, SupervisorCadItens, \
        SupervisorCdaCapa, SupervisorCdaCorpo, SupervisorCdaItensFavoritos
    from sigpdv.models.cfg_configuracao_data import CaixaCfgConfiguracao
    from sigpdv.models.db_utils import exec_sql

    from elixir import create_all, setup_all
    # setup_all  e create_all criam as tabelas que nao existem no banco de dados
    # dentro de cada arquivo do model tinha uma chamada dessas funcoes, foram retiradas pois quando o supervisor não está
    # na mesma loja ou o ip trocou, demora muito para inicializar o frente.
    try:
        setup_all()
        create_all()
    except Exception as e:
        log.error(str(e))

    from decimal import Decimal
    from sigpdv import root_path
    from sigpdv.tools.config import config
    import traceback
    from sigpdv.control.utils import checar_conexao, config_dados_cliente
except Exception as e:
    import traceback
    from sigpdv.tools.log_restaurante import traced as log

    str_err = traceback.format_exc()
    log.error(str_err)
    try:
        app = QtGui.QApplication(sys.argv)
    except:
        pass
    from sigpdv.ui.messagebox_timeout import message_dlg

    message_dlg(None, MSG_ERROR, 'A Aplicação será encerrada', str_err)
    sys.exit()

# STATUS DA TELA
page, page_2 = range(2)

STATE_SCREEN_NAMES = {page: 'page', page_2: 'page_2'}

# Stacked Actions Pages
stk_page, stk_page_2 = range(2)

TWOPLACES = Decimal(10) ** -2
TREEPLACES = Decimal(10) ** -3


@log
def exception_message_box(text, btn_quit=False):
    """
    Extract exception information and display a message box with exception details in
    a details extension of the message.
    """
    msg = QtGui.QMessageBox()
    msg.setWindowTitle(QtGui.QApplication.applicationName())
    msg.setText(text)
    exc_type, exc_value, exc_traceback = sys.exc_info()
    traceback.print_exc()
    msg.setDetailedText(''.join(traceback.format_exception(exc_type, exc_value, exc_traceback)))
    msg.setStandardButtons(QtGui.QMessageBox.Ok)
    msg.setIcon(QtGui.QMessageBox.Critical)
    if btn_quit:
        msg.addButton(QtGui.QMessageBox.Close)
    ret = msg.exec_()
    if ret == QtGui.QMessageBox.Close:
        sys.exit()


@log
def get_loja():
    res = CaixaCfgConfiguracao.get_by()
    if not res:
        config_dados = config_dados_cliente('pdvsig.sig')
        res = CaixaCfgConfiguracao.get_by()
        if not config_dados or not res or not res.supcfc_loja:
            msg = 'Empresa não cadastrada na tabela caixa_cfg_configuracao\n\n' +\
                  'A Aplicação não pode continuar'
            if not res.supcfc_loja:
                msg = "Atenção!\n\nLoja não está cadastrada no parâmetro do Supervisor!" + \
                      '\nA Aplicação não pode continuar'

            message_dlg(None, MSG_ERROR, msg)
            sys.exit()

    return res

class ConsultaRestForm(QtGui.QMainWindow):
    def __init__(self, parent=None):

        super(ConsultaRestForm, self).__init__()

        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.ui.stackedWidget.setCurrentIndex(stk_page)
        self.estacionamento = False
        self.teclado = False
        self.btn_cancel_info_dialog = False
        self.cancel_pinpad = False

        self._CONTROL_FOCUS = None
        self._tab_order = []

        self.status_conexao_servidor = False
        checar_conexao(parent=self, var=True)
        scanner_timer = QtCore.QTimer(self)
        scanner_timer.timeout.connect(self.timer)
        scanner_timer.start(1000)

        self._tempo_ocioso = 0
        self.loja = get_loja()
        self.ui.btn_voltar.pressed.connect(self.key_button_voltar)
        self.ui.btn_fechar.pressed.connect(self.key_button_fechar)

        self.filtro_consulta()

    @log
    def filtro_consulta(self, *args):
        self.setControlFocus(self._CONTROL_FOCUS, self.ui.edt_comanda)

    def timer(self):
        st = self.status_conexao_servidor
        self.ui.radioButton.setChecked(st)
        time_cda = config['time_comanda']

        if self._tempo_ocioso >= time_cda:
            self.ui.labelItem.setText(u'SiG Informática')
            self.key_button_voltar()
            self._tempo_ocioso = 0
        else:
            self._tempo_ocioso += 1

        try:
            if not st:
                raise Exception('Sem conexao com o Banco de Dados Supervisor.')
        except Exception as e:
            exception_message_box(str(e), True)
        return st

    @log
    def setControlFocus(self, old, new, is_escape=False):
        old_name, new_name = ('', '')
        if old:
            old_name = old.objectName()
        if new:
            new_name = new.objectName()
        if not is_escape:
            self._tab_order.append(new)
        if old:
            old.setStyleSheet("")

        self._CONTROL_FOCUS = new

        if type(new) in [QtGui.QLineEdit, QtGui.QDoubleSpinBox, QtGui.QComboBox, QtGui.QDateEdit]:
            new.setStyleSheet("background-color: rgba(153, 204, 204, 255); color: rgb(0, 0, 127)")

            if type(old) in [QtGui.QDoubleSpinBox, QtGui.QDateEdit]:
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Up,
                                     QtCore.Qt.KeyboardModifiers())
                old.keyPressEvent(ev)
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Down,
                                     QtCore.Qt.KeyboardModifiers())
                old.keyPressEvent(ev)
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_End,
                                     QtCore.Qt.KeyboardModifiers())
                old.keyPressEvent(ev)

            elif type(new) in [QtGui.QDoubleSpinBox, QtGui.QDateEdit]:
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Up,
                                     QtCore.Qt.KeyboardModifiers())
                new.keyPressEvent(ev)
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Down,
                                     QtCore.Qt.KeyboardModifiers())
                new.keyPressEvent(ev)
        elif type(new) == QtGui.QTableWidget:
            new.setStyleSheet('background-color: rgb(255, 252, 205)')
        try:
            new.selectAll()
            new.setFocus()
        except Exception as e:
            pass

        return True

    @log
    def keyPressEvent(self, ev):
        if ev.key() in (QtCore.Qt.Key_Return.__int__(), QtCore.Qt.Key_Enter.__int__()):
            if self.ui.edt_comanda.hasFocus():
                self.consulta_comanda()

    @log
    def key_button_voltar(self):
        if self.ui.btn_voltar.text() == 'Voltar':
            self.ui.stackedWidget.setCurrentIndex(stk_page)

    @log
    def key_button_fechar(self):
        if self.ui.btn_fechar.text() == 'Fechar':
            os.system('killall -9 python')
            sys.exit(-1)

    @log
    def consulta_comanda(self):
        codigo_comanda = self.ui.edt_comanda.text()
        self.clear_widgets()
        try:
            cd = int(codigo_comanda)

            comanda = SupervisorCdaCapa.query.filter("supcda_codigo_comanda = %s" % cd).order_by('id DESC').limit(1).all()

            if not comanda:
                return self.ui.labelItem.setText(u'Comanda não encontrada')

            if isinstance(comanda, list):
                comanda = comanda[0]

            comanda_id = int(comanda.id)

            restricao = SupervisorCdaCapa.get_by(id=comanda_id)

            if restricao:
                st = restricao.supcda_restricao
                if st == 0:
                    return self.itens_comanda(comanda_id)
                if st == 2:
                    return message_dlg(self, MSG_INFO,
                                       u'Comanda Nº %d já foi faturada.' % cd)
                if st == 4:
                    return message_dlg(self, MSG_INFO, u'Comanda Nº %d foi anulada.' % cd)
                if st == 5:
                    return message_dlg(self, MSG_INFO,
                                       u'Comanda Nº %d está sendo faturada no caixa.' % cd)
                if st == -1:
                    return message_dlg(self, MSG_INFO, u'Comanda Nº %d foi cancelada.' % cd)

        except ValueError:
            pass
        except Exception as err:
            log.error(str(err))

    @log
    def itens_comanda(self, comanda_id, ro=True):
        self.clear_widgets()
        self.ui.stackedWidget.setCurrentIndex(stk_page_2)
        self.comanda_capa = comanda = SupervisorCdaCapa.get_by(id=comanda_id)
        self.ui.editNumCda.setText('%d' % comanda.supcda_codigo_comanda)
        d = comanda.supcda_data_hora.timetuple()
        d = QtCore.QDateTime(d[0], d[1], d[2], d[3], d[4], d[5])
        self.ui.dateEmissaoCda.setDateTime(d)
        self.ui.comboStatusCda.setCurrentIndex(0)
        self.ui.spinTotalCda.setValue(comanda.supcda_total_comanda)
        self.ui.comboStatusCda.setCurrentIndex(comanda.supcda_restricao)
        self.update_table_item(comanda_id)
        self.ui.frame_esquerdo.setEnabled(ro)
        self._tempo_ocioso = 0

    @log
    def update_table_item(self, id_cda):
        self.clear_widget_table(self.ui.tableItens)
        corpos = SupervisorCdaCorpo.query.filter("supcdo_capa_id = %s" % id_cda).order_by('supcdo_sequencia_id')

        for c in corpos:
            item = SupervisorCadItens.get_by(supite_codigo_item=c.supcdo_item,
                                             supite_loja=self.loja.supcfc_loja)
            vend = SupervisorCadVendedor.get_by(supven_codigo=c.supcdo_cod_vendedor)
            icon3 = QtGui.QIcon()
            if c.supcdo_venda_cancelamento == 'V':
                icon3.addPixmap(
                    QtGui.QPixmap(os.path.join(root_path, "resources/16x16/check2.png")),
                    QtGui.QIcon.Normal, QtGui.QIcon.Off)
            else:
                icon3.addPixmap(
                    QtGui.QPixmap(os.path.join(root_path, "resources/16x16/delete.png")),
                    QtGui.QIcon.Normal, QtGui.QIcon.Off)

            table_item = self.ui.tableItens
            row = table_item.rowCount()
            table_item.insertRow(row)

            # Codigo Item
            item_check = QtGui.QTableWidgetItem('')
            item_check.setIcon(icon3)

            # Codigo Item
            item_cod = QtGui.QTableWidgetItem(c.supcdo_item)

            # Descrição
            item_name = item.name if config['descricao_resumida'] else item.supite_descricao_completa
            item_descr = QtGui.QTableWidgetItem(item_name)
            # size = QtCore.QSize(650, 400)
            # item_descr.setSizeHint(size)

            # Quantidade
            item_qtda = QtGui.QTableWidgetItem('%1.3f' % c.supcdo_quantidade)

            # Valor Unitário
            item_vuni = QtGui.QTableWidgetItem('R$ %1.2f' % item.supite_preco)

            # Total
            item_total = QtGui.QTableWidgetItem('R$ %1.2f' % c.supcdo_preco_total)

            # Vendedor
            if vend:
                item_vend = QtGui.QTableWidgetItem(vend.name)
            else:
                item_vend = QtGui.QTableWidgetItem(None)

            # WidGET OS NEW ITENS NA TABLE ITENS
            table_item.setItem(row, 0, item_check)
            table_item.setItem(row, 1, item_cod)
            table_item.setItem(row, 2, item_descr)
            table_item.setItem(row, 3, item_qtda)
            table_item.setItem(row, 4, item_vuni)
            table_item.setItem(row, 5, item_total)
            table_item.setItem(row, 6, item_vend)
            table_item.resizeColumnsToContents()
            table_item.scrollToBottom()

    @log
    def clear_widgets(self):
        self.clear_widget_table(self.ui.tableItens)
        self.ui.edt_comanda.clear()

    @log
    def clear_widget_table(self, table):
        # Apaga lista de itens
        rows_count = table.rowCount()
        # Verifica se hà dados
        if rows_count:
            rows = range(0, rows_count + 1)
            rows.reverse()
            for r in rows:
                table.removeRow(r)
        return rows_count


@log
def run():
    app = QtGui.QApplication(sys.argv)
    form = ConsultaRestForm()
    form.showMaximized()
    app.exec_()


if __name__ == "__main__":
    run()