__author__ = 'christian.douglas'

from datetime import datetime
from PySide import QtCore, QtGui
from decimal import Decimal
from sigpdv.models.cad_mesas_data import CaixaCadMesas
from sigpdv.models.cad_itens_data import CaixaCadItens
from sigpdv.models.cad_tributacao_data import CaixaCadTributacao
from sigpdv.models.cfg_parametros_data import CaixaCfgParametros
from sigpdv.models.mov_mesas_data import CaixaMovMesas
from sigpdv.models.mov_itens_mesa_data import CaixaMovItensMesa
from sigpdv.models.db_utils import sess
from sigpdv.ui.messagebox import YES_ROLE, MSG_QUESTION, MSG_INFO, message_dlg

from sigpdv.control.utils import rel_mesas_abertas, rel_gerencial_conf_mesas
from tools.config import config


TWOPLACES = Decimal(10) ** -2
TREPLACES = Decimal(10) ** -3


class ControleMesa():
    def __init__(self, parent=None):
        self.parent = parent
        self.estacionamento = False
        self.teclado = False
        if self.parent:
            self.atualizar_mesas()

    def _returnPressed_edtCodigoRes(self):
        cod = self.parent.ui.edtCodigoRes.text()
        if cod:
            num_mesa = self.parent.ui.tableMesa.item(self.parent.ui.tableMesa.currentRow(), 0)
            mov_mesa = CaixaMovMesas.get_by(supmes_status='A',
                                            supmes_num_mesa=int(num_mesa.text()))

            # TRANSFERENCIA DE MESA/ITENS DA MESA
            if self.parent._state_coupon in (
                    self.parent.mesa_transf, self.parent.itens_mesa_transf):
                if not CaixaCadMesas.get_by(Numero=int(cod)):
                    self.parent._state_coupon = self.atualizar_mesas()
                    self.parent.ui.edtCodigoRes.clear()
                    self.parent.ui.lblCodigoRest.setText(u'Codigo do Item:')
                    self.parent.ui.spinQtdaRes.setValue(1)
                    self.parent._tab_order = []
                    self.parent.setControlFocus(self.parent._CONTROL_FOCUS,
                                                self.parent.ui.tableMesa)
                    return message_dlg(self.parent,
                                       text='Mesa No. %03d nao existe!' % int(cod))
                if num_mesa.text() == cod:
                    self.parent._state_coupon = self.parent.coupon_closed
                    self.parent.ui.edtCodigoRes.clear()
                    self.parent.ui.lblCodigoRest.setText(u'Codigo do Item:')
                    self.parent.ui.spinQtdaRes.setValue(1)
                    return message_dlg(self.parent, MSG_INFO,
                                       'Mesa de Destino nao pode Ser igual a Mesa de Origem')
                else:
                    if self.parent._state_coupon == self.parent.mesa_transf:
                        self.parent.transf_mesa(int(num_mesa.text()), int(cod))
                    else:
                        id_movimento = self.parent.ui.tableItensMesa. \
                            item(self.parent.ui.tableItensMesa.currentRow(), 0)
                        self.parent.transf_item_mesa(int(cod), int(id_movimento.text()))
                    self.atualizar_mesas()
                self.parent.ui.edtCodigoRes.clear()
                self.parent.ui.lblCodigoRest.setText(u'Codigo do Item:')
                self.parent.ui.spinQtdaRes.setValue(1)
                self.parent._tab_order = []
                self.parent.setControlFocus(self.parent._CONTROL_FOCUS, self.parent.ui.tableMesa)
                self.parent._state_coupon = self.parent.coupon_closed
            else:
                frmcod = cod.rjust(14, '0')  # 14 zeros a esquerda
                item = CaixaCadItens.get_by(supite_codigo_item=frmcod,
                                            Loja=self.parent._loja.supcfc_loja,
                                            Ativo='A')
                if not item:
                    self.parent.ui.lblCodigoRest.setText(u'Codigo do Item:')
                    self.parent.ui.spinQtdaRes.setValue(1)
                    self.parent.ui.edtCodigoRes.clear()
                    return message_dlg(self, MSG_INFO, 'Codigo do produto nao cadastrado!')
                else:
                    sequencia = self.parent.ui.tableItensMesa.rowCount() + 1
                    if self.parent.ui.spinQtdaRes.value() > 0:
                        qtda = Decimal(str(self.parent.ui.spinQtdaRes.value()))
                    else:
                        qtda = Decimal(1)
                    if not mov_mesa:
                        if message_dlg(self, MSG_QUESTION, 'Deseja Efetuar a Abertura da Mesa %s?'
                                % num_mesa) == YES_ROLE:
                            sess.begin()
                            mov_mesa = CaixaMovMesas(
                                Loja=self.parent._loja.supcfc_loja,
                                Caixa=self.parent._caixa_id,
                                supmes_num_mesa=int(num_mesa.text())
                            )
                            sess.commit()
                            self.atualizar_mesas()
                        else:
                            self.parent.ui.lblCodigoRest.setText(u'Codigo do Item:')
                            self.parent.ui.spinQtdaRes.setValue(1)
                            self.parent.ui.edtCodigoRes.clear()
                            return

                    sess.begin()
                    CaixaMovItensMesa(
                        Loja=self.parent._loja.supcfc_loja,
                        Caixa=self.parent._caixa_id,
                        IdMovMesa=mov_mesa.Id,
                        Sequencia=sequencia,
                        Item=frmcod,
                        Descricao=item.name,
                        Quantidade=qtda,
                        Unidade=item.supite_unidade,
                        VrUnit=item.supite_preco,
                        VrTotal=item.supite_preco * qtda,
                        DescontoItem=0,
                        AcrescimoItem=0,
                        Tributacao=item.Tributacao
                    )
                    sess.commit()
                    self.up_itens_in_table_movitens(mov_mesa.Id)
                    self.parent.ui.edtCodigoRes.clear()
                    self.parent.ui.lblCodigoRest.setText(u'Codigo do Item:')
                    self.parent.ui.spinQtdaRes.setValue(1)
        else:
            message_dlg(self, MSG_INFO, 'Informe o codigo do produto!')
        return

    def _returnPressed_spinQtdaRes(self):
        value = Decimal(
            str(self.parent.ui.spinQtdaRes.value())) if self.parent.ui.spinQtdaRes.text() else 0
        min_max = CaixaCfgParametros.get_by()
        if min_max and value:
            msg = ''
            if value < min_max.supcfp_qtd_minima and not self.parent._state_item == self.parent.item_balanca \
                    and not self.parent._IdDav:
                msg = 'Qtda. digitada menor que a permitida!'
            elif value > min_max.supcfp_qtd_maxima and not self.parent._state_item == self.parent.item_balanca \
                    and not self.parent._IdDav:
                msg = 'Qtda. digitada maior que a permitida!'
            if msg and value:
                if message_dlg(self, MSG_QUESTION, msg + '\nDeseja tentar novamente?') == YES_ROLE:
                    self.parent.ui.spinQtdaRes.setValue(1)
                    self.parent.setControlFocus(self.parent._CONTROL_FOCUS,
                                                self.parent.ui.spinQtdaRes)
                    return
                else:
                    self.parent.ui.edtCodigoRes.clear()
                    self.parent.ui.lblCodigoRest.setText(u'Codigo do Item:')
                    self.parent.ui.spinQtdaRes.setValue(1)
                    self.parent.setControlFocus(self.parent._CONTROL_FOCUS,
                                                self.parent.ui.edtCodigoRes)
        if value <= 0:
            if message_dlg(self, MSG_QUESTION,
                           'Qtda. do Item nao pode ser menor ou igual a zero!'
                           '\nDeseja tentar novamente?') == YES_ROLE:
                self.parent.ui.spinQtdaRes.clear()
            else:
                self.parent.ui.spinQtdaRes.setValue(1)
                self.parent.ui.edtCodigoRes.clear()
                self.parent.ui.lblCodigoRest.setText(u'Codigo do Item:')
                self.parent.ui.spinQtdaRes.setValue(1)
                self.parent.setControlFocus(self.parent._CONTROL_FOCUS, self.parent.ui.edtCodigoRes)
        else:
            self.parent.setControlFocus(self.parent._CONTROL_FOCUS, self.parent.ui.edtCodigoRes)

        self.parent.check_button_status()
        return value

    def _returnPressed_tableItensMesa(self, row):
        id_movimento = self.parent.ui.tableItensMesa.item(row, 0)

        item =CaixaMovItensMesa.get_by(Id=int(id_movimento.text()))

        if item.supcor_venda_cancelamento in ('V', 'T'):
            if self.parent._state_coupon == self.parent.itens_mesa_transf:
                self.parent._tab_order = [self.parent.ui.tableMesa]
                self.parent.infoText('Informe o numero da mesa para qual deseja transferir o item')
                self.parent.ui.lblCodigoRest.setText('Informe o No. da Mesa de Destino')
                return self.parent.setControlFocus(self.parent._CONTROL_FOCUS,
                                                   self.parent.ui.edtCodigoRes)
            else:
                sess.begin()
                item.supcor_venda_cancelamento = 'C'
                sess.commit()
                self.up_itens_in_table_movitens(item.IdMovMesa)
                self.parent.infoText('O Item: [%s] %s cancelado!' % (item.Item, item.name))
        else:
            self.parent.infoText('O Item selecionado ja foi cancelado!')

        self.parent.ui.tableItensMesa.clearSelection()
        self.parent._tab_order = [self.parent.ui.tableMesa]
        self.parent.setControlFocus(self.parent._CONTROL_FOCUS, self.parent.ui.edtCodigoRes)
        self.parent._state_item = self.parent.item_none
        self.parent.ui.tableItensMesa.scrollToBottom()
        self.parent.check_button_status()
        return row

    def _returnPressed_tableMesa(self, row):
        self.parent.setControlFocus(self.parent._CONTROL_FOCUS, self.parent.ui.edtCodigoRes)
        self.parent.infoText('::: Informe o codigo do item para vender :::')

    def _cell_activated_tableMesa(self, row, b):
        try:
            num_caixa = self.parent.ui.tableMesa.item(row, 0).text()
        except Exception as e:
            return 0
        mov_mesa = CaixaMovMesas.get_by(supmes_status='A', supmes_num_mesa=int(num_caixa))
        if mov_mesa:
            self.parent.ui.btnKEY_AbrirMesaRes.setEnabled(False)
            self.parent.ui.btnKEY_FecharMesaRes.setEnabled(True)
            self.parent.ui.btnKEY_TransfMesaRes.setEnabled(
                True)  # Todo: Habilitar somente se houver itens
            self.parent.ui.btnKEY_ConferMesaRes.setEnabled(False)
            self.parent.ui.btnKEY_ConferMesaRes.setEnabled(True)
            self.parent._state_coupon = self.parent.mesa_opened
            self.up_itens_in_table_movitens(mov_mesa.Id)
            self.parent.ui.label_11.setText('<span style=" font-size:16pt; font-weight:600; '
                                            'color:#55aa00;">Aberta</span>')
        else:
            self.parent.clear_widget_table(self.parent.ui.tableItensMesa)
            self.parent.ui.btnKEY_AbrirMesaRes.setEnabled(True)
            self.parent.ui.btnKEY_FecharMesaRes.setEnabled(False)
            self.parent.ui.btnKEY_TransfMesaRes.setEnabled(False)
            self.parent.ui.btnKEY_TransfItensMesaRes.setEnabled(False)
            self.parent.ui.btnKEY_ConferMesaRes.setEnabled(False)
            self.parent._state_coupon = self.parent.mesa_closed
            self.parent.ui.lblVlrTotalMesa.setText('%1.2f' % 0)
            self.parent.ui.label_11.setText('<span style=" font-size:16pt; font-weight:600; '
                                            'color:#ff0000;">Fechada</span>')
        self.parent.ui.lblsupmes_num_mesa.setText('%03d' % int(num_caixa))
        self.parent.check_button_status()
        return mov_mesa

    def add_itens_in_table_movitens(self, p_cod_item, p_insert=True, p_id_mov_mesa=0):
        frmcod = p_cod_item.rjust(14, '0')  # 14 zeros a esquerda
        item = CaixaCadItens.get_by(supite_codigo_item=frmcod, Loja=self.parent._loja.supcfc_loja,
                                    Ativo='A')

        wdgt_table = self.parent.ui.tableItensMesa
        if item:
            # CODIGO DO ITEM
            i_item_cod = QtGui.QTableWidgetItem('%s' % str(item.supite_codigo_item))
            i_item_cod.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
            i_item_cod.setIcon(self.parent.icon_round_green)
            # DESCRICAO DO ITEM
            i_item_desc = QtGui.QTableWidgetItem('%s' % item.name)
            i_item_desc.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
            # DESCRICAO DO ITEM
            i_item_desc = QtGui.QTableWidgetItem('%s' % item.name)
            i_item_desc.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)

    def up_itens_in_table_movitens(self, p_id_mov_mesa):
        table = self.parent.ui.tableItensMesa
        self.parent.clear_widget_table(table)
        row = 0

        itens = CaixaMovItensMesa.query. \
            filter(CaixaMovItensMesa.IdMovMesa == p_id_mov_mesa). \
            order_by(CaixaMovItensMesa.DataHora.asc())
        total = []
        indicador_cancelamento = False
        for item in itens:
            status_item = '*'
            brush = QtGui.QBrush(QtCore.Qt.darkGray)
            icon = self.parent.icon_red
            if item.supcor_venda_cancelamento == 'C':
                status_item = 'Cancelado'
                brush = QtGui.QBrush(QtCore.Qt.darkRed)
                icon = self.parent.icon_delete
                indicador_cancelamento = True
            elif item.supcor_venda_cancelamento == 'V':
                status_item = 'Venda'
                brush = QtGui.QBrush(QtCore.Qt.darkGreen)
                icon = self.parent.icon_add
                total.append(item.VrTotal)
            elif item.supcor_venda_cancelamento == 'T':
                status_item = 'Transf. da Mesa %3d' % item.TransfMesa
                brush = QtGui.QBrush(QtCore.Qt.darkBlue)
                icon = self.parent.icon_replace
                total.append(item.VrTotal)

            # 0 - SEQUENCIA
            i_item_seq = QtGui.QTableWidgetItem('%s' % item.Id)
            i_item_seq.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
            i_item_seq.setForeground(QtCore.Qt.transparent)
            i_item_seq.setIcon(icon)
            # 1 - CODIGO DO ITEM
            i_item_cod = QtGui.QTableWidgetItem('%s' % item.Item)
            i_item_cod.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
            i_item_cod.setForeground(brush)
            # 2 - DESCRICAO DO ITEM
            i_item_desc = QtGui.QTableWidgetItem('%s' % item.name)
            i_item_desc.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
            i_item_desc.setForeground(brush)
            # 3 - UNIDADE DO ITEM
            i_item_un = QtGui.QTableWidgetItem('%s' % item.supite_unidade)
            i_item_un.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
            i_item_un.setTextAlignment(QtCore.Qt.AlignHCenter | QtCore.Qt.AlignVCenter)
            i_item_un.setForeground(brush)
            # 4 - QUANTIDADE DO ITEM
            i_item_qt = QtGui.QTableWidgetItem('%1.3f' % item.Quantidade)
            i_item_qt.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
            i_item_qt.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            i_item_qt.setForeground(brush)
            # 5 - VR UNIT DO ITEM
            i_item_vr_unit = QtGui.QTableWidgetItem('%1.2f' % item.VrUnit)
            i_item_vr_unit.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
            i_item_vr_unit.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            i_item_vr_unit.setForeground(brush)
            # 6 - VR UNIT DO ITEM
            i_item_vr_tot = QtGui.QTableWidgetItem('%1.2f' % item.VrTotal)
            i_item_vr_tot.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
            i_item_vr_tot.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            i_item_vr_tot.setForeground(brush)
            # 7 - SITUACAO TRIBUTARIA
            tributacao = CaixaCadTributacao.get_by(suptri_codigo_trib_ecf=item.Tributacao)
            aliquota = 0
            if tributacao:
                aliquota = tributacao.Aliquota
            i_item_vr_st = QtGui.QTableWidgetItem('%1.2f' % aliquota)
            i_item_vr_st.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
            i_item_vr_st.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            i_item_vr_st.setForeground(brush)
            # 8 - STATUS
            i_item_st = QtGui.QTableWidgetItem(status_item)
            i_item_st.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
            i_item_st.setForeground(brush)

            table.insertRow(row)
            table.setItem(row, 0, i_item_seq)
            table.setItem(row, 1, i_item_cod)
            table.setItem(row, 2, i_item_desc)
            table.setItem(row, 3, i_item_un)
            table.setItem(row, 4, i_item_qt)
            table.setItem(row, 5, i_item_vr_unit)
            table.setItem(row, 6, i_item_vr_tot)
            table.setItem(row, 7, i_item_vr_st)
            table.setItem(row, 8, i_item_st)
            row += 1
        table.scrollToBottom()
        table.resizeColumnsToContents()
        table.horizontalHeader().setStretchLastSection(True)
        if total:
            self.parent.ui.lblVlrTotalMesa.setText('%1.2f' % sum(total))
            self.parent.ui.btnKEY_TransfItensMesaRes.setEnabled(True)
            self.parent.ui.btnKEY_FecharMesaRes.setEnabled(True)
        else:
            self.parent.ui.lblVlrTotalMesa.setText('%1.2f' % 0)
            self.parent.ui.btnKEY_TransfItensMesaRes.setEnabled(False)
            self.parent.ui.btnKEY_FecharMesaRes.setEnabled(indicador_cancelamento)

        table.setColumnWidth(0, 32)
        return row

    def transf_item_mesa(self, p_num_mesa_destino, p_id_movimento):
        item_mov_mesa_origem = CaixaMovItensMesa.get_by(Id=p_id_movimento)
        mov_mesa_origem = CaixaMovMesas.get_by(Id=item_mov_mesa_origem.IdMovMesa)
        num_mesa = mov_mesa_origem.supmes_num_mesa

        mov_mesa_destino = CaixaMovMesas.get_by(supmes_status='A',
                                                      supmes_num_mesa=p_num_mesa_destino)

        if not mov_mesa_destino:
            sess.begin()
            mov_mesa_destino = CaixaMovMesas(
                Loja=self.parent._loja.supcfc_loja,
                Caixa=self.parent._caixa_id,
                supmes_num_mesa=int(p_num_mesa_destino)
            )
            sess.commit()

        sess.begin()
        item_mov_mesa_origem.IdMovMesa = mov_mesa_destino.Id
        item_mov_mesa_origem.DataHora = datetime.now()
        item_mov_mesa_origem.TransfMesa = num_mesa
        item_mov_mesa_origem.supcor_venda_cancelamento = 'T'
        sess.commit()

    def transf_mesa(self, p_num_mesa_origem, p_num_mesa_destino):
        # ORIGEM
        mesa_origem = CaixaCadMesas.get_by(Ativo='S', Numero=p_num_mesa_origem)
        mov_mesa_origem = CaixaMovMesas.get_by(supmes_status='A',
                                                     supmes_num_mesa=p_num_mesa_origem)
        itens_mov_mesa_origem = CaixaMovItensMesa.query.filter(
            CaixaMovItensMesa.IdMovMesa == mov_mesa_origem.Id
        )
        if not itens_mov_mesa_origem:
            raise Exception('Nao ha itens na mesa de origem para efetuar a transferencia')
        # DESTINO
        mesa_destino = CaixaCadMesas.get_by(Numero=p_num_mesa_destino)
        mov_mesa_destino = CaixaMovMesas.get_by(supmes_status='A',
                                                      supmes_num_mesa=p_num_mesa_destino)
        if not mov_mesa_destino:
            sess.begin()
            mov_mesa_destino = CaixaMovMesas(
                Loja=self.parent._loja.supcfc_loja,
                Caixa=self.parent._caixa_id,
                supmes_num_mesa=p_num_mesa_destino
            )
            sess.commit()
        # TRANSFERIR ITENS
        sess.begin()
        for item in itens_mov_mesa_origem:
            item.IdMovMesa = mov_mesa_destino.Id
            item.DataHora = datetime.now()
            if item.supcor_venda_cancelamento != 'C':
                item.TransfMesa = p_num_mesa_origem
                item.supcor_venda_cancelamento = 'T'
        mov_mesa_origem.TransfMesa = p_num_mesa_destino
        mov_mesa_origem.DataHoraFechamento = datetime.now()
        mov_mesa_origem.Status = 'F'
        sess.commit()
        return mov_mesa_destino

    def KEY_TransfItensMesaRes_Click(self):
        self.parent.infoText('Selecione o item para transferir e pressione enter')
        self.parent._state_coupon = self.parent.itens_mesa_transf
        self.parent._tab_order = []
        self.parent.ui.tableItensMesa.selectRow(0)
        self.parent.setControlFocus(self.parent._CONTROL_FOCUS, self.parent.ui.tableItensMesa)

    def KEY_TransfMesaRes_Click(self):
        self.parent.infoText('Informe o No. da Mesa de Destino')
        self.parent.ui.lblCodigoRest.setText('Informe o No. da Mesa de Destino')
        self.parent._state_coupon = self.parent.mesa_transf
        self.parent.setControlFocus(self.parent._CONTROL_FOCUS, self.parent.ui.edtCodigoRes)

    def KEY_ConferMesaRes_Click(self):
        num_mesa = self.parent.ui.tableMesa.item(self.parent.ui.tableMesa.currentRow(), 0)
        mov_mesa = CaixaMovMesas.get_by(supmes_status='A',
                                              supmes_num_mesa=int(num_mesa.text()))
        text = rel_gerencial_conf_mesas(mov_mesa.Id)
        self.parent.infoText('Imprimindo Gerencial')
        self.parent.send_command("gerencial_report_open(config['confe_mesas'])")
        self.parent.send_command("gerencial_report_print(%r)" % text)
        self.parent.send_command("gerencial_report_close()")
        capa = self.parent.gera_capa_rg(tipo='G')
        sess.begin()
        mov_mesa.NroCupom = capa.NroCupom
        sess.commit()

    def _returnPressed_cbMFMesas(self):
        num_mesa = self.parent.ui.cbMFMesas.currentText()
        mov_mesa = CaixaMovMesas.get_by(supmes_status='A', supmes_num_mesa=int(num_mesa))
        text = rel_gerencial_conf_mesas(mov_mesa.Id)
        self.parent.infoText('Imprimindo Gerencial')
        self.parent.send_command("gerencial_report_open(config['confe_mesas'])")
        self.parent.send_command("gerencial_report_print(%r)" % text)
        self.parent.send_command("gerencial_report_close()")
        capa = self.parent.gera_capa_rg(tipo='G')
        sess.begin()
        mov_mesa.NroCupom = capa.NroCupom
        sess.commit()
        self.parent.KEY_ESCAPE_Click()

    def KEY_AbrirMesaRes_Click(self):
        try:
            num_mesa = self.parent.ui.tableMesa.item(self.parent.ui.tableMesa.currentRow(), 0)
            # mov_mesa = CaixaMovMesas.get_by(supmes_status='A', supmes_num_mesa=int(num_mesa.text()))
            sess.begin()
            CaixaMovMesas(
                Loja=self.parent._loja.supcfc_loja,
                Caixa=self.parent._caixa_id,
                supmes_num_mesa=int(num_mesa.text())
            )
            sess.commit()
            self.atualizar_mesas()
            self.parent.setControlFocus(self.parent._CONTROL_FOCUS, self.parent.ui.edtCodigoRes)
            self.parent.infoText('::: Mesa %s Aberta :::' % num_mesa)
        except Exception as e:
            self.parent.tratar_except()

    def KEY_FecharMesaRes_Click(self):
        num_mesa = self.parent.ui.tableMesa.item(self.parent.ui.tableMesa.currentRow(), 0)
        mov_mesa = CaixaMovMesas.get_by(supmes_status='A',
                                              supmes_num_mesa=int(num_mesa.text()))
        itens_mov_mesa = CaixaMovItensMesa.query.filter(
            CaixaMovItensMesa.IdMovMesa == mov_mesa.Id
        ).all()
        if not itens_mov_mesa:
            return message_dlg(self, text='Nao ha itens lancado na Mesa No. %03d'
                                          % int(num_mesa.text()))
        else:
            if not message_dlg(self, MSG_QUESTION, text='Deseja fechar a Mesa No. %03d'
                    % int(num_mesa.text())) == YES_ROLE:
                return

        self.parent._IdMesa = mov_mesa
        if mov_mesa.Cliente:
            self.parent._search_cliente(mov_mesa.Cliente, False)

        self.parent._set_screen_state(self.parent.in_coupon)

        def add_item(item):
            self.parent.setControlFocus(self.parent._CONTROL_FOCUS, self.parent.ui.edtCodigo)
            self.parent.ui.edtCodigo.setText(item)
            QtGui.QApplication.processEvents()
            return self.parent.KEY_RETURN_Click()

        total = []
        for x in itens_mov_mesa:
            QtGui.QApplication.processEvents()
            if x.Quantidade > 1:
                self.parent.ui.spinQtda.setValue(x.Quantidade)
                add_item(x.Item)
                self.parent.KEY_RETURN_Click()
            else:
                add_item(x.Item)
            if x.supcor_venda_cancelamento == 'C':
                self.parent.KEY_CANCEL_ITENS_Click()
                self.parent.KEY_RETURN_Click()
            else:
                total.append(x.VrTotal)

        try:
            value = Decimal(str(sum(total))).quantize(TWOPLACES)

            totalize = self.parent.send_command('totalize()')
            self.parent._coupon_totalize = Decimal(totalize)
            self.parent._state_coupon = self.parent.coupon_finish
            self.parent.infoText('Cupom totalizado! Efetue o pagamento.')
            self.parent.ui.lblVlrReceber.setText(u'%.2f' % self.parent._coupon_amount)
            self.parent.ui.lblVlrTotal.setText(u'%.2f' % self.parent._coupon_amount)
            self.parent.ui.lblVlrTotalR.setText(u'%.2f' % self.parent._coupon_amount)
            self.parent.check_button_status()
            self.parent.setControlFocus(self.parent._CONTROL_FOCUS, self.parent.ui.stackedActions)
            if not totalize:
                sess.begin()
                self.parent._IdMesa.DataHoraFechamento = datetime.now()
                self.parent._IdMesa.Status = 'F'
                sess.commit()
                self.parent.show_info_proccess(False)
                self.parent._payment_receipt_g = []
                self.parent._payment_receipt = []
                self.parent._libera_restricao = False
                self._libera_desconto = False
                self.parent._clear_globals(self.parent.coupon_closed)
                self.parent._set_controls_default_values()
                self.parent._set_screen_state(self.parent.in_restaurante)
        except Exception:
            self.parent.tratar_except()
            self.parent._coupon_totalize = False
            return
        self.parent.show_info_proccess(False)

    def KEY_VenderItemRes_Click(self):
        self.parent.setControlFocus(self.parent._CONTROL_FOCUS, self.parent.ui.edtCodigoRes)
        self.parent.infoText('::: Informe o codigo do item para vender :::')

    def verificar_mesas_abertas(self, imprimir_gerencial=False):
        qry_mov_mesas = CaixaMovMesas.query. \
            filter(CaixaMovMesas.supmes_status == 'A').all()
        if qry_mov_mesas and imprimir_gerencial and config['ativar_restaurante']:
            text = rel_mesas_abertas()
            self.parent.infoText('Imprimindo Gerencial')
            self.parent.send_command("gerencial_report_open(config['mesas_abertas'])")
            self.parent.send_command("gerencial_report_print(%r)" % text)
            self.parent.send_command("gerencial_report_close()")
            self.parent.gera_capa_rg(tipo='G')
        return qry_mov_mesas

    def atualizar_mesas(self):
        qry_mesas = CaixaCadMesas.query. \
            filter(CaixaCadMesas.supmes_loja == self.parent._loja.supcfc_loja). \
            filter(CaixaCadMesas.active == 't').all()
        table = self.parent.ui.tableMesa
        current_row = table.currentRow()

        self.parent.clear_widget_table(table)

        row = 0

        for mesa in qry_mesas:
            mov_mesa = CaixaMovMesas.get_by(supmes_status='A', supmes_num_mesa=mesa.Numero)
            data_mov = 'Fechada'
            icon = self.parent.icon_media_stop_red

            table.insertRow(row)

            if mov_mesa:
                icon = self.parent.icon_media_play_green
                data_mov = mov_mesa.supcax_data_hora_abertura

            # Numero da Mesa
            iMesa = QtGui.QTableWidgetItem('%d' % mesa.Numero)
            iMesa.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
            iMesa.setIcon(icon)

            # Data/Hora Abertura
            iMesa2 = QtGui.QTableWidgetItem('%s' % data_mov)
            iMesa2.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)

            table.setItem(row, 0, iMesa)
            table.setItem(row, 1, iMesa2)
            row += 1
        table.resizeColumnsToContents()
        table.horizontalHeader().setStretchLastSection(True)
        if current_row < 0:
            table.selectRow(0)
        else:
            table.selectRow(current_row)
        return row

