#!/usr/bin/python
# -*- coding: utf-8 -*-

from datetime import datetime
from sigpdv.tools.log_restaurante import traced as log
from sigpdv.control.utils import remove_accents, converte_tz
from sigpdv.models.db_utils import b_sess, exec_sql
from sigpdv.models.cad_comandos_impressoras_data import CaixaCadComandosImpressoras
from sigpdv.tools.config import config
import os
from sigpdv.models.supervisor import SupervisorCadItens, SupervisorCdaCapa, SupervisorCdaCorpo, \
    SupervisorCfgConfiguracao
import subprocess
import time

class ImprimirComanda(object):

    def __init__(self, parent):
        self.parent = parent
        self.set_impressoras()

    def set_impressoras(self):
        self.impressoras = CaixaCadComandosImpressoras.query.all()


    def _get_realacao_locais(self):
        locais = {}
        map(lambda x: locais.update({x.get('sucloc_codigo'): x.get('sucloc_impressora')}),
            exec_sql("""SELECT sucloc_impressora, sucloc_codigo 
                          FROM supervisor_producao_local
                          WHERE sucloc_impressora IS NOT NULL """, 'S', ret=True)
            )
        return locais

    def imprimir_pedidos(self, impressoras):
        path = '/tmp/pedido_cozinha.txt'

        if not self.impressoras:
            self.set_impressoras()


        cmd_impressora = False
        for i in self.impressoras:
            marca , modelo = config['impressoras'].split(' | ')
            if i.supcci_marca == marca and i.supcci_modelo == modelo:
                cmd_impressora = i
                break

        for impressora, impressoes in impressoras.items():
            try:
                self.parent.obj_impressoras.supcci_imprimir('\n'.join(impressoes))
                file = open(path, 'w')
                file.write('\n'.join(impressoes))
                file.close()
                # if cmd_impressora.supcci_marca.upper() in ('ELGIN', 'EPSON'):
                #     os.system("echo %s | sudo -S chmod 776 %s; cat %s > %s" %
                #               (config['senha_usuario'] or 'sig',
                #                config['porta_impressora'],
                #                path,
                #                config['porta_impressora']))
                # else:
                #     subprocess.call(["lp", "-o raw", "-n1", '-d%s' % config['impressora'], path])
            except Exception as e:
                log.error('Erro ao imprimir pedido de cozinha %s' % e)

            time.sleep(1)

    def _get_impressora_impressao(self, dados_corpo, dict_itens, dict_capa):
        """Retorna um  dict com a chave sendo a impressora e o valor as impressoes"""
        remove_acentuacao = remove_accents
        dict_locais = self._get_realacao_locais()
        impressoras = {}
        if not dict_locais:
            return impressoras

        if not self.impressoras:
            self.set_impressoras()

        nome_empresa = remove_accents(SupervisorCfgConfiguracao.get_by().supcfc_cliente or '')
        data_sem_fuso = datetime.now()
        data = datetime.strftime(data_sem_fuso, '%d/%m/%Y %H:%M:%S')[:19]
        ultima_capa_id = False
        cabecalho = ['{b1}%s{b2}' % ((nome_empresa).center(48)),
                     ('DATA IMPRESSAO: %s' % data).rjust(48),
                     '',
                     '{b1}--------------* PEDIDO DE COZINHA *-------------{b2}',
                     '']
        rodape = ['DESENVOLVIDO POR:',
                  '{b1}Sig Solucoes em Informatica Ltda. (31)2106-1700{b2}',
                  '',
        ]
        impressao_comanda = []
        ultima_printer = False
        for corpo in dados_corpo:
            # itens cancelados nao imprime
            if corpo.supcdo_venda_cancelamento == 'C':
                b_sess.begin()
                corpo.supcdo_imprimir = '1'
                b_sess.commit()
                continue

            create_date = converte_tz(self, (datetime.strptime(('%s' % (corpo.create_date
                                            or ('%s' % data_sem_fuso)))[:19], '%Y-%m-%d %H:%M:%S')),
                     tz_origem='UTC', tz_destino='America/Sao_Paulo').strftime('%d/%m/%Y %H:%M:%S')
            item = dict_itens.get(corpo.supcdo_item)
            printer = False
            if item.supite_codigo_local and dict_locais.get(item.supite_codigo_local):
                printer = dict_locais.get(item.supite_codigo_local)

            nome_cliente = remove_acentuacao(dict_capa.get(corpo.supcdo_capa_id)[1] or '')
            comanda = dict_capa.get(corpo.supcdo_capa_id)[2]
            garcom = remove_acentuacao(corpo.supcdo_nome_vendedor or '')
            mesa = dict_capa.get(corpo.supcdo_capa_id)[3]
            seq_item = corpo.supcdo_sequencia_id
            descricao = remove_acentuacao(item.supite_descricao_completa)
            qtd = corpo.supcdo_quantidade
            if len(descricao) > 42:
                descricao = '\n      '.join([descricao[:42], descricao[42:]])

            obs = remove_acentuacao(corpo.supcdo_observacao or '')
            if len(obs) > 48:
                obs = '\n'.join([obs[:48], obs[48:]])

            seq_impressao = (corpo.supcdo_seq_impressao or 0) + 1
            if ultima_capa_id != corpo.supcdo_capa_id:
                if impressao_comanda:
                    impressao_comanda.extend(rodape)
                    if ultima_printer:
                        if not impressoras.get(ultima_printer):
                            impressoras[ultima_printer] = ['\n'.join(impressao_comanda)]
                        else:
                            impressoras[ultima_printer].append('\n'.join(impressao_comanda))
                    else:
                        log.error('Impressora nao cadastrada no local do item: %s' %
                                  item.supite_codigo_item)


                ultima_capa_id = corpo.supcdo_capa_id
                impressao_comanda = []
                impressao_comanda.extend(cabecalho)
                if seq_impressao > 1:
                    impressao_comanda.append('{f1} Reimpressao de pedido{f2}')

                if nome_cliente:
                    impressao_comanda.append('CLIENTE: %s' % nome_cliente)

                impressao_comanda.extend([
                             'GARCOM: {b1}%s{b2}' % (garcom),
                             'COMANDA: {b1}%06d{b2}' % (comanda),
                             'MESA: {b1}%03d{b2}' % (mesa),
                             'DATA: %s' % create_date,
                             '',
                             '{b1}------------------------------------------------{b2}',
                             ])

            impressao_comanda.extend([
                             'Seq.Item: {b1}%03d{b2}' % (seq_item),
                             'SEQ.IMPRESSAO: %s' % seq_impressao,
                             'ITEM: {f1}%s{f2}' % (descricao),
                             'QTD: {f1}{b1}%s{b1}{f2}' % (qtd),
                             '',
                             'OBS:\n{f1}*%s\n{f2}' % (obs),
                             '',])
            ultima_printer = printer
            if not printer:
                continue

            b_sess.begin()
            data = converte_tz(self, (datetime.strptime(data, '%d/%m/%Y %H:%M:%S')),
                     tz_origem='America/Sao_Paulo', tz_destino='UTC').strftime('%d/%m/%Y %H:%M:%S')
            corpo.supcdo_seq_impressao = seq_impressao
            corpo.supcdo_data_hora_impressao = data
            corpo.supcdo_imprimir = '4'
            b_sess.commit()
            
        if impressao_comanda:
            impressao_comanda.extend(rodape)
            if ultima_printer:
                if not impressoras.get(ultima_printer):
                    impressoras[ultima_printer] = ['\n'.join(impressao_comanda)]
                else:
                    impressoras[ultima_printer].append('\n'.join(impressao_comanda))
            else:
                log.error('Impressora nao cadastrada no local do item: %s' %
                          item.supite_codigo_item)

        return impressoras

    def imprimir_pedidos_cozinha(self):
        dados_corpo = SupervisorCdaCorpo.query.filter(
            SupervisorCdaCorpo.supcdo_capa_id == self.parent._pve.id). \
            filter(SupervisorCdaCorpo.supcdo_imprimir == '3').order_by('id')

        if not dados_corpo.all():
            return True

        codigos = []
        capa_ids = []
        for i in dados_corpo:
            codigos.append(i.supcdo_item)
            capa_ids.append(i.supcdo_capa_id)

        # ------------ pega a relacao dos itens ---------------
        dados_item = SupervisorCadItens.query.filter(
                                                SupervisorCadItens.supite_codigo_item.in_(codigos))

        dict_itens = {}
        map(lambda x: dict_itens.update( {x.supite_codigo_item: x}), dados_item)

        # ------------ pega a relacao das capas ---------------
        dict_capa = {}
        
        for i in SupervisorCdaCapa.query.filter(SupervisorCdaCapa.id.in_(capa_ids)):
            cliente = i.supcda_nome_cliente or ''
            comanda = i.supcda_codigo_comanda
            mesa = i.supcda_mesa or 0
            chave = '%s-%s' % (i.supcda_loja, i.supcda_caixa)
            dict_capa.update({i.id: (chave, cliente, comanda, mesa)})

        impressoras = self._get_impressora_impressao(dados_corpo, dict_itens, dict_capa)
        if not impressoras:
            log.error('Nenhuma impressora cadastrada nos locais')
            return True

        self.imprimir_pedidos(impressoras)
        return True