#!/usr/bin/python
# -*- encoding: utf-8 -*-

__author__ = 'Dener Rodrigues'

from kiwi.log import set_log_file, Logger
import logging
import os
import sys
from config import LOG_FILE_EXPORTAR
from datetime import datetime


BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE, _NOTHING, DEFAULT = range(10)
# The background is set with 40 plus the number of the color, and the foreground with 30
# These are the sequences need to get colored ouput
RESET_SEQ = "\033[0m"
COLOR_SEQ = "\033[1;%dm"
BOLD_SEQ = "\033[1m"
COLOR_PATTERN = "%s%s%%s%s: " % (COLOR_SEQ, COLOR_SEQ, RESET_SEQ)
LEVEL_COLOR_MAPPING = {
    logging.DEBUG: (BLUE, DEFAULT),
    logging.INFO: (GREEN, DEFAULT),
    logging.WARNING: (YELLOW, DEFAULT),
    logging.ERROR: (RED, DEFAULT),
    logging.CRITICAL: (WHITE, RED),
}

LEVEL_NAMES = {
    logging.DEBUG: 'DEBUG',
    logging.INFO: 'INFO',
    logging.WARNING: 'WARNING',
    logging.ERROR: 'ERROR',
    logging.CRITICAL: 'CRITICAL',
}
global LOG
LOG = Logger('sigpdv')


def _(string):
    if type(string) == unicode:
        string = string.encode('utf-8', 'replace')
    return string


class Singleton:
    """
    A non-thread-safe helper class to ease implementing singletons.
    This should be used as a decorator -- not a metaclass -- to the
    class that should be a singleton.

    The decorated class can define one `__init__` function that
    takes only the `self` argument. Other than that, there are
    no restrictions that apply to the decorated class.

    To get the singleton instance, use the `Instance` method. Trying
    to use `__call__` will result in a `TypeError` being raised.

    Limitations: The decorated class cannot be inherited from.

    """

    def __init__(self, decorated):
        self._decorated = decorated

    def Instance(self):
        """
        Returns the singleton instance. Upon its first call, it creates a
        new instance of the decorated class and calls its `__init__` method.
        On all subsequent calls, the already created instance is returned.

        """
        try:
            return self._instance
        except AttributeError:
            self._instance = self._decorated()
            return self._instance

    def __call__(self):
        raise TypeError('Singletons must be accessed through `Instance()`.')

    def __instancecheck__(self, inst):
        return isinstance(inst, self._decorated)


class SigLog():
    def __init__(self, stdout=True, log=LOG):
        set_log_file(LOG_FILE_EXPORTAR)
        self._log_name = 'exportar'
        self._log = log

        self._stdout = stdout

    def stdout_(self, level, text):
        fg_color, bg_color = LEVEL_COLOR_MAPPING[level]
        text_format = '[%s] %s %s ' % (
        os.getpid(), datetime.now().strftime('%H:%M:%S,%f'), self._log_name[:10].ljust(10, ' '))
        text_format += COLOR_PATTERN % (30 + fg_color, 40 + bg_color, LEVEL_NAMES[level].ljust(8))
        sys.stdout.write(_(text_format) + _(text) + '\n')
        return text_format

    def debug(self, text):
        self._log.debug(text)
        if self._stdout:
            self.stdout_(logging.DEBUG, text)

    def info(self, text):
        self._log.info(text)
        if self._stdout:
            self.stdout_(logging.INFO, text)

    def warning(self, text):
        self._log.warning(text)
        if self._stdout:
            self.stdout_(logging.WARNING, text)

    def error(self, text):
        self._log.error(text)
        if self._stdout:
            self.stdout_(logging.ERROR, text)

    def critical(self, text):
        self._log.critical(text)
        if self._stdout:
            self.stdout_(logging.CRITICAL, text)

    def set_logger(self, name):
        self._log_name = name
        self._log = Logger(name)


if __name__ == '__main__':
    l = SigLog()
    l.debug('Debug')
    l.set_logger(__name__)
    l.info('Info')
    l.warning('Warning')
    l.error('Error')
    import traceback

    try:
        print '>', 10 / 0
    except Exception as e:
        a = traceback.format_exc()
        l.critical('Critical')
