import os
import tempfile

from sigpdv.tools.log import traced
from sigpdv.ui.messagebox import message_dlg, MSG_INFO


class SingleInstance:
    def __init__(self, aplicacao):
        import sys

        self.lockfile = os.path.normpath(tempfile.gettempdir() + '/' + '%s'.replace("/", "-")
                                         .replace(":", "").replace("\\", "-") % aplicacao + '.lock')
        traced.debug("SingleInstance lockfile: " + self.lockfile)
        self.teste = False
        if sys.platform == 'win32':
            try:
                # file already exists, we try to remove (in case previous execution was interrupted)
                if os.path.exists(self.lockfile):
                    os.unlink(self.lockfile)
                self.fd = os.open(self.lockfile, os.O_CREAT | os.O_EXCL | os.O_RDWR)
            except OSError as e:
                if e.errno == 13:
                    msg = u"Ja existe outra instancia do %s em execucao." % aplicacao
                    # log.error(msg)
                    message_dlg(None, MSG_INFO, msg)
                    sys.exit(-1)
                traced.error(e.errno)
                raise
        else:  # non Windows
            import fcntl, sys

            self.fp = open(self.lockfile, 'w')

            self.fpid = open('%s.pid' % self.lockfile, 'w')
            self.fpid.write(str(os.getpid()))
            self.fpid.close()
            try:
                fcntl.lockf(self.fp, fcntl.LOCK_EX | fcntl.LOCK_NB)
            except IOError:
                msg = "Ja existe outra instancia do %s em execucao." % aplicacao
                traced.warning(msg)
                message_dlg(None, MSG_INFO, msg)
                sys.exit(-1)

    def __del__(self):
        if self.teste:
            try:
                traced.debug('Apagando arquivo temporario: %r' % self.lockfile)
                os.unlink(self.lockfile)
                traced.debug('Apagando arquivo temporario: %r' % '%s.pid' % self.lockfile)
                os.unlink('%s.pid' % self.lockfile)
            except Exception as e:
                traced.error(str(e))
