#!/usr/bin/env python
# -*- Mode: Python; coding: utf-8 -*-

import sys
from PySide import QtCore, QtGui
from sigpdv.models.supervisor import SupervisorCadItens
from sigpdv.tools.config import config
from busca_produto_ui import Ui_Dialog


class BuscaProduto(QtGui.QDialog):
    def __init__(self, parent=None):
        super(BuscaProduto, self).__init__()

        self.ui = Ui_Dialog()

        self.ui.setupUi(self)

        self.parent = parent
        self.retorno = False
        # SIGNALS
        self.ui.buttonBuscarProduto.pressed.connect(self._button_buscar_produto_click)
        self.ui.tableBuscaProduto.cellClicked.connect(self.table_busca_produto_click)
        # AJUSTES
        self.ui.tableBuscaProduto.resizeColumnsToContents()
        self.ui.editBuscaProduto.setFocus()

    def accept(self):
        if self.ui.tableBuscaProduto.selectedItems():
            cod_item = self.ui.tableBuscaProduto.item(self.ui.tableBuscaProduto.currentRow(), 0).text()
            self.retorno = SupervisorCadItens.get_by(supite_codigo_item=cod_item.rjust(14, '0'),
                                                     supite_loja=self.parent.loja.supcfc_loja)
            super(BuscaProduto, self).accept()

    def _button_buscar_produto_click(self):
        texto_busca = self.ui.editBuscaProduto.text()
        if texto_busca:
            if self.ui.comboFiltroBusca.currentIndex() == 0:  # Busca por Código do Produto
                cod = texto_busca.rjust(14, '0')
                res = SupervisorCadItens.query.filter(
                    SupervisorCadItens.supite_codigo_item == cod).filter(
                    SupervisorCadItens.active == 'T')
            else:
                flike = SupervisorCadItens.name.\
                    ilike('%s%s%s' % ('%', texto_busca, '%'))
                res = SupervisorCadItens.query.filter(flike).filter(
                    SupervisorCadItens.active == 'T').order_by(SupervisorCadItens.name).limit(20)
            if res and res.count() > 0:
                self._update_table_busca(res)
                self.ui.tableBuscaProduto.setFocus()
            else:
                QtGui.QMessageBox.information(None, u'Pesquisa produto.', u'Item não cadastrado!',
                                              QtGui.QMessageBox.Ok)
                self.ui.editBuscaProduto.clear()
                self.ui.editBuscaProduto.setFocus()
        else:
            QtGui.QMessageBox.information(None, u'Pesquisa produto.',
                                          u'Digite algum texto para efetuar a busca.',
                                          QtGui.QMessageBox.Ok)
            self.ui.editBuscaProduto.setFocus()

    def _update_table_busca(self, itens):
        tableBuscaProduto = self.ui.tableBuscaProduto
        # Apaga lista de itens
        rows_count = tableBuscaProduto.rowCount()
        # Verifica se hà dados
        if rows_count:
            rows = range(0, rows_count + 1)
            rows.reverse()
            for r in rows:
                tableBuscaProduto.removeRow(r)
        rows = 0
        for produto in itens:
            tableBuscaProduto.insertRow(rows)
            # Codigo Item
            newItem = QtGui.QTableWidgetItem(produto.supite_codigo_item)
            # Descrição
            item_name = produto.name if config['descricao_resumida'] else produto.supite_descricao_completa
            newDesc = QtGui.QTableWidgetItem(item_name)
            # newDesc.setFlags(QtCore.Qt.ItemIsEnabled)
            # Unidade
            newUnid = QtGui.QTableWidgetItem(produto.supite_unidade)
            newUnid.setTextAlignment(QtCore.Qt.AlignHCenter | QtCore.Qt.AlignVCenter)
            # Valor Unitário
            newVUni = QtGui.QTableWidgetItem('%1.2f' % produto.supite_preco)
            newVUni.setTextAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            # WIDGET OS NEW ITENS NA TABLE ITENS
            tableBuscaProduto.setItem(rows, 0, newItem)
            tableBuscaProduto.setItem(rows, 1, newDesc)
            tableBuscaProduto.setItem(rows, 2, newUnid)
            tableBuscaProduto.setItem(rows, 3, newVUni)
            rows += 1
        self.ui.tableBuscaProduto.resizeColumnsToContents()

    def table_busca_produto_click(self, row):
        print row


if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    form = BuscaProduto()
    form.show()
    sys.exit(app.exec_())
