# -*- coding: utf-8 -*-
__author__ = 'christian.douglas'

from PySide import QtGui, QtCore
from info_dialog_ui import Ui_Dialog
import sys
import os
import io
import segno

ROOT_PATH = os.path.dirname(os.path.abspath(__file__))


class InfoDialog(QtGui.QDialog):
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent, QtCore.Qt.Dialog | QtCore.Qt.FramelessWindowHint)

        self.ui = Ui_Dialog()
        self.ui.setupUi(self)
        self.setAttribute(QtCore.Qt.WA_TranslucentBackground)
        self.setModal(True)
        self._done = False
        self._done = False
        self.parent = parent
        self._not_close = False
        self.parent._close_window = False
        self.ui.buttonBox.setVisible(False)
        self.ui.buttonBox.clicked.connect(self._button_click)
        self.ui.lbl_qrcode.setText('')
        self.ui.lbl_qrcode.setVisible(False)

    def _button_click(self, b):
        self.parent.cancel_pinpad = True
        return True


    def set_caption(self, text):
        self.ui.label_caption.setText(text)
        return text

    def set_info_text(self, text):
        self.ui.label_info.setText(text)
        if self.parent.btn_cancel_info_dialog:
            self.ui.buttonBox.setVisible(True)
            self.ui.buttonBox.setFocus()
            self.parent.btn_cancel_info_dialog = False
        else:
            self.ui.buttonBox.setVisible(False)

        if 'QRCODE' in text:
            out = io.BytesIO()
            # Save PNG into "out"
            self.ui.label_info.setText('Aguardando leitura do QRCode...')
            segno.make(text.replace('QRCODE', '')).save(out, kind='png', scale=2)
            out.seek(0)
            # Read PNG from "out"
            qim = QtGui.QImage.fromData(out.getvalue())
            qr_pixmap = QtGui.QPixmap.fromImage(qim)
            self.ui.lbl_qrcode.setVisible(True)
            self.ui.lbl_qrcode.setPixmap(qr_pixmap)
            self.ui.lbl_qrcode.show()
            self.ui.lbl_qrcode.raise_()
        else:
            self.ui.lbl_qrcode.clear()
            self.ui.lbl_qrcode.setVisible(False)

        return text

    def done(self, ev):
        if self._not_close:
            pass
        else:
            self.parent._close_window = True
            self.parent.activateWindow()
            return super(InfoDialog, self).done(ev)


    def keyPressEvent(self, ev):
        if not ev.key():
            return

#def message_dlg(parent=None, type=MSG_INFO, text='', detail='', file=None):
#    msg_box = MessageDialog(parent=parent)
#    msg_box.set_type(type)
#    if type == MSG_ERROR and not detail:
#        msg_box.set_info_text(str(text).decode('utf-8'))
#        if len(text.args) > 1:
#            msg_box.set_detail(text.args[1].decode('utf-8'))
#    else:
#        msg_box.set_info_text(text)
#    if detail:
#        msg_box.set_detail(detail)
#    if file:
#        msg_box.set_file(file)
#    msg_box.exec_()
#    return msg_box.retval
#
if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)

    InfoDialog().exec_()
