#!/usr/bin/env python
# -*- Mode: Python; coding: utf-8 -*-
from sqlalchemy.exc import OperationalError

import sys
from PySide import QtCore, QtGui
from ui_login_sig_pdv import Ui_Dialog
from messagebox import message_dlg, MSG_INFO, MSG_ERROR
from sigpdv.tools.config import config
from sigpdv.models.supervisor import SupervisorCadOperadores

K1 = ord('1') if not config['usarcodificado'] else config['bt_num_1']
K2 = ord('2') if not config['usarcodificado'] else config['bt_num_2']
K3 = ord('3') if not config['usarcodificado'] else config['bt_num_3']
K4 = ord('4') if not config['usarcodificado'] else config['bt_num_4']
K5 = ord('5') if not config['usarcodificado'] else config['bt_num_5']
K6 = ord('6') if not config['usarcodificado'] else config['bt_num_6']
K7 = ord('7') if not config['usarcodificado'] else config['bt_num_7']
K8 = ord('8') if not config['usarcodificado'] else config['bt_num_8']
K8 = ord('8') if not config['usarcodificado'] else config['bt_num_8']
K9 = ord('9') if not config['usarcodificado'] else config['bt_num_9']
K0 = ord('0') if not config['usarcodificado'] else config['bt_num_0']
KDICT = {
    K1: '1',
    K2: '2',
    K3: '3',
    K4: '4',
    K5: '5',
    K6: '6',
    K7: '7',
    K8: '8',
    K9: '9',
    K0: '0'
}


class LoginFormSigPdv(QtGui.QDialog):
    def __init__(self, parent=None, db='C', superuser=False):
        QtGui.QDialog.__init__(self, parent, QtCore.Qt.Dialog | QtCore.Qt.FramelessWindowHint)
        self.setAttribute(QtCore.Qt.WA_TranslucentBackground)
        self.setModal(True)
        self.estacionamento = False
        self.teclado = False
        self.btn_cancel_info_dialog = False
        self.cancel_pinpad = False

        self._tbl_operadores = SupervisorCadOperadores

        self.ui = Ui_Dialog()
        self.ui.setupUi(self)
        try:
            # self._tbl_operadores.get_by(supope_codigo=1)
            pass
        except Exception as e:
            from sigpdv.dav_pve.pve import exception_message_box

            exception_message_box(str(e))
            sys.exit()
        self.parent = parent
        self.retorno = False
        # SIGNALS
        self._CONTROL_FOCUS = self.ui.editPwd
        self.ui.editPwd.setFocus()
        # KeysNumPad
        self.ui.btnKEY_0.clicked.connect(self.KEY_0_Click)
        self.ui.btnKEY_1.clicked.connect(self.KEY_1_Click)
        self.ui.btnKEY_2.clicked.connect(self.KEY_2_Click)
        self.ui.btnKEY_3.clicked.connect(self.KEY_3_Click)
        self.ui.btnKEY_4.clicked.connect(self.KEY_4_Click)
        self.ui.btnKEY_5.clicked.connect(self.KEY_5_Click)
        self.ui.btnKEY_6.clicked.connect(self.KEY_6_Click)
        self.ui.btnKEY_7.clicked.connect(self.KEY_7_Click)
        self.ui.btnKEY_8.clicked.connect(self.KEY_8_Click)
        self.ui.btnKEY_9.clicked.connect(self.KEY_9_Click)
        self.ui.btnKEY_RETURN.clicked.connect(self.KEY_RETURN_Click)
        self.ui.btnKEY_BACKSPACE.clicked.connect(self.KEY_BACKSPACE_Click)

    def setControlFocus(self, old, new, is_escape=False):
        old_name, new_name = ('', '')
        if old:
            old_name = old.objectName()
        if new:
            new_name = new.objectName()
        if not is_escape:
            self._tab_order.append(new)
        if old:
            old.setStyleSheet("")

        self._CONTROL_FOCUS = new

        if type(new) in [QtGui.QLineEdit, QtGui.QDoubleSpinBox, QtGui.QComboBox, QtGui.QDateEdit]:
            new.setStyleSheet("background-color: rgba(153, 204, 204, 255); color: rgb(0, 0, 127)")

            if type(old) in [QtGui.QDoubleSpinBox, QtGui.QDateEdit]:
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Up,
                                     QtCore.Qt.KeyboardModifiers())
                old.keyPressEvent(ev)
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Down,
                                     QtCore.Qt.KeyboardModifiers())
                old.keyPressEvent(ev)
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_End,
                                     QtCore.Qt.KeyboardModifiers())
                old.keyPressEvent(ev)

            elif type(new) in [QtGui.QDoubleSpinBox, QtGui.QDateEdit]:
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Up,
                                     QtCore.Qt.KeyboardModifiers())
                new.keyPressEvent(ev)
                ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_Down,
                                     QtCore.Qt.KeyboardModifiers())
                new.keyPressEvent(ev)
        elif type(new) == QtGui.QTableWidget:
            new.setStyleSheet('background-color: rgb(255, 252, 205)')
        try:
            new.selectAll()
            new.setFocus()
        except Exception as e:
            pass

        return True

    def _returnPressed_editPwd(self):
        return True


    def get_key_name(self, key):
        try:
            name = QtCore.Qt.Key(key).name[4:]
        except Exception as e:
            message_dlg(self, MSG_ERROR, 'Verifique no PDVCONFIG se todas as teclas do '
                                       + 'teclado codificado estao configuradas corretamente')
            return ''

        if name == 'Comma':
            name = ','
        if name == 'Period':
            name = '.'
        return name

    def keyPressEvent(self, ev):
        if not ev.key():
            return ''

        if self._CONTROL_FOCUS == self.ui.editPwd:
            ks = [K0, K1, K2, K3, K4, K5, K6, K7, K8, K9]
            if ev.key() in ks:
                self._CONTROL_FOCUS.keyPressEvent(ev)
                return self.get_key_name(ev.key())
            if not config.get('usarcodificado'):
                if ev.key() not in ks:
                    if ev.key() in [QtCore.Qt.Key_Enter.__int__(), QtCore.Qt.Key_Return.__int__()]:
                        self.KEY_RETURN_Click()
                    else:
                        self._CONTROL_FOCUS.keyPressEvent(ev)
                    return
            else:
                if ev.key() == config['bt_entrar']:
                    self.KEY_RETURN_Click()
                    return
            if ev.key() in [QtCore.Qt.Key_Enter.__int__(), QtCore.Qt.Key_Return.__int__()]:
                self.KEY_RETURN_Click()
                return

        return self.get_key_name(ev.key())

    def _on_key_click(self, key):
        self._tempo_ocioso = 0
        text = chr(key) if key < 256 else ''
        text = text if text != '-' else ''
        ev = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, key, QtCore.Qt.KeyboardModifiers(), text=text)
        obj_name = str(self._CONTROL_FOCUS.objectName()).lower()
        if type(self._CONTROL_FOCUS) in [QtGui.QLineEdit, QtGui.QDoubleSpinBox, QtGui.QDateEdit,
                                         QtGui.QComboBox] and key == QtCore.Qt.Key_Return:
            try:
                return eval('self.%s()' % obj_name)
            except Exception as err:

                self.keyPressEvent(ev)

        # BACKSPACE
        if type(self._CONTROL_FOCUS) in [QtGui.QLineEdit, QtGui.QDoubleSpinBox, QtGui.QDateEdit] and \
                        key == QtCore.Qt.Key_Backspace.__int__():
            self._CONTROL_FOCUS.selectAll()
            self._CONTROL_FOCUS.clear()
            self._CONTROL_FOCUS.selectAll()
        if type(self._CONTROL_FOCUS) in [QtGui.QLineEdit, QtGui.QDoubleSpinBox, QtGui.QDateEdit] and \
                        key == QtCore.Qt.Key_Down.__int__():
            return

        self._CONTROL_FOCUS.keyPressEvent(ev)
        if type(self._CONTROL_FOCUS) == QtGui.QListWidget:
            self._CONTROL_FOCUS.clearFocus()

    def KEY_0_Click(self):
        return self._on_key_click(QtCore.Qt.Key_0)

    def KEY_1_Click(self):
        return self._on_key_click(QtCore.Qt.Key_1)

    def KEY_2_Click(self):
        return self._on_key_click(QtCore.Qt.Key_2)

    def KEY_3_Click(self):
        return self._on_key_click(QtCore.Qt.Key_3)

    def KEY_4_Click(self):
        return self._on_key_click(QtCore.Qt.Key_4)

    def KEY_5_Click(self):
        return self._on_key_click(QtCore.Qt.Key_5)

    def KEY_6_Click(self):
        return self._on_key_click(QtCore.Qt.Key_6)

    def KEY_7_Click(self):
        return self._on_key_click(QtCore.Qt.Key_7)

    def KEY_8_Click(self):
        return self._on_key_click(QtCore.Qt.Key_8)

    def KEY_9_Click(self):
        return self._on_key_click(QtCore.Qt.Key_9)

    def KEY_BACKSPACE_Click(self):
        self.ui.editPwd.setFocus()
        return self._on_key_click(QtCore.Qt.Key_Backspace)

    def KEY_RETURN_Click(self):
        if self._CONTROL_FOCUS == self.ui.editPwd:
            self._CONTROL_FOCUS = self.ui.editPwd
        return self.accept()

    def accept(self):
        str_pwd = self.ui.editPwd.text()
        if not str_pwd:
            QtGui.QMessageBox.StandardButtons(QtGui.QMessageBox.Ok)
            self.ui.editPwd.setFocus()
            QtGui.QMessageBox.information(None, 'Login', self.trUtf8(
                'Senha inválida.'),
                                          QtGui.QMessageBox.StandardButtons(
                                              QtGui.QMessageBox.Ok))
        else:
            if str_pwd == '7012':
                super(LoginFormSigPdv, self).accept()
            else:
                QtGui.QMessageBox.information(None, 'Login', self.trUtf8(
                    'Senha inválida.'),
                                              QtGui.QMessageBox.StandardButtons(
                                                  QtGui.QMessageBox.Ok))


def start():
    app = QtGui.QApplication(sys.argv)
    form = LoginFormSigPdv(db='S')
    form.show()
    app.exec_()

if __name__ == "__main__":
    try:
        start()
    except:
        sys.exit()
