# -*- coding: utf-8 -*-
__author__ = 'christian.douglas'

from PySide import QtGui, QtCore
from message_dialog import Ui_Dialog
import os
import sys

from sigpdv.tools.config import config
from sigpdv.tools.log import traced
import time


ROOT_PATH = os.path.dirname(os.path.abspath(__file__))
from sigpdv import root_path
MSG_INFO, MSG_ERROR, MSG_QUESTION, MSG_ALL, MSG_NONE = range(5)

MSG_NAMES = {
    MSG_INFO: u'Informação',
    MSG_ERROR: u'Erro',
    MSG_QUESTION: u'Questão',
    MSG_ALL: u'None',
    MSG_NONE: u'Informação: Pressione a Tecla SAIR/ESC'
}

NO_ROLE = QtGui.QDialogButtonBox.NoRole
YES_ROLE = QtGui.QDialogButtonBox.YesRole
ACCEPT_ROLE = QtGui.QDialogButtonBox.AcceptRole
CANCEL_ROLE = QtGui.QDialogButtonBox.RejectRole
APPLY_ROLE = QtGui.QDialogButtonBox.ApplyRole
NOME_TECLA = {config['bt_num_0']: '0',
              config['bt_num_1']: '1',
              config['bt_num_2']: '2',
              config['bt_num_3']: '3',
              config['bt_num_4']: '4',
              config['bt_num_5']: '5',
              config['bt_num_6']: '6',
              config['bt_num_7']: '7',
              config['bt_num_8']: '8',
              config['bt_num_9']: '9',
              config['bt_entrar']: 'Enter',
              config['bt_limpar']: 'Limpa',
              config['bt_sair']: 'Sair',
              16777220: 'Enter',
              16777221: 'Enter',
              16777219: 'Limpa',
              16777216: 'Sair',}

def _(string):
    if type(string) == unicode:
        string = string.encode('utf-8', 'replace')
    return string


class MessageDialog(QtGui.QDialog):
    def __init__(self, parent=None, focus=2):
        QtGui.QDialog.__init__(self, parent, QtCore.Qt.Dialog | QtCore.Qt.FramelessWindowHint)
        self.focus_question = focus
        self.ui = Ui_Dialog()
        self.ui.setupUi(self)
        self.setAttribute(QtCore.Qt.WA_TranslucentBackground)
        self.setModal(True)

        self.ui.buttonBox.clicked.connect(self._button_click)
        self.ui.plain_text_detail.setVisible(False)
        self.ui.lbl_codigo_barras.setVisible(False)
        self._filename = None
        self.retval = None
        self.botao_selecionado = '' # texto do botao selecionado da caixa de mensagem
        if parent:
            parent._close_window = False
        self.parent = parent
        self._close = False
        self._timer = QtCore.QTimer(self)
        self.ui.buttonBox.setFocus()
        if self.parent and self.parent.estacionamento and self.parent.barcode_active:
            scanner_timer = QtCore.QTimer(self)
            scanner_timer.timeout.connect(self._active_scanner_timer_msg)
            self._timer = scanner_timer
            self._timer.start(100)
#        self._timer.timeout.connect(self.timer)
#        self._timer.start(1000)

    @traced
    def _button_click(self, b):
        ret_cmd = False
        print self.ui.buttonBox.buttonRole(b), self._filename
        if self._filename and self.ui.buttonBox.buttonRole(b) == APPLY_ROLE:
            if os.path.exists(self._filename):
                dirapp = os.path.join(root_path, 'qtedit', 'edit.py')
                if not os.path.exists(dirapp):
                    dirapp = os.path.join(root_path, 'qtedit', 'edit.pyc')
                cmd = 'python %s %s&' % (dirapp, self._filename)
                traced.info('Executar editor de texto: %s' % cmd)
                ret_cmd = os.system(cmd)
            else:
                traced.warning('Arquivo não encontrado: %s' % self._filename)
            self.close()
        if self.ui.buttonBox.buttonRole(b) == YES_ROLE and self.parent \
                and  self.parent.btn_cancel_info_dialog:
            self.parent.btn_cancel_info_dialog = False
            self.parent.cancel_pinpad = True

        return ret_cmd

    def set_caption(self, text):
        self.ui.label_caption.setText(text)
        return text

    def set_info_text(self, text):
        text = self.trUtf8(text)
        msg = '<h3><font color="white">%s</font></h3>' % text.replace('\n', '<br></br>')
        self.ui.label_info.setText(msg)
        return _(text)

    def set_cb_text(self, text):
        msg = '<h3><font color="white">%s</font></h3>' % text.replace('\n', '<br></br>')
        self.ui.lbl_codigo_barras.setText(msg)
        self.ui.lbl_codigo_barras.setVisible(True)
        return _(text)

    def set_detail(self, text):
        if text:
            self.ui.plain_text_detail.setVisible(True)
            self.ui.plain_text_detail.setPlainText(self.trUtf8(text))
        else:
            self.ui.plain_text_detail.setVisible(False)

    def set_file(self, filename):
        self.set_detail(filename)
        if not config.get('usarcodificado'):
            self.ui.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.NoButton)
            self.ui.buttonBox.addButton(u'&Abrir', APPLY_ROLE)
            b = self.ui.buttonBox.addButton(u'&Ok', YES_ROLE)
            self._filename = filename
            b.setFocus()
        # if filename:
        #     text = filename
        #     self._filename = filename
        #     self.ui.plain_text_detail.setVisible(False)
        #     self.ui.label_file.setText(text)
        #     self.ui.label_file.setVisible(True)
        # else:
        #     self.ui.plain_text_detail.setVisible(False)

    def set_type(self, type):
        str_type = MSG_NAMES[type]
        if type == MSG_INFO:
            self.ui.label_icon.setPixmap(QtGui.QPixmap(os.path.join(ROOT_PATH, 'img', 'dialog-information.svg')))
            self.set_caption(u'Informação')
            self.ui.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Ok)
            if self.parent and self.parent.teclado:
                self.parent.teclado.envia_botoes_display('Ok')

        elif type == MSG_ERROR:
            self.ui.label_icon.setPixmap(QtGui.QPixmap(os.path.join(ROOT_PATH, 'img', 'dialog-error.svg')))
            self.set_caption('Erro')
            self.ui.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel)
            if self.parent and self.parent.teclado:
                self.parent.teclado.envia_botoes_display('Cancel')

        elif type == MSG_QUESTION:
            self.ui.label_icon.setPixmap(QtGui.QPixmap(os.path.join(ROOT_PATH, 'img', 'dialog-warning.svg')))
            self.set_caption(u'Questão')
            self.ui.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.NoButton)
            b = self.ui.buttonBox.addButton(u'&Sim', YES_ROLE)
            b_no = self.ui.buttonBox.addButton(u'&Não', NO_ROLE)
            if self.focus_question == 2:
                b.setFocus()
            else:
                b_no.setFocus()

            if self.parent and self.parent.teclado:
                self.parent.teclado.envia_botoes_display('Sim')

        elif type == MSG_ALL:
            self.ui.label_icon.setPixmap(QtGui.QPixmap("img/dialog-information.svg"))
            self.set_caption(u'Informação')
            self.ui.buttonBox.setStandardButtons(
                QtGui.QDialogButtonBox.Ok | QtGui.QDialogButtonBox.Cancel | QtGui.QDialogButtonBox.Yes | QtGui.QDialogButtonBox.No | QtGui.QDialogButtonBox.Open)

        elif type == MSG_NONE:
            self.set_caption(u'Informação: Pressione a Tecla SAIR/ESC')
            self.ui.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.NoButton)
            if self.parent and self.parent.teclado:
                self.parent.teclado.envia_botoes_display('Pressione a Tecla SAIR/ESC')


    def done(self, ev):
#        self._timer.stop()
        if self.parent:
            self.parent._close_window = True
            self.parent.activateWindow()
        self._close = True
        try:
            self.retval = self.ui.buttonBox.buttonRole(self.ui.buttonBox.focusWidget())
        except:
            self.retval = self.ui.buttonBox.NoButton
            pass
        return super(MessageDialog, self).done(ev)

    def _active_scanner_timer_msg(self):
        cod = ''
        if self.parent.estacionamento:
            try:
                if QtGui.QApplication.activeWindow():
                    cod = self.parent.barcode.get_code()
            except Exception as e:
                cod = ''

            if cod:
                self.parent.codigo_estacionamento = cod
                self.set_cb_text(self.parent.codigo_estacionamento)
                self.accept()
                self.retval = self.ui.buttonBox.YesRole


    def get_tecla(self, ev):
        traced.info('Tecla pressionada: %s' % (NOME_TECLA.get(ev.key())) or ev.key())
        if ev.key() == config['bt_num_0'] or ev.key() == QtCore.Qt.Key_0.__int__():
            self.parent.codigo_estacionamento += '0'
        elif ev.key() == config['bt_num_1']:
            self.parent.codigo_estacionamento += '1'
        elif ev.key() == config['bt_num_2']:
            self.parent.codigo_estacionamento += '2'
        elif ev.key() == config['bt_num_3']:
            self.parent.codigo_estacionamento += '3'
        elif ev.key() == config['bt_num_4']:
            self.parent.codigo_estacionamento += '4'
        elif ev.key() == config['bt_num_5']:
            self.parent.codigo_estacionamento += '5'
        elif ev.key() == config['bt_num_6']:
            self.parent.codigo_estacionamento += '6'
        elif ev.key() == config['bt_num_7']:
            self.parent.codigo_estacionamento += '7'
        elif ev.key() == config['bt_num_8']:
            self.parent.codigo_estacionamento += '8'
        elif ev.key() == config['bt_num_9']:
            self.parent.codigo_estacionamento += '9'
        elif ev.key() in (config['bt_entrar'], 16777220, QtCore.Qt.Key_Enter.__int__()):
            self.accept()
            self.retval = self.ui.buttonBox.YesRole
        elif ev.key() in (config['bt_limpar'], 16777219):
            self.parent.codigo_estacionamento = self.parent.codigo_estacionamento[:-1]

        self.set_cb_text(self.parent.codigo_estacionamento)


    def keyPressEvent(self, ev):
        if not ev.key():
            return

        if self.parent and self.parent.estacionamento:
            self.get_tecla(ev)

        if config.get('usarcodificado'):
            if ev.key() in [config['bt_cima'], config['bt_direita']]:
                self.ui.buttonBox.focusNextChild()
            elif ev.key() in [config['bt_baixo'], config['bt_esquerda']]:
                self.ui.buttonBox.focusPreviousChild()
            elif ev.key() == config['bt_entrar']:
                bt_focus = self.ui.buttonBox.focusWidget()
                bt_focus.emit(QtCore.SIGNAL("clicked()"))
            elif ev.key() == config['bt_sair']:
                self.reject()
                self.retval = self.ui.buttonBox.NoRole
            if self.parent and self.parent.teclado:
                # evita que envie varias vezes o mesmo texto para o display
                if self.botao_selecionado != self.focusWidget().text():
                    self.botao_selecionado = self.focusWidget().text()
                    self.parent.teclado.envia_botoes_display(self.focusWidget().text())
        else:
            if ev.key() == QtCore.Qt.Key_Escape.__int__():
                self.reject()
                self.retval = self.ui.buttonBox.NoRole

@traced
def message_dlg(parent=None, type=MSG_INFO, text='', detail='', file='', focus=2):
    def _limpa_texto(texto):
        '''
        Retira as tegs do texto
        :param texto:
        :return: retorna o texto limpo
        '''
        aux = ''
        grava = True
        for i in texto:
            if i == '<':
                grava = False

            elif i == '>':
                if aux:
                    aux += ' '
                grava = True

            elif grava:
                aux += i

        texto = aux
        return texto

    qtTranslator = QtCore.QTranslator()
    if parent and parent.teclado:
        limpa_display = True
        if text:
            limpa_display = parent.linhas_display == 4 and False or True
            texto = text
            if '<' in text and '>' in text: # quando tem alguma tag no texto
                texto = _limpa_texto(text)
            parent.teclado.envia_display(texto)
        if detail:
            texto = detail if not text else '\n' + detail
            if '<' in text and '>' in text: # quando tem alguma tag no texto
                texto = _limpa_texto(text)
            parent.teclado.envia_display(texto, limpa_display=limpa_display)

    msg_box = MessageDialog(parent=parent, focus=focus)
    msg_box.set_type(type)
    text = _(text)
    if type == MSG_ERROR and not detail:
        msg_box.set_info_text(text)
#        if len(text.args) > 1:
#            msg_box.set_detail(text.args[1].decode('utf-8'))
    else:
        msg_box.set_info_text(text)
    if detail:
        msg_box.set_detail(detail)
    if file:
        msg_box.set_file(file)
    if parent:
        parent._message_box = msg_box

    msg_box.exec_()
    res = msg_box.retval
    if parent and parent.teclado:
        parent.teclado.limpar_display() # limpa o display do teclado

    return res

if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    qtTranslator = QtCore.QTranslator()
    print '----', os.path.join(root_path,  'qt_pt.qm')
    if qtTranslator.load(os.path.join(root_path,  'qt_pt.qm')):
        print('INSTALADO LN')
        #app.installTranslator(qtTranslator)
    #message_dlg(None, MSG_ERROR, 'O Dav #%010d possui restrições.', ' ')
    msg = '<center><h2><font color="red">A Quantidade informada está incorreta.</font></h2> ' \
          '<center><h2><font color="white">Pressione <font color="yellow">[Sair]</font> para digitar a quantidade ' \
          'correta.</font></h2>'



    print '>>>', message_dlg(None, MSG_QUESTION, msg, file=__file__)

    application = app
