# -*- coding: utf-8 -*-
__author__ = 'felipe.franca'

from PySide import QtGui, QtCore
from ui_message_dialog_autoatende import Ui_Dialog
import os
import sys

from sigpdv.tools.config import config
from sigpdv.tools.log import traced


ROOT_PATH = os.path.dirname(os.path.abspath(__file__))
from sigpdv import root_path
MSG_INFO, MSG_ERROR, MSG_QUESTION, MSG_ALL, MSG_NONE = range(5)

MSG_NAMES = {
    MSG_INFO: u'Informação',
    MSG_ERROR: u'Erro',
    MSG_QUESTION: u'Questão',
    MSG_ALL: u'None',
    MSG_NONE: u'Informação: Pressione a Tecla SAIR/ESC'
}

NO_ROLE = QtGui.QDialogButtonBox.NoRole
YES_ROLE = QtGui.QDialogButtonBox.YesRole
ACCEPT_ROLE = QtGui.QDialogButtonBox.AcceptRole
CANCEL_ROLE = QtGui.QDialogButtonBox.RejectRole
APPLY_ROLE = QtGui.QDialogButtonBox.ApplyRole
NOME_TECLA = {config['bt_num_0']: '0',
              config['bt_num_1']: '1',
              config['bt_num_2']: '2',
              config['bt_num_3']: '3',
              config['bt_num_4']: '4',
              config['bt_num_5']: '5',
              config['bt_num_6']: '6',
              config['bt_num_7']: '7',
              config['bt_num_8']: '8',
              config['bt_num_9']: '9',
              config['bt_entrar']: 'Enter',
              config['bt_limpar']: 'Limpa',
              config['bt_sair']: 'Sair',
              16777220: 'Enter',
              16777221: 'Enter',
              16777219: 'Limpa',
              16777216: 'Sair',}

def _(string):
    if type(string) == unicode:
        string = string.encode('utf-8', 'replace')
    return string


class MessageDialogAutoAtende(QtGui.QDialog):
    def __init__(self, parent=None, focus=2):
        QtGui.QDialog.__init__(self, parent, QtCore.Qt.Dialog | QtCore.Qt.FramelessWindowHint)
        self.focus_question = focus
        self.ui = Ui_Dialog()
        self.ui.setupUi(self)
        self.setAttribute(QtCore.Qt.WA_TranslucentBackground)
        self.setModal(True)

        self.ui.btnKEY_CPF.clicked.connect(self._button_CPF_click)
        self.ui.btnKEY_CANCELAR.clicked.connect(self._button_CANCELA_click)
        self._filename = None
        self.retval = None
        self.botao_selecionado = '' # texto do botao selecionado da caixa de mensagem
        if parent:
            parent._close_window = False
        self.parent = parent
        self._close = False
        self._timer = QtCore.QTimer(self)
        self.ui.btnKEY_CANCELAR.setFocus()


    @traced
    def _button_CPF_click(self):
        self.retval = 1
        return self.done(0)

    @traced
    def _button_CANCELA_click(self):
        self.retval = 0
        return self.done(0)

    def set_caption(self, text):
        self.ui.label_caption.setText(text)
        return text

    def set_info_text(self, text):
        text = self.trUtf8(text)
        return _(text)

    def set_cb_text(self, text):
        msg = '<h3><font color="white">%s</font></h3>' % text.replace('\n', '<br></br>')
        self.ui.lbl_codigo_barras.setText(msg)
        self.ui.lbl_codigo_barras.setVisible(True)
        return _(text)

    def set_detail(self, text):
        if text:
            self.ui.plain_text_detail.setVisible(True)
            self.ui.plain_text_detail.setPlainText(self.trUtf8(text))
        else:
            self.ui.plain_text_detail.setVisible(False)

    def set_file(self, filename):
        self.set_detail(filename)
        self._filename = filename

    def set_type(self, type):
        str_type = MSG_NAMES[type]
        if type == MSG_INFO:
            self.ui.label_icon.setPixmap(QtGui.QPixmap(os.path.join(ROOT_PATH, 'img', 'dialog-information.svg')))
            self.set_caption(u'Informação')


    def done(self, ev):
#        self._timer.stop()
        if self.parent:
            self.parent._close_window = True
            self.parent.activateWindow()
        self._close = True
        try:
            self.ui.btnKEY_CANCELAR.buttonRole(self.ui.btnKEY_CANCELAR.focusWidget())
        except:
            pass
        return super(MessageDialogAutoAtende, self).done(ev)


    def keyPressEvent(self, ev):
        if not ev.key():
            return

        if ev.key() == QtCore.Qt.Key_Escape.__int__():
            self.reject()

@traced
def message_dlg_autoatende(parent=None, type=MSG_INFO, text='', detail='', file='', focus=2):
    qtTranslator = QtCore.QTranslator()
    msg_box = MessageDialogAutoAtende(parent=parent, focus=focus)
    msg_box.set_type(type)
    text = _(text)
    if type == MSG_ERROR and not detail:
        msg_box.set_info_text(text)
#        if len(text.args) > 1:
#            msg_box.set_detail(text.args[1].decode('utf-8'))
    else:
        msg_box.set_info_text(text)
    if detail:
        msg_box.set_detail(detail)
    if file:
        msg_box.set_file(file)
    if parent:
        parent._message_box = msg_box

    msg_box.exec_()
    res = msg_box.retval
    return res

if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    qtTranslator = QtCore.QTranslator()
    print '----', os.path.join(root_path,  'qt_pt.qm')
    if qtTranslator.load(os.path.join(root_path,  'qt_pt.qm')):
        print('INSTALADO LN')
        #app.installTranslator(qtTranslator)
    #message_dlg(None, MSG_ERROR, 'O Dav #%010d possui restrições.', ' ')
    msg = '<center><h2><font color="red">A Quantidade informada está incorreta.</font></h2> ' \
          '<center><h2><font color="white">Pressione <font color="yellow">[Sair]</font> para digitar a quantidade ' \
          'correta.</font></h2>'



    print '>>>', message_dlg(None, MSG_QUESTION, msg, file=__file__)

    application = app
